@@ -50,7 +50,7 @@ def main():
   instHandle = "cubit"
   serverID = "cubit-server"
 
-  s = ilu.CreateServer(serverID, ("tcp_0_0",), "IIOP_1_0")
+  s = ilu.CreateServer(serverID, ("tcp_0_0",), "iiop_1_0_1")
   uc = realCubit(instHandle, s)
 
   uc.IluPublish()
