@@ -1,4 +1,4 @@
-/* $Id: webserver.c,v 1.6 1996/07/16 23:19:34 janssen Exp $ */
+/* $Id: webserver.c,v 1.13 1997/11/25 05:44:10 janssen Exp $ */
 
 /* very simple brute force web server program
 
@@ -13,6 +13,11 @@
 #include <stdio.h>
 #include <time.h>
 
+/* for file information */
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+
 #if (defined WIN32 || defined WIN16)
 #include <winsock.h>
 #include <io.h>
@@ -26,12 +31,12 @@
 #define CLOSE_FUNCTION _close
 #define RDONLY_MODE _O_RDONLY
 #else
-#include <sys/types.h>
+#include <unistd.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
-#include <unistd.h>
+#include <ctype.h>
 #define CHDIR_FUNTION chdir
 #define GETCWD_FUNTION getcwd
 #define FSTAT_FUNCTION fstat
@@ -40,20 +45,19 @@
 #define READ_FUNCTION read
 #define CLOSE_FUNCTION close
 #define RDONLY_MODE O_RDONLY
-extern int gethostname(char *name, int namelen);
 #endif
 
-/* for file information */
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
 
 /* include internals header (for NIL) */
 #include <iluntrnl.h>
 
+#ifndef HAVE_GETHOSTNAME_PROTOTYPE
+extern int gethostname(char *name, int namelen);
+#endif /* ndef HAVE_GETHOSTNAME_PROTOTYPE */
+
+
 /* include header(s) defining the interfaces */
-#include "http.h"
+#include "iluhttp.h"
 
 
 #define ILU_TEST_DOMAIN "parc.xerox.com"	/* string used in our ilu server name */
@@ -70,9 +74,10 @@ ilu_boolean g_b_verbose = ilu_FALSE;		/* whether to output info */
 char gc_file_base[1024];					/* base location of files */
 
 char g_c_usage[] =							/* Usage string */
-"Usage:  webserver [port_number [file_base [verbose] ] ]\n\n\
+"Usage:  webserver [port_number [file_base [pinfo [verbose] ] ] ]\n\n\
 \tport_number - tcp port the webserver should use - default 80\n\
 \tfile_base - Root location of where server files are stored - default \\htmldocs \n\n\
+\tpinfo - one of http_1_0 (default), or http_1_0p (persistent connections)\n\
 \tverbose - if anthing there as 2nd arg, the program\n\
 \toutputs information about what's going on.";
 
@@ -211,8 +216,8 @@ static CORBA_Object object_of_instance_handle (ilu_string str_object_id,
 											   ilu_private p_iluserver) {
 
 	/* simply return a new true object with this id  */
-	return (http_Resource__OTCreateTrue (str_object_id,
-				       *((ilu_Server *) p_iluserver),
+	return (iluhttp_Resource__OTCreateTrue (str_object_id,
+				       *((ILU_C_Server *) p_iluserver),
 				       NIL)); 
 }
 
@@ -231,15 +236,16 @@ int main(int ac, char **av) {
 
 	char pc_hostname[1024];						/* holds the name of the host the server is on */
 	char pc_serverid[1024];						/* holds the server id */
-	ilu_Server webserver;						/* server for http web objects */
-	char* pc_protocol_info = "http_1_0";		/* denotes use of http as the protocol */
+	ILU_C_Server webserver;						/* server for http web objects */
+	char* pc_protocol_info_default = "http_1_0p";		/* denotes use of http (with persistence) as the protocol */
+	char* pc_protocol_info = pc_protocol_info_default;
 	ILU_C_ObjectTable object_table;				/* will be our object table */
 
 	/* webserver defaults to run at port 80 on top of tcp/ip */
 	char* ppc_base_transport_info[2] = { "tcp_0_80", ILU_NIL };
 	char  c_alternative_tcp_port_info[64];
 	
-	if (ac > 4) {			/* check for proper command line args */
+	if (ac > 5) {			/* check for proper command line args */
 		fprintf (stderr, "%s\n", g_c_usage);
 		exit(1);
 	}
@@ -255,13 +261,16 @@ int main(int ac, char **av) {
 	}
 	else strcpy(gc_file_base, "\\htmldocs");
 
+	if (ac > 3) /* pinfo */
+		pc_protocol_info = av[3];
+
 	gethostname (pc_hostname, sizeof(pc_hostname));
 
-	if (ac > 3)
+	if (ac > 4)
 		g_b_verbose = ilu_TRUE;
 
 
-	http__InitializeServer();	/* perform server initialization */
+	iluhttp__InitializeServer();	/* perform server initialization */
 
 	/* make up an object table we can use to create up objects representing
 	the resources (usually documents stored in files) on the fly */
@@ -379,21 +388,21 @@ char* get_current_time_string () {
 /* ********************************************************* */
 /* build up a simple response                                */
 
-http_Response* build_simple_response (http_StatusCode status, char* pc_body_contents, 
+iluhttp_Response* build_simple_response (iluhttp_StatusCode status, char* pc_body_contents, 
 									  ilu_cardinal card_body_length) {
 
-	http_Response* p_the_response;			/* holds our response */ 
-	http_Header* p_date_header;				/* holds our Date header */
-	http_Header* p_length_header;			/* holds Content-length header */
+	iluhttp_Response* p_the_response;			/* holds our response */ 
+	iluhttp_Header* p_date_header;				/* holds our Date header */
+	iluhttp_Header* p_length_header;			/* holds Content-length header */
 	char c_buffer[64];
 
-	p_the_response = (http_Response*) ilu_malloc(sizeof(http_Response));	/* create empty response */
+	p_the_response = (iluhttp_Response*) ilu_malloc(sizeof(iluhttp_Response));	/* create empty response */
 
 	p_the_response->status = status;					/* set it to indicate the passed status */ 
 
 	if (card_body_length != 0) { /* we don't want ilu http to auto generate content-length */
 
-		p_date_header = (http_Header*) ilu_malloc(2*sizeof(http_Header));		/* create headers */
+		p_date_header = (iluhttp_Header*) ilu_malloc(2*sizeof(iluhttp_Header));		/* create headers */
 
 		p_date_header->name = duplicate_c_string("Date");				/* make a General header containing Date */
 		p_date_header->value = get_current_time_string();
@@ -410,7 +419,7 @@ http_Response* build_simple_response (http_StatusCode status, char* pc_body_cont
 	}
 	else { /* let ilu http auto generate content-length header */
 
-		p_date_header = (http_Header*) ilu_malloc(sizeof(http_Header));		/* create headers */
+		p_date_header = (iluhttp_Header*) ilu_malloc(sizeof(iluhttp_Header));		/* create headers */
 
 		p_date_header->name = duplicate_c_string("Date");				/* make a General header containing Date */
 		p_date_header->value = get_current_time_string();
@@ -422,7 +431,7 @@ http_Response* build_simple_response (http_StatusCode status, char* pc_body_cont
 
 
 	if (pc_body_contents != NIL) {
-		p_the_response->body = (http_OptionalEntityBody) ilu_malloc (sizeof(http_EntityBody));
+		p_the_response->body = (iluhttp_OptionalEntityBody) ilu_malloc (sizeof(iluhttp_EntityBody));
 		p_the_response->body->_buffer = (ilu_bytes) duplicate_c_string(pc_body_contents);
 		p_the_response->body->_length = strlen(pc_body_contents);
 		p_the_response->body->_maximum = p_the_response->body->_length;
@@ -438,9 +447,9 @@ http_Response* build_simple_response (http_StatusCode status, char* pc_body_cont
 /* ********************************************************* */
 /* printout a request                                        */
 
-void print_request (http_Request* p_the_request) {
+void print_request (iluhttp_Request* p_the_request) {
 
-	http_Header* p_the_header;			/* used to point to headers */
+	iluhttp_Header* p_the_header;			/* used to point to headers */
 	unsigned long ul_index;				/* used as index into sequences */
 
 	printf("Request: (Note all values are shown between >< s)\n");
@@ -479,10 +488,150 @@ void print_request (http_Request* p_the_request) {
 
 
 /* ********************************************************* */
-/* this function basically does the real work for either get or post */
+/*  (brute force) Routine used when the file being retrieved has the extension
+    iluhttpresponse (a file containing the complete http response)
 
-http_Response* server_http_Resource_GETorPOST (http_Resource http_obj, 
-												  http_Request* p_http_req,
+Line format of an .iluhttpresponse file:
+
+Status
+Number of headers
+headername headervalue
+...
+headername headervalue
+body
+
+
+
+For example:
+
+302
+2
+Location: http://pundit.parc.xerox.com/foo.html
+Content-Length: 62
+<a href=http://pundit.parc.xerox.com/foo.html>New Location</a>
+
+
+*/
+
+iluhttp_Response* build_response_from_file(char* pc_filename, ilu_boolean b_include_body) {
+	
+	iluhttp_Response* p_the_response;	/* for building up our response */
+	int i_response_code;
+	int i_num_headers;
+	iluhttp_Header* p_header;
+	int i_index;
+	char c_header_name[128];
+	char c_header_value[256];
+	int i_num_scanned;
+	char* pc_value;
+	FILE* p_file;
+	int i_temp;
+	fpos_t filepos_current;
+	long l_current_pos;
+	long l_end_pos;
+	long l_body_size;
+	unsigned char* pc_body_bytes;
+	char* pc_walker;
+				
+	if ((p_file = fopen(pc_filename, "rb")) == NULL)
+		return NULL;
+	
+	/* get response code and number of headers */
+	if (fscanf(p_file, "%i\n%i\n", &i_response_code, &i_num_headers) != 2) {
+		fclose(p_file);
+		return NULL;
+	}
+	
+	/* build up our response */
+	p_the_response = (iluhttp_Response*) ilu_malloc(sizeof(iluhttp_Response));	/* create empty response */
+	
+	p_the_response->status = (iluhttp_StatusCode)i_response_code;
+	
+	
+	if (i_num_headers > 0) /* create space for headers */
+		p_the_response->headers._buffer = (iluhttp_Header*) ilu_malloc(i_num_headers * sizeof(iluhttp_Header));
+	else 
+		p_the_response->headers._buffer = NULL;
+	
+	p_the_response->headers._maximum = i_num_headers;
+	p_the_response->headers._length = i_num_headers;
+	p_header = p_the_response->headers._buffer;
+	
+	/* read in the headers */
+	for (i_index = 0; i_index < i_num_headers; i_index++) {
+		c_header_name[0] = '\0';
+		c_header_value[0] = '\0';
+		
+		/* read in header name */
+		i_num_scanned = fscanf(p_file, "%[^:] :", c_header_name);
+		
+		if (i_num_scanned != 1) { /* problem - free things up */
+			for (i_temp = 0; i_temp < i_index; i_temp++) {
+				ilu_free(((p_the_response->headers._buffer)[i_temp]).name);
+				ilu_free(((p_the_response->headers._buffer)[i_temp]).value);
+			}
+			ilu_free(p_the_response->headers._buffer);
+			ilu_free(p_the_response);
+			fclose (p_file);
+			return NULL;
+		}
+		
+		/* read in header value if any */
+		pc_value = fgets(c_header_value, 256, p_file);
+		if (pc_value) /* non empty header value */ {
+			c_header_value[strlen(c_header_value) - 1] = '\0';  /* remove \n */
+			while (isspace(*pc_value)) /* remove any leading whitespace */
+				pc_value++;
+			if (pc_value[0] == '\0')
+				pc_value = NULL;
+			if (pc_value) { /* remove any trailing whitespace */
+				pc_walker = pc_value + strlen(pc_value) - 1;
+				while (isspace(*pc_walker))
+					pc_walker--;
+				*(pc_walker + 1) = '\0';
+			}
+		}
+		
+		/* assign copies of name and value to header */
+		p_header->name = duplicate_c_string(c_header_name);
+		p_header->value = pc_value ? duplicate_c_string(c_header_value) : NULL;
+		p_header++;
+	}
+	
+	if (b_include_body) { /* if we should add the body */
+		/* get a buffer of the file contents */
+		
+		l_current_pos = ftell(p_file);
+		fgetpos(p_file, &filepos_current);
+		fseek(p_file, 0, SEEK_END);
+		l_end_pos = ftell(p_file);
+		fsetpos(p_file, &filepos_current);
+		l_body_size = l_end_pos - l_current_pos;	
+		
+		pc_body_bytes = ilu_malloc(l_body_size);
+		fread(pc_body_bytes, l_body_size, l_body_size, p_file);
+		
+		/* and stuff on our body bytes */
+		p_the_response->body = (iluhttp_OptionalEntityBody) ilu_malloc (sizeof(iluhttp_EntityBody));
+		p_the_response->body->_buffer = pc_body_bytes;
+		p_the_response->body->_length = l_body_size;
+		p_the_response->body->_maximum = l_body_size;
+	}
+	else p_the_response->body = NIL;
+	
+	/* close the file */
+	fclose(p_file);
+
+	return p_the_response;
+}
+
+
+
+/* ********************************************************* */
+/* this function basically does the real work for either get or head */
+
+iluhttp_Response* server_iluhttp_Resource_GETorHEAD (iluhttp_Resource http_obj, 
+												  iluhttp_Request* p_http_req,
 												  ilu_boolean b_include_body,
 												  ILU_C_ENVIRONMENT* ilu_env) {
 
@@ -494,8 +643,8 @@ http_Response* server_http_Resource_GETorPOST (http_Resource http_obj,
 	char* pc_type_string;			/* points to MIME type string */
 	char* pc_extension;				/* points to file extension */
 	ilu_bytes pc_body_bytes;		/* points to body buffer */
-	http_Response* p_the_response;	/* for building up our response */
-	http_Header* p_header;
+	iluhttp_Response* p_the_response;	/* for building up our response */
+	iluhttp_Header* p_header;
 
 	/* make up the full file name, defaulting to the default file name if necessary */
 	if (strcmp(p_http_req->URI, "/") == 0) /*  */
@@ -511,23 +660,53 @@ http_Response* server_http_Resource_GETorPOST (http_Resource http_obj,
 
 	if (validate_path(c_filename, gc_file_base) == ilu_FALSE) {
 		/* return not found if we've gone outside our root */
-		return build_simple_response(http_NotFound, NIL, 0);
+		return build_simple_response(iluhttp_NotFound, NIL, 0);
 	}
 
 	/* open up the file */
 	h_file = OPEN_FUNCTION(c_filename, RDONLY_MODE); 
 	if ((h_file == -1) || (FSTAT_FUNCTION(h_file, &stat_struct) == -1)) {
 		/* return not found if there's a problem opening the file */
-		return build_simple_response(http_NotFound, NIL, 0);
+		return build_simple_response(iluhttp_NotFound, NIL, 0);
 	}
 
-	/* build up our response */
-	p_the_response = (http_Response*) ilu_malloc(sizeof(http_Response));	/* create empty response */
+	/* determine file extension */
+	pc_extension = c_filename + strlen(c_filename) - 1;
+	while (pc_extension > c_filename) {
+		if (*pc_extension == '.') {
+			pc_extension++;
+			break;
+		}
+		pc_extension--;
+	}
 
-	p_the_response->status = http_OK;	/* set it to indicate success */ 
+	if (pc_extension == c_filename)  /* ensure we didn't walk all the way back */
+		pc_extension = NULL;
+
+
+	if (strcmp(pc_extension, "iluhttpresponse") == 0) {
+		/* it's a file containing the complete response to send back */
+		
+		/* close the file */
+		CLOSE_FUNCTION(h_file);
+		
+		p_the_response = build_response_from_file(c_filename, b_include_body);
+
+		if (!p_the_response)  /* if had an error - respond appropriate error iluhttp_NoContent? */
+			return  build_simple_response(iluhttp_NoContent, NIL, 0);
+
+		/* return the response the file contained */
+		return p_the_response;
+	}
+
+
+	 /* build up our response */
+	p_the_response = (iluhttp_Response*) ilu_malloc(sizeof(iluhttp_Response));	/* create empty response */
+
+	p_the_response->status = iluhttp_OK;	/* set it to indicate success */ 
 
 	/* create space for headers */
-	p_the_response->headers._buffer = (http_Header*) ilu_malloc(5*sizeof(http_Header));
+	p_the_response->headers._buffer = (iluhttp_Header*) ilu_malloc(5*sizeof(iluhttp_Header));
 	p_the_response->headers._maximum = 5;
 	p_the_response->headers._length = 5;
 	p_header = p_the_response->headers._buffer;
@@ -549,19 +728,6 @@ http_Response* server_http_Resource_GETorPOST (http_Resource http_obj,
 	p_header->value = get_time_string(&(stat_struct.st_mtime));
 	p_header++;
 
-	/* determine file extension */
-	pc_extension = c_filename + strlen(c_filename) - 1;
-	while (pc_extension > c_filename) {
-		if (*pc_extension == '.') {
-			pc_extension++;
-			break;
-		}
-		pc_extension--;
-	}
-
-	if (pc_extension == c_filename)  /* ensure we didn't walk all the way back */
-		pc_extension = NULL;
-
 	if ((pc_extension != NULL) &&		/* create Content-Type header if known */
 		(pc_type_string = content_type_from_extension(pc_extension)) != NULL) {
 		p_header->name = duplicate_c_string("Content-Type");
@@ -578,7 +744,7 @@ http_Response* server_http_Resource_GETorPOST (http_Resource http_obj,
 		READ_FUNCTION(h_file, pc_body_bytes, stat_struct.st_size);
 
 		/* and stuff on our body bytes */
-		p_the_response->body = (http_OptionalEntityBody) ilu_malloc (sizeof(http_EntityBody));
+		p_the_response->body = (iluhttp_OptionalEntityBody) ilu_malloc (sizeof(iluhttp_EntityBody));
 		p_the_response->body->_buffer = pc_body_bytes;
 		p_the_response->body->_length = stat_struct.st_size;
 		p_the_response->body->_maximum = stat_struct.st_size;
@@ -597,8 +763,8 @@ http_Response* server_http_Resource_GETorPOST (http_Resource http_obj,
 /* ********************************************************* */
 /* GET                                                       */
 
-http_Response* server_http_Resource_GET (http_Resource http_obj, 
-												  http_Request* p_http_req, 
+iluhttp_Response* server_iluhttp_Resource_GET (iluhttp_Resource http_obj, 
+												  iluhttp_Request* p_http_req, 
 												  ILU_C_ENVIRONMENT* ilu_env) {
 
 	if (g_b_verbose) {
@@ -607,15 +773,15 @@ http_Response* server_http_Resource_GET (http_Resource http_obj,
 		print_request (p_http_req);
 	}
 
-	return server_http_Resource_GETorPOST(http_obj, p_http_req, TRUE, ilu_env);
+	return server_iluhttp_Resource_GETorHEAD(http_obj, p_http_req, TRUE, ilu_env);
 }
 
 
 /* ********************************************************* */
 /* HEAD                                                      */
 
-http_Response* server_http_Resource_HEAD (http_Resource http_obj, 
-												   http_Request* p_http_req, 
+iluhttp_Response* server_iluhttp_Resource_HEAD (iluhttp_Resource http_obj, 
+												   iluhttp_Request* p_http_req, 
 												   ILU_C_ENVIRONMENT* ilu_env) {
 	if (g_b_verbose) {
 		printf("\n------------------------------------------\n");
@@ -623,23 +789,82 @@ http_Response* server_http_Resource_HEAD (http_Resource http_obj,
 		print_request (p_http_req);
 	}
 
-	return server_http_Resource_GETorPOST(http_obj, p_http_req, FALSE, ilu_env);
+	return server_iluhttp_Resource_GETorHEAD(http_obj, p_http_req, FALSE, ilu_env);
 }
 
 
 /* ********************************************************* */
 /* POST                                                      */
 
-http_Response* server_http_Resource_POST (http_Resource http_obj, 
-												   http_Request* p_http_req, 
+iluhttp_Response* server_iluhttp_Resource_POST (iluhttp_Resource http_obj, 
+												   iluhttp_Request* p_http_req, 
 												   ILU_C_ENVIRONMENT* ilu_env) {
+	char c_filename[2048];			/* holds the name of of the referenced file */
+	int h_file;						/* for obtaining file information */
+	STAT_STRUCT stat_struct;
+	char* pc_walker;
+	char* pc_extension;				/* points to file extension */
+	iluhttp_Response* p_the_response;	/* for building up our response */
+	
 	if (g_b_verbose) {
 		printf("\n------------------------------------------\n");
 		printf("POST called on %s\n", p_http_req->URI);
 		print_request (p_http_req);
 	}
+	/* make up the full file name, defaulting to the default file name if necessary */
+	if (strcmp(p_http_req->URI, "/") == 0) /*  */
+		sprintf(c_filename, "%s%s", gc_file_base, DEFAULT_SERVER_FILE_NAME);
+	else
+		sprintf(c_filename, "%s%s", gc_file_base, p_http_req->URI);
 
-	return build_simple_response(http_NotImplemented, "POST not implemented", 0);
+	/* terminate properly if params or queries are present*/
+	pc_walker = c_filename;
+	while (*pc_walker && (*pc_walker != ';') && (*pc_walker != '?'))
+		pc_walker++;
+	*pc_walker = '\0';
+
+	if (validate_path(c_filename, gc_file_base) == ilu_FALSE) {
+		/* return not found if we've gone outside our root */
+		return build_simple_response(iluhttp_NotFound, NIL, 0);
+	}
+
+	/* open up the file */
+	h_file = OPEN_FUNCTION(c_filename, RDONLY_MODE); 
+	if ((h_file == -1) || (FSTAT_FUNCTION(h_file, &stat_struct) == -1)) {
+		/* return not found if there's a problem opening the file */
+		return build_simple_response(iluhttp_NotFound, NIL, 0);
+	}
+	
+	/* close the file */
+	CLOSE_FUNCTION(h_file);
+
+	/* determine file extension */
+	pc_extension = c_filename + strlen(c_filename) - 1;
+	while (pc_extension > c_filename) {
+		if (*pc_extension == '.') {
+			pc_extension++;
+			break;
+		}
+		pc_extension--;
+	}
+
+	if (pc_extension == c_filename)  /* ensure we didn't walk all the way back */
+		pc_extension = NULL;
+
+	if (strcmp(pc_extension, "iluhttpresponse") == 0) {
+		/* it's a file containing the complete response to send back */
+		
+		p_the_response = build_response_from_file(c_filename, ilu_TRUE);
+
+		if (!p_the_response)  /* if had an error - respond appropriate error iluhttp_NoContent? */
+			return  build_simple_response(iluhttp_NoContent, NIL, 0);
+
+		/* return the response the file contained */
+		return p_the_response;
+	}
+
+
+	return build_simple_response(iluhttp_NotImplemented, "POST not implemented", 0);
 }
 
 
