@@ -1,11 +1,14 @@
-/* $Id: multlang.c,v 1.5 1996/05/04 05:16:38 janssen Exp $ */
+/* $Id: multlang.c,v 1.7 1997/09/09 19:37:05 janssen Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "multlang.h"
 
-ilu_Server theServer;
+extern int Py_Initialize(void);
+extern int PyRun_SimpleString(char *);
+
+ILU_C_Server theServer;
 
 CORBA_unsigned_long
   server_multlang_Squarer_ObtainSquare (multlang_Squarer handle,
@@ -15,6 +18,7 @@ CORBA_unsigned_long
   /* dummy routine; never called */
   fprintf (stderr, "C routine server_multlang_Squarer_ObtainSquare called!\n");
   exit(1);
+  return 1;
 }
 
 CORBA_unsigned_long
