@@ -1,8 +1,11 @@
-/* $Id: server.c,v 1.5 1996/05/04 05:16:10 janssen Exp $ */
+/* $Id: server.c,v 1.7 1997/05/27 16:32:21 larner Exp $ */
 /* Last edited by Mike Spreitzer December 18, 1995 2:28 pm PST */
 
 #include <stdio.h>
+
+#ifndef WIN32
 #include <unistd.h>	/* for gethostname */
+#endif
 
 #include "objtable.h"
 
@@ -12,7 +15,7 @@ static CORBA_Object
   object_of_ih (ilu_string ih, ilu_private rock)
 {
   return (objtable_file__OTCreateTrue (ih,
-				       *((ilu_Server *) rock),
+				       *((ILU_C_Server *) rock),
 				       ILU_C_Strdup(ih)));
 }
 
@@ -43,7 +46,7 @@ objtable_file
 
 int main (int ac, char **av)
 {
-  static ilu_Server ks;
+  static ILU_C_Server ks;
   objtable_server s;
   objtable_file f;
   char *sid;
