@@ -1,5 +1,5 @@
-/* $Id: clnt.c,v 1.23 1996/07/17 16:00:49 larner Exp $ */
-/* Last edited by Mike Spreitzer June 20, 1996 10:14 am PDT */
+/* $Id: clnt.c,v 1.32 1997/11/14 00:31:31 janssen Exp $ */
+/* Last edited by Mike Spreitzer November 25, 1996 12:45 pm PST */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +40,20 @@ static void Print_RO (Test1_RO ro)
     OUTPUT("nil");
 }
 
+#define Test1_TheU__R 0
+#define Test1_TheU__RO 1
+#define Test1_TheU__CSS 2
+#define Test1_TheU__O1 3
+#define Test1_TheU__OO 4
+#define Test1_TheU__boolean 5
+
+#define Test1_U__R 0
+#define Test1_U__RO 1
+#define Test1_U__CSS 2
+#define Test1_U__O1 3
+#define Test1_U__OO 4
+#define Test1_U__boolean 5
+
 static void Print_U(Test1_U u)
 {
   switch (u._d) {
@@ -74,17 +88,19 @@ static void Print_U(Test1_U u)
 
 int doit()
 {
-  Test1_O1        handle;
-  Test1_O2        o2;
-  Test1_O3        o3;
-  float           f;
-  ILU_C_ENVIRONMENT s = {0};
-  Test1_U         u, *pu;
-  Test1_CSS       *css;
-  Test1_ScS       scs[3];
-  Test1_RO        ro;
-  Test1_R         r;
-  ilu_byte       *ap;
+#define local 
+
+local Test1_O1        handle;
+local Test1_O2        o2;
+local Test1_O3        o3;
+local float           f;
+local ILU_C_ENVIRONMENT s = {0};
+local Test1_U         u, *pu;
+local Test1_CSS       *css;
+local Test1_RO        ro;
+local Test1_R         r;
+local ilu_byte       *ap;
+local ilu_CString	foobar = "foobar";
 
 #ifdef SECURE_TRANSPORT
   /* we need to call these initializers to make sure any incoming
@@ -109,31 +125,46 @@ int doit()
 #endif
   u._d = Test1_U__boolean;
   u._u.boolean = 1;
-  scs[0] = "hello world";
-  scs[1] = "hello mars";
-  css = Test1_CSS_Create(2, scs);
+  css = Test1_CSS_Create(2, NULL);
+  Test1_CSS_Append(css, ILU_C_Strdup("hello world"));
+  Test1_CSS_Append(css, ILU_C_Strdup("hello mars"));
   pu = Test1_O1_U_CSS_to_U(handle, &u, css, &s);
   OUTPUT("Test1.O1.U-CSS-to-U() => ");
   if (!ILU_C_SUCCESSFUL(&s))
-    OUTPUT("Exception \"%s\"", s.returnCode);
+    OUTPUT("Exception \"%s\"\n", s.returnCode);
   else if (!pu)
-    OUTPUT("NIL!");
-  else
+    OUTPUT("NIL!\n");
+  else {
     Print_U(*pu);
-  OUTPUT("\n");
+    OUTPUT("\n");
+    Test1_TheU__Free(pu);
+    ilu_free(pu);
+    pu = NULL;
+  }
+  
   ro = Test1_O1_f_CSS_to_RO(handle, css, &s);
   OUTPUT("Test1.O1.f-CSS-to-RO() => ");
-  if (!ILU_C_SUCCESSFUL(&s))
-    OUTPUT("Exception \"%s\"", s.returnCode);
-  else
+  if (!ILU_C_SUCCESSFUL(&s)) {
+    OUTPUT("Exception \"%s\"\n", s.returnCode);
+    ro = NULL;
+  } else {
     Print_RO(ro);
-  OUTPUT("\n");
-
-  f = Test1_O1_R_ScS_to_F(handle, ro, scs[0], &s);
-  if (ILU_C_SUCCESSFUL(&s))
-    OUTPUT("f=%f\n", f);
-  else
-    OUTPUT("Test1.O1.R-ScS-to-F() raises \"%s\"\n", s.returnCode);
+    OUTPUT("\n");
+  }
+  Test1_CSS__Free(css);
+  ilu_free(css);
+  css = NULL;
+  
+  if (ro) {
+    f = Test1_O1_R_ScS_to_F(handle, ro, "a shortchar seq", &s);
+    if (ILU_C_SUCCESSFUL(&s))
+      OUTPUT("f=%f\n", f);
+    else
+      OUTPUT("Test1.O1.R-ScS-to-F() raises \"%s\"\n", s.returnCode);
+    Test1_TheR__Free(ro);
+    ilu_free(ro);
+    ro = NULL;
+  }
 
   Test1_O1_a_RO(handle, ILU_NIL, &s);
 
@@ -156,6 +187,8 @@ int doit()
       OUTPUT("Test1.O2.OO-A0-to-CSS() => ");
       Print_CSS(*css2);
       OUTPUT("\n");
+      Test1_CSS__Free(css2);
+      ilu_free(css2);
     }
 
     r.css._length = 0;
@@ -176,6 +209,7 @@ int doit()
       for (j = 0; j < 8; j++)
 	OUTPUT(" %d", ap[j]);
       OUTPUT(">\n");
+      ilu_free(ap);
     } else {
       OUTPUT("O2.R-I-A1-to-I-A0() raises \"%s\"\n", s.returnCode);
     }
@@ -198,9 +232,13 @@ int doit()
       OUTPUT("instance of class %s received!\n", ILU_C_ClassName(o3));
     } else {
       i2 = Test1_O3_RS_R_to_R_IS(o3, rs, &pr, &s);
-      if (ILU_C_SUCCESSFUL(&s))
+      if (ILU_C_SUCCESSFUL(&s)) {
 	OUTPUT("Test1.O3.RS-R-to-R-IS succeeds.\n");
-      else
+	Test1_TheR__Free(pr);
+	ilu_free(pr);
+	Test1_IS__Free(i2);
+	ilu_free(i2);
+      } else
 	OUTPUT("Test1.O3.RS-R-to-R-IS raises %s\n", s.returnCode);
       Test1_O3_O1_U_to_U(o3, handle, &u, &s);
       if (ILU_C_SUCCESSFUL(&s)) {
@@ -211,6 +249,7 @@ int doit()
 	OUTPUT("Test1.O3.O1-U-to-U raises %s\n", s.returnCode);
       }
     }
+    ilu_free(rs);
   } else {
     OUTPUT("couldn't get an instance of O3.  Exception is \"%s\".\n",
 	   s.returnCode);
@@ -227,10 +266,15 @@ int doit()
 	   ILU_C_ClassName(o3));
 
     i2 = Test1_O3_RS_R_to_R_IS(o3, rs, &pr, &s);
-    if (ILU_C_SUCCESSFUL(&s))
-	OUTPUT("Test1.O3.RS-R-to-R-IS succeeds.\n");
-      else
-	OUTPUT("Test1.O3.RS-R-to-R-IS raises %s\n", s.returnCode);
+    if (ILU_C_SUCCESSFUL(&s)) {
+      OUTPUT("Test1.O3.RS-R-to-R-IS succeeds.\n");
+      Test1_TheR__Free(pr);
+      ilu_free(pr);
+      Test1_IS__Free(i2);
+      ilu_free(i2);
+    } else
+      OUTPUT("Test1.O3.RS-R-to-R-IS raises %s\n", s.returnCode);
+    ilu_free(rs);
     Test1_O3_O1_U_to_U(o3, handle, &u, &s);
       if (ILU_C_SUCCESSFUL(&s)) {
 	OUTPUT("Test1.O3.O1-U-to-U => ");
@@ -254,6 +298,8 @@ int doit()
 	OUTPUT("Test3.O.I-to-Test1U() => ");
 	Print_U(*u2);
 	OUTPUT("\n");
+	Test1_TheU__Free(u2);
+	ilu_free(u2);
       }
     }
   } else {
@@ -283,5 +329,39 @@ int doit()
 	   s.returnCode);
   }
 
+#if (defined(TCPIP_TRANSPORT) && defined(ENABLE_DEBUGGING))
+  {
+    ilu_cardinal bytes_sent;
+    ilu_cardinal bytes_received;
+    ilu_cardinal moorings_created;
+    ilu_cardinal connections_received;
+    ilu_cardinal connections_created;
+    ilu_cardinal current_connections;
+    ilu_cardinal max_connections;
+    ilu_tcp_GetStats (&bytes_sent,
+		      &bytes_received,
+		      &moorings_created,
+		      &connections_received,
+		      &connections_created,
+		      &current_connections,
+		      &max_connections);
+    OUTPUT("TCP/IP usage stats:\n"
+	   "  bytes sent:  %lu\n"
+	   "  bytes read:  %lu\n"
+	   "  moorings created:  %lu\n"
+	   "  connections accepted:  %lu\n"
+	   "  connections opened:  %lu\n"
+	   "  currently open connections:  %lu\n"
+	   "  max connections open during test:  %lu\n",
+	   bytes_sent,
+	   bytes_received,
+	   moorings_created,
+	   connections_received,
+	   connections_created,
+	   current_connections,
+	   max_connections);
+  }
+#endif
+
   return (0);
 }
