@@ -1,9 +1,9 @@
-/* $Id: clnt.h,v 1.2 1996/04/05 23:52:35 mdavidso Exp $ */
+/* $Id: clnt.h,v 1.3 1997/01/24 20:41:44 larner Exp $ */
 /* Last edited by Mike Spreitzer April 3, 1996 2:42 pm PST */
 
 extern int doit();
 
-#if defined (_WINIO)
+#if defined (_WINDOWS)
 #define OUTPUT	WIN_PRINTF
 extern void WIN_PRINTF(char *format, ...);
 #else
