@@ -1,4 +1,4 @@
-/* $Id: cppsrvr.cpp,v 1.12 1996/07/03 00:45:56 janssen Exp $ */
+/* $Id: cppsrvr.cpp,v 1.13 1997/03/06 03:04:13 janssen Exp $ */
 /* Last edited by Mike Spreitzer June 5, 1996 10:48 pm PDT */
 
 #include <stdio.h>
@@ -434,11 +434,32 @@ Test1_T_U * Test3_T_O_impl::I_to_Test1U (Test3Status *_status, ilu_Integer i)
 
 int main (int, char **)
 {
-  extern void Test1__InitializeServer(void);
+#ifdef macintosh
+	/*
+	 * On the Mac, the C++ auto-initialization is disabled in both the stubbers
+	 * and the runtime. This makes it easy (and mandatory) to specify when and where
+	 * ILU should be initialized. Perhaps in the future all platforms will have this
+	 * option?
+	 */
+
+	extern void Test1__InitializeServer(void);
+	extern void Test2__InitializeServer(void);
+	extern void Test3__InitializeServer(void);
+
+	ILUStartup();
+	Test1__Initialize();
+	Test2__Initialize();
+	Test3__Initialize();
+	Test1__InitializeServer();
+	Test2__InitializeServer();
+	Test3__InitializeServer();
+#endif
+
   iluServer s ("Test1-Server", NULL);
   Test1_T_O1 *uc;
   Test1_T_O1 *uc2;
 
+
   s.AddPort(NULL, NULL, ilu_TRUE);
   ilu::SetDefaultServer(&s);
 
