@@ -13,6 +13,10 @@ import Test3__skel
 
 singleO2 = None
 
+the_Test1_O3 = None
+the_Test1_O4 = None
+the_Test3_O  = None
+
 class O1(Test1__skel.O1):
   def __init__(self, ih, srvr):
     self.IluInstHandle = ih
@@ -48,22 +52,28 @@ class O1(Test1__skel.O1):
     return uc
 
   def get_O3(self, subclass):
+    global the_Test1_O3, the_Test1_O4, the_Test3_O
     print "Test1.O1.get-O3"
     try:
       if subclass:
-	uc = O()
+	if not the_Test3_O:
+	  the_Test3_O = O()
+	return the_Test3_O
       else:
 	if self.one == 0:
 	  self.one = 1
-	  print "making O3..."
-	  uc = O3()
+	  if not the_Test1_O3:
+	    print "making O3..."
+	    the_Test1_O3 = O3()
+	  return the_Test1_O3
 	else:
 	  self.one = 0
-	  print "making O4..."
-	  uc = O4()
+	  if not the_Test1_O4:
+	    print "making O4..."
+	    the_Test1_O4 = O4()
+	  return the_Test1_O4
     except:
       raise Test1.CantCreate
-    return uc
 
 class O2(Test1__skel.O2):
   def OO_A0_to_CSS(self, o, a):
@@ -79,8 +89,17 @@ class O2(Test1__skel.O2):
 
 class O3(Test1__skel.O3):
   def RS_R_to_R_IS(self, r):
+    passport = ilu.CallerIdentity()
+    print "passport is", passport
+    identity = {}
+    val = passport.lookupIdentity("ConnectionIdentity")
+    if val:
+	    identity["ConnectionIdentity"] = val
+    val = passport.lookupIdentity("SunRPCAuthUnixIdentity")
+    if val:
+	    identity["SunRPCAuthUnixIdentity"] = val
     print "Test1.O3.RS-R-to-R-IS"
-    print "(Caller is " + str(ilu.CallerIdentity()) + ")"
+    print "(Caller is " + str(identity) + ")"
     r2 = Test1.TheR(("a", "b", "c"), ["just", "a", "string"], -133567)
     ret = []
     return ret, r2
