@@ -1,5 +1,5 @@
-/* $Id: srvr.c,v 1.44 1996/07/02 07:20:30 janssen Exp $ */
-/* Last edited by Mike Spreitzer June 20, 1996 10:12 am PDT */
+/* $Id: srvr.c,v 1.57 1997/11/17 20:19:25 janssen Exp $ */
+/* Last edited by Mike Spreitzer March 14, 1997 9:30 pm PST */
 
 #include <stdio.h>
 #include <math.h>
@@ -10,8 +10,8 @@
 
 #include "srvr.h"
 
-ilu_Server theServer = ILU_NIL;
-ilu_Server SunRPCServer = ILU_NIL;
+ILU_C_Server theServer = ILU_NIL;
+ILU_C_Server SunRPCServer = ILU_NIL;
 
 #if (defined(SECURE_TRANSPORT) && defined(SUNRPCRM_TRANSPORT) && defined(TCPIP_TRANSPORT))
 
@@ -84,16 +84,19 @@ ilu_TransportInfo DefaultTInfo(ilu_boolean secure)
 }
 
 
+static ilu_string proof = NULL;
+
 int
 doit(char *pinfo, ilu_TransportInfo tinfo, ilu_boolean threadly,
      ilu_boolean init_credentials)
 {
-#if defined(_WINIO)
+#if defined(_WINDOWS)
   extern void     set_process_windows_messages_alarm(int *pi_stop);
   static int      i_stop = 0;
 #endif
   Test1_O1        uc, uc2;
   ilu_Passport    pp = ILU_NIL;
+  CORBA_Environment env;
 
   if (threadly) {
 #if (defined(ILU_OS_THREADED))
@@ -143,7 +146,7 @@ doit(char *pinfo, ilu_TransportInfo tinfo, ilu_boolean threadly,
     OUTPUT("*** Error, couldn't create object!\n");
     exit(1);
   }
-  if (ILU_C_PublishObject(uc) == NULL) {
+  if ((proof = ILU_C_PublishObject(uc)) == NULL) {
     fprintf(stderr, "*** Error, can't publish Test1-Server object!\n");
     exit(1);
   }
@@ -154,8 +157,11 @@ doit(char *pinfo, ilu_TransportInfo tinfo, ilu_boolean threadly,
     fprintf(stderr, "*** Error, lookup returns wrong object!\n");
     exit(1);
   }
+  CORBA_Object_release(uc2, &env);
+  uc2 = ILU_NIL;
+  CORBA_exception_free(&env);
   OUTPUT("exported %s\n", ILU_C_SBHOfObject(uc));
-#if defined(_WINIO)
+#if defined(_WINDOWS)
   set_process_windows_messages_alarm(&i_stop);
   ILU_C_Stoppable_Run(&i_stop);
   return 0;
@@ -167,16 +173,24 @@ doit(char *pinfo, ilu_TransportInfo tinfo, ilu_boolean threadly,
 #endif
 }
 
+#define Test1_U__R 0
+#define Test1_U__RO 1
+#define Test1_U__CSS 2
+#define Test1_U__O1 3
+#define Test1_U__OO 4
+#define Test1_U__boolean 5
+
 Test1_U        *
 server_Test1_TheO1_U_CSS_to_U(Test1_O1 h, Test1_U * u, Test1_CSS * css,
 			      ILU_C_ENVIRONMENT * s)
 {
-  ilu_Passport    p = s->callerPassport;
+  ilu_Passport    p;
   ilu_IdentityInfo ident;
   int             first;
   ilu_Error       err;
   Test1_U        *ans = CORBA_sequence_Test1_U_allocbuf(1);
 
+  p = ILU_C_CallerIdentity();
   OUTPUT("Test1.O1.U-CSS-to-U");
   if (p != ILU_NIL) {
     first = 0;
@@ -230,21 +244,21 @@ server_Test1_TheO1_U_CSS_to_U(Test1_O1 h, Test1_U * u, Test1_CSS * css,
   }
   OUTPUT("\n");
   ans->_d = Test1_U__O1;
-  ans->_u.O1 = h;
+  ans->_u.O1 = CORBA_Object_duplicate(h, s);
   return ans;
 }
 
 Test1_RO server_Test1_TheO1_f_CSS_to_RO( Test1_O1 h, Test1_CSS *css, ILU_C_ENVIRONMENT *s )
 {
-  Test1_RO	x = (Test1_RO) ilu_must_malloc( sizeof( Test1_R ));
+  Test1_RO        x = (Test1_RO) ilu_must_malloc(sizeof(*x));
 
   x->i = 9;
-  Test1_CSS_Init( &x->css , 0, NULL );
-  x->a[0]= ILU_C_Strdup("hello");
-  x->a[1]= ILU_C_Strdup("world");
-  x->a[2]= ILU_C_Strdup("!\n");
-  OUTPUT ("Test1.O1.f-CSS-to-R0\n");
-  return( x );
+  Test1_CSS_Init(&x->css, 0, NULL);
+  x->a[0] = ILU_C_Strdup("hello");
+  x->a[1] = ILU_C_Strdup("world");
+  x->a[2] = ILU_C_Strdup("!\n");
+  OUTPUT("Test1.O1.f-CSS-to-R0\n");
+  return (x);
 }
 
 float server_Test1_TheO1_R_ScS_to_F( Test1_O1 h, Test1_R *r, Test1_ScS str, ILU_C_ENVIRONMENT *s )
@@ -263,61 +277,74 @@ void server_Test1_TheO1_a_RO( Test1_O1 h, Test1_RO ro, ILU_C_ENVIRONMENT *s )
 Test1_O2 server_Test1_TheO1_get_O2 ( Test1_O1 h, ILU_C_ENVIRONMENT *s)
 {
   static Test1_O2 uc = NULL;
+  static ILU_C_Server ts = NULL;
 
-  /* note that Test1.O2 is a singleton class, of cinfo-type "sunrpc_2_0x3458_3".
-     This means that it has to be exported via a kernel server with a "sunrpc"
-     port... */
+  /*
+   * Note that Test1.O2 is a singleton class, of cinfo-type
+   * "sunrpc_2_0x3458_3".  This means that it has to be exported via
+   * a kernel server with a "sunrpc" port...
+   */
 
-  OUTPUT ("Test1.O1.get-O2\n");
-  if (uc == NULL)
-    {
-      ilu_Server s;
-      ilu_string sunrpctinfo[] = { "sunrpcrm", "tcp_0_0", ILU_NIL };
-      s = ILU_C_InitializeServer("Test1-SunRPC-Server", NULL, "sunrpc", sunrpctinfo, ILU_NIL,
-				 ilu_TRUE);
-      uc = Test1_O2__CreateTrue( NULL, s, NULL );
-    }
-  if (uc == NULL)
-    {
-      s->_major = ILU_C_USER_EXCEPTION;
-      s->returnCode = ex_Test1_CantCreate;
-      return (NULL);
-    }
-  else
-    return (uc);
+  OUTPUT("Test1.O1.get-O2\n");
+  if (uc == NULL) {
+    ilu_string      sunrpctinfo[] = {"sunrpcrm", "tcp_0_0", ILU_NIL};
+    if (!ts)
+      ts = ILU_C_InitializeServer("Test1-SunRPC-Server", NULL, "sunrpc",
+				  sunrpctinfo, ILU_NIL, ilu_TRUE);
+    uc = Test1_O2__CreateTrue(NULL, ts, NULL);
+  }
+  if (uc == NULL) {
+    s->_major = ILU_C_USER_EXCEPTION;
+    s->returnCode = ex_Test1_CantCreate;
+    return (NULL);
+  }
+  return (CORBA_Object_duplicate(uc, s));
 }
 
 Test1_O3 server_Test1_TheO1_get_O3 ( Test1_O1 h, CORBA_boolean b, ILU_C_ENVIRONMENT *s )
 {
-  Test1_O3 uc;
-  static int one = 0;
+  static Test1_O3 theObj = ILU_NIL;
+  static int      one = 0;
+  static Test1_O3 theO3 = ILU_NIL;
+  static Test1_O4 theO4 = ILU_NIL;
+  static Test3_O  the3_O = ILU_NIL;
 
-  OUTPUT ("Test1.O1.get-O3\n");
-  if (b)
-    uc = Test3_O__CreateTrue( NULL, theServer, NULL );
-  else
-    {
-      if (one == 0)
-	{
-	  one = 1;
-	  OUTPUT ("making O3...\n");
-	  uc = Test1_O3__CreateTrue( NULL, theServer, NULL );
-	}
-      else
-	{
-	  one = 0;
-	  OUTPUT ("making O4...\n");
-	  uc = Test1_O4__CreateTrue (NULL, theServer, NULL);
-	}
+  OUTPUT("Test1.O1.get-O3\n");
+  if (b) {
+    if (the3_O == ILU_NIL) {
+      OUTPUT("making Test3.O...\n");
+      theObj = the3_O = Test3_O__CreateTrue(NULL, theServer, NULL);
+    } else {
+      OUTPUT("returning Test3.O...\n");
+      theObj = the3_O;
     }
-  if (uc == NULL)
-    {
-      s->_major = ILU_C_USER_EXCEPTION;
-      s->returnCode = ex_Test1_CantCreate;
-      return (NULL);
+  } else {
+    if (one == 0) {
+      one = 1;
+      if (theO3 == ILU_NIL) {
+	OUTPUT("making O3...\n");
+	theObj = theO3 = Test1_O3__CreateTrue(NULL, theServer, NULL);
+      } else {
+	OUTPUT("returning O3...\n");
+	theObj = theO3;
+      }
+    } else {
+      one = 0;
+      if (theO4 == ILU_NIL) {
+	OUTPUT("making O4...\n");
+	theObj = theO4 = Test1_O4__CreateTrue(NULL, theServer, NULL);
+      } else {
+	OUTPUT("returning O4...\n");
+	theObj = theO4;
+      }
     }
-  else
-    return (uc);
+  }
+  if (theObj == NULL) {
+    s->_major = ILU_C_USER_EXCEPTION;
+    s->returnCode = ex_Test1_CantCreate;
+    return (NULL);
+  }
+  return (CORBA_Object_duplicate(theObj, s));
 }
 
 Test1_CSS      *
@@ -422,8 +449,9 @@ server_Test1_O4_RS_R_to_R_IS(Test1_O4 self, Test1_RS * r, Test1_R ** pr2,
 void server_Test1_O4_O1_U_to_U (Test1_O4 self, Test1_O1 o, Test1_U *u, ILU_C_ENVIRONMENT *s)
 {
   OUTPUT ("Test1.O4.O1-U-to-U\n");
+  Test1_TheU__Free(u);
   u->_d = Test1_U__O1;
-  u->_u.O1 = o;
+  u->_u.O1 = CORBA_Object_duplicate(o, s);
   return;
 }
 
@@ -470,16 +498,18 @@ server_Test1_P_m2(Test1_P self, ilu_integer j, ILU_C_ENVIRONMENT * s)
 void server_Test1_O3_O1_U_to_U (Test1_O3 self, Test1_O1 o, Test1_U *u, ILU_C_ENVIRONMENT *s)
 {
   OUTPUT ("Test1.O3.O1-U-to-U\n");
+  Test1_TheU__Free(u);
   u->_d = Test1_U__O1;
-  u->_u.O1 = o;
+  u->_u.O1 = CORBA_Object_duplicate(o, s);
   return;
 }
 
 void server_Test1_P_O1_U_to_U (Test1_P self, Test1_O1 o, Test1_U *u, ILU_C_ENVIRONMENT *s)
 {
   OUTPUT ("Test1.P.O1-U-to-U\n");
+  Test1_TheU__Free(u);
   u->_d = Test1_U__O1;
-  u->_u.O1 = o;
+  u->_u.O1 = CORBA_Object_duplicate(o, s);
   return;
 }
 
@@ -506,8 +536,9 @@ server_Test3_O_RS_R_to_R_IS(Test3_O self, Test1_RS * r, Test1_R ** pr2,
 void server_Test3_O_O1_U_to_U (Test1_O3 self, Test1_O1 o, Test1_U *u, ILU_C_ENVIRONMENT *s)
 {
   OUTPUT ("Test3.O.O1-U-to-U(0x%lx, {%d})\n", (unsigned long) o, u->_d);
+  Test1_TheU__Free(u);
   u->_d = Test1_U__O1;
-  u->_u.O1 = o;
+  u->_u.O1 = CORBA_Object_duplicate(o, s);
   return;
 }
 
