@@ -1,4 +1,4 @@
-/* $Id: srvr.h,v 1.3 1996/06/16 23:17:01 spreitze Exp $ */
+/* $Id: srvr.h,v 1.4 1997/01/24 20:41:44 larner Exp $ */
 /* Last edited by Mike Spreitzer June 16, 1996 3:13 pm PDT */
 
 #include <iluxport.h>
@@ -18,7 +18,7 @@ extern ilu_TransportInfo DefaultTInfo(ilu_boolean secure);
  */
 
 
-#if defined (_WINIO)
+#if defined (_WINDOWS)
 #define OUTPUT	WIN_PRINTF
 extern void WIN_PRINTF(char *format, ...);
 #else
