@@ -1,4 +1,4 @@
-/* $Id: srvru.c,v 1.5 1996/06/17 05:40:56 spreitze Exp $ */
+/* $Id: srvru.c,v 1.6 1996/07/31 02:31:14 janssen Exp $ */
 /* Last edited by Mike Spreitzer June 16, 1996 10:29 pm PDT */
 
 #include "srvr.h"
@@ -36,6 +36,10 @@ int main (int ac, char **av)
     } else if (strcmp(av[i], "-nosec") == 0) {
       cred = 0; i++;
       tinfo = DefaultTInfo(0);
+    } else if (strcmp(av[i], "-errexit") == 0) {
+      ilu_SetAssertionFailureAction(1);
+      ilu_SetCheckFailureAction(1);
+      i++;
     } else if (strcmp(av[i], "-sec") == 0) {
       cred = 1; i++;
       newti = DefaultTInfo(1);
