@@ -1,15 +1,20 @@
 /* bsvr.c */
-/* $Id: bsvr.c,v 1.8 1996/06/17 06:09:22 spreitze Exp $ */
-/* Last edited by Mike Spreitzer June 16, 1996 11:04 pm PDT */
+/* $Id: bsvr.c,v 1.13 1997/09/16 18:12:59 larner Exp $ */
+/* Last edited by Mike Spreitzer September 11, 1997 11:40 am PDT */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>	/* for exit() */
 
+#ifndef WIN32
+#include <unistd.h>
+#endif
+
 #include "Batcher.h"
 
 static Batcher_TimeSeq times;
 static int verbose = 0;
+static unsigned int tosleep = 0;
 
 void
 server_Batcher_T_Send(Batcher_T self,
@@ -37,6 +42,13 @@ server_Batcher_T_Sync(Batcher_T self,
   Batcher_TimeSeq *ans = Batcher_TimeSeq__alloc();
   ilu_cardinal    i;
   server_Batcher_T_Send(self, s, Env);
+  if (tosleep) {
+#if defined(WIN32)
+    _sleep(tosleep);
+#else
+    sleep(tosleep);
+#endif
+  }
   Batcher_TimeSeq_Init(ans, times._length, NULL);
   for (i = 0; i < times._length; i++)
     Batcher_TimeSeq_Append(ans, &times._buffer[i]);
@@ -47,7 +59,7 @@ server_Batcher_T_Sync(Batcher_T self,
 int
 main(int argc, char **argv)
 {
-  ilu_Server s;
+  ILU_C_Server s;
   Batcher_T t;
   int i;
   ilu_string progname, sbh, pinfo = NULL;
@@ -61,14 +73,16 @@ main(int argc, char **argv)
   for (i = 0;  i < argc;  i++) {
     if (strcmp(argv[i], "-p") == 0 && ((i+1) < argc))
       pinfo = argv[++i];
-    else if (strcmp(argv[0], "-t") == 0 && ((i+1) < argc)) {
+    else if (strcmp(argv[i], "-s") == 0 && ((i+1) < argc))
+      tosleep = atoi(argv[++i]);
+    else if (strcmp(argv[i], "-t") == 0 && ((i+1) < argc)) {
       int j = 0;
       ++i;
       while ((i < argc) && (argv[i][0] != '-'))
 	tinfo[j++] = argv[i++];
       tinfo[j] = NULL;
     }
-    else if (strcmp(argv[0], "-v") == 0)
+    else if (strcmp(argv[i], "-v") == 0)
       verbose = 1;
     else
       goto usage;
@@ -89,9 +103,12 @@ main(int argc, char **argv)
     exit(1);
   }
   sbh = ILU_C_SBHOfObject(t);
+  printf("tosleep = %u\n", tosleep);
   printf("SBH = '%s'\n", sbh);
   ILU_C_Run();
 usage:
-  fprintf(stderr, "Usage: %s [-v] [-p pinfo] [-t tinfo [tinfo...]]\n", progname);
+  fprintf(stderr,
+    "Usage: %s [-v] [-s sleep] [-p pinfo] [-t tinfo [tinfo...]]\n",
+	  progname);
   return 1;
 }
