@@ -1,7 +1,10 @@
-/* $Id: server.c,v 1.12 1996/05/04 05:12:45 janssen Exp $ */
+/* $Id: server.c,v 1.15 1997/11/21 21:10:57 larner Exp $ */
 /* Last edited by Mike Spreitzer April 2, 1996 9:46 pm PST */
 
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+
 #ifdef WIN32
 /* dll - to pick up gethostname */
 #include <winsock.h>
@@ -14,25 +17,56 @@
 #define ILU_TEST_DOMAIN "parc.xerox.com"
 ilu_boolean verbose = ilu_FALSE;
 
+static void usage() {
+  fprintf (stderr, "Usage:  timeit-server [-verbose] [-host HOSTNAME] [-p PROTOCOL] [-t TINFO]\n");
+}
+
 int main (int ac, char **av)
 {
   extern void Test1__InitializeServer(void);
-  ilu_Server s;
+  ilu_boolean mt = ilu_FALSE;		/* true for multi-threaded */
+  ILU_C_Server s;
   timeit_p  uc;
+  int i;
   char *pinfo = "sunrpc";
   char *tinfo[10] = { "sunrpcrm", "tcp_0_0", ILU_NIL };
   char hostname[1000], serverid[1000];
 
+  gethostname (hostname, sizeof(hostname));
+
+  for (i = 1;  i < ac; ) {
+    if (strcmp(av[i], "-p") == 0) {
+      if (i++ < ac)
+	pinfo = av[i++];
+      else {
+	usage();
+	exit(1);
+      }	
+    } else if (strcmp(av[i], "-t") == 0) {
+      int             j = 0;
+      ++i;
+      while ((i < ac) && (av[i][0] != '-'))
+	tinfo[j++] = av[i++];
+      tinfo[j] = ILU_NIL;
+    } else if (strcmp(av[i], "-st") == 0) {
+      mt = 0; i++;
+    } else if (strcmp(av[i], "-mt") == 0) {
+      mt = 1; i++;
+    } else if (strcmp(av[i], "-host") == 0) {
+      i++;
+      strcpy (hostname, av[i]);
+      i++;
+    } else if (strcmp(av[i], "-verbose") == 0) {
+      verbose = ilu_TRUE;
+      i++;
+    } else {
+      usage();
+      exit(1);
+    }
+  }
+
   timeit__InitializeServer();
 
-  if (ac > 1)
-    strcpy (hostname, av[1]);
-  else
-    gethostname (hostname, sizeof(hostname));
-
-  if (ac > 2)
-    verbose = ilu_TRUE;
-
   sprintf (serverid, "TimingTest.%s.%s", hostname, ILU_TEST_DOMAIN);
   s = ILU_C_InitializeServer (serverid, ILU_NIL, pinfo, tinfo, ILU_NIL, ilu_TRUE);
   uc = timeit_p__CreateTrue ( "0", s, ILU_NIL );
@@ -86,7 +120,7 @@ static timeit_pageimg pageimg_Create (ilu_cardinal size)
   timeit_pageimg *p = timeit_pageimg__alloc();
   p->_maximum = size;
   p->_length = size;
-  p->_buffer = CORBA_sequence_octet_bufalloc(size);
+  p->_buffer = CORBA_sequence_octet_allocbuf(size);
   return *p;
 }
 
