@@ -1,5 +1,5 @@
 /* TapeCalculator-impl.c */
-/* $Id: TapeCalculator-impl.c,v 1.4 1996/04/09 20:07:59 spreitze Exp $ */
+/* $Id: TapeCalculator-impl.c,v 1.5 1997/04/30 22:03:32 janssen Exp $ */
 /* Last edited by Mike Spreitzer April 9, 1996 12:55 pm PDT */
 
 /* The first thing we need to do is to include the generated header
@@ -48,6 +48,7 @@ static Tutorial2_Operation AddOp (Tutorial2_RegisterTape *tape, Tutorial2_OpType
   n.value = value;
   n.accumulator = acc;
   Tutorial2_RegisterTape_Append (tape, &n);
+  return n;
 }
 
 static Tutorial2_RegisterTape *CopyTape (Tutorial2_RegisterTape *orig)
@@ -81,7 +82,7 @@ server_Tutorial2_TapeCalculator_SetValue (
    * any object instance, so we'll just set it to be "v".
    */
 
-  struct calc_state *s = ((struct calc_state *)(self->data));
+  struct calc_state *s = ((struct calc_state *)(self->instanceData));
 
   s->the_value = v;
   AddOp (s->tape, Tutorial2_SetValue, v, s->the_value);
@@ -92,7 +93,7 @@ server_Tutorial2_TapeCalculator_GetValue (
    Tutorial2_TapeCalculator self,
    ILU_C_ENVIRONMENT *env)
 {
-  return (((struct calc_state *) (self->data))->the_value);
+  return (((struct calc_state *) (self->instanceData))->the_value);
 }
   
   void
@@ -101,7 +102,7 @@ server_Tutorial2_TapeCalculator_Add (
   CORBA_double v,
   ILU_C_ENVIRONMENT *env)
 {
-  struct calc_state *s = ((struct calc_state *)(self->data));
+  struct calc_state *s = ((struct calc_state *)(self->instanceData));
 
   s->the_value += v;
   AddOp (s->tape, Tutorial2_Add, v, s->the_value);
@@ -113,7 +114,7 @@ server_Tutorial2_TapeCalculator_Subtract (
   CORBA_double v,
   ILU_C_ENVIRONMENT *env)
 {
-  struct calc_state *s = ((struct calc_state *)(self->data));
+  struct calc_state *s = ((struct calc_state *)(self->instanceData));
 
   s->the_value -= v;
   AddOp (s->tape, Tutorial2_Subtract, v, s->the_value);
@@ -125,7 +126,7 @@ server_Tutorial2_TapeCalculator_Multiply (
   CORBA_double v,
   ILU_C_ENVIRONMENT *env)
 {
-  struct calc_state *s = ((struct calc_state *)(self->data));
+  struct calc_state *s = ((struct calc_state *)(self->instanceData));
 
   s->the_value *= v;
   AddOp (s->tape, Tutorial2_Multiply, v, s->the_value);
@@ -157,7 +158,7 @@ server_Tutorial2_TapeCalculator_Divide (
     RAISE(env, ex_Tutorial_DivideByZero)
   else
     {
-      struct calc_state *s = ((struct calc_state *)(self->data));
+      struct calc_state *s = ((struct calc_state *)(self->instanceData));
 
       s->the_value /= v;
       AddOp (s->tape, Tutorial2_Divide, v, s->the_value);
@@ -169,5 +170,5 @@ server_Tutorial2_TapeCalculator_GetTape(
 				     Tutorial2_TapeCalculator self,
 					ILU_C_ENVIRONMENT * env)
 {
-  return (CopyTape(((struct calc_state *) (self->data))->tape));
+  return (CopyTape(((struct calc_state *) (self->instanceData))->tape));
 }
