@@ -29,7 +29,8 @@ def main(argv):
 	print "Factory instance published."
 	print "Its SBH is '" + theFactory.IluSBH() + "'"
 
-	ilu.RunMainLoop ()
+	handle = ilu.CreateLoopHandle()
+	ilu.RunMainLoop (handle)
 
 
 main(sys.argv)
