@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: ilu.cpp,v 1.67 1996/03/11 23:34:52 mdavidso Exp $ */
-/* Last edited by Mike Spreitzer December 8, 1995 10:42 am PST */
+/* $Id: ilu.cpp,v 1.84 1997/11/25 22:19:44 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 11, 1997 10:11 am PDT */
 
 #include "ilu.hh"
 #include <stdio.h>
@@ -82,11 +82,11 @@ void ilu::ObjectTypeDefined(ilu_Class t)
   ILU_MUST_BE_SUCCESS(lerr);
 }
 
-ilu_Exception ilu::DefineException(ilu_CString i, ilu_CString e)
+ilu_Exception ilu::DefineException(ilu_CString i, ilu_CString e, ilu_CString typeidstring)
 {
   ILU_ERRS((internal, no_memory)) lerr;
   ilu_Exception   ans;
-  ans = ilu_DefineException(i, e, &lerr);
+  ans = ilu_DefineException(i, e, typeidstring, &lerr);
   ILU_MUST_BE_SUCCESS(lerr);
   return ans;
 }
@@ -122,12 +122,15 @@ ilu::DefineMethod(ilu_Class c,
 		  ilu_Boolean me_cacheable,
 		  ilu_Boolean me_asynchronous,
 		  ilu_Cardinal me_exceptionCount,
-		  ilu_Exception * me_exceptionVector)
+		  ilu_Exception * me_exceptionVector,
+		  ilu_Cardinal me_nargs,
+		  ilu_CString me_ret_typeid)
 {
   ILU_ERRS((internal, no_memory)) lerr;
   (void) ilu_DefineMethod(c, i, me_name, me_id, ILU_BDn(me_cacheable),
 			  ILU_BDn(me_asynchronous), me_exceptionCount,
-			  me_exceptionVector, &lerr);
+			  me_exceptionVector, me_nargs, me_ret_typeid,
+			  &lerr);
   ILU_MUST_BE_SUCCESS(lerr);
 }
 
@@ -294,7 +297,6 @@ ilu_Boolean	ilu::FinishException (iluCall call)
 
   ans = ilu_FinishException(&call->call, &call->err);
   ilu_PreferSuccess(&call->err);
-  iluServer::EnableRequests(call, conn);
   return ans;
 }
 
@@ -320,7 +322,6 @@ ilu_Boolean	ilu::FinishReply	(iluCall call)
 
   ans = ilu_FinishReply(&call->call, &call->err);
   ilu_PreferSuccess(&call->err);
-  iluServer::EnableRequests(call, conn);
   return ans;
 }
 
@@ -332,8 +333,6 @@ void		ilu::NoReply	(iluCall call)
 
   ans = ilu_NoReply(&call->call, &call->err);
   ilu_PreferSuccess(&call->err);
-  if (!ilu_ThreadPerRequest(conn))
-    iluServer::EnableRequests(call, conn);
   return;
 }
 
@@ -390,7 +389,7 @@ ilu_Boolean	ilu::InputBoolean	(iluCall call, ilu_Boolean *b)
 ilu_Boolean	ilu::InputOptional	(iluCall call, ilu_Boolean *b)
 {
   ilu_boolean b2;
-  ilu_InputOptional(&call->call, &b2, &call->err);
+  ilu_InputOptional(&call->call, &b2, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   if (b != NULL)
     *b = b2;
@@ -420,10 +419,21 @@ ilu_Character	ilu::InputCharacter	(iluCall call, ilu_Character *b)
 }
 
 
+ilu_ShortCharacter	ilu::InputShortCharacter	(iluCall call, ilu_ShortCharacter *b)
+{
+  ilu_ShortCharacter b2;
+  ilu_InputShortCharacter(&call->call, &b2, &call->err);
+  ilu_PreferSuccess(&call->err);
+  if (b != NULL)
+    *b = b2;
+  return(b2);
+}
+
+
 ilu_ShortCardinal	ilu::InputEnum		(iluCall call, ilu_ShortCardinal *b)
 {
   ilu_shortcardinal b2;
-  ilu_InputEnum(&call->call, &b2, &call->err);
+  ilu_InputEnum(&call->call, &b2, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   if (b != NULL)
     *b = b2;
@@ -593,17 +603,17 @@ ilu_KernelObject	ilu::InputObjectID	(iluCall call, ilu_Boolean discriminator_p,
 ilu_Cardinal	ilu::InputSequence	(iluCall call, ilu_Cardinal *count, ilu_Cardinal limit)
 {
   ilu_cardinal c2;
-  ilu_InputSequence(&call->call, &c2, limit, &call->err);
+  ilu_InputSequence(&call->call, &c2, limit, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   *count = c2;
   return(c2);
 }
 
 
-ilu_Cardinal	ilu::InputUnion	(iluCall call, ilu_Cardinal *discriminator, ilu_Cardinal discriminator_size)
+ilu_Cardinal	ilu::InputUnion	(iluCall call, ilu_Cardinal *discriminator, ilu_TypeKind discriminator_typekind)
 {
   ilu_cardinal d2;
-  ilu_InputUnion(&call->call, &d2, discriminator_size, &call->err);
+  ilu_InputUnion(&call->call, &d2, discriminator_typekind, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   *discriminator = d2;
   return d2;
@@ -612,7 +622,7 @@ ilu_Cardinal	ilu::InputUnion	(iluCall call, ilu_Cardinal *discriminator, ilu_Car
 
 ilu_Boolean	ilu::InputRecord	(iluCall call)
 {
-  ilu_InputRecord(&call->call, &call->err);
+  ilu_InputRecord(&call->call,(ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return 1;
 }
@@ -620,7 +630,7 @@ ilu_Boolean	ilu::InputRecord	(iluCall call)
 
 ilu_Boolean	ilu::InputArray	(iluCall call)
 {
-  ilu_InputArray(&call->call, &call->err);
+  ilu_InputArray(&call->call, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return 1;
 }
@@ -691,7 +701,7 @@ ilu_Boolean	ilu::OutputBoolean (iluCall call, ilu_Boolean b)
 
 ilu_Boolean	ilu::OutputOptional	(iluCall call, ilu_Boolean opt)
 {
-  ilu_OutputOptional(&call->call, ILU_BDn(opt), &call->err);
+  ilu_OutputOptional(&call->call, ILU_BDn(opt), (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return((ilu_Boolean) 1);
 }
@@ -715,7 +725,7 @@ ilu_Boolean	ilu::OutputCharacter	(iluCall call, ilu_Character val)
 
 ilu_Boolean	ilu::OutputEnum		(iluCall call, ilu_ShortCardinal val)
 {
-  ilu_OutputEnum(&call->call, val, &call->err);
+  ilu_OutputEnum(&call->call, val, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return((ilu_Boolean) 1);
 }
@@ -815,22 +825,18 @@ ilu_Boolean	ilu::OutputStringVec	(iluCall call, ilu_CString buf, ilu_Cardinal le
 }
 
 
-ilu_Boolean	ilu::OutputWString	(iluCall call, ilu_WString buf, ilu_Cardinal len, ilu_Cardinal limit, ilu_CString *buf2, ilu_Cardinal *buf2len)
+ilu_Boolean	ilu::OutputWString	(iluCall call, ilu_WString buf, ilu_Cardinal len, ilu_Cardinal limit)
 {
-  ilu_cardinal    buf2len2;
-  ilu_OutputWString(&call->call, buf, len, limit, buf2, &buf2len2, &call->err);
+  ilu_OutputWString(&call->call, buf, len, limit, &call->err);
   ilu_PreferSuccess(&call->err);
-  *buf2len = buf2len2;
   return ((ilu_Boolean) 1);
 }
 
 
-ilu_Boolean	ilu::OutputWStringVec(iluCall call, ilu_WString buf, ilu_Cardinal len, ilu_CString *buf2, ilu_Cardinal *buf2len)
+ilu_Boolean	ilu::OutputWStringVec(iluCall call, ilu_WString buf, ilu_Cardinal len)
 {
-  ilu_cardinal    buf2len2;
-  ilu_OutputWStringVec(&call->call, buf, len, buf2, &buf2len2, &call->err);
+  ilu_OutputWStringVec(&call->call, buf, len, &call->err);
   ilu_PreferSuccess(&call->err);
-  *buf2len = buf2len2;
   return ((ilu_Boolean) 1);
 }
 
@@ -847,15 +853,15 @@ ilu_Boolean	ilu::OutputObjectID	(iluCall call, ilu_KernelObject obj, ilu_Boolean
 
 ilu_Boolean	ilu::OutputSequence	(iluCall call, ilu_Cardinal length, ilu_Cardinal limit)
 {
-  ilu_OutputSequence(&call->call, length, limit, &call->err);
+  ilu_OutputSequence(&call->call, length, limit, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return((ilu_Boolean) 1);
 }
 
 
-ilu_Boolean	ilu::OutputUnion	(iluCall call, ilu_Cardinal discriminator, ilu_Cardinal discriminator_size)
+ilu_Boolean	ilu::OutputUnion	(iluCall call, ilu_Cardinal discriminator,  ilu_TypeKind discriminator_typekind)
 {
-  ilu_OutputUnion(&call->call, discriminator, discriminator_size, &call->err);
+  ilu_OutputUnion(&call->call, discriminator, discriminator_typekind, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return((ilu_Boolean) 1);
 }
@@ -865,7 +871,7 @@ ilu_Boolean	ilu::OutputArray	(iluCall call)
 {
 	/***** this 0 needs to change to the actual number of elements to output */
 
-  ilu_OutputArray(&call->call, 0, &call->err);
+  ilu_OutputArray(&call->call, 0, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ((ilu_Boolean) 1);
 }
@@ -873,7 +879,7 @@ ilu_Boolean	ilu::OutputArray	(iluCall call)
 
 ilu_Boolean	ilu::OutputRecord	(iluCall call)
 {
-  ilu_OutputRecord(&call->call, &call->err);
+  ilu_OutputRecord(&call->call, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return((ilu_Boolean) 1);
 }
@@ -900,7 +906,7 @@ ilu_Cardinal	ilu::SizeOfBoolean	(iluCall call, ilu_Boolean b)
 ilu_Cardinal	ilu::SizeOfOptional	(iluCall call, ilu_Boolean opt)
 {
   ilu_cardinal    ans;
-  ans = ilu_SizeOfOptional(&call->call, ILU_BDn(opt), &call->err);
+  ans = ilu_SizeOfOptional(&call->call, ILU_BDn(opt), (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ans;
 }
@@ -927,7 +933,7 @@ ilu_Cardinal	ilu::SizeOfCharacter	(iluCall call, ilu_Character val)
 ilu_Cardinal	ilu::SizeOfEnum		(iluCall call, ilu_ShortCardinal val)
 {
   ilu_cardinal    ans;
-  ans = ilu_SizeOfEnum(&call->call, val, &call->err);
+  ans = ilu_SizeOfEnum(&call->call, val, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ans;
 }
@@ -1041,22 +1047,20 @@ ilu_Cardinal	ilu::SizeOfStringVec	(iluCall call, ilu_CString buf, ilu_Cardinal l
 }
 
 
-ilu_Cardinal	ilu::SizeOfWString	(iluCall call, ilu_WString buf, ilu_Cardinal len, ilu_Cardinal limit, ilu_CString *buf2, ilu_Cardinal *buf2len)
+ilu_Cardinal	ilu::SizeOfWString	(iluCall call, ilu_WString buf, ilu_Cardinal len, ilu_Cardinal limit)
 {
-  ilu_cardinal    ans, buf2len2;
-  ans = ilu_SizeOfWString(&call->call, buf, len, limit, buf2, &buf2len2, &call->err);
+  ilu_cardinal    ans;
+  ans = ilu_SizeOfWString(&call->call, buf, len, limit, &call->err);
   ilu_PreferSuccess(&call->err);
-  *buf2len = buf2len2;
   return ans;
 }
 
 
-ilu_Cardinal	ilu::SizeOfWStringVec	(iluCall call, ilu_WString buf, ilu_Cardinal len, ilu_CString *buf2, ilu_Cardinal *buf2len)
+ilu_Cardinal	ilu::SizeOfWStringVec	(iluCall call, ilu_WString buf, ilu_Cardinal len)
 {
-  ilu_cardinal    ans, buf2len2;
-  ans = ilu_SizeOfWStringVec(&call->call, buf, len, buf2, &buf2len2, &call->err);
+  ilu_cardinal    ans;
+  ans = ilu_SizeOfWStringVec(&call->call, buf, len, &call->err);
   ilu_PreferSuccess(&call->err);
-  *buf2len = buf2len2;
   return ans;
 }
 
@@ -1078,16 +1082,16 @@ ilu_Cardinal	ilu::SizeOfObjectID	(iluCall call, ilu_KernelObject obj, ilu_Boolea
 ilu_Cardinal	ilu::SizeOfSequence	(iluCall call, ilu_Cardinal len, ilu_Cardinal limit)
 {
   ilu_cardinal    ans;
-  ans = ilu_SizeOfSequence(&call->call, len, limit, &call->err);
+  ans = ilu_SizeOfSequence(&call->call, len, limit, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ans;
 }
 
 
-ilu_Cardinal	ilu::SizeOfUnion	(iluCall call, ilu_Cardinal discriminator, ilu_Cardinal discriminator_size)
+ilu_Cardinal	ilu::SizeOfUnion	(iluCall call, ilu_Cardinal discriminator, ilu_TypeKind discriminator_typekind)
 {
   ilu_cardinal    ans;
-  ans = ilu_SizeOfUnion(&call->call, discriminator, discriminator_size, &call->err);
+  ans = ilu_SizeOfUnion(&call->call, discriminator, discriminator_typekind, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ans;
 }
@@ -1097,7 +1101,7 @@ ilu_Cardinal	ilu::SizeOfArray	(iluCall call)
 	/****** this 0 needs to change to the actual number of elements to output */
 
   ilu_cardinal    ans;
-  ans = ilu_SizeOfArray(&call->call, 0, &call->err);
+  ans = ilu_SizeOfArray(&call->call, 0, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ans;
 }
@@ -1105,7 +1109,7 @@ ilu_Cardinal	ilu::SizeOfArray	(iluCall call)
 ilu_Cardinal	ilu::SizeOfRecord	(iluCall call)
 {
   ilu_cardinal    ans;
-  ans = ilu_SizeOfRecord(&call->call, &call->err);
+  ans = ilu_SizeOfRecord(&call->call, (ilu_Type)ILU_NIL, &call->err);
   ilu_PreferSuccess(&call->err);
   return ans;
 }
@@ -1115,7 +1119,11 @@ ilu_ProtocolException ilu::WaitForReply (iluCall call,
 {
   ilu_ProtocolException ans;
   ilu_cardinal    sc;
-  ans = ilu_GetReply(&call->call, &sc, &call->err);
+  ilu_Connection newconn;
+  
+  ans = ilu_GetReply(&call->call, &sc, &newconn, &call->err);
+  if (newconn != NULL)
+    iluServer::MonitorConn(newconn);
   ilu_PreferSuccess(&call->err);
   *successCode = sc;
   return ans;
@@ -1133,18 +1141,25 @@ class iluServer *ilu::GetDefaultServer()
   return DefaultServer;
 }
 
-static char *DefaultProtocol = "sunrpc";
-static ilu_string DefaultTransport[] = { "sunrpcrm", "tcp_0_0", 0 };
-
 ilu_Port ilu::CreatePort (ilu_Server server, char *protocolType, ilu_TransportInfo transportType)
 {
   char           *p;
+  ilu_Error	err;
   ilu_TransportInfo t;
+  ilu_Port	port;
+
   if ((p = protocolType) == NULL)
-    p = DefaultProtocol;
+    p = ilu_DefaultProtocolInfo();
   if ((t = transportType) == NULL)
-    t = DefaultTransport;
-  return (ilu_CreatePort(server, p, t));
+    t = ilu_DefaultTransportInfo();
+  port = ilu_CreatePort(server, p, t, (_ilu_Passport_s*)ILU_NIL, &err);
+  if (ILU_ERROK(err))
+    return port;
+  else
+    {
+      ILU_HANDLED(err);
+      return NULL;
+    }
 }
 
 void ilu::SetServerDefaultPort(ilu_Server s, ilu_Port p)
@@ -1163,11 +1178,12 @@ ilu::ParseSBH(ilu_CString sbh, ilu_CString * plainInstH,
   ilu_boolean     ans;
   ilu_cardinal    cinfolen2;
   ilu_Error	  lerr;
+  ilu_boolean	  pass_cinfo = ilu_FALSE;
 
   ans = ilu_ParseSBH(sbh, plainInstH, plainServerID, plainMstid,
 		     encodedContactInfo,
-		     cinfolen ? &cinfolen2 : NULL,
-		     &lerr);
+		     cinfolen ? &cinfolen2 : (ilu_cardinal *) 0,
+		     &pass_cinfo, &lerr);
   ilu_PreferSuccess(&lerr);
   if (cinfolen)
     *cinfolen = cinfolen2;
@@ -1187,26 +1203,33 @@ ilu_RcvReqStat ilu::ReceiveRequest (ilu_Connection conn, iluCall call, ilu_Boole
   return ans;
 }
 
-ilu_Connection	ilu::HandleNewConnection	(ilu_Port port)
+ilu_Connection	ilu::HandleNewConnection	(ilu_Port port,
+		                       ilu_boolean * closed,
+		                       ILU_ERRS((IoErrs, bad_locks,
+			                     no_resources)) * err)
 {
-  ilu_Connection c;
-  ilu_boolean closed;
-
-  c = ilu_HandleNewConnection(port, &closed);
-  if (closed)
-    return (NULL);
-  else
-    return (c);
+  return ilu_HandleNewConnection(port, closed, err);
 }
 
-void		ilu::CloseConnection		(ilu_Connection conn)
+ilu_Boolean	ilu::DoneWithPort	(ilu_Port port,
+		                       ILU_ERRS((bad_param, bad_locks,
+					   internal)) * err)
 {
-  ilu_CloseConnection(conn);
+  return ilu_DoneWithPort(port, err);
 }
 
-ilu_Boolean	ilu::WaitForPortConnectionRequest	(ilu_Port p)
+ilu_Boolean	ilu::DoneServingConnection	(ilu_Connection conn,
+				ILU_ERRS((bad_param, broken_locks,
+					  bad_locks, internal)) * err)
 {
-  return (ilu_WaitForPortConnectionRequest(p) != ilu_FALSE);
+  return (ilu_DoneServingConnection(conn, err) != ilu_FALSE);
+}
+
+ilu_Boolean	ilu::WaitForPortConnectionRequest	(ilu_Port p,
+				 ILU_ERRS((bad_locks, broken_locks,
+					   interrupted)) * err)
+{
+  return (ilu_WaitForPortConnectionRequest(p, err) != ilu_FALSE);
 }
 
 /* Inside(s, c) */
