@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iluobj.cpp,v 1.50 1996/07/11 18:02:31 spreitze Exp $ */
-/* Last tweaked by Mike Spreitzer July 11, 1996 11:02 am PDT */
+/* $Id: iluobj.cpp,v 1.54 1997/03/20 03:51:18 janssen Exp $ */
+/* Last tweaked by Mike Spreitzer August 1, 1996 3:46 pm PDT */
 
 #include "ilu.hh"
 
@@ -24,6 +24,8 @@ extern "C" {
 
 #if ((defined(WIN32) || defined(WIN16)) && defined(_WINIO))
 #include <winiodef.h>
+#elif defined( macintosh )
+#include	<macos.h>
 #endif /* ((defined(WIN32) || defined(WIN16)) && defined(_WINIO)) */
 
 extern char *_ilu_Strdup(const char *);
@@ -191,28 +193,16 @@ ilu_KernelObject iluObject::ILUEnsureKernelObject()
     return 0;
   if (this->ILURPCObject == NULL) {
     /* Should be a true object. */
-    static ilu_cardinal      idcounter = 0;
+    static unsigned long idcounter = 0;
     char           *id;
     ilu_Error       lerr;
+    char            idbuf[10];
 
-    if ((id = this->ILUGetInstanceHandle()) != NULL) {
-      id = ilu_StrdupE(id, &lerr);
-      if (ILU_ERRNOK(lerr)) {
-	ILU_HANDLED(lerr);
-	return 0;
-      }
-    } else {
-      char            idbuf[10];
-
-      sprintf(idbuf, "%lu", (unsigned long) (++idcounter));
-      id = ilu_StrdupE(idbuf, &lerr);
-      if (ILU_ERRNOK(lerr)) {
-	ILU_HANDLED(lerr);
-	return 0;
-      }
+    if ((id = this->ILUGetInstanceHandle()) == NULL) {
+      id = idbuf;
+      sprintf(idbuf, "%lu", ++idcounter);
       this->ILUInstanceHandle = ilu_StrdupE(idbuf, &lerr);
       if (ILU_ERRNOK(lerr)) {
-	ilu_free(id);
 	ILU_HANDLED(lerr);
 	return 0;
       }
@@ -345,6 +335,48 @@ void iluObject::_ILU_RegisterAsGCCallback (class iluObject *c)
     }
 }
 
+//////////////////////////////////////////////////////////////////////
+// Implement ilu.CORBA-Object
+//////////////////////////////////////////////////////////////////////
+
+ilu_Class ilu_T_CORBA_Object::ILUClassRecord = NULL;
+
+class ilu_T_CORBA_Object * ilu_T_CORBA_Object::ILUCreateFromSBH (ilu_CString sbh)
+{
+  return (ilu_T_CORBA_Object *) ilu::SBHToObject(sbh, ilu_T_CORBA_Object::ILUClassRecord);
+}
+
+class ilu_T_CORBA_Object * ilu_T_CORBA_Object::ILUQuaT (class iluObject *from)
+{
+  return((class ilu_T_CORBA_Object *) (from->ILUCastDown (ilu_T_CORBA_Object::ILUClassRecord)));
+}
+
+void * ilu_T_CORBA_Object::ILUCastDown (ilu_Class cast_to)
+{
+  if (cast_to == NULL)
+    return((void *)((class iluObject *) this));
+  else if (cast_to == ilu_T_CORBA_Object::ILUClassRecord)
+    return ((void *) this);
+  else return (NULL);
+}
+
+ilu_T_CORBA_Object::ilu_T_CORBA_Object ()
+{
+  this->ILUInstanceClassRecord = ilu_T_CORBA_Object::ILUClassRecord;
+  this->ILUSetMostSpecificObject((void *) this);
+}
+
+ilu_T_CORBA_Object::~ilu_T_CORBA_Object ()
+{
+}
+
+static class iluObject * Create_ilu_T_CORBA_Object(ilu_KernelObject obj) {
+  class ilu_T_CORBA_Object *nobj = new ilu_T_CORBA_Object;
+  nobj->ILUSetRPCObject(obj);
+  ilu::SetLanguageSpecificObject(obj, (class iluObject *) nobj);
+  return ((class iluObject *) nobj);
+}
+
 //////////////////////////////////////////////////////////////////////
 // code to do the GC keepalive acknowedgements
 //////////////////////////////////////////////////////////////////////
@@ -416,18 +448,62 @@ class _ilu_GCCallbackInit {
     _ilu_GCCallbackInit();
 };
 
-static _ilu_GCCallbackInit do_init;
 static _ilu_GCCallback * GCCallbackObject = NULL;
 
+#ifdef macintosh
+static _ilu_GCCallbackInit* do_init = NULL;
+
+void ILUStartup()
+{
+	StartupWinsock();
+	
+	do_init = new _ilu_GCCallbackInit;
+}
+
+void ILUShutdown()
+{
+	delete do_init;
+	do_init = 0;
+	ShutdownWinsock();
+}
+
+#else
+static _ilu_GCCallbackInit do_init;
+#endif
+
 _ilu_GCCallbackInit::_ilu_GCCallbackInit ()
 {
   class iluServer *server;
 
+  if (ilu_IsGcClientSet())
+	// some other runtime must have already set things up for this process
+	return;
+
   // Initialize GCCallback class
 
   _ilu_GCCallback::ILUClassRecord = ilu_GetGcCallbackClass();
   iluObject::RegisterSurrogateCreator (_ilu_GCCallback::ILUClassRecord, Create__ilu_GCCallback);
 
+  // Initialize CORBA-Object class
+  ilu::EnterOTMu();
+  {	// for definition of ilu_T_CORBA_Object
+    ilu_Class t;
+    ilu_T_CORBA_Object::ILUClassRecord = t = ilu::DefineObjectType(
+	"ilu.CORBA-Object",	/* ILU name */
+	"",	/* Brand */
+	(char *) ilu_TypeID_ilu_CORBA_Object,
+	NULL,	/* singleton */
+	ilu_kernelTRUE,	/* optional? */
+	ilu_kernelFALSE,	/* collectible? */
+	NULL,	/* authentication */
+	0,	/* number of methods */
+	0,	/* number of superclasses */
+	NULL	/* no superclass uids */);
+    iluObject::RegisterSurrogateCreator(t, Create_ilu_T_CORBA_Object);
+    ilu::ObjectTypeDefined(t);
+  }	// end definition of ilu_T_CORBA_Object
+  ilu::ExitOTMu();
+
   // create instance of GCCallback on server
 
   server = new iluServer(NULL, NULL);
