@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: ilusrvr.cpp,v 1.42 1996/06/17 13:59:27 spreitze Exp $ */
-/* Last edited by Mike Spreitzer June 17, 1996 6:51 am PDT */
+/* $Id: ilusrvr.cpp,v 1.59 1997/11/06 04:14:09 janssen Exp $ */
+/* Last edited by Mike Spreitzer December 18, 1996 10:34 am PST */
 
 #include <stdio.h>	/* I/O defs (including popen and pclose) */
 
@@ -32,6 +32,10 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #include "ilu.hh"
 
+extern "C" {
+#include <iludebug.h>		/* ILU_NOTE */
+}
+
 /* portability hack since we don't have access to kernel headers */
 #ifndef ANSI_STRERROR
 #ifdef SUNOS
@@ -41,6 +45,52 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #endif
 #endif
 
+#define ERRFMTSIZE 128
+
+static const char *FmtError(char buf[ERRFMTSIZE], ilu_Error * err)
+{
+  int             mc, est;
+  char            nbuf[24], *fmt;
+  const char     *errname = ILU_ERR_NAME(*err);
+  const char     *errfile = ilu_ErrorFile(err);
+  static char    *fmt0 = "%s from line %d of %s";
+  static char    *fmt1 = "%s (minor=%s) from line %d of %s";
+  static char    *fmt2 = "interrupted (bitset=%s) from line %d of %s";
+  static char    *fmt3 = "no_memory (nbytes=%s) from line %d of %s";
+  ILU_ERR_SWITCH(*err) {
+    ILU_SUCCESS_CASE return "SUCCESS";
+    ILU_ERR_CASE(no_memory, x) {
+      sprintf(nbuf, "%lu", x->nbytes);
+      fmt = fmt3;
+    }
+    ILU_ERR_CASE(interrupted, x) {
+      sprintf(nbuf, "0x%x", x->ilu_interruptSet);
+      fmt = fmt2;
+    }
+    ILU_ERR_CASE(bad_param, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(imp_limit, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(comm_failure, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(inv_objref, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(internal, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(marshal, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(bad_typecode, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(bad_operation, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_CASE(no_resources, x) fmt = fmt1, mc = x->minor;
+    ILU_ERR_ELSE    fmt = fmt0;
+  } ILU_ERR_ENDSWITCH;
+  if (fmt == fmt1)
+    sprintf(nbuf, "%d", mc);
+  est = (strlen(fmt) + strlen(errname) + 20 + strlen(errfile) +
+	 ((fmt != fmt0) ? strlen(nbuf) : 0));
+  if (est > ERRFMTSIZE)
+    return errname;
+  if (fmt == fmt0)
+    sprintf(buf, fmt, errname, ilu_ErrorLine(err), errfile);
+  else
+    sprintf(buf, fmt, errname, nbuf, ilu_ErrorLine(err), errfile);
+  return buf;
+}
+
 iluObjectTable::~iluObjectTable()
 {
   fprintf(stderr, "default destructor for iluObjectTable was called!\n");
@@ -62,28 +112,33 @@ static void CallExit (int *stop)
 }
 
 static ilu_boolean CallRegisterInput(int fd,
-				     void (*proc)(int fd, void * rock),
-				     void * rock)
+				     ilu_IOHandler proc,
+				     ilu_private rock)
 {
   return ((ilu_boolean) theML->RegisterInputHandler(fd, proc, rock));
 }
 
-static ilu_boolean CallUnregisterInput(int fd)
+static ilu_boolean CallUnregisterInput(int fd,
+				       ilu_IOHandler *proc,
+				       ilu_private *rock)
 {
-  return ((ilu_boolean) theML->UnregisterInputHandler(fd));
+  theML->UnregisterInputHandler(fd, proc, rock);
+  return ilu_FALSE;
 }
 
 static ilu_boolean CallRegisterOutput(int fd,
-				     void (*proc)(int fd,
-						  void * rock),
-				     void * rock)
+				      ilu_IOHandler proc,
+				      ilu_private rock)
 {
   return ((ilu_boolean) theML->RegisterOutputHandler(fd, proc, rock));
 }
 
-static ilu_boolean CallUnregisterOutput(int fd)
+static ilu_boolean CallUnregisterOutput(int fd,
+					ilu_IOHandler *proc,
+					ilu_private *rock)
 {
-  return ((ilu_boolean) theML->UnregisterOutputHandler(fd));
+  theML->UnregisterOutputHandler(fd, proc, rock);
+  return ilu_FALSE;
 }
 
 static ilu_Alarm CallCreateAlarm (void)
@@ -167,7 +222,7 @@ ilu_Boolean iluServer::SetFork(void (*fork)(void (*proc)(void *arg), void *arg))
 }
 
 /* Main Invariant holds for invocations */
-static ilu_ForkProc *theForkProc = 0;
+static ilucpp_ForkProc *theForkProc = 0;
 
 /* Main Invariant holds */
 static void     ErrlessFork(void (*proc) (void *arg), void *arg)
@@ -184,7 +239,7 @@ static void     ErrlessFork(void (*proc) (void *arg), void *arg)
 
 /* Main Invariant holds */
 ilu_Boolean iluServer::StartThreading(ilu_ThreadSetupProc *s,
-                                       ilu_ForkProc *f)
+                                       ilucpp_ForkProc *f)
 {
   ILU_ERRS((bad_param, no_memory, no_resources, internal)) lerr;
   if (!(*s) (&lerr)) {
@@ -208,9 +263,8 @@ ilu_Boolean iluServer::StartThreading(ilu_ThreadSetupProc *s,
   after:  L2     >=   {conn's callmu, iomu} if result==ilu_good_request,
   after:  L2 disjoint {conn's callmu, iomu} if result!=ilu_good_request */
 /*If !threaded: ReadServiceRequest registered for fd of conn */
-static ilu_boolean FinalServiceRequest (ilu_private arg)
+static ilu_boolean FinalServiceRequest (ilu_Connection conn)
 {
-  ilu_Connection  conn = (ilu_Connection) arg;
   ilu_RcvReqStat  s;
   iluCall_s	  call;
   ilu_Class       ptype;
@@ -223,8 +277,8 @@ static ilu_boolean FinalServiceRequest (ilu_private arg)
   s = ilu::ReceiveRequest(conn, &call, &initted, &ptype, &method,
                           &serial_number);
   if (s == ilu_RcvReqStat_request) {
-    ilu_refany      stubproc = ilu_GetMethodStubProc(method);
-    _ilu_Assert(stubproc != NULL, "ilusrvr.cpp:FinalServiceRequest");
+    ilu_StubProc      stubproc = ilu_GetMethodStubProc(method, ilu::CppLangIdx());
+    _ilu_Assert(stubproc != ((ilu_StubProc) 0), "ilusrvr.cpp:FinalServiceRequest");
     (*(void (*) (ilu_Call)) stubproc) (&call.call);
   }
   if (initted) {
@@ -232,8 +286,15 @@ static ilu_boolean FinalServiceRequest (ilu_private arg)
     ilu_PreferSuccess(&call.err);
     ILU_HANDLED(call.err);
   }
-  if (s == ilu_RcvReqStat_quit)
+  if (s == ilu_RcvReqStat_quit) {
+    ILU_ERRS((bad_param, broken_locks, bad_locks, internal)) lerr;
+    if (!ilu_ConnectionServingP(conn)) {
+      ilu::DoneServingConnection(conn, &lerr);
+      ilu_PreferSuccess(&lerr);
+      ILU_HANDLED(lerr);
+    }
     return ilu_TRUE;
+  }
   if (!call.call.ca_reqs_enabled) {
     iluServer::EnableRequests(&call, conn);
   }
@@ -244,7 +305,7 @@ static ilu_boolean FinalServiceRequest (ilu_private arg)
 
 static void ReadServiceRequest (void *arg)
 {
-  (void) FinalServiceRequest(arg);
+  (void) FinalServiceRequest((ilu_Connection) arg);
 }
 
 /* When a request for a new connection to the server is received,
@@ -254,15 +315,43 @@ static void ReadServiceRequest (void *arg)
 void ReadConnectionRequest (void *arg)
 {
   ilu_Port        p = (ilu_Port) arg;
+  ilu_boolean     closed;
   ilu_Error       lerr;
   ilu_Connection  conn;
+  char            ef[ERRFMTSIZE];
 
-  if ((conn = ilu::HandleNewConnection(p)) == NULL) {
-    fprintf(stderr, "iluServer::ReadConnectionRequest:  Error:  Unable to accept incoming connection.\n");
-    return;
+  conn = ilu::HandleNewConnection(p, &closed, &lerr);
+  if (closed) {
+    ilu_DebugPrintf("iluServer::ReadConnectionRequest(%p) port closed.\n",
+		    p);
+    goto abandon;
+  }
+  if (conn == NULL) {
+    ILU_ERR_SWITCH(lerr) {
+      ILU_SUCCESS_CASE {
+	ILU_NOTE(LSR_DEBUG, ("Spurrious input call on port %p.\n", p));
+	goto ok;
+      }
+      ILU_ERR_CASE(no_resources, x)
+	ILU_NOTE(LSR_DEBUG, ("Connection request on port %p"
+			" ran over FD limit --- port abandoned.\n",
+			p));
+      ILU_ERR_ELSE
+	ILU_NOTE(LSR_DEBUG, ("Got error %s"
+			" trying to accept connection on port %p;"
+			" abandoning port.\n",
+			FmtError(ef, &lerr), p));
+    } ILU_ERR_ENDSWITCH;
+    ILU_HANDLED(lerr);
+    goto abandon;
   }
   ilu_SetConnectionInputHandler(conn, ReadServiceRequest, conn, &lerr);
   ILU_MUST_BE_SUCCESS(lerr);
+ok:
+  return;
+abandon:
+  ilu_DoneWithPort(p, &lerr);
+  ilu_PreferSuccess(&lerr);
 }
 
 
@@ -272,24 +361,74 @@ static void RunConnection (void *arg)
   ilu_Connection conn   = (ilu_Connection) arg;
 
   for (;;) {
-    if (!ilu_BlockingWaitForInputOnConnection(conn, (ilu_FineTime *) 0))
+    if (!ilu_BlockingWaitForInputOnConnection(conn, (ilu_FineTime *) 0)) {
+      ILU_ERRS((bad_param, broken_locks, bad_locks, internal)) lerr;
+      ILU_NOTE(LSR_DEBUG, ("ILU-C++ done serving connection %p.\n",
+			conn));
+      ilu_DoneServingConnection(conn, &lerr);
+      ilu_PreferSuccess(&lerr);
+      ILU_HANDLED(lerr);
       break;
+    }
     if (FinalServiceRequest(conn))
       break;
   }
-  ilu::CloseConnection(conn);
+  return;
 }
 
 /* Main invariant holds */
 static void ReadConnectionRequests (void *arg)
 {
   ilu_Port        p = (ilu_Port) arg;
+  ilu_boolean     closed;
+  ILU_ERRS((IoErrs, bad_locks, no_resources)) lerr;
   ilu_Connection  conn;
-  while (ilu::WaitForPortConnectionRequest(p)) {
-    conn = ilu::HandleNewConnection(p);
-    if (conn != ILU_NIL)
+  char            ef[ERRFMTSIZE];
+  while (1) {
+    if (!ilu::WaitForPortConnectionRequest(p, &lerr)) {
+      ILU_NOTE(LSR_DEBUG, (
+	  "No longer waiting for connection requests on port %p,\n"
+	   "because ilu_WaitForPortConnectionRequest raised %s.\n",
+			p, FmtError(ef, &lerr)));
+      ILU_HANDLED(lerr);
+      goto dun;
+    }
+    conn = ilu::HandleNewConnection(p, &closed, &lerr);
+    if (closed) {
+      ILU_NOTE(LSR_DEBUG, (
+	    "No longer waiting for connection requests on port %p,"
+			"because it's closed.\n",
+			p));
+      break;
+    }
+    if (conn == ILU_NIL) {
+      ILU_ERR_SWITCH(lerr) {
+	ILU_SUCCESS_CASE {
+	  ILU_NOTE(LSR_DEBUG, ("Spurrious input call on port %p.\n",
+			    p));
+	  goto nechst;
+	}
+	ILU_ERR_CASE(no_resources, x)
+	  ILU_NOTE(LSR_DEBUG, ("Connection request on port %p"
+			" ran over FD limit --- port abandoned.\n",
+			  p));
+	ILU_ERR_ELSE
+	  ILU_NOTE(LSR_DEBUG, ("Got error %s"
+			  " trying to accept connection on port %p;"
+			  " abandoning port.\n",
+			  FmtError(ef, &lerr), p));
+      } ILU_ERR_ENDSWITCH;
+      ILU_HANDLED(lerr);
+      goto dun;
+    } else {
+      /* fork thread to run new connection */
       Fork(RunConnection, conn);
+    }
+nechst:0;
   }
+dun:
+  if (!ilu_DoneWithPort(p, &lerr))
+    ilu_PreferSuccess(&lerr);
 }
 
 /* Inside(obj->server, obj->class) */
@@ -312,12 +451,11 @@ static void Call_free_self(ilu_ObjectTable self)
   return;
 }
 
-static ilu_string inmemTinfo[] = { "inmem_", 0 };
-
 iluServer::iluServer (	char *serviceID /* = NULL */,
 			iluObjectTable *objtab /* = NULL */)
 {
   ilu_ObjectTable kot;
+  ilu_Error       an_error;
   threadedSet = ilu_TRUE;
   if (objtab == NULL)
     kot = NULL;
@@ -329,12 +467,61 @@ iluServer::iluServer (	char *serviceID /* = NULL */,
   }
   if ((serviceID == NULL) || (strlen(serviceID) == 0))
     serviceID = ilu_InventID();
-  ks = ilu_CreateTrueServer(serviceID, kot, ilu::CppLangIdx());
-  (void) AddPort("sunrpc", inmemTinfo, 0);
+
+  ks = ilu_CreateTrueServer(serviceID, kot, ilu::CppLangIdx(), &an_error);
+
+  if (ILU_ERRNOK(an_error)) {
+    ILU_ERRPRINTF("iluServer::iluServer Error %s  calling ilu_CreateTrueServer, file %s, line %i\n", 
+		  ILU_ERR_NAME(an_error), __FILE__, __LINE__);
+    ILU_HANDLED(an_error); 
+    ILU_CLER(an_error);	
+  }
+
+  // set the language specific server of the kernel server to this iluServer
+  ilu_SetLSS(ks, this, ilu::CppLangIdx(), &an_error);
+
+  if (ILU_ERRNOK(an_error)) {	
+    ILU_ERRPRINTF("iluServer::iluServer Error %s calling ilu_SetLSS, file %s, line %i\n", 
+		  ILU_ERR_NAME(an_error), __FILE__, __LINE__);
+    ILU_HANDLED(an_error); 
+    ILU_CLER(an_error);
+  }
+
+  ilu_ExitServer(ks, ilu_rootClass);
+
+  (void) AddPort(ilu_DefaultProtocolInfo(), ilu_LocalTransportInfo(), 0);
   /* What to do if this fails?? */
   return;
 }
 
+
+
+iluServer::~iluServer () {
+
+  // XXX note: really should call ilu_BankAndScanServer here to ensure all is
+  // properly cleaned up, but we'll assume that the application has already deleted
+  // any objects served by this server 
+
+  // unlink this iluServer from the kernel server
+  ilu_Error an_error;
+
+  ilu_EnterServer(ks, ilu_rootClass);
+
+  ilu_SetLSS(ks, NULL,  ilu::CppLangIdx(), &an_error);
+
+  if (ILU_ERRNOK(an_error)) {	
+    ilu_ExitServer(ks, ilu_rootClass);
+    ILU_ERRPRINTF("iluServer::~iluServer Error %s calling ilu_SetLSS, file %s, line %i\n", 
+		  ILU_ERR_NAME(an_error), __FILE__, __LINE__);
+    ILU_HANDLED(an_error); 
+    ILU_CLER(an_error);
+  }
+}
+
+
+
+
+
 ilu_Boolean iluServer::AddPort(	char *protocolType,
 				ilu_TransportInfo transportType,
 				ilu_Boolean be_default)
@@ -343,11 +530,11 @@ ilu_Boolean iluServer::AddPort(	char *protocolType,
   ilu_Port        nu;
   nu = ilu::CreatePort(ks, protocolType, transportType);
   if (nu == NULL) {
-    fprintf(stderr, "iluServer::Start:  Error:  Couldn't open port.\n");
+    fprintf(stderr, "iluServer::AddPort:  Error:  Couldn't open port.\n");
     return (ilu_FALSE);
   }
   if (be_default) ilu::SetServerDefaultPort(ks, nu);
-  if (threaded && transportType != inmemTinfo) {
+  if (threaded && (transportType != ilu_LocalTransportInfo())) {
     Fork(ReadConnectionRequests, nu);
   } else {
     ilu_SetConnectionRequestHandler(nu, ReadConnectionRequest, nu, &lerr);
@@ -426,3 +613,24 @@ ilu_Boolean iluServer::DisableRequests(iluCall call, ilu_Connection conn)
     call->call.ca_reqs_enabled = ilu_FALSE;
   return ans;
 }
+
+
+/* ********************************************************************** */
+/* Call this to extend the dispatch in the  main loop implemented by Run. */
+
+ilu_Boolean iluServer::RegisterInputHandler (int fd,
+					     void (*handlerProc)(int,void *),
+					     void *handlerArg) {
+
+  return (ilu_RegisterInputSource (fd, handlerProc, handlerArg));
+}
+
+
+/* ***************************************************************************** */
+/* Call this to cancel an extension installed by a call on RegisterInputHandler. */
+
+ilu_Boolean iluServer::UnregisterInputHandler (int fd) {
+
+  return (ilu_UnregisterInputSource (fd));
+}
+
