@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: bsdmnlp.c,v 1.33 1996/05/07 23:11:47 mdavidso Exp $ */
-/* Last edited by Mike Spreitzer March 13, 1996 12:11 pm PST */
+/* $Id: bsdmnlp.c,v 1.43 1997/11/08 01:35:10 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 25, 1997 11:47 pm PDT */
 
 #include "iluntrnl.h"
 #include "oscalls.h"
@@ -28,12 +28,14 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #include <sys/types.h>
 
-#if (defined(WIN32) || defined(WIN16))
+#if (defined(WIN32) || defined(WIN16) || defined(macintosh))
 #include <time.h>
+#define FD_CAST(fd)		((unsigned int)fd)
 /* DLL sez: pick up FD_ZERO, etc. */
 #include <winsock.h>
-#else /* (defined(WIN32) || defined(WIN16)) */
+#else /* (defined(WIN32) || defined(WIN16) || defined(macintosh) */
 #include <sys/time.h>
+#define FD_CAST(fd)		(fd)
 #endif /* not WIN32 or WIN16 */
 
 #ifndef WIN16
@@ -52,11 +54,11 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 /* ================ default Main Loop for UNIX ================ */
 
 struct io_reg {
-  /*L1, L2, Main unconstrained --- single-threaded*/
-  
-  int fd, input;
-  ilu_private rock;
-  ilu_IOHandler proc;
+  /* L1, L2, Main unconstrained --- single-threaded */
+
+  int             fd, input;
+  ilu_IOHandler   proc;
+  ilu_private     rock;
 };
 
 #define IOTABSZ		256
@@ -67,9 +69,9 @@ static struct io_reg IOTab[IOTABSZ];
 static int nIdx = 0, lastIdx = 0;
 static fd_set readfds, writefds, excnfds;
 
-typedef ilu_boolean(*registerer) (int fd,
-				  ilu_IOHandler handler,
-				  ilu_private rock);
+typedef 
+ilu_boolean(*registerer) (int fd,
+			  ilu_IOHandler handler, ilu_private rock);
 typedef ilu_boolean(*canceller) (int fd);
 
 static registerer extra_reg_inp = NULLFN;
@@ -108,8 +110,8 @@ ilu_AddRegisterersToDefault(registerer reg_inp,
 
   /* Externally register any previous registrations. */
   for (i = 0; i < nIdx; i += 1) {
-    if ((*(IOTab[i].input ? extra_reg_inp : extra_reg_out))(IOTab[i].fd,
-       IOTab[i].proc, IOTab[i].rock) != ilu_TRUE)
+    if (!((*(IOTab[i].input ? extra_reg_inp : extra_reg_out))
+	  (IOTab[i].fd, IOTab[i].proc, IOTab[i].rock)))
       return ilu_FALSE;
   }
 
@@ -121,60 +123,58 @@ static void MaybeDumpIOTab(void)
   int             i;
   _ilu_AutoSetDebugLevel();
 #ifdef ENABLE_DEBUGGING
-  if (!(_ilu_DebugLevel & MAINLOOP_DEBUG))
+  if (!(ilu_DebugLevel & MAINLOOP_DEBUG))
     return;
-  ILU_ERRPRINTF("IOTab size is %d.\n", nIdx);
+  ILU_ERRPRINTF("(ILU/MaybeDumpIOTab):  IOTab size is %d.\n", nIdx);
   for (i = 0; i < nIdx; i += 1)
-    ILU_ERRPRINTF("IOTab[%d] = {%d, %d, %p, %p}\n", i,
-       IOTab[i].fd, IOTab[i].input, IOTab[i].proc, IOTab[i].rock);
+    ILU_ERRPRINTF("(ILU/MaybeDumpIOTab):  IOTab[%d] ="
+		  " {%d, %s, %p(%p)}\n", i,
+		  IOTab[i].fd,
+		  IOTab[i].input?"I":"O",
+		  IOTab[i].proc, IOTab[i].rock);
   return;
 #else
   return;
 #endif
 }
 
-static ilu_boolean 
-Default_UnregisterInputSource(int fd)
+static          ilu_boolean
+Default_UnregisterInputSource(int fd, ilu_IOHandler * proc,
+			      ilu_refany * rock)
 {
   register int    i;
   static ilu_boolean also = TRUE;
   if (extra_can_inp != NULLFN)
     also = (*extra_can_inp) (fd);
-#if (defined (WIN32) || defined (WIN16))
-  FD_CLR(((unsigned int) fd), &readfds);
-  FD_CLR(((unsigned int) fd), &excnfds);
-#else
-  FD_CLR(fd, &readfds);
-  FD_CLR(fd, &excnfds);
-#endif
+  FD_CLR(FD_CAST(fd), &readfds);
+  FD_CLR(FD_CAST(fd), &excnfds);
   for (i = 0; i < nIdx; i += 1)
     if ((IOTab[i].fd == fd) && IOTab[i].input) {
+      *proc = IOTab[i].proc;
+      *rock = IOTab[i].rock;
       nIdx--;
       if (lastIdx > nIdx)
 	lastIdx = nIdx;
-      if (i != nIdx) {
-	IOTab[i].fd = IOTab[nIdx].fd;
-	IOTab[i].input = IOTab[nIdx].input;
-	IOTab[i].proc = IOTab[nIdx].proc;
-	IOTab[i].rock = IOTab[nIdx].rock;
-      }
+      if (i != nIdx)
+	IOTab[i] = IOTab[nIdx];
       MaybeDumpIOTab();
       return (also);
     }
-  DEBUG(MAINLOOP_DEBUG, (stderr,
-		    "UnregisterInputSource: FD %d not in table!\n",
-			 fd));
+  ILU_NOTE(MAINLOOP_DEBUG,
+     ("(ILU/Default_UnregisterInputSource): FD %d not in table!\n",
+      fd));
+  *proc = NULLFN;
+  *rock = NIL;
   return (ilu_FALSE);
 }
 
 static          ilu_boolean
 Default_RegisterInputSource(int fd,
-			    ilu_IOHandler proc,
-			    ilu_private rock)
+			    ilu_IOHandler proc, ilu_private rock)
 {
   if (nIdx >= IOTABSZ) {
-    DEBUG(MAINLOOP_DEBUG, (stderr,
-			   "RegisterInputSource: table full!\n"));
+    ILU_NOTE(MAINLOOP_DEBUG,
+	     ("(ILU/Default_RegisterInputSource): table full!\n"));
     return (ilu_FALSE);
   }
   IOTab[nIdx].fd = fd;
@@ -190,51 +190,51 @@ Default_RegisterInputSource(int fd,
    */
   if (extra_reg_inp != NULLFN)
     if (!(*extra_reg_inp) (fd, proc, rock)) {
-      Default_UnregisterInputSource(fd);
+      ilu_IOHandler   proc2;
+      ilu_private     rock2;
+      Default_UnregisterInputSource(fd, &proc2, &rock2);
       return ilu_FALSE;
     }
   return (ilu_TRUE);
 }
 
-static ilu_boolean Default_UnregisterOutputSource (int fd)
+static ilu_boolean 
+Default_UnregisterOutputSource(int fd, ilu_IOHandler * proc,
+			       ilu_refany * rock)
 {
   register int    i;
   static ilu_boolean also = ilu_TRUE;
   if (extra_can_out != NULLFN)
     also = (*extra_can_out) (fd);
-#if (defined (WIN32) || defined(WIN16))
-  FD_CLR(((unsigned int) fd), &writefds);
-#else
-  FD_CLR(fd, &writefds);
-#endif
+  FD_CLR(FD_CAST(fd), &writefds);
   for (i = 0; i < nIdx; i += 1)
     if ((IOTab[i].fd == fd) && !IOTab[i].input) {
+      *proc = IOTab[i].proc;
+      *rock = IOTab[i].rock;
       nIdx--;
       if (lastIdx > nIdx)
 	lastIdx = nIdx;
       if (i != nIdx) {
-	IOTab[i].fd = IOTab[nIdx].fd;
-	IOTab[i].input = IOTab[nIdx].input;
-	IOTab[i].proc = IOTab[nIdx].proc;
-	IOTab[i].rock = IOTab[nIdx].rock;
+	IOTab[i] = IOTab[nIdx];
       }
       MaybeDumpIOTab();
       return (also);
     }
-  DEBUG(MAINLOOP_DEBUG, (stderr,
-		   "UnregisterOutputSource: FD %d not in table!\n",
-			 fd));
+  ILU_NOTE(MAINLOOP_DEBUG,
+    ("(ILU/Default_UnregisterOutputSource): FD %d not in table!\n",
+     fd));
+  *proc = NULLFN;
+  *rock = NIL;
   return (ilu_FALSE);
 }
 
-static ilu_boolean 
+static          ilu_boolean
 Default_RegisterOutputSource(int fd,
-			     ilu_IOHandler proc,
-			     ilu_private rock)
+			     ilu_IOHandler proc, ilu_private rock)
 {
   if (nIdx >= IOTABSZ) {
-    DEBUG(MAINLOOP_DEBUG, (stderr,
-			   "RegisterOutputSource: table full!\n"));
+    ILU_NOTE(MAINLOOP_DEBUG,
+	     ("(ILU/Default_RegisterOutputSource): table full!\n"));
     return (ilu_FALSE);
   }
   IOTab[nIdx].fd = fd;
@@ -245,7 +245,9 @@ Default_RegisterOutputSource(int fd,
   MaybeDumpIOTab();
   if (extra_reg_out != NULLFN)
     if (!(*extra_reg_out) (fd, proc, rock)) {
-      Default_UnregisterOutputSource(fd);
+      ilu_IOHandler   proc2;
+      ilu_private     rock2;
+      Default_UnregisterOutputSource(fd, &proc2, &rock2);
       return ilu_FALSE;
     }
   return (ilu_TRUE);
@@ -286,6 +288,8 @@ DefaultAlarm_s _ilu_grDefaultAlarm_s
 	= {{NIL, NIL, FALSE, {0, 0}}, NULLFN, NIL};
 DefaultAlarm_s _ilu_udpDefaultAlarm_s
 	= {{NIL, NIL, FALSE, {0, 0}}, NULLFN, NIL};
+DefaultAlarm_s _ilu_soonDefaultAlarm_s
+	= {{NIL, NIL, FALSE, {0, 0}}, NULLFN, NIL};
 
 static void Callem(ilu_FineTime t)
 {
@@ -325,7 +329,7 @@ static void daUrcancel(void)
 
 static ilu_refany Default_CreateAlarm(void)
 {
-  DefaultAlarm    da = (DefaultAlarm) ilu_malloc(sizeof(DefaultAlarm_s));
+  DefaultAlarm    da = (DefaultAlarm) ilu_must_malloc(sizeof(*da));
   ilu_Alarmette_s ae = {NIL, NIL, FALSE, {0, 0}};
   da->ae = ae;
   da->proc = NULLFN;
@@ -357,6 +361,16 @@ static void Default_UnsetAlarm(ilu_refany thisalarm)
   return;
 }
 
+static void Default_DestroyAlarm(ilu_refany thisalarm)
+{
+  DefaultAlarm    da = (DefaultAlarm) thisalarm;
+  _ilu_AcquireMutex(ilu_daimu);
+  ilu_MXAClear(&dar, &da->ae);
+  _ilu_ReleaseMutex(ilu_daimu);
+  ilu_free(da);
+  return;
+}
+
 /*L1, L2 unconstrained*/
 static float MyRand(void)
 {
@@ -399,6 +413,8 @@ static void Default_RunMainLoop(int *stop)
       t = ilu_FineTime_Now();
       if (ilu_FineTime_Cmp(t, alarmTime) >= 0) {
 	alarmSet = 0;
+	ILU_NOTE(MAINLOOP_DEBUG,
+		 ("(ILU/Default_RunMainLoop):  calling alarms.\n"));
 	ilu_MXAProc(t, &dar);
 	_ilu_ReleaseMutex(ilu_daimu);
 	goto next;		/* find next thing to do */
@@ -428,25 +444,20 @@ static void Default_RunMainLoop(int *stop)
       ilu_boolean     forEx, forIo;
       fd_set         *iofds;
       --lastIdx;
-      forEx = FD_ISSET(IOTab[lastIdx].fd, &excnfds) != 0;
+      forEx = (IOTab[lastIdx].fd >= 0) && (FD_ISSET(IOTab[lastIdx].fd, &excnfds) != 0);
       iofds = IOTab[lastIdx].input ? &readfds : &writefds;
-      forIo = FD_ISSET(IOTab[lastIdx].fd, iofds) != 0;
+      forIo = (IOTab[lastIdx].fd >= 0) && (FD_ISSET(IOTab[lastIdx].fd, iofds) != 0);
       if (forEx || forIo) {
 	lastreg = IOTab[lastIdx];
-	DEBUG(MAINLOOP_DEBUG,
-	      (stderr,
-	       "Default MainLoop: calling IOTab[%d](%ld,%ld) = {%d, %d, %p, %p}\n",
+	ILU_NOTE(MAINLOOP_DEBUG,
+	      ("(ILU/Default_RunMainLoop):  calling IOTab[%d](%ld,%ld) = {%d, %s, %p(%p)}\n",
 	       lastIdx, (long) forEx, (long) forIo,
 	       lastreg.fd,
-	       lastreg.input,
+	       lastreg.input?"I":"O",
 	       lastreg.proc,
 	       lastreg.rock));
 	if (forIo)
-#if (defined (WIN32) || defined (WIN16))
-          FD_CLR(((unsigned int) lastreg.fd), iofds);
-#else
-	  FD_CLR(lastreg.fd, iofds);
-#endif
+          FD_CLR(FD_CAST(lastreg.fd), iofds);
 	(*lastreg.proc) (lastreg.fd, lastreg.rock);
 	goto next;
       }
@@ -457,12 +468,14 @@ static void Default_RunMainLoop(int *stop)
     FD_ZERO(&readfds);
     FD_ZERO(&writefds);
     FD_ZERO(&excnfds);
-    DEBUG(MAINLOOP_DEBUG, (stderr, "Default_RunMainLoop: select "));
+    ILU_NOTE(MAINLOOP_DEBUG, ("(ILU/Default_RunMainLoop): select "));
     for (i = 0; i < nIdx; i++) {
-      DEBUG(MAINLOOP_DEBUG, (stderr, "%d%s ", IOTab[i].fd,
+      ILU_NOTE(MAINLOOP_DEBUG, ("%d%s ", IOTab[i].fd,
 			     IOTab[i].input ? "I" : "O"));
-      FD_SET(IOTab[i].fd, (IOTab[i].input ? &readfds : &writefds));
-      FD_SET(IOTab[i].fd, &excnfds);
+      if (IOTab[i].fd >= 0) {
+	FD_SET(IOTab[i].fd, (IOTab[i].input ? &readfds : &writefds));
+	FD_SET(IOTab[i].fd, &excnfds);
+      };
       if (IOTab[i].fd >= width)
 	width = IOTab[i].fd + 1;
     }
@@ -470,12 +483,9 @@ static void Default_RunMainLoop(int *stop)
 
     status = select(width, &readfds, &writefds, &excnfds, ptv);
     theerr = sockerrno;
-    ASSERT(status >= 0 || theerr == SOCKERRID(INTR), buf,
-	   (buf, "default mainloop:select failed, errno=%d=%s",
-	    theerr, strerror(theerr)));
 
 #ifdef ENABLE_DEBUGGING
-    if (_ilu_DebugLevel & MAINLOOP_DEBUG) {
+    if (ilu_DebugLevel & MAINLOOP_DEBUG) {
       ILU_ERRPRINTF("=> %d %d:", status, theerr);
       for (i = 0; i < width; i++) {
 	if (FD_ISSET(i, &readfds))
@@ -489,15 +499,25 @@ static void Default_RunMainLoop(int *stop)
     }
 #endif /* ENABLE_DEBUGGING */
     
+    if (status == -1 && theerr == EINTR &&
+	(_ilu_SignalCallbackHandler != NULLFN)) {
+      ILU_NOTE(MAINLOOP_DEBUG, ("%s", "(ILU/Default_RunMainLoop):"
+			     "  Calling signal handler callback\n"));
+      (*_ilu_SignalCallbackHandler)(_ilu_SignalCallbackHandlerArg);
+    } else {
+      ASSERT(status >= 0 || theerr == SOCKERRID(INTR), buf,
+            (buf, "default mainloop:select failed, errno=%d=%s",
+             theerr, strerror(theerr)));
+    }
+
     if (TestIluMainLoop) {
       int             victim = (int) (nIdx * MyRand());
       struct io_reg   vior;
       _ilu_Assert(0 <= victim && victim < nIdx, "TestIluMainLoop");
       vior = IOTab[victim];
       FD_SET(vior.fd, (vior.input ? &readfds : &writefds));
-      DEBUG(MAINLOOP_DEBUG,
-	    (stderr,
-	     "Hacking IOTab[%d]={fd=%d, input=%d}\n",
+      ILU_NOTE(MAINLOOP_DEBUG,
+	    ("(ILU/Default_RunMainLoop):  Hacking IOTab[%d]={fd=%d, input=%d}\n",
 	     victim, vior.fd, vior.input));
     }
     
@@ -523,5 +543,5 @@ ilu_MainLoop    _ilu_DefaultMainLoop = {
   Default_RegisterInputSource, Default_UnregisterInputSource,
   Default_RegisterOutputSource, Default_UnregisterOutputSource,
   Default_CreateAlarm,
-  Default_SetAlarm, Default_UnsetAlarm
+  Default_SetAlarm, Default_UnsetAlarm, Default_DestroyAlarm
 };
