@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1991-1994 Xerox Corporation.  All Rights Reserved.  
+Copyright (c) 1991-1997 Xerox Corporation.  All Rights Reserved.  
 
 Unlimited use, reproduction, and distribution of this software is
 permitted.  Any copy of this software must include both the above
@@ -14,8 +14,9 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: bsdutils.c,v 1.34 1996/07/16 23:15:03 spreitze Exp $ */
-/* Last edited by Mike Spreitzer July 16, 1996 4:14 pm PDT */
+/* $Id: bsdutils.c,v 1.47 1997/11/25 01:34:01 janssen Exp $ */
+/* Last edited by Mike Spreitzer July 8, 1997 11:30 pm PDT */
+/* Chris Jacobi, May 2, 1997 3:54 pm PDT */
 
 #include <signal.h>	/* for SIGPIPE, SIG_IGN, SIG_DFL */
 #include <limits.h>	/* for INT_MAX */
@@ -24,31 +25,45 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include "oscalls.h"
 #include "ilusock.h"
 
+/* oscalls.h will have included unistd.h, which may or may not define
+ * a prototype for gethostname().  If not, define it here.
+ */
+#ifndef HAVE_GETHOSTNAME_PROTOTYPE
+extern int gethostname(char *name, int namelen);
+#endif /* HAVE_GETHOSTNAME_PROTOTYPE */
+
+#ifdef macintosh
+#define SEND_RECV_BUF_TYPE(b)	((char *)b)
+#else
+#define SEND_RECV_BUF_TYPE(b)	((char *)b)
+#endif
+
 
 /* ================ BSD/WIN Hostname ================ */
 
 /*L1, L2, Main unconstrained*/
 
-#if (defined(_IS_BSD) || defined(WIN32) || defined(WIN16))
-
-#if 0
-/*
- * Linux doesn't like "int", and IRIX doesn't like "size_t", so
- * there's no way to make this declaration without getting a fatal
- * error.  So can it.  Who wanted it, anyway?
- */
-#ifdef _IS_BSD
-extern int      gethostname(char *, int);
-#endif
-#endif
+#if (defined(_IS_BSD) || defined(WIN32) || defined(WIN16) || defined(macintosh))
 
 ilu_string _ilu_Hostname(void)
 {
   static ilu_string name = NIL;
+
   if (name == NIL) {
     char            hostname[1000];
+#ifdef macintosh
+	if ( gethostname(hostname, sizeof(hostname)) == 0 )
+	{
+	}
+	else
+	{
+		strcpy( hostname, ILU_MAC_DEFAULT_HOST );
+	}
+
+#else
     _ilu_Assert(gethostname(hostname, sizeof(hostname)) == 0,
 		"gethostname failed");
+#endif
     name = _ilu_Strdup(hostname);
   }
   return (name);
@@ -82,18 +97,18 @@ _ilu_CurrentHostIPAddrString(ilu_string * host_out,
   struct in_addr *hea;
   if (inetname == NIL) {
     if ((myname = _ilu_Hostname()) == NIL) {
-#if (defined(WIN32) || defined(WIN16))
+#if (defined(WIN32) || defined(WIN16) || defined(macintosh))
       ILU_ERRPRINTF("no hostname for this machine! WSAGetLastError() = %i\n",
 		    WSAGetLastError());
 #else
       perror("no hostname for this machine!");
 #endif
-      return ILU_ERR_CONS1(internal, err, minor, ilu_im_noHostName, NIL);
+      return ILU_ERR_CONS1(internal, err, minor, ilu_im_noHostName, (ilu_string) NIL);
     }
     he = gethostbyname(myname);
     if (he == NIL || he->h_addr_list == NIL || he->h_addr_list[0] == NIL)
       return ILU_ERR_CONS1(internal, err, minor, ilu_im_noHostIpAddr,
-			   NIL);
+			   (ilu_string) NIL);
     hea = (struct in_addr *) (he->h_addr_list[0]);
     tempname = (ilu_string) inet_ntoa(myaddr = *hea);
     inetname = ilu_StrdupE(tempname, err);
@@ -108,11 +123,18 @@ _ilu_CurrentHostIPAddrString(ilu_string * host_out,
   return (inetname);
 }
 
+/*L1 >= {trmu}; L2 unconstrained*/
+ilu_string
+_ilu_CurrentHostIPAddrStringOnly(ILU_ERRS((IoErrs)) * err)
+{
+  return _ilu_CurrentHostIPAddrString(NIL, NIL, err);
+}
+
 /* ================ BSD/WIN ilu_FineTime ================ */
 
 /*L1, L2, Main unconstrained*/
 
-#if (defined(_IS_BSD) || defined(WIN32) || defined(WIN16))
+#if (defined(_IS_BSD) || defined(WIN32) || defined(macintosh))
 
 /*
  * "ilu_FineTime_Now" and "ilu_CoarseTime_Now" should be implemented
@@ -160,6 +182,22 @@ ilu_FineTime ilu_FineTime_Now()
   return ans;
 }
 
+#elif defined( macintosh )
+
+#include <unistd.h>	/* for time() */
+#include <types.h>	/* for time() */
+#include <time.h>	/* for time(), gettimeofday */
+
+const ilu_cardinal    ilu_FineTimeRate = 60;
+
+ilu_FineTime ilu_FineTime_Now()
+{
+  ilu_FineTime    ans;
+  ans.ft_s = clock() / CLOCKS_PER_SEC;
+  ans.ft_t =  clock() % CLOCKS_PER_SEC;
+  return ans;
+}
+
 #else
 
 #error "Don't know how to get FineTime for this OS!"
@@ -175,9 +213,10 @@ ilu_integer ilu_CoarseTime_Now()
 
 /*L1, L2, Main unconstrained*/
 
-static int need_seed = 1;
+/* monotonically increasing counter used in generating unique id's */
+static unsigned long l_id_counter = 0;
 
-/*L1_sup < smu*/
+/*L1_sup < trmu*/
 ilu_string ilu_InventID ()
 {
   char			buf[1000];
@@ -185,11 +224,7 @@ ilu_string ilu_InventID ()
   static ilu_string	iphostname = NIL;
   ilu_Error		lerr;
 
-  _ilu_AcquireMutex(ilu_smu);
-  if (need_seed) {
-    srand(OS_GETPID());
-    need_seed = 0;
-  }
+  _ilu_AcquireMutex(ilu_trmu);
   if (iphostname == NIL)
     {
       /* use HostIPAddr instead of Hostname to get domain info */
@@ -204,8 +239,11 @@ ilu_string ilu_InventID ()
   sprintf(buf, "%s.%lx.%lx.%lx", iphostname,
 	  (long unsigned) OS_GETPID(),
 	  (long unsigned) now.ft_s,
-	  (long unsigned) rand());
-  _ilu_ReleaseMutex(ilu_smu);
+	  (long unsigned) l_id_counter);
+
+  l_id_counter++;
+
+  _ilu_ReleaseMutex(ilu_trmu);
   return _ilu_Strdup(buf);
 }
 
@@ -250,9 +288,9 @@ _ilu_HandleSigPIPE(void)
   SigPIPEHandler = TRUE;
   _ilu_ReleaseMutex(ilu_trmu);
 
-#elif (defined(WIN32) || defined(WIN16))
+#elif (defined(WIN32) || defined(WIN16) || defined(macintosh))
 
- /* do nothing for win32 as there is no SIGPIPE */
+ /* do nothing for these archs as there is no SIGPIPE */
 
 #else		/* not really POSIX -- use ANSI C form */
 
@@ -287,6 +325,45 @@ ilu_SIGPIPE_Handled(void)
 
 /* ================ BSD/WIN Socket I/O ================ */
 
+/* We have two function pointers which hold the actual bindings
+   for recv() and send().
+*/
+#ifdef HAS_SOLARIS2_TCP_SOCKET_BUG
+static int
+_local_recv(int s, char* buf, int len, int flags)
+{
+    return read(s, buf, len);
+}
+
+static int
+_local_send(int s, const char* msg, int len, int flags)
+{
+    return write(s, msg, len);
+}
+
+static ilu_RecvProc LocalRecvProc = (ilu_RecvProc) &_local_recv;
+static ilu_SendProc LocalSendProc = (ilu_SendProc) &_local_send;
+
+#else
+
+static ilu_RecvProc LocalRecvProc = (ilu_RecvProc) &recv;
+static ilu_SendProc LocalSendProc = (ilu_SendProc) &send;
+
+#endif
+
+/*Main Invariant holds*/
+
+/* The Java JDK 1.0.2 code doesn't do non-blocking correctly.  So we
+   need to provide a way to let the Java LSR override the system calls
+   read() and write().
+*/
+void
+  ilu_SetRecvSendProcs (ilu_RecvProc rp, ilu_SendProc sp)
+{
+  LocalRecvProc = rp;
+  LocalSendProc = sp;
+}
+
 /*Main Invariant holds*/
 /*L2 >= {fd's connection's callmu, iomu}*/
 
@@ -302,19 +379,15 @@ _ilu_NbSockRead(int fd, unsigned char *buf, ilu_cardinal bufLen,
   req = (int) reql;
   rpt->tr_eof = FALSE;
   while (1) {
-#ifdef HAS_SOLARIS2_TCP_SOCKET_BUG
-    got = read(fd, buf, req);
-#else
-    got = recv(fd, buf, req, 0);
-#endif
+    got = (*LocalRecvProc)(fd, SEND_RECV_BUF_TYPE(buf), req, 0);
     theerr = sockerrno;
     if (!OS_SOCKERR(got)) {
       rpt->tr_eof = (got == 0);
-      if (got == 0)
-	DEBUG(CONNECTION_DEBUG,
-	      (stderr,
-	       "_ilu_NbSockRead: Clean EOF detected on FD %d.\n",
+      if (got == 0) {
+	ILU_NOTE(CONNECTION_DEBUG,
+	      ("_ilu_NbSockRead: Clean EOF detected on FD %d.\n",
 	       fd));
+      }
       goto dun;
     }
     switch (theerr) {
@@ -334,20 +407,19 @@ _ilu_NbSockRead(int fd, unsigned char *buf, ilu_cardinal bufLen,
     case SOCKERRID(NETDOWN):
     case SOCKERRID(CONNABORTED):
     case SOCKERRID(CONNRESET):
-#if !(defined(WIN32) || defined(WIN16))
+    case SOCKERRID(SHUTDOWN):
+#if !(defined(WIN32) || defined(WIN16) || defined(macintosh))
     case SOCKERRID(PIPE):
 #endif
-      DEBUG(CONNECTION_DEBUG,
-	    (stderr,
-	     "_ilu_NbSockRead: Connection failure (errno %d) from recv(%d).\n",
+      ILU_NOTE(CONNECTION_DEBUG,
+	    ("_ilu_NbSockRead: Connection failure (errno %d) from recv(%d).\n",
 	     theerr, fd));
       rpt->tr_eof = TRUE;
       got = 0;
       goto dun;
     default:
-      DEBUG(CONNECTION_DEBUG,
-	    (stderr,
-	 "_ilu_NbSockRead: Impossible errno (%d) from recv(%d).\n",
+      ILU_NOTE(CONNECTION_DEBUG,
+	    ("_ilu_NbSockRead: Impossible errno (%d) from recv(%d).\n",
 	     theerr, fd));
       return ILU_ERR_CONS1(internal, err, minor, ilu_im_errno,
 			   0);
@@ -378,11 +450,7 @@ _ilu_SockWrite(int fd, unsigned char *buf, ilu_cardinal nbytes,
     if (reql > INT_MAX)
       reql = INT_MAX;
     req = (int) reql;
-#ifdef HAS_SOLARIS2_TCP_SOCKET_BUG
-    got = write(fd, buf + sofar, req);
-#else
-    got = send(fd, buf + sofar, req, 0);
-#endif
+    got = (*LocalSendProc)(fd, SEND_RECV_BUF_TYPE(buf) + sofar, req, 0);
     theerr = sockerrno;
     if (!OS_SOCKERR(got)) {
       sofar += (unsigned) got;
@@ -397,9 +465,8 @@ _ilu_SockWrite(int fd, unsigned char *buf, ilu_cardinal nbytes,
 #error "no applicable errno error for 'Would Block'"
 #endif
       if (sure)
-	DEBUG(CONNECTION_DEBUG,
-	      (stderr,
-	       "_ilu_SockWrite: Sure write to FD %d WOULDBLOCK!\n",
+	ILU_NOTE(CONNECTION_DEBUG,
+	      ("_ilu_SockWrite: Sure write to FD %d WOULDBLOCK!\n",
 	       fd));
     case SOCKERRID(INTR):
       break;
@@ -408,23 +475,23 @@ _ilu_SockWrite(int fd, unsigned char *buf, ilu_cardinal nbytes,
     case SOCKERRID(NETDOWN):
     case SOCKERRID(CONNABORTED):
     case SOCKERRID(CONNRESET):
-#if !(defined(WIN32) || defined(WIN16))
+    case SOCKERRID(SHUTDOWN):
+#if !(defined(WIN32) || defined(WIN16) || defined(macintosh))
     case SOCKERRID(PIPE):
 #endif
-      DEBUG(CONNECTION_DEBUG,
-	    (stderr,
-       "_ilu_SockWrite: Connection failure (errno %d) on FD %d.\n",
+      ILU_NOTE(CONNECTION_DEBUG,
+	    ("_ilu_SockWrite: Connection failure (errno %d) on FD %d.\n",
 	     theerr, fd));
-      return ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_conn_lost,
-			   FALSE);
+      return (ilu_boolean) ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_conn_lost,
+					 FALSE);
     default:
-      DEBUG(CONNECTION_DEBUG,
-	    (stderr,
-	     "_ilu_SockWrite: Impossible errno (%d) for FD %d.\n",
+      ILU_NOTE(CONNECTION_DEBUG,
+	    ("_ilu_SockWrite: Impossible errno (%d) for FD %d.\n",
 	     theerr, fd));
-      return ILU_ERR_CONS1(internal, err, minor, ilu_im_errno, FALSE);
+      return (ilu_boolean) ILU_ERR_CONS1(internal, err, minor, ilu_im_errno, FALSE);
     }
   }
   return TRUE;
 }
 
+
