@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: courier.c,v 1.117 1996/07/09 23:47:04 janssen Exp $ */
-/* Last edited by Mike Spreitzer June 27, 1996 4:30 pm PDT */
+/* $Id: courier.c,v 1.134 1997/11/13 23:13:41 spreitze Exp $ */
+/* Last edited by Mike Spreitzer November 13, 1997 2:36 pm PST */
 
 #ifdef MACOS
 #pragma segment ilu
@@ -63,6 +63,11 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #endif
 
 #define callport(call)	((call)->ca_prTrans)
+/*
+ * callport(call)'s xmu == call's conn's iomu;
+ * callport(call)'s ymu == (call->ca_prbit1 ? conn's iomu :
+ * conn's waitmu).
+ */
 
 typedef struct {
   ilu_Class cri_class;
@@ -70,12 +75,15 @@ typedef struct {
   ilu_cardinal cri_pnumber, cri_version;
 } courierinfo;
 
-static ilu_cardinal CourierMaxStringSize = 100000;
+static ilu_cardinal CourierMaxStringSize = 0xFFFFFFFF;
 
 static          ilu_boolean
 _courier_InitCall(ilu_Call call, ILU_ERRS((IoErrs)) * err)
 {
+  if (!transport_boundaried(connection_transport(call->ca_connection)))
+    return ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_tr_non_boundaried, ilu_FALSE);
   call->ca_prTrans = connection_transport(call->ca_connection);
+  call->ca_prbit1 = FALSE;
   return ILU_CLER(*err);
 }
 
@@ -83,16 +91,23 @@ static          ilu_boolean
 _courier_FinishCall(ilu_Call call, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
-  if (call->ca_prTrans != NIL &&
-   call->ca_prTrans != connection_transport(call->ca_connection)) {
+  if (call->ca_prbit1) {
     ilu_integer     cdfd;
     transport_close(call->ca_prTrans, &cdfd, err);
+    ILU_ERR_SWITCH(*err) {
+      ILU_ERR_CASE(bad_locks, x) {
+	ILU_HANDLED(*err);
+	(void) ilu_Check(FALSE, err);
+      }
+      ILU_ERR_ELSE 0;
+    } ILU_ERR_ENDSWITCH;
     _ilu_Assert(cdfd == 0, "courier FinishCall");
   }
   return ILU_ERROK(*err);
 }
 
-static ilu_refany _courier_CreateDataBlock (ILU_ERRS((no_memory)) *err)
+static ilu_refany _courier_CreateDataBlock (ilu_ProtocolInfo pinfo,
+					    ILU_ERRS((no_memory)) *err)
 {
   return (ILU_CLER(*err), NIL);
 }
@@ -133,9 +148,9 @@ static void _courier_EnsureRegistries(void)
 {
   if (RegistryHashTable == NIL)
     {
-      RegistryHashTable = _ilu_hash_MakeNewTable (137, NULLFN, NULLFN);
+      RegistryHashTable = ilu_hash_MakeNewTable (137, NULLFN, NULLFN);
       ProgramNumberHashTable =
-	_ilu_hash_MakeNewTable (137,
+	ilu_hash_MakeNewTable (137,
 				(ILU_hash_proc)_courier_HashCourier,
 				(ILU_find_proc)_courier_CompareCourier);
       _courier_AddClassInformation (_ilu_rootClass);
@@ -179,15 +194,15 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
     }
   else
     {
-      ilu_cardinal hash = _ilu_CRC32 ((ilu_bytes) class->cl_unique_id, strlen(class->cl_unique_id));
+      ilu_cardinal hash = ilu_CRC32 ((ilu_bytes) class->cl_unique_id, strlen(class->cl_unique_id));
 
       s->cri_pnumber = ILU_COURIER_PROGRAM_NUMBER | ((hash >> 16) & 0xFFFF);
       s->cri_version = hash & 0xFFFF;
     }
-  _ilu_Assert((int) _ilu_hash_AddToTable(RegistryHashTable,
+  _ilu_Assert((int) ilu_hash_AddToTable(RegistryHashTable,
 				   s->cri_type_id, s),
 	      "Courier AddToTable RegistryHashTable");
-  _ilu_Assert((int) _ilu_hash_AddToTable(ProgramNumberHashTable, s, s),
+  _ilu_Assert((int) ilu_hash_AddToTable(ProgramNumberHashTable, s, s),
 	      "Courier AddToTable ProgramNumberHashTable");
   return (s);
 }
@@ -198,13 +213,13 @@ static courierinfo *_courier_CourierInformationForClass (ilu_Class class)
 
   _courier_EnsureRegistries();
 
-  s = (courierinfo *) _ilu_hash_FindInTable (RegistryHashTable,
+  s = (courierinfo *) ilu_hash_FindInTable (RegistryHashTable,
 					    class->cl_unique_id);
   if (s == NIL)
     s = _courier_AddClassInformation (class);
 
-  DEBUG(COURIER_DEBUG,
-	(stderr, "%s \"%s:%s\", pnumber is 0x%lx, version is %lu.\n",
+  ILU_NOTE(COURIER_DEBUG,
+	("%s \"%s:%s\", pnumber is 0x%lx, version is %lu.\n",
 	 "_courier_CourierInformationForClass:  Class",
 	 class->cl_name, class->cl_unique_id,
 	 (s == NIL) ? 0L : s->cri_pnumber,
@@ -229,7 +244,7 @@ static courierinfo *_courier_ClassFromProgramNumber (ilu_cardinal pnumber,
 
   dummy.cri_pnumber = pnumber;
   dummy.cri_version = version;
-  s = (courierinfo *) _ilu_hash_FindInTable (ProgramNumberHashTable,
+  s = (courierinfo *) ilu_hash_FindInTable (ProgramNumberHashTable,
 					      (ilu_refany) &dummy);
   if (s == NIL)
     {
@@ -246,8 +261,8 @@ static courierinfo *_courier_ClassFromProgramNumber (ilu_cardinal pnumber,
     }
   else
     {
-      DEBUG(COURIER_DEBUG,
-	    (stderr, "%s \"%s:%s\", pnumber is 0x%lx, version is %lu.\n",
+      ILU_NOTE(COURIER_DEBUG,
+	    ("%s \"%s:%s\", pnumber is 0x%lx, version is %lu.\n",
 	     "_courier_ClassFromProgramNumber:  Class",
 	     s->cri_class->cl_name, s->cri_class->cl_unique_id,
 	     s->cri_pnumber, s->cri_version));
@@ -278,8 +293,8 @@ static ilu_cardinal ilu_courier_SetMaxStringSize (ilu_cardinal size)
 }
 
 /*L1, L2 unconstrained for sizing, end*/
-/*Main holds, L2 >= {call's connection's callmu, iomu} for output*/
-/*L1 unconstrained, L2 >= {call's connection's callmu, iomu} for input*/
+/*Main Invariant, Call-OHi(call) for output*/
+/*Main Invariant, Call-IHi(call) for input*/
 
 /* ==================== short cardinal ==================== */
 
@@ -493,31 +508,6 @@ _courier_SizeOfLongCardinal(ilu_Call call, ilu_longcardinal i,
   return (8);
 }
 
-/* ==================== enumeration ==================== */
-
-static void 
-_courier_OutputEnumeration(ilu_Call call, ilu_shortcardinal i,
-			   ILU_ERRS((IoErrs)) * err)
-{
-  _courier_OutputShortCardinal(call, i, err);
-}
-
-static void 
-_courier_InputEnumeration(ilu_Call call, ilu_shortcardinal * i,
-			  ILU_ERRS((IoErrs)) * err)
-{
-  _courier_InputShortCardinal(call, i, err);
-}
-
-/*ARGSUSED*/
-static          ilu_cardinal
-_courier_SizeOfEnumeration(ilu_Call call, ilu_shortcardinal i,
-			   ILU_ERRS((IoErrs)) * err)
-{
-  ILU_CLER(*err);
-  return (2);
-}
-
 /* ==================== real ==================== */
 
 static void 
@@ -670,24 +660,45 @@ _courier_SizeOfBytes(ilu_Call call, ilu_bytes i, ilu_cardinal l,
 /* ==================== string ==================== */
 
 static void 
-_courier_OutputString(ilu_Call call, ilu_string s, ilu_cardinal len,
-		      ilu_cardinal limit, ILU_ERRS((IoErrs)) * err)
+_courier_OutputString(ilu_Call call, void * s, ilu_cardinal len,
+		      ilu_cardinal limit,
+		      ilu_cardinal expected_encoding,
+		      ilu_cardinal current_encoding,
+		      ILU_ERRS((IoErrs)) * err)
 {
+  if ((expected_encoding != ILU_StringEncoding_latin1) ||
+      (current_encoding != ILU_StringEncoding_latin1)) {
+    ILU_ERR_CONS1(imp_limit, err, minor, ilu_ilm_unsupported_charset_encoding, 0);
+    return;
+  }
   _courier_OutputBytes(call, (ilu_bytes) s, len, limit, err);
 }
 
 static void 
-_courier_InputString(ilu_Call call, ilu_string * s, ilu_cardinal * len,
-		     ilu_cardinal limit, ILU_ERRS((IoErrs)) * err)
+_courier_InputString(ilu_Call call, void ** s, ilu_cardinal * len,
+		     ilu_cardinal limit, ilu_cardinal expected_encoding,
+		     ilu_cardinal *current_encoding,
+		     ILU_ERRS((IoErrs)) * err)
 {
+  if (expected_encoding != ILU_StringEncoding_latin1) {
+    ILU_ERR_CONS1(imp_limit, err, minor, ilu_ilm_unsupported_charset_encoding, 0);
+    return;
+  };
   _courier_InputBytes(call, (ilu_bytes *) s, len, limit, err);
 }
 
   /*ARGSUSED*/
 static          ilu_cardinal
-_courier_SizeOfString(ilu_Call call, ilu_string i, ilu_cardinal l,
-		      ilu_cardinal limit, ILU_ERRS((IoErrs)) * err)
+_courier_SizeOfString(ilu_Call call, void * i, ilu_cardinal l,
+		      ilu_cardinal limit,
+		      ilu_cardinal expected_encoding,
+		      ilu_cardinal current_encoding,
+		      ILU_ERRS((IoErrs)) * err)
 {
+  if ((expected_encoding != ILU_StringEncoding_latin1) ||
+      (current_encoding != ILU_StringEncoding_latin1)) {
+    return ILU_ERR_CONS1(imp_limit, err, minor, ilu_ilm_unsupported_charset_encoding, 0);
+  }
   return _courier_SizeOfBytes(call, (ilu_bytes) i, l, limit, err);
 }
 
@@ -778,7 +789,7 @@ _courier_SizeOfOpaque(ilu_Call call, ilu_bytes o, ilu_cardinal len,
 
 static void
 _courier_OutputSequence(ilu_Call c, ilu_cardinal seqLen,
-		      ilu_cardinal limit, ILU_ERRS((IoErrs)) * err)
+		      ilu_cardinal limit, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   if (limit != 0 && limit <= 0xFFFF)
     _courier_OutputShortCardinal(c, (ilu_shortcardinal) seqLen, err);
@@ -802,7 +813,7 @@ _courier_InputSequenceMark(ilu_Call c, ilu_cardinal extent,
 
 static void 
 _courier_InputSequence(ilu_Call c, ilu_cardinal * sequenceLength,
-		       ilu_cardinal limit, ILU_ERRS((IoErrs)) * err)
+		       ilu_cardinal limit, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ilu_shortcardinal len = 0;
   if (limit != 0 && limit <= 0xFFFF) {
@@ -820,7 +831,7 @@ static void _courier_EndSequence (ilu_Call c, ILU_ERRS((IoErrs)) * err)
 
 static          ilu_cardinal
 _courier_SizeOfSequence(ilu_Call c, ilu_cardinal length,
-		      ilu_cardinal limit, ILU_ERRS((IoErrs)) * err)
+		      ilu_cardinal limit, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   if (limit != 0 && limit <= 0xFFFF)
     return _courier_SizeOfShortCardinal(c, (ilu_shortcardinal) length,
@@ -829,23 +840,56 @@ _courier_SizeOfSequence(ilu_Call c, ilu_cardinal length,
     return _courier_SizeOfCardinal(c, length, err);
 }
 
+/* ==================== enumeration ==================== */
+
+static void 
+_courier_OutputEnumeration(ilu_Call call, ilu_shortcardinal i,
+			   ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
+{
+  _courier_OutputShortCardinal(call, i, err);
+}
+
+static void 
+_courier_InputEnumeration(ilu_Call call, ilu_shortcardinal * i,
+			  ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
+{
+  _courier_InputShortCardinal(call, i, err);
+}
+
+/*ARGSUSED*/
+static          ilu_cardinal
+_courier_SizeOfEnumeration(ilu_Call call, ilu_shortcardinal i,
+			   ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
+{
+  ILU_CLER(*err);
+  return (2);
+}
+
 /* ==================== union ==================== */
 
 static void 
 _courier_OutputUnion(ilu_Call c, ilu_cardinal typeIndex,
-		     ilu_cardinal dsize,
-		     ILU_ERRS((IoErrs)) * err)
+		     ilu_TypeKind tk,
+		     ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
+  if ((tk == ilu_cardinal_tk) || (tk == ilu_integer_tk)) {
+    ILU_ERR_CONS1(imp_limit,err,minor,ilu_ilm_bad_type_for_protocol,0);
+    return;
+  };
   _courier_OutputShortCardinal(c, (ilu_shortcardinal) typeIndex, err);
 }
 
 static void 
 _courier_InputUnion(ilu_Call c, ilu_cardinal * typeIndex,
-		    ilu_cardinal dsize,
-		    ILU_ERRS((IoErrs)) * err)
+		    ilu_TypeKind tk,
+		    ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ilu_shortcardinal temp;
 
+  if ((tk == ilu_cardinal_tk) || (tk == ilu_integer_tk)) {
+    ILU_ERR_CONS1(imp_limit,err,minor,ilu_ilm_bad_type_for_protocol,0);
+    return;
+  };
   _courier_InputShortCardinal(c, &temp, err);
   *typeIndex = temp;
   return;
@@ -858,9 +902,13 @@ static void _courier_EndUnion (ilu_Call c, ILU_ERRS((IoErrs)) * err)
 
 static          ilu_cardinal
 _courier_SizeOfUnion(ilu_Call c, ilu_cardinal typeIndex,
-		     ilu_cardinal dsize,
-		     ILU_ERRS((IoErrs)) * err)
+		     ilu_TypeKind tk,
+		     ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
+  if ((tk == ilu_cardinal_tk) || (tk == ilu_integer_tk)) {
+    ILU_ERR_CONS1(imp_limit,err,minor, ilu_ilm_bad_type_for_protocol,0);
+    return 0;
+  };
   return (_courier_SizeOfShortCardinal(c, (ilu_shortcardinal) typeIndex,
 				       err));
 }
@@ -869,13 +917,13 @@ _courier_SizeOfUnion(ilu_Call c, ilu_cardinal typeIndex,
 
 static void
 _courier_OutputArray(ilu_Call c, ilu_cardinal len,
-		     ILU_ERRS((IoErrs)) * err)
+		     ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
 }
 
 static void 
-_courier_InputArray(ilu_Call c, ILU_ERRS((IoErrs)) * err)
+_courier_InputArray(ilu_Call c, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
 }
@@ -887,7 +935,7 @@ static void _courier_EndArray (ilu_Call c, ILU_ERRS((IoErrs)) * err)
 
 static          ilu_cardinal
 _courier_SizeOfArray(ilu_Call c, ilu_cardinal len,
-		     ILU_ERRS((IoErrs)) * err)
+		     ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
   return (0);
@@ -895,12 +943,12 @@ _courier_SizeOfArray(ilu_Call c, ilu_cardinal len,
 
 /* ==================== record ==================== */
 
-static void _courier_OutputRecord (ilu_Call c, ILU_ERRS((IoErrs)) * err)
+static void _courier_OutputRecord (ilu_Call c, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
 }
 
-static void _courier_InputRecord (ilu_Call c, ILU_ERRS((IoErrs)) * err)
+static void _courier_InputRecord (ilu_Call c, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
 }
@@ -911,7 +959,7 @@ static void _courier_EndRecord (ilu_Call c, ILU_ERRS((IoErrs)) * err)
 }
 
 static          ilu_cardinal
-_courier_SizeOfRecord(ilu_Call c, ILU_ERRS((IoErrs)) * err)
+_courier_SizeOfRecord(ilu_Call c, ilu_Type prefix, ILU_ERRS((IoErrs)) * err)
 {
   ILU_CLER(*err);
   return (0);
@@ -925,7 +973,7 @@ _courier_SizeOfRecord(ilu_Call c, ILU_ERRS((IoErrs)) * err)
 
 /*L2, Main unconstrained*/
 /*L1_sup < prmu*/
-static ilu_string _courier_FormProtocolHandle (ilu_Object obj)
+static ilu_string _courier_FormProtocolHandle (ilu_refany pdata, ilu_Object obj)
 {
   char            buf[100];
   courierinfo    *s = NULL;
@@ -964,9 +1012,8 @@ _courier_ReadHeader(ilu_Call call, ilu_PacketType * type,
   case ilu_rhrc_ok:
     break;
   case ilu_rhrc_error:
-    DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	  (stderr,
-	 "courier:  error %s on transport_begin_message (input)\n",
+    ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	  ("courier:  error %s on transport_begin_message (input)\n",
 	   ILU_ERR_NAME(*err)));
   case ilu_rhrc_eof:
   case ilu_rhrc_nothing:
@@ -976,9 +1023,8 @@ _courier_ReadHeader(ilu_Call call, ilu_PacketType * type,
   }
   packet = transport_get_input_buffer(bs, 4, err);
   if (packet == NIL) {
-    DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	  (stderr,
-	   "courier:  error %s reading header.\n",
+    ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	  ("courier:  error %s reading header.\n",
 	   ILU_ERR_NAME(*err)));
     return (ilu_rhrc_error);
   }
@@ -986,9 +1032,8 @@ _courier_ReadHeader(ilu_Call call, ilu_PacketType * type,
   packetType = (packet[0] << 8) | packet[1];
   serialNumber = (packet[2] << 8) | packet[3];
 
-  DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	(stderr,
-	 "courier:  read header with SN %lu, type %lu (%s).\n",
+  ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	("courier:  read header with SN %lu, type %lu (%s).\n",
 	 serialNumber, packetType,
 	 ((packetType == 0) ? "request"
 	  : ((packetType == 2) ? "reply"
@@ -1016,7 +1061,7 @@ _courier_DelayInterp(ilu_Call call,
   if (q == NIL)
     return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*q), NIL);
   q->replyType = call->ca_prdata1;
-  q->trans = _ilu_BufferInputMessage(t, err);
+  q->trans = _ilu_BufferInputMessage(t, 0, FALSE, err);
   return (ILU_ERROK(*err) ? q : NIL);
 }
 
@@ -1025,6 +1070,8 @@ static void _courier_ResumeInterp(ilu_Call call, ilu_refany x)
   Queued          q = (Queued) x;
   call->ca_prTrans = q->trans;
   call->ca_prdata1 = q->replyType;
+  call->ca_prbit1 = TRUE;
+  ilu_free(q);
   return;
 }
 
@@ -1050,7 +1097,8 @@ _courier_DiscardMessage(ilu_Call call, ILU_ERRS((internal)) * err)
 }
 
 static          ilu_boolean
-_courier_AbandonDelayedInterp(ilu_refany x, ILU_ERRS((internal)) * err)
+_courier_AbandonDelayedInterp(ilu_Connection conn, ilu_refany x,
+			      ILU_ERRS((internal)) * err)
 {
   Queued          q = (Queued) x;
   ilu_boolean     ans;
@@ -1058,6 +1106,13 @@ _courier_AbandonDelayedInterp(ilu_refany x, ILU_ERRS((internal)) * err)
   ilu_integer     cdfd;
   ilu_free(q);
   ans = transport_close(trans, &cdfd, err);
+  ILU_ERR_SWITCH(*err) {
+    ILU_ERR_CASE2(bad_locks, broken_locks) {
+      ILU_HANDLED(*err);
+      (void) ilu_Check(FALSE, err);
+    }
+    ILU_ERR_ELSE 0;
+  } ILU_ERR_ENDSWITCH;
   _ilu_Assert(cdfd == 0, "courier AbandonDelayedInterp");
   return ans;
 }
@@ -1077,9 +1132,8 @@ _courier_InterpretRequest(ilu_Call call,
   if (ILU_ERRNOK(*err)) return FALSE;
   _courier_InputShortCardinal(call, &methodID, err);
   if (ILU_ERRNOK(*err)) return FALSE;
-  DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	(stderr,
-	 "courier:InterpretRequest:  (call SN %lu) class ID = 0x%lx (version %u), method ID = %u\n",
+  ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	("courier:InterpretRequest:  (call SN %lu) class ID = 0x%lx (version %u), method ID = %u\n",
 	 call->ca_SN, programNumber, programVersion, methodID));
 
   _ilu_AcquireMutex(ilu_prmu);
@@ -1091,35 +1145,31 @@ _courier_InterpretRequest(ilu_Call call,
   call_intro_type(call) = class;
   _ilu_ReleaseMutex(ilu_prmu);
   if (NULL == s || NULL == class) {
-    DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	  (stderr,
-	   "_courier_InterpretRequest:  (call %lu) Can't find ilu_Class with pn 0x%lx, version %u\n",
+    ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	  ("_courier_InterpretRequest:  (call %lu) Can't find ilu_Class with pn 0x%lx, version %u\n",
 	   call->ca_SN, programNumber, programVersion));
     call->ca_pe = ilu_ProtocolException_NoSuchClassAtServer;
     return (FALSE);
   }
-  DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	(stderr,
-    "_courier_InterpretRequest:  (call SN %lu) intro_type is %s\n",
+  ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	("_courier_InterpretRequest:  (call SN %lu) intro_type is %s\n",
 	 call->ca_SN, class->cl_name));
 
   call->ca_method = ilu_FindMethodByID(class, methodID);
 
   if (call->ca_method == NULL) {
-    DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	  (stderr,
-	   "_courier_InterpretRequest:  (call %lu) Can't find method on class 0x%lx with methodID = %u.\n",
+    ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	  ("_courier_InterpretRequest:  (call %lu) Can't find method on class 0x%lx with methodID = %u.\n",
 	   call->ca_SN, programNumber, methodID));
     call->ca_pe = ilu_ProtocolException_NoSuchMethodOnClass;
     return (FALSE);
   } else {
-    DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	  (stderr,
-	   "_courier_InterpretRequest:  record for method %d is %p (%s).\n",
+    ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	  ("_courier_InterpretRequest:  record for method %d is %p (%s).\n",
 	   methodID, call->ca_method, call->ca_method->me_name));
   }
-  DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	(stderr, "_courier_InterpretRequest:  returning TRUE\n"));
+  ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	("_courier_InterpretRequest:  returning TRUE\n"));
   return (TRUE);
 }
 
@@ -1138,9 +1188,8 @@ _courier_InterpretReply(ilu_Call call, ilu_cardinal * estatus,
     ilu_ProtocolException_GarbageArguments
   };
 
-  DEBUG((INCOMING_DEBUG | COURIER_DEBUG),
-	(stderr,
-	 "courier:InterpretReply:  SN %lu, replyStatus is %u\n",
+  ILU_NOTE((INCOMING_DEBUG | COURIER_DEBUG),
+	("courier:InterpretReply:  SN %lu, replyStatus is %u\n",
 	 call->ca_SN, replyStatus));
 
   if (replyStatus == 2) {	/* MSG_ACCEPTED */
@@ -1206,9 +1255,8 @@ _courier_StartRequest(ilu_Call call, ilu_cardinal argSize,
   ilu_Class       pclass = call->ca_intro_type;
   ilu_Method      method = call->ca_method;
 
-  DEBUG(COURIER_DEBUG,
-	(stderr,
-	 "_courier_StartRequest:  call %p (sn %lu), argSize %lu, class %s (%s), method %s (%ld)\n",
+  ILU_NOTE(COURIER_DEBUG,
+	("_courier_StartRequest:  call %p (sn %lu), argSize %lu, class %s (%s), method %s (%ld)\n",
 	 call, call_serial_number(call), argSize,
 	 class_name(pclass), class_unique_id(pclass),
 	 method_name(method), method_id(method)));
@@ -1219,8 +1267,8 @@ _courier_StartRequest(ilu_Call call, ilu_cardinal argSize,
 	    pclass->cl_name, call);
     return (FALSE);
   }
-  DEBUG(COURIER_DEBUG,
-	(stderr, "_courier_StartRequest:  pnum/vers 0x%lx/%ld\n",
+  ILU_NOTE(COURIER_DEBUG,
+	("_courier_StartRequest:  pnum/vers 0x%lx/%ld\n",
 	 pnumber, version));
 
   packetSize = argSize
@@ -1246,9 +1294,8 @@ _courier_StartRequest(ilu_Call call, ilu_cardinal argSize,
   if (ILU_ERRNOK(*err))
     goto dun;
 
-  DEBUG(COURIER_DEBUG,
-	(stderr,
-	 "_courier_StartRequest:  request %lu begun (size %lu).\n",
+  ILU_NOTE(COURIER_DEBUG,
+	("_courier_StartRequest:  request %lu begun (size %lu).\n",
 	 call->ca_SN, packetSize));
   return (TRUE);
 dun:
@@ -1317,9 +1364,8 @@ _courier_BeginReply(ilu_Call call, ilu_boolean exceptions,
   ilu_cardinal    packetSize;
   ilu_Transport   trans = callport(call);
 
-  DEBUG(PACKET_DEBUG,
-	(stderr,
-	 "courier:BeginReply:  SN %lu, argSize %lu, exceptions %s, trans %p.\n",
+  ILU_NOTE(PACKET_DEBUG,
+	("courier:BeginReply:  SN %lu, argSize %lu, exceptions %s, trans %p.\n",
 	 call->ca_SN, argSize, exceptions ? "TRUE" : "FALSE",
 	 trans));
 
@@ -1334,9 +1380,8 @@ _courier_BeginReply(ilu_Call call, ilu_boolean exceptions,
   _courier_OutputCardShouldBeShort(call, call->ca_SN, err);
   if (ILU_ERRNOK(*err)) goto faild;
 
-  DEBUG(PACKET_DEBUG,
-	(stderr,
-	 "_courier_BeginReply:  started reply %lu (size %lu).\n",
+  ILU_NOTE(PACKET_DEBUG,
+	("_courier_BeginReply:  started reply %lu (size %lu).\n",
 	 call->ca_SN, packetSize));
   return (TRUE);
 faild:
@@ -1379,9 +1424,8 @@ _courier_BeginException(ilu_Call call, ilu_cardinal evalue,
   /* exception value */
   if (ILU_ERRNOK(*err)) goto faild;
 
-  DEBUG(PACKET_DEBUG,
-	(stderr,
-	 "courier:BeginException:  exception started to peer %s:  SN %lu, size %lu, evalue %lu.\n",
+  ILU_NOTE(PACKET_DEBUG,
+	("courier:BeginException:  exception started to peer %s:  SN %lu, size %lu, evalue %lu.\n",
 	 conn_peerinfo(conn),
 	 call_serial_number(call), packetSize,
 	 (evalue == 0) ? sysExnIdx : evalue));
@@ -1450,14 +1494,14 @@ static ilu_Protocol _courier_NewCourier (void)
   new->pr_size_of_longreal = _courier_SizeOfLongReal;
 
   new->pr_output_optional =
-	(void (*)(ilu_Call,ilu_boolean,ilu_Error*))
-	_courier_OutputCardinal;
+	(void (*)(ilu_Call,ilu_boolean,ilu_Type,ilu_Error*))
+	_courier_OutputEnumeration;
   new->pr_input_optional =
-	(void (*)(ilu_Call,ilu_boolean *,ilu_Error*))
-	_courier_InputCardinal;
+	(void (*)(ilu_Call,ilu_boolean *,ilu_Type,ilu_Error*))
+	_courier_InputEnumeration;
   new->pr_size_of_optional =
-	(ilu_cardinal (*)(ilu_Call,ilu_boolean,ilu_Error*))
-	_courier_SizeOfCardinal;
+	(ilu_cardinal (*)(ilu_Call,ilu_boolean,ilu_Type,ilu_Error*))
+	_courier_SizeOfEnumeration;
 
   new->pr_output_enum_code = _courier_OutputEnumeration;
   new->pr_input_enum_code = _courier_InputEnumeration;
@@ -1541,6 +1585,14 @@ static ilu_Protocol _courier_NewCourier (void)
   new->pr_end_union = _courier_EndUnion;
   new->pr_size_of_union = _courier_SizeOfUnion;
 
+#ifdef ADD_VARIANT_SUPPORT
+
+  new->pr_output_pickle = _ilu_OutputPickle;
+  new->pr_input_pickle = _ilu_InputPickle;
+  new->pr_size_of_pickle = _ilu_SizeOfPickle;
+
+#endif /* ADD_VARIANT_SUPPORT */
+
   new->pr_form_handle = _courier_FormProtocolHandle;
 
   new->pr_read_header = _courier_ReadHeader;
@@ -1557,13 +1609,15 @@ static ilu_Protocol _courier_NewCourier (void)
 
   new->pr_create_data_block = _courier_CreateDataBlock;
   new->pr_free_data_block = (void (*)(void *)) _courier_FreeDataBlock;
+  new->pr_conn_closing = NULLFN;
 
   return (new);
 }
 
 /*L1_sup < prmu*/
 
-ilu_Protocol _ilu_courier_Protocol(void)
+ilu_Protocol _ilu_courier_Protocol(ilu_ProtocolInfo pinfo,
+				   ilu_Error *err)
 {
   /*L1 >= {prmu}*/
   static ilu_Protocol StandardCourier = NULL;
@@ -1571,5 +1625,7 @@ ilu_Protocol _ilu_courier_Protocol(void)
   if (StandardCourier == NULL)
     StandardCourier = _courier_NewCourier();
   _ilu_ReleaseMutex(ilu_prmu);
+  ILU_CLER(*err);
   return (StandardCourier);
 }
+
