@@ -14,23 +14,61 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: debug.c,v 1.70 1996/06/11 03:57:41 janssen Exp $ */
-/* Last edited by Mike Spreitzer December 11, 1995 1:58 pm PST */
+/* $Id: debug.c,v 1.85 1997/09/16 18:30:09 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 1, 1997 11:22 pm PST */
+
 
 #define _POSIX_SOURCE
 
+#if defined(WIN32)
+#include <windows.h>
+#include <stdio.h>
+#endif
+
 #include <ctype.h>
 #include <stdarg.h>
 
 #include "iluntrnl.h"
 #include "oscalls.h"	/* for OS_SLEEP */
 
+/* the following ifdef is an *very ugly hack* to get around the problem 
+of trying to include "threads.h" here.  Trying to do so introduces
+problems because of the #define _POSIX_SOURCE above.  If you're
+inclined to sort this all out, please do so - right now all I want
+to do is have the endless sleep loop put out a thread id! */
+#ifdef ILU_OS_THREADED
+
+#ifdef ILU_SOLARIS2_THREADS
+extern unsigned int thr_self();
+#define  GET_CURRENT_THREAD() thr_self()
+#endif
+
+#if  (defined(ILU_POSIX_THREADS) || defined(ILU_DCE_THREADS))
+extern  unsigned int pthread_self();
+#define GET_CURRENT_THREAD() (pthread_self())
+#endif
+
+#ifdef ILU_WIN32_THREADS
+WINBASEAPI DWORD WINAPI GetCurrentThreadId(VOID);
+#define GET_CURRENT_THREAD() GetCurrentThreadId()
+#endif
+
+#endif
+/* end of *very ugly hack* */
+
 /*L1, L2, Main unconstrained*/
 
-ilu_cardinal _ilu_DebugLevel = 0;
+ilu_cardinal ilu_DebugLevel = 0;
 
 static ilu_boolean debug_initialized = FALSE;
 
+#if defined(WIN32)
+static ilu_boolean Win32DebugHandlerInitialized = FALSE;
+static ilu_boolean CurrentProgramIsNotConsoleApp = FALSE;
+static HANDLE OutputHandle;
+static char *OutputBuffer;
+#endif
+
 static void 
 printErrorRaises(ilu_ErrorType et,
 		 const char *filename,
@@ -57,10 +95,14 @@ printErrorRaises(ilu_ErrorType et,
 #define IS_POSIX_VAL "not POSIX"
 #endif
 
-#ifdef ILU_SOLARIS2_THREADS
+#if   defined(ILU_SOLARIS2_THREADS)
 #define THREAD_VAL "Solaris 2 threads"
 #elif defined(ILU_POSIX_THREADS)
 #define THREAD_VAL "POSIX threads"
+#elif defined(ILU_WIN32_THREADS)
+#define THREAD_VAL "Win32 threads"
+#elif defined(ILU_DCE_THREADS)
+#define THREAD_VAL "DCE threads"
 #else
 #define THREAD_VAL "no threads"
 #endif
@@ -95,6 +137,12 @@ printErrorRaises(ilu_ErrorType et,
 #define W3NG_PVALUE ""
 #endif
 
+#ifdef JAVARMI_PROTOCOL
+#define JAVARMI_PVALUE " javarmi"
+#else
+#define JAVARMI_PVALUE ""
+#endif
+
 #ifdef TCPIP_TRANSPORT
 #define TCPIP_PVALUE " tcp"
 #else
@@ -119,6 +167,12 @@ printErrorRaises(ilu_ErrorType et,
 #define W3MUX_PVALUE ""
 #endif
 
+#ifdef BATCHING_TRANSPORT
+#define BATCHING_PVALUE " batching"
+#else
+#define BATCHING_PVALUE ""
+#endif
+
 #ifdef SECURE_TRANSPORT
 #define SECURE_PVALUE " secure"
 #else
@@ -139,32 +193,104 @@ printErrorRaises(ilu_ErrorType et,
 #define SIZE_T_PVALUE	"size_t"
 #endif
 
+#ifdef ADD_VARIANT_SUPPORT
+#define VAR_PVALUE ""
+#else
+#define VAR_PVALUE "no "
+#endif /* ADD_VARIANT_SUPPORT */
+
+#if defined(WIN32)
+void Win32MessageHandler(char *format, va_list args)
+{
+  int numWrote;
+
+  (void) vsprintf(OutputBuffer, format, args);
+  WriteConsole(OutputHandle, OutputBuffer, strlen(OutputBuffer), &numWrote, NULL);
+}
+
+void InitializeWin32ErrorHandler(ilu_boolean FullInitialize)
+{
+	/* Initialize the handling of ILU_DEBUG output for WIN32 */
+	/* If ilu_SetDebugLevel() gets called, and the user hasn't hooked in an error trapping */
+	/* function at this point, then this code will provide an output medium for the error */
+	/* messages iff the current application is NOT a console app.  If it is a console app, */
+	/* then the regular debug routine will work just fine. */
+	/* Special case exists if Debug Level is never set, and a message comes from the kernel. */
+
+	Win32DebugHandlerInitialized = TRUE;
+
+	/* Attempt to create a console for this process */
+	if (AllocConsole() == FALSE)
+		return;		/* Console exists for this process; nothing more needs to be done */
+
+	CurrentProgramIsNotConsoleApp = TRUE;	// Means that our program is not a console-app
+
+	if ((OutputHandle = GetStdHandle(STD_OUTPUT_HANDLE)) == INVALID_HANDLE_VALUE)
+	{
+		LPVOID lpMsgBuf;
+ 
+		FormatMessage( 
+    		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
+    		NULL,
+    		GetLastError(),
+    		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
+    		(LPTSTR) &lpMsgBuf,
+    		0,
+    		NULL 
+		);
+
+		MessageBox(NULL, lpMsgBuf, "Unable to obtain console handle", MB_OK | MB_ICONINFORMATION);
+		LocalFree(lpMsgBuf);
+	}
+
+	SetConsoleTitle("ILU_DEBUG output console");
+
+	OutputBuffer = (char *) LocalAlloc(LPTR, 1024);
+	if (OutputBuffer == NULL)
+	{
+		MessageBox(NULL, "Unable to allocate 1K for output formatting buffer!", "Out of memory", MB_OK | MB_ICONINFORMATION);
+	}
+	if (!FullInitialize)
+		return;
+
+	ilu_SetDebugMessageHandler(Win32MessageHandler);
+}
+#endif
+
 ilu_cardinal ilu_SetDebugLevel(ilu_cardinal level)
 {
-  ilu_cardinal old_level = _ilu_DebugLevel;
+  ilu_cardinal old_level = ilu_DebugLevel;
+
+  if (level != 0 || ilu_DebugLevel != 0) {
+
+#if defined(WIN32)
+	if (Win32DebugHandlerInitialized == FALSE)
+		InitializeWin32ErrorHandler(TRUE);
+#endif
 
-  if (level != 0 || _ilu_DebugLevel != 0) {
     ILU_ERRPRINTF(
       "ILU version %s.  Copyright 1990-1996 Xerox Corporation.\n",
 	    ilu_GetILUVersion());
     ILU_ERRPRINTF(
      "------------------------------------------------------------\n");
-    ILU_ERRPRINTF("Configuration info: %s-endian, %s, %s, %s, size_t=%s\n",
+    ILU_ERRPRINTF("Configuration info: %s-endian, %s, %s, %s, %svariant, size_t=%s,\n",
 #ifdef WORDS_BIGENDIAN
 "big",
 #else
 "little",
 #endif
-		  IS_BSD_VAL, IS_POSIX_VAL, THREAD_VAL, SIZE_T_PVALUE);
+		  IS_BSD_VAL, IS_POSIX_VAL, THREAD_VAL, VAR_PVALUE, SIZE_T_PVALUE);
     ILU_ERRPRINTF("  char=%u%s, short=%u, int=%u, long=%u, void *=%u, fnptr=%u,",
 		  (unsigned) SIZEOF_CHAR, CHAR_SIGN_PVALUE,
 		  (unsigned) SIZEOF_SHORT, (unsigned) SIZEOF_INT, (unsigned) SIZEOF_LONG,
 		  (unsigned) SIZEOF_VOID_P, (unsigned) SIZEOF_FN_P);
     ILU_ERRPRINTF(" long long=%u, long double=%u, enum=%u,\n",
 		  (unsigned) SIZEOF_LONG_LONG, (unsigned) SIZEOF_LONG_DOUBLE, (unsigned) SIZEOF_ENUM);
+    ILU_ERRPRINTF("  arch=%s, compiler=\"%s\",\n  ANSI C lib=\"%s\", sys aux libs=\"%s\",\n",
+		  ILU_MACHINE_TYPE, ILU_COMPILE_COMMAND, ILU_ANSI_C_LIBRARY, ILU_SYSAUX_LIBRARIES);
     ILU_ERRPRINTF("  protocols =%s, transports =%s,\n",
-		  SUNRPC_PVALUE COURIER_PVALUE IIOP_PVALUE HTTP_PVALUE W3NG_PVALUE,
-		  " inmem" TCPIP_PVALUE UDPSOCKET_PVALUE SUNRPCRM_PVALUE SECURE_PVALUE W3MUX_PVALUE);
+		  SUNRPC_PVALUE COURIER_PVALUE IIOP_PVALUE HTTP_PVALUE W3NG_PVALUE JAVARMI_PVALUE,
+		  " inmem" TCPIP_PVALUE UDPSOCKET_PVALUE SUNRPCRM_PVALUE SECURE_PVALUE W3MUX_PVALUE BATCHING_PVALUE);
 #ifdef ILU_BINDING_DIRECTORY
     {
       char *binding_dir;
@@ -183,14 +309,14 @@ ilu_cardinal ilu_SetDebugLevel(ilu_cardinal level)
      "------------------------------------------------------------\n");
     ILU_ERRPRINTF(
      "ilu_SetDebugLevel:  setting debug mask from 0x%x to 0x%lx\n",
-	    _ilu_DebugLevel, level);
+	    ilu_DebugLevel, level);
   }
 
-  _ilu_DebugLevel = level;
+  ilu_DebugLevel = level;
 
 #ifdef ENABLE_DEBUGGING
 
-  if ((_ilu_DebugLevel & ERROR_DEBUG) != 0)
+  if ((ilu_DebugLevel & ERROR_DEBUG) != 0)
     {
       ILU_ERRPRINTF("ilu_SetDebugLevel:  noting error raises via <debug.c:printErrorRaises>\n");
       ilu_SetRaiseDebugHook (printErrorRaises);
@@ -198,7 +324,7 @@ ilu_cardinal ilu_SetDebugLevel(ilu_cardinal level)
 
 #else
 
-  if (_ilu_DebugLevel != 0)
+  if (ilu_DebugLevel != 0)
     ILU_ERRPRINTF("ilu_SetDebugLevel:  ILU kernel was compiled without debugging.  No debugging messages available.\n");
 
 #endif
@@ -331,20 +457,25 @@ ilu_cardinal ilu_SetDebugLevelViaString (ilu_string s)
     if (((debug = _ilu_atoi(s, &p)) == 0 && p == s) || *p != '\0') {
       strcpy(buf, s);
       for (debug = 0, p = buf, s = strchr(buf, ':'),
-       debugcount = (sizeof(debugs) / sizeof(struct debug_entry));
+	debugcount = (sizeof(debugs) / sizeof(struct debug_entry));
 	   p != NIL;
 	   p = s + 1, s = strchr(s + 1, ':')) {
+	int             negate = (*p == '-');
 	if (s != NIL)
 	  *s = '\0';
+	p += negate;
 	for (i = 0; i < debugcount; i += 1)
 	  if (_ilu_casefree_cmp(debugs[i].name, p) == 0) {
-	    debug |= debugs[i].value;
+	    if (negate)
+	      debug &= ~debugs[i].value;
+	    else
+	      debug |= debugs[i].value;
 	    break;
 	  }
 	if (i >= debugcount) {
-	  ILU_ERRPRINTF(
-		  "_ilu_AutoSetDebugLevel:  Bad debug option \"%s\" specified.  Valid flags are:  ",
-		  p);
+	  ILU_ERRPRINTF("_ilu_AutoSetDebugLevel:  Bad debug option"
+			" \"%s\" specified.  Valid flags are:  ",
+			p);
 	  for (i = 0; i < debugcount; i++)
 	    ILU_ERRPRINTF(" %s", debugs[i].name);
 	  ILU_ERRPRINTF("\n");
@@ -355,11 +486,24 @@ ilu_cardinal ilu_SetDebugLevelViaString (ilu_string s)
     }
     return (ilu_SetDebugLevel(debug));
   }
-  return _ilu_DebugLevel;
+  return ilu_DebugLevel;
 }
 
 static void DebugPrint (char *formatSpec, va_list ap)
 {
+#if defined(WIN32)
+  if (!Win32DebugHandlerInitialized)
+  {
+    InitializeWin32ErrorHandler(FALSE);		/* initialize enough to determine program type */
+  }
+  if (CurrentProgramIsNotConsoleApp)
+  {
+    /* Send the message out to the error console */
+	Win32MessageHandler(formatSpec, ap);
+	return;
+  }
+#endif
+
   /* vfprintf is ANSI C, section 4.9.6.7 */
   (void) vfprintf (stderr, formatSpec, ap);
 }
@@ -378,24 +522,20 @@ void ilu_DebugPrintf (char *formatSpec, ...)
   va_end(ap);
 }
 
-void _ilu_DebugPrintfToFileStar (void *fp, char *formatSpec, ...)
-{
-  va_list ap;
-
-  va_start (ap, formatSpec);
-
-  if (debugMessageRoutine != NULLFN)
-    (*debugMessageRoutine) (formatSpec, ap);
-
-  va_end(ap);
-}
-
 void ilu_SetDebugMessageHandler (void (*handler)(char *, va_list))
 {
   if (((ilu_cardinal) handler) == 1)
     debugMessageRoutine = DebugPrint;
   else
     debugMessageRoutine = handler;
+
+#if defined(WIN32)
+  /* If the user sets up their own message hook, then make sure our error code */
+  /* is never called...                                                        */
+
+  Win32DebugHandlerInitialized = TRUE;
+  CurrentProgramIsNotConsoleApp = FALSE;
+#endif
 }
 
 static FILE * theDebugOutputFile = ILU_NIL;
@@ -417,6 +557,10 @@ void ilu_SendDebugOutputToFile (ilu_string filename)
     ilu_SetDebugMessageHandler (defaultFileOutput);
 }
 
+#ifdef macintosh
+static char mac_debug_string[] = ILU_MAC_DEBUG_STRING;
+#endif /* macintosh */
+
 void _ilu_AutoSetDebugLevel (void)
 {
   if (!debug_initialized)	/* just do it once */
@@ -427,6 +571,10 @@ void _ilu_AutoSetDebugLevel (void)
       if (file != NIL)
 	ilu_SendDebugOutputToFile(file);
 
+#ifdef macintosh
+      s = &mac_debug_string[ 0 ];	
+#endif
+
       if (s != NIL)
 	(void) ilu_SetDebugLevelViaString (s);
 
@@ -485,8 +633,8 @@ static _ilu_FailureHandler theAFC = {_ilu_ConsumeByLoop, TRUE};
 
 void ilu_SetAssertionFailureAction (int afa)
 {
-  DEBUG(ERROR_DEBUG,
-	(stderr, "ilu_SetAssertionFailureAction: to %d.\n",
+  ILU_NOTE(ERROR_DEBUG,
+	("ilu_SetAssertionFailureAction: to %d.\n",
 	 afa));
   theAFC = _ilu_FailureActionToConsumer(afa, 1);
   return;
@@ -495,8 +643,8 @@ void ilu_SetAssertionFailureAction (int afa)
 void ilu_SetAssertionFailConsumer (ilu_FailureConsumer afc)
 {
   _ilu_Assert(afc != NULLFN, "SetAssertionFailConsumer(NIL)");
-  DEBUG(ERROR_DEBUG,
-	(stderr, "ilu_SetAssertionFailConsumer: to %p.\n",
+  ILU_NOTE(ERROR_DEBUG,
+	("ilu_SetAssertionFailConsumer: to %p.\n",
 	 afc));
   theAFC.fc = afc;
   theAFC.printMsg = FALSE;
@@ -529,8 +677,8 @@ static _ilu_FailureHandler theCFC = {_ilu_ConsumeByLoop, TRUE};
 
 void ilu_SetCheckFailureAction (int cfa)
 {
-  DEBUG(ERROR_DEBUG,
-	(stderr, "ilu_SetCheckFailureAction: to %d.\n",
+  ILU_NOTE(ERROR_DEBUG,
+	("ilu_SetCheckFailureAction: to %d.\n",
 	 cfa));
   theCFC = _ilu_FailureActionToConsumer(cfa, 2);
   return;
@@ -539,8 +687,8 @@ void ilu_SetCheckFailureAction (int cfa)
 void ilu_SetCheckFailureConsumer (ilu_CheckFailureConsumer cfc)
 {
   _ilu_Assert(cfc != NULLFN, "SetCheckFailureConsumer(NIL)");
-  DEBUG(ERROR_DEBUG,
-	(stderr, "ilu_SetCheckFailureConsumer: to %p.\n",
+  ILU_NOTE(ERROR_DEBUG,
+	("ilu_SetCheckFailureConsumer: to %p.\n",
 	 cfc));
   theCFC.fc = cfc;
   theCFC.printMsg = FALSE;
@@ -574,6 +722,9 @@ void     _ilu_ConsumeByLoop(const char *f, int l)
   ILU_ERRPRINTF(
 	  "Entering endless sleep loop (at line %d of %s)",
 	  __LINE__ + 4, __FILE__);
+#ifdef ILU_OS_THREADED
+    ILU_ERRPRINTF(", Thread ID = 0x%x,", GET_CURRENT_THREAD());
+#endif
   ILU_ERRPRINTF(" for debugging purposes...\n");
   while (1)
 #ifdef WIN16
