@@ -14,8 +14,42 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: error.c,v 1.35 1996/03/29 18:44:28 janssen Exp $ */
+/* $Id: error.c,v 1.44 1997/08/29 16:15:01 spreitze Exp $ */
 /* $Log: error.c,v $
+ * Revision 1.44  1997/08/29  16:15:01  spreitze
+ * Started adding serializers and pipelines.
+ * Pipelines not fully there yet: ilu_GetReply doesn't release
+ * callmu while waiting for a reply to a pipelined call over a
+ * serial connection.
+ * Passing of ilu_Serializer and ilu_Pipeline is via explicit
+ * arguments to new proc, ilu_FullStartCall.
+ *
+ * Revision 1.43  1997/06/27  00:36:48  spreitze
+ * More relocation work.
+ * Retired the possibility of permanant relocation.
+ *
+ * Revision 1.42  1997/06/11  17:03:15  spreitze
+ * Added relocate (aka redirect) support.
+ * Client side only.
+ *
+ * Revision 1.41  1997/06/06  15:04:42  spreitze
+ * Finished adding codeset_incompatible.
+ *
+ * Revision 1.40  1997/06/06  04:58:39  janssen
+ * Added DEF_ERR for relocate
+ *
+ * Revision 1.39  1997/06/06  04:48:30  janssen
+ * Added relocate error.
+ *
+ * Revision 1.38  1997/06/03  21:48:20  spreitze
+ * Tracked change in TRANSIENT error.
+ *
+ * Revision 1.37  1997/04/03  23:59:12  larner
+ * changed _ilu_hash to ilu_hash
+ *
+ * Revision 1.36  1997/03/06  02:58:33  janssen
+ * Added Mac patches from Dan Keith
+ *
  * Revision 1.35  1996/03/29  18:44:28  janssen
  * Fixed typo
  *
@@ -132,20 +166,18 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  * Modifications to TransportClass to allow this.
  * Note that tr_flush_buffer now serves as end-of-message marker.
  * */
-/* Last edited by Mike Spreitzer December 7, 1995 9:31 am PST */
+/* Last edited by Mike Spreitzer August 29, 1997 9:05 am PDT */
 
 /* Standard error machinery, and defs of standard errors */
 
 #define _POSIX_SOURCE
 
-#ifdef MACOS
-#pragma segment ilu
-#endif
-
 #include "iluntrnl.h"
 
 ilu_Error       ilu_success_err = {NIL, 0, 0};
 
+ilu_string      ilu_RelocateScope_Names[2] = {"call", "conn"};
+
 static ilu_ErrorTypeDetails typeDetails[ILU_ERRTYP(ErrListLen)];
 
 ilu_ErrorTypeDetails ilu_GetErrorTypeDetails(int et)
@@ -185,7 +217,7 @@ ilu_CORBAizeSystemErr(ilu_Error * err,
     ILU_ERR_CASE(no_response, v)	minor = v->ignoreme*0;
     ILU_ERR_CASE(persist_store, v)	minor = v->ignoreme*0;
     ILU_ERR_CASE(bad_inv_order, v)	minor = v->ignoreme*0;
-    ILU_ERR_CASE(transient, v)		minor = v->ignoreme*0;
+    ILU_ERR_CASE(transient, v)		minor = v->minor;
     ILU_ERR_CASE(free_mem, v)		minor = v->ignoreme*0;
     ILU_ERR_CASE(inv_ident, v)		minor = v->ignoreme*0;
     ILU_ERR_CASE(inv_flag, v)		minor = v->ignoreme*0;
@@ -193,11 +225,12 @@ ilu_CORBAizeSystemErr(ilu_Error * err,
     ILU_ERR_CASE(bad_context, v)	minor = v->ignoreme*0;
     ILU_ERR_CASE(obj_adapter, v)	minor = v->ignoreme*0;
     ILU_ERR_CASE(data_conversion, v)	minor = v->ignoreme*0;
+    ILU_ERR_CASE(barrier, v)		minor = v->ignoreme*0;
     ILU_ERR_CASE(internal, v) {
       if (v->minor == ilu_im_check) {
 	const char     *cf = err->ilu_file;
 	char           *f = (char *) cf;
-	minor = (_ilu_hash_HashString(f, 32771) * 10000
+	minor = (ilu_hash_HashString(f, 32771) * 10000
 		 + err->ilu_line + 1000);
       } else
 	minor = v->minor;
@@ -319,11 +352,17 @@ ILU_DEF_ERR(intf_repos, "(a standard CORBA system exception)") {}
 ILU_DEF_ERR(bad_context, "(a standard CORBA system exception)") {}
 ILU_DEF_ERR(obj_adapter, "(a standard CORBA system exception)") {}
 ILU_DEF_ERR(data_conversion, "(a standard CORBA system exception)") {}
+ILU_DEF_ERR(codeset_incompatible, "(a CORBA 2.1 standard system exception)") {}
+ILU_DEF_ERR(barrier, "break in guaranteed serialization") {}
 ILU_DEF_ERR(bad_locks, "caller violated locking precondition") {}
 ILU_DEF_ERR(broken_locks, "locking meta-object broken") {}
 ILU_DEF_ERR(interrupted, "caller asked for RPC interruption") {}
 ILU_DEF_ERR(gss_security, "a standard GSS security error") {}
 
+ILU_DEF_ERR(relocate, "new location for object's server") {
+  ilu_free(e->rel_pinfo);
+  ilu_free(e->rel_tinfo);
+}
 
 ILU_DEF_ERR(KernelBroken, "Internal bug in kernel") {
   /* nothing to free */
