@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: gc.c,v 1.63 1996/07/01 23:08:41 jacobi Exp $ */
-/* Last tweaked by Mike Spreitzer March 8, 1996 9:04 am PST */
+/* $Id: gc.c,v 1.70 1997/08/01 03:41:13 janssen Exp $ */
+/* Last tweaked by Mike Spreitzer July 8, 1997 11:15 pm PDT */
 
 #define _POSIX_SOURCE
 
@@ -55,14 +55,18 @@ const ilu_Class _ilu_GcCallbackClass = &GCCallbackClassRecord;
   obj true && collectible => L1 >= {gcmu}*/
 ilu_boolean ilu_VeryInterested(ilu_Object obj)
 {
-  if (object_gclist(obj) != NIL &&
-      _ilu_vector_size(VECTOR(object_gclist(obj))) != 0)
-    return TRUE;
-  if (   class_collectible(obj->ob_class)
-      && ilu_CoarseTime_Now() < object_lastRemote(obj) + obj->ob_timeout)
-    return TRUE;
-  if (object_notifying(obj) && !object_known(obj))
-    return TRUE;
+  if (object_is_true(obj)) {
+    if (object_gclist(obj) != NIL &&
+	_ilu_vector_size(VECTOR(object_gclist(obj))) != 0)
+      return TRUE;
+    if (class_collectible(obj->ob_class)
+	&& (ilu_CoarseTime_Now()
+	    < object_lastRemote(obj) + obj->ob_timeout))
+      return TRUE;
+  } else {
+    if (object_notifying(obj) && !object_known(obj))
+      return TRUE;
+  }
   if (object_holds(obj) != 0)
     return TRUE;
   return FALSE;
@@ -72,6 +76,15 @@ ilu_boolean ilu_VeryInterested(ilu_Object obj)
 /* ====================== Server Side Stuff ====================== */
 
 static const ilu_FineTime pingPeriod = {300, 0};
+/* server pings callback client handles every "pingPeriod" seconds
+ * to see if they are still there or have crashed.
+ */
+
+ilu_integer		  _ilu_GCTimeOut = 1200;
+/* after all client registrations have been removed from a true object,
+ * the server waits this number of seconds for possible new but slow
+ * client registrations to reach it before losing interest in the object
+ */
 
 typedef struct {
   /*L1 >= {gcmu}; L2, Main unconstrained*/
@@ -124,8 +137,7 @@ static ilu_Vector Objects = NIL;
 ILU_ERRS((BadDataStructure, KernelBroken))
 _ilu_TouchedObj(ilu_Object obj)
 {
-  int             should = (object_gclist(obj) == NIL)
-  || (_ilu_vector_size(VECTOR(object_gclist(obj))) == 0);
+  int             should;
   ilu_Error       ans;
   ASSERT(object_collectible(obj), buf,
 	 (buf, "gc.c:TouchedObj: !collectible(%s/%s)",
@@ -133,6 +145,8 @@ _ilu_TouchedObj(ilu_Object obj)
   ASSERT(object_is_true(obj), buf,
 	 (buf, "gc.c:TouchedObj: surrogate(%s/%s)",
 	  obj->ob_server->sr_id, obj->ob_ih));
+  should = ((object_gclist(obj) == NIL)
+	    || (_ilu_vector_size(VECTOR(object_gclist(obj))) == 0));
   if (should) {
     ilu_FineTime    t = {0, 0};
     t.ft_s = obj->ob_timeout + object_lastRemote(obj);
@@ -179,17 +193,17 @@ DecrementObjCount(counted_client * cc, ilu_boolean fullFlush)
   ilu_Error       err = ILU_NO_ERR;
   cc->nObjs -= 1;
   if (cc->nObjs == 0 && cc->sweeping == FALSE) {
-    if (_ilu_hash_RemoveFromTable(Clients, cc->client) != cc)
+    if (ilu_hash_RemoveFromTable(Clients, cc->client) != cc)
       return ILU_ERR_CONS1(KernelBroken, &err, what,
 			   "DecrementObjCount: cc wasn't in table",
 			   err);
     ilu_MXAClear(&gccr, &cc->gcc);
     if (fullFlush == TRUE) {
       ilu_Server      s = object_server(cc->client);
-      _ilu_AcquireMutex(server_lock(s));
+      _ilu_AcquireServerMutex(s);
       err = _ilu_DeltaHolds(cc->client, -1);
       /* Won't raise GcRegFailed because clients aren't GCed */
-      _ilu_ReleaseMutex(server_lock(s));
+      _ilu_ReleaseServerMutex(s);
       ilu_free(cc);
     }
   }
@@ -203,10 +217,11 @@ gcoInvoke(ilu_Alarmette a)
   ilu_Error       err;
   static struct _ilu_Object_s dumby;
   ilu_Object      obj = (ilu_Object) (((char *) a) -
-		   (((char *) &dumby.ob_collectibleInfo.ob_true.ob_gco) - ((char *) &dumby)));
+	       (((char *) &dumby.ob_collectibleInfo.ob_true.ob_gco)
+		- ((char *) &dumby)));
   ilu_Server      s = object_server(obj);
   _ilu_AcquireMutex(ilu_cmu);
-  _ilu_AcquireMutex(server_lock(s));
+  _ilu_AcquireServerMutex(s);
   ASSERT(object_collectible(obj), buf,
 	 (buf, "gc.c:gcoInvoke: !collectible(%s/%s)",
 	  s->sr_id, obj->ob_ih));
@@ -215,7 +230,7 @@ gcoInvoke(ilu_Alarmette a)
 	  s->sr_id, obj->ob_ih));
   err = _ilu_VIUpdate(obj);
   ILU_MUST_BE_SUCCESS(err);
-  _ilu_ReleaseMutex(server_lock(s));
+  _ilu_ReleaseServerMutex(s);
   _ilu_ReleaseMutex(ilu_cmu);
   return;
 }
@@ -284,23 +299,23 @@ AddGCInterest(ilu_Object obj, ilu_Object interest)
   /* Because StartGCingTrueObj must have been called on obj. */
   _ilu_AcquireMutex(ilu_gcmu);
   _ilu_AcquireMutex(ilu_cmu);
-  _ilu_AcquireMutex(server_lock(s));
+  _ilu_AcquireServerMutex(s);
   if (object_gclist(obj) == NIL)
     object_gclist(obj) = (void *) _ilu_vector_new(INITIAL_GCLIST_SIZE);
   if (Clients == NIL)
-    Clients = _ilu_hash_MakeNewTable(INITIAL_GCCLIENTS_SIZE,
-		 _ilu_hash_HashPointer, _ilu_hash_PointerCompare);
+    Clients = ilu_hash_MakeNewTable(INITIAL_GCCLIENTS_SIZE,
+		 ilu_hash_HashPointer, ilu_hash_PointerCompare);
   for (i = 0, j = _ilu_vector_size(VECTOR(object_gclist(obj))),
        p = (counted_client **)
        _ilu_vector_elements(VECTOR(object_gclist(obj)));
        i < j; i++)
     if (p[i]->client == interest) {
-      _ilu_ReleaseMutex(server_lock(s));
+      _ilu_ReleaseServerMutex(s);
       _ilu_ReleaseMutex(ilu_cmu);
       _ilu_ReleaseMutex(ilu_gcmu);
       return ILU_NO_ERR;			/* already there */
     }
-  cc = (counted_client *) _ilu_hash_FindInTable(Clients, interest);
+  cc = (counted_client *) ilu_hash_FindInTable(Clients, interest);
   if (cc == NIL) {
     ilu_Alarmette_s gcc = {NIL, NIL, FALSE, {0, 0}};
     cc = (counted_client *) ilu_malloc(sizeof(counted_client));
@@ -308,7 +323,7 @@ AddGCInterest(ilu_Object obj, ilu_Object interest)
     cc->client = interest;
     cc->sweeping = FALSE;
     cc->gcc = gcc;
-    _ilu_Assert((int)_ilu_hash_AddToTable(Clients, interest, cc),
+    _ilu_Assert((int)ilu_hash_AddToTable(Clients, interest, cc),
 		"AddGCInterest AddToTable Clients");
     ilu_MXASet(&gccr, &cc->gcc,
 	       ilu_FineTime_Add(ilu_FineTime_Now(), pingPeriod));
@@ -333,7 +348,7 @@ AddGCInterest(ilu_Object obj, ilu_Object interest)
     } ILU_ERR_ENDSWITCH;
   }
 fini:
-  _ilu_ReleaseMutex(server_lock(s));
+  _ilu_ReleaseServerMutex(s);
   _ilu_ReleaseMutex(ilu_cmu);
   _ilu_ReleaseMutex(ilu_gcmu);
   return err;
@@ -356,7 +371,7 @@ DropGCInterest(ilu_Object obj,		/* non-NIL true obj of collectible type */
     return ILU_ERR_CONS1(internal, &err, minor, ilu_im_broken, err);
   else if (object_gclist(obj) == NIL)
     {
-      _ilu_ReleaseMutex(server_lock(object_server(obj)));
+      _ilu_ReleaseServerMutex(object_server(obj));
       return err;
     }
 
@@ -375,13 +390,13 @@ DropGCInterest(ilu_Object obj,		/* non-NIL true obj of collectible type */
 	  ILU_ERR_CASE2(BadDataStructure, KernelBroken) return err;
 	} ILU_ERR_ENDSWITCH;
       }
-      _ilu_ReleaseMutex(server_lock(object_server(obj)));
+      _ilu_ReleaseServerMutex(object_server(obj));
 
       err = DecrementObjCount(p2, fullFlush);
       return err;
     }
   }
-  _ilu_ReleaseMutex(server_lock(object_server(obj)));
+  _ilu_ReleaseServerMutex(object_server(obj));
   return err;
 }
 
@@ -487,7 +502,7 @@ void _ilu_HandleGCInterestDeregistration (ilu_Call call)
     s = object_server(disc);
     _ilu_AcquireMutex(ilu_gcmu);
     _ilu_AcquireMutex(ilu_cmu);
-    _ilu_AcquireMutex(server_lock(s));
+    _ilu_AcquireServerMutex(s);
     lerr = DropGCInterest(disc, interest, TRUE);
     ILU_MUST_BE_SUCCESS(lerr);
     _ilu_ReleaseMutex(ilu_cmu);
@@ -562,16 +577,16 @@ static void gccInvoke (ilu_Alarmette a)
       m = _ilu_vector_size(Objects);
       for ( k = 0;  k < m;  k++) {
 	  ilu_Object obj = q[k];
-	  _ilu_AcquireMutex(server_lock(object_server(obj)));
+	  _ilu_AcquireServerMutex(object_server(obj));
 	  lerr = DropGCInterest (obj, cc->client, FALSE);
 	  ILU_MUST_BE_SUCCESS(lerr);
 	}
       ilu_MXAClear(&gccr, &cc->gcc);
       if (cc->nObjs == 0) {
-          _ilu_AcquireMutex(server_lock(cs));
+          _ilu_AcquireServerMutex(cs);
           lerr = _ilu_DeltaHolds(cc->client, -1);
           ILU_MUST_BE_SUCCESS(lerr);
-          _ilu_ReleaseMutex(server_lock(cs));
+          _ilu_ReleaseServerMutex(cs);
           ilu_free(cc);
         }
       else
@@ -596,6 +611,10 @@ void ilu_SetGcClient(ilu_Object i)
   return;
 }
 
+ilu_boolean ilu_IsGcClientSet(void) {
+  return (gcInterest != NIL);
+}
+
 /*Main Invariant holds; L2 otherwise unconstrained*/
 
 static ILU_ERRS((GcRegFailed)) RegWork (ilu_Object obj, ilu_Method which)
@@ -630,17 +649,17 @@ static ILU_ERRS((GcRegFailed)) RegWork (ilu_Object obj, ilu_Method which)
   }
   os = object_server(obj);
   is = object_server(gci);
-  _ilu_AcquireMutex(server_lock(os));
+  _ilu_AcquireServerMutex(os);
   reqSize += ilu_SizeOfObjectID(call, obj, TRUE, _ilu_rootClass, &lerr);
+  _ilu_ReleaseServerMutex(os);
   if (ILU_ERRNOK(lerr))
     goto faild;
-  _ilu_ReleaseMutex(server_lock(os));
-  _ilu_AcquireMutex(server_lock(is));
+  _ilu_AcquireServerMutex(is);
   reqSize += ilu_SizeOfObjectID(call, gci, FALSE, _ilu_GcCallbackClass,
 				&lerr);
+  _ilu_ReleaseServerMutex(is);
   if (ILU_ERRNOK(lerr))
     goto faild;
-  _ilu_ReleaseMutex(server_lock(is));
   if (!ilu_StartRequest(call, reqSize, &lerr))
     goto faild;
   ilu_EnterServer(os, object_class(obj));
@@ -653,7 +672,10 @@ static ILU_ERRS((GcRegFailed)) RegWork (ilu_Object obj, ilu_Method which)
     goto faild;
   if (!ilu_FinishRequest(call, &lerr))
     goto faild;
-  pe = ilu_GetReply(call, &errorStatus, &lerr);
+  pe = ilu_GetReply(call, &errorStatus, &newconn, &lerr);
+  if (newconn != NIL)
+    _ilu_HandOffNewConnection(newconn, &hoerr);
+  ILU_MUST_BE_SUCCESS(hoerr);
   if (pe != ilu_ProtocolException_Success)
     goto faild;
   else if (errorStatus != 0) {
