@@ -14,11 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: hash.c,v 1.48 1996/04/26 21:52:53 mdavidso Exp $ */
-
-#ifdef MACOS
-#pragma segment ilu
-#endif
+/* $Id: hash.c,v 1.52 1997/08/29 21:26:54 janssen Exp $ */
 
 /* void *ilu_malloc (unsigned long size); */
 /* void *ilu_realloc (void *p, unsigned long size); */
@@ -60,7 +56,7 @@ struct hashTableSlot {
   struct hashEntry *hs_entries;
 };
 
-struct hashTable {
+struct _ilu_HashTable_s {
   /*L1 >= {some mutex that protects the table}*/
 
   ilu_cardinal ht_size, ht_nPairs;
@@ -80,7 +76,7 @@ struct hashTable {
 /*L2, Main unconstrained*/
 /*L1 >= {some mutex that protects the table}*/
 
-ilu_refany _ilu_hash_FindInTable(HashTable ht, ilu_refany key)
+ilu_refany ilu_hash_FindInTable(HashTable ht, ilu_refany key)
 {
   struct hashTableSlot *slot;
   ilu_shortcardinal count;
@@ -95,7 +91,7 @@ ilu_refany _ilu_hash_FindInTable(HashTable ht, ilu_refany key)
   return (ILU_NIL);
 }
 
-ilu_refany _ilu_hash_RemoveFromTable (HashTable ht, ilu_refany key)
+ilu_refany ilu_hash_RemoveFromTable (HashTable ht, ilu_refany key)
 {
   struct hashTableSlot *slot;
   ilu_shortcardinal count;
@@ -124,7 +120,7 @@ ilu_refany _ilu_hash_RemoveFromTable (HashTable ht, ilu_refany key)
 }
 
 ilu_boolean 
-_ilu_hash_AddToTable(HashTable ht, ilu_refany key, ilu_refany obj)
+ilu_hash_AddToTable(HashTable ht, ilu_refany key, ilu_refany obj)
 {
   struct hashTableSlot *slot;
   ilu_shortcardinal count;
@@ -156,7 +152,7 @@ _ilu_hash_AddToTable(HashTable ht, ilu_refany key, ilu_refany obj)
   return (ilu_TRUE);
 }
 
-void _ilu_hash_BeginEnumeration (HashTable ht, HashEnumerator *he)
+void ilu_hash_BeginEnumeration (HashTable ht, HashEnumerator *he)
 {
   he->hn_ht = ht;
   he->hn_index = 0;
@@ -164,7 +160,7 @@ void _ilu_hash_BeginEnumeration (HashTable ht, HashEnumerator *he)
   return;
 }
 
-ilu_boolean _ilu_hash_Next(HashEnumerator *he, ilu_refany *key,
+ilu_boolean ilu_hash_Next(HashEnumerator *he, ilu_refany *key,
 					       ilu_refany *data)
 {
   if (he->hn_index >= he->hn_ht->ht_size)
@@ -185,7 +181,7 @@ ilu_boolean _ilu_hash_Next(HashEnumerator *he, ilu_refany *key,
 #ifdef ENABLE_DEBUGGING
 /* This function is provided solely for use with a debugger.  It is
    never called by the ILU runtime. */
-static void _ilu_hash_PrintHashTable (HashTable ht)
+static void ilu_hash_PrintHashTable (HashTable ht)
 {
   HashEnumerator he;
   ilu_refany key, data;
@@ -194,10 +190,10 @@ static void _ilu_hash_PrintHashTable (HashTable ht)
     ilu_DebugPrintf ("NIL specified for hash table\n");
   else
     {
-      _ilu_hash_BeginEnumeration (ht, &he);
-      while (_ilu_hash_Next(&he, &key, &data))
+      ilu_hash_BeginEnumeration (ht, &he);
+      while (ilu_hash_Next(&he, &key, &data))
 	{
-	  if (ht->ht_compfn == _ilu_hash_StringCompare)
+	  if (ht->ht_compfn == ilu_hash_StringCompare)
 	    ilu_DebugPrintf ("  %s (%p) : %p\n", (ilu_string) key, key, data);
 	  else
 	    ilu_DebugPrintf ("  %p : %p\n", key, data);
@@ -207,7 +203,7 @@ static void _ilu_hash_PrintHashTable (HashTable ht)
 #endif /* ENABLE_DEBUGGING */
 
 void 
-_ilu_hash_TableEnumerate(HashTable ht,
+ilu_hash_TableEnumerate(HashTable ht,
 	    void (*proc) (ilu_refany entry_data, ilu_refany rock),
 			 ilu_refany rock)
 {
@@ -226,7 +222,7 @@ _ilu_hash_TableEnumerate(HashTable ht,
 }
 
 ilu_refany 
-_ilu_hash_FindViaProc(HashTable ht,
+ilu_hash_FindViaProc(HashTable ht,
       ilu_boolean(*proc) (ilu_refany entry_data, ilu_refany rock),
 		      ilu_refany rock)
 {
@@ -248,19 +244,19 @@ _ilu_hash_FindViaProc(HashTable ht,
 
 /*L1 unconstrained*/
 
-ilu_boolean _ilu_hash_StringCompare (ilu_refany key1, ilu_refany key2)
+ilu_boolean ilu_hash_StringCompare (ilu_refany key1, ilu_refany key2)
 {
   ilu_string str1 = (ilu_string) key1;
   ilu_string str2 = (ilu_string) key2;
   return (strcmp(str1, str2) == 0);
 }
 
-ilu_boolean _ilu_hash_PointerCompare (ilu_refany key1, ilu_refany key2)
+ilu_boolean ilu_hash_PointerCompare (ilu_refany key1, ilu_refany key2)
 {
   return (key1 == key2);
 }
 
-ilu_cardinal _ilu_hash_HashString (ilu_refany key, ilu_cardinal size)
+ilu_cardinal ilu_hash_HashString (ilu_refany key, ilu_cardinal size)
 {
   ilu_string      str = (ilu_string) key;
 #if 0
@@ -274,12 +270,12 @@ ilu_cardinal _ilu_hash_HashString (ilu_refany key, ilu_cardinal size)
     m = (m * 256) % 0xFFFFFD;
   }
 #else
-  ilu_cardinal crc = _ilu_CRC32 ((ilu_bytes) str, (ilu_cardinal) strlen(str));
+  ilu_cardinal crc = ilu_CRC32 ((ilu_bytes) str, (ilu_cardinal) strlen(str));
   return (crc % size);
 #endif
 }
 
-ilu_cardinal _ilu_hash_HashPointer (ilu_refany key, ilu_cardinal size)
+ilu_cardinal ilu_hash_HashPointer (ilu_refany key, ilu_cardinal size)
 {
   ilu_cardinal i = 0;
   ilu_cardinal val = 0;
@@ -289,20 +285,20 @@ ilu_cardinal _ilu_hash_HashPointer (ilu_refany key, ilu_cardinal size)
   return (val % size);
 }
 
-HashTable _ilu_hash_MakeNewTable (ilu_cardinal size,
+HashTable ilu_hash_MakeNewTable (ilu_cardinal size,
 	ilu_hashfnptr hashfn, ilu_compfnptr compfn)
 {
-  HashTable       new = (HashTable) ilu_malloc(sizeof(struct hashTable));
+  HashTable       new = (HashTable) ilu_malloc(sizeof(struct _ilu_HashTable_s));
   new->ht_size = size;
   new->ht_nPairs = 0;
   new->ht_slots = (struct hashTableSlot *) ilu_malloc(
 			     sizeof(struct hashTableSlot) * size);
   if (hashfn == NULLFN)
-    new->ht_hashfn = _ilu_hash_HashString;
+    new->ht_hashfn = ilu_hash_HashString;
   else
     new->ht_hashfn = hashfn;
   if (compfn == NULLFN)
-    new->ht_compfn = _ilu_hash_StringCompare;
+    new->ht_compfn = ilu_hash_StringCompare;
   else
     new->ht_compfn = compfn;
   memset((void *) (new->ht_slots), 0,
@@ -312,7 +308,7 @@ HashTable _ilu_hash_MakeNewTable (ilu_cardinal size,
 
 /*L1 >= {some mutex that protects the table}*/
 
-void _ilu_hash_FreeHashTable (HashTable ht, void (*freeKey)(ilu_refany),
+void ilu_hash_FreeHashTable (HashTable ht, void (*freeKey)(ilu_refany),
 					    void (*freeData)(ilu_refany))
 {
   register ilu_cardinal i;
@@ -343,7 +339,7 @@ void _ilu_hash_FreeHashTable (HashTable ht, void (*freeKey)(ilu_refany),
   return;
 }
 
-ilu_cardinal _ilu_hash_PairsInTable (HashTable ht)
+ilu_cardinal ilu_hash_PairsInTable (HashTable ht)
 {
   if (ht == ILU_NIL)
        return (0);
