@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: identity.c,v 1.12 1996/07/03 01:02:50 janssen Exp $ */
+/* $Id: identity.c,v 1.19 1997/09/18 00:58:28 janssen Exp $ */
 /* Last edited by Mike Spreitzer June 6, 1996 8:50 am PDT */
 
 #include "iluntrnl.h"
@@ -52,22 +52,22 @@ ilu_boolean
     {
       if (IdentityTypes[i] == NIL)
 	{ 
-	  DEBUG(AUTHENTICATION_DEBUG,
-		(stderr, "ilu_RegisterIdentityType:  registered identity type \"%s\".\n",
+	  ILU_NOTE(AUTHENTICATION_DEBUG,
+		("ilu_RegisterIdentityType:  registered identity type \"%s\".\n",
 		 type->it_name));
 	  IdentityTypes[i] = type;
 	  return ILU_CLER(*err);
 	}
       else if (strcmp(IdentityTypes[i]->it_name, type->it_name) == 0)
 	{
-	  DEBUG(AUTHENTICATION_DEBUG,
-		(stderr, "ilu_RegisterIdentityType:  identity type \"%s\" already registered.\n",
+	  ILU_NOTE(AUTHENTICATION_DEBUG,
+		("ilu_RegisterIdentityType:  identity type \"%s\" already registered.\n",
 		 type->it_name));
 	  return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_identity_type_registered, ilu_FALSE);
 	}
     }
-  DEBUG(AUTHENTICATION_DEBUG,
-	(stderr, "ilu_RegisterIdentityType:  too many identity types, registration of \"%s\" failed."
+  ILU_NOTE(AUTHENTICATION_DEBUG,
+	("ilu_RegisterIdentityType:  too many identity types, registration of \"%s\" failed."
 	 "  Max is %d.\n",
 	 type->it_name, MAX_IDENTITY_TYPES));
   return ILU_ERR_CONS1(imp_limit, err, minor, ilu_ilm_max_identity_types, ilu_FALSE);
@@ -81,7 +81,7 @@ ilu_IdentityType
 
   ILU_CLER(*err);
   for (i = 0;  i < MAX_IDENTITY_TYPES;  i++)
-    if (strcmp(IdentityTypes[i]->it_name, name) == 0)
+    if ((IdentityTypes[i] != NIL) && (strcmp(IdentityTypes[i]->it_name, name) == 0))
       return IdentityTypes[i];
   return NIL;
 }
@@ -175,46 +175,45 @@ void _ilu_AddConnIdentities (ilu_Call call, ilu_Error *err)
 {
   ilu_Error lerr;
   ilu_Passport ppn = call->ca_caller;
+  int n;
+  ilu_IdentityInfo info;
+  ilu_Vector vec;
 
   if (ppn == NIL)
     {
       ppn = ilu_CreatePassport(NIL, err);
       if (ILU_ERRNOK(*err)) return;
     }
-
-  if (call_connection(call)->co_auth_info != NIL)
-    {
-      ilu_Passport pp = (ilu_Passport) (call_connection(call)->co_auth_info);
-      ilu_IdentityInfo ident;
-
-      if ((ident = ilu_FindIdentity(pp, ilu_ConnectionIdentity)) != NIL)
-	{
-	  ilu_AddIdentity (ppn, ident, err);
-	  if (ILU_ERRNOK(*err))
-	    {
-	      ilu_DestroyPassport (ppn, &lerr);
-	      ILU_HANDLED(lerr);
-	      return;
-	    }
+  vec = (ilu_Vector) call_connection(call)->co_auth_info;
+  if (vec != NIL) {
+    for (n = 0;  n < vec->ve_size;  n++)
+      {
+	/* optimize use of non-owned identities by not copying them */
+	if (((ilu_IdentityInfo)(vec->ve_elements[n]))->ii_owned_by_passport == ilu_FALSE)
+	  info = (ilu_IdentityInfo)(vec->ve_elements[n]);
+	else {
+	  /* make copy of identity for this passport */
+	  if ((info = ilu_CopyIdentity((ilu_IdentityInfo)(vec->ve_elements[n]), err)) == NIL)
+	    goto errpoint;
+	  info->ii_owned_by_passport = ilu_TRUE;
 	}
+	if (!ilu_AddIdentity(ppn, info, err))
+	  goto errpoint;
+      }
+  }
 
-#ifdef SECURE_TRANSPORT
-      if ((ident = ilu_FindIdentity(pp, ilu_GSSIdentity)) != NIL)
-	{
-	  ilu_AddIdentity (ppn, ident, ilu_FALSE, err);
-	  if (ILU_ERRNOK(*err))
-	    {
-	      ilu_DestroyPassport(ppn, &lerr);
-	      ILU_HANDLED(lerr);
-	      return;
-	    }
-	}
-#endif
-
-    }
   if (call->ca_caller == NIL)
     call->ca_caller = ppn;
   ILU_CLER(*err);
+  return;
+
+ errpoint:
+  if ((call->ca_caller == NIL) && (ppn != NIL)) {
+    ilu_Error lerr;
+    ilu_DestroyPassport(ppn, &lerr);
+    ILU_HANDLED(lerr);
+  }
+  return;
 }
 
 /*L1, L2, Main unconstrained*/
@@ -249,7 +248,7 @@ extern ilu_IdentityInfo
   ilu_IdentityInfo i;
   ilu_refany dupinfo;
 
-  dupinfo = (*info->ii_type->it_duplicate_data)(info->ii_info, err);
+  dupinfo = (*info->ii_type->it_duplicate_data)((ilu_IdentityInfo) info, err);
   if (ILU_ERRNOK(*err))
     return NIL;
 
