@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iiop.h,v 1.8 1996/05/31 18:23:58 janssen Exp $ */
+/* $Id: iiop.h,v 1.17 1997/11/19 05:20:07 janssen Exp $ */
 /* Last tweaked by Mike Spreitzer November 7, 1993 9:08 pm PST */
 
 #include <stdio.h>
@@ -38,9 +38,14 @@ struct IIOP_IOR_IOR {
 struct IIOP_DataBlock {
   ilu_cardinal major;
   ilu_cardinal minor;
+  ilu_cardinal mapping;
+  ilu_bytes    key;
+  ilu_cardinal keylen;
+  ilu_boolean  relocate_checked;
 };
 
 #define IIOP_TAG_INTERNET_IOP		0
+#define IIOP_TAG_MULTIPLE_COMPONENT_IOP	1
 #define IIOP_TAG_ILU_IOP		0x494c5500
 #define IIOP_TAG_ILU_SUNRPC_IOP		0x494c5501
 #define IIOP_TAG_ILU_COURIER_IOP	0x494c5502
@@ -49,6 +54,10 @@ struct IIOP_DataBlock {
 #define IIOP_TAG_ILU_UNUSED3_IOP	0x494c5505
 #define IIOP_TAG_ILU_UNUSED4_IOP	0x494c5506
 #define IIOP_TAG_ILU_UNUSED5_IOP	0x494c5507
+#define IIOP_TAG_CODE_SETS		1
+
+#define ilu_IIOP_NATIVE_SHORT_CHAR_CODE_SET_ID	0x00010001
+#define ilu_IIOP_NATIVE_CHAR_CODE_SET_ID	0x00010100
 
 #define GIOP_PacketType_Request		0
 #define GIOP_PacketType_Reply		1
@@ -70,6 +79,7 @@ enum byte_order { BigEndian, LittleEndian };
 typedef struct ilu_packet_s {
   struct ilu_packet_methods_s *methods;
   ilu_Transport bs;
+  ilu_boolean bs_needs_closing_on_finish_call;
   ilu_byte *vop;
   enum byte_order byteorder;
   int ptype;
@@ -78,6 +88,8 @@ typedef struct ilu_packet_s {
   ilu_cardinal size;
   ilu_bytes principal;
   ilu_cardinal principalLen;
+  ilu_cardinal short_char_codeset;
+  ilu_cardinal char_codeset;
 } * PACKET;
 
 struct ilu_packet_methods_s {
@@ -146,9 +158,11 @@ struct ilu_packet_methods_s {
 #define packet_get_opaque(p,b,l,e)	((*((p)->methods)->get_opaque)((p),(b),(l),(e)))
 
 #define packet_destroy(p,e)	((*((p)->methods)->destroy)((p),(e)))
+#define packet_vop(p)		((p)->vop)
 
 #define iiop_packet(x)		((PACKET)((x)->ca_prdata2))
 #define iiop_set_packet(x,p)	((x)->ca_prdata2 = ((ilu_refany)(p)))
+#define iiop_transport(x)	(iiop_packet(x)->bs)
 #define iiop_packetType(x)	(iiop_packet(x)->ptype)
 #define iiop_objKey(x)		(iiop_packet(x)->objKey)
 #define iiop_objKeyLen(x)	(iiop_packet(x)->objKeyLen)
@@ -158,8 +172,11 @@ struct ilu_packet_methods_s {
 #define iiop_byte_order(x)	(iiop_packet(x)->byteorder)
 #define iiop_principal(x)	(iiop_packet(x)->principal)
 #define iiop_principalLen(x)	(iiop_packet(x)->principalLen)
+#define iiop_data_block(x)	((struct IIOP_DataBlock *)(call_connection(x)->co_protocol_data))
 #define iiop_major_version(x)	(((struct IIOP_DataBlock *)(call_connection(x)->co_protocol_data))->major)
 #define iiop_minor_version(x)	(((struct IIOP_DataBlock *)(call_connection(x)->co_protocol_data))->minor)
+#define iiop_short_char_codeset(x)	(iiop_packet(x)->short_char_codeset)
+#define iiop_char_codeset(x)	(iiop_packet(x)->char_codeset)
 
 #define ODD(x)		(((x)&0x1)!=0)
 #define EVEN(x)		(((x)&0x1)==0)  
@@ -205,4 +222,34 @@ struct ilu_packet_methods_s {
 
 #define ENDIAN_MATCH(p)		((p)->byteorder == NATIVE_BYTE_ORDER)
 
-#define iiop_transport(x)	((x)->ca_prTrans)
+/* typecode values */
+
+#define CORBA_tk_null		0
+#define CORBA_tk_void		1
+#define CORBA_tk_shortinteger	2
+#define CORBA_tk_integer	3
+#define CORBA_tk_shortcardinal	4
+#define CORBA_tk_cardinal	5
+#define CORBA_tk_shortreal	6
+#define CORBA_tk_real		7
+#define CORBA_tk_boolean	8
+#define CORBA_tk_shortcharacter	9
+#define CORBA_tk_byte		10
+#define CORBA_tk_variant	11
+#define CORBA_tk_type		12
+#define CORBA_tk_Principal	13
+#define CORBA_tk_object		14
+#define CORBA_tk_record		15
+#define CORBA_tk_union		16
+#define CORBA_tk_enumeration	17
+#define CORBA_tk_string		18
+#define CORBA_tk_sequence	19
+#define CORBA_tk_array		20
+#define CORBA_tk_alias		21
+#define CORBA_tk_except		22
+
+#define CORBA_tk_longinteger	23
+#define CORBA_tk_longcardinal	24
+#define CORBA_tk_longreal	25
+#define CORBA_tk_character	26
+#define CORBA_tk_optional	27
