@@ -15,11 +15,8 @@
  * TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX
  * CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  */
-/*
- * $Id: ilubufxp.c,v 1.41 1995/08/24 23:25:56 spreitze Exp spreitze
- * $
- */
-/* Last edited by Mike Spreitzer November 1, 1995 3:11 pm PST */
+/* $Id: ilubufxp.c,v 1.16 1997/09/19 05:21:43 spreitze Exp $ */
+/* Last edited by Mike Spreitzer September 18, 1997 8:54 pm PDT */
 
 #include "iluntrnl.h"
 #include "transprt.h"
@@ -54,8 +51,7 @@ CloseDFd(ilu_Transport t)
 
 static          ilu_boolean
 SetInputHandler(ilu_Transport t,
-		ilu_TransportInputHandler tih,
-		ilu_refany tih_rock,
+		ilu_TIH tih,
 		ILU_ERRS((no_memory, internal,
 			  no_resources)) * err)
 {
@@ -67,15 +63,20 @@ SetInputHandler(ilu_Transport t,
 }
 
 static          ilu_boolean
-WaitForInput(ilu_Transport t,
-	     ilu_FineTime * limit,
-	     ILU_ERRS((interrupted)) * err)
+WaitForInput(ilu_Transport t, int *disabled,
+	     ilu_FineTime * limit, ILU_ERRS(()) * err)
+{
+  return (*disabled = 0, ILU_CLER(*err));
+}
+
+static ilu_boolean 
+InterruptInpWait(ilu_Transport t, ILU_ERRS(()) * err)
 {
   return ILU_CLER(*err);
 }
 
 static ilu_boolean 
-Interrupt(ilu_Transport t, ILU_ERRS((bad_param)) * err)
+AbleInpWait(ilu_Transport t, ILU_ERRS(()) * err)
 {
   return ILU_CLER(*err);
 }
@@ -134,9 +135,8 @@ ReadBytes(ilu_Transport t, ilu_bytes buf, ilu_cardinal bufLen,
   tp->buffs = next = head->next;
   ilu_free(head);
   if (next != NIL) {
-    DEBUG(CONNECTION_DEBUG,
-	  (stderr,
-	   "ilubufxp.ReadBytes(%p): next chunk has %lu bytes\n",
+    ILU_NOTE(CONNECTION_DEBUG,
+	  ("ilubufxp.ReadBytes(%p): next chunk has %lu bytes\n",
 	   t, (long unsigned) next->len));
     t->tr_inBuff = next->bytes;
     t->tr_inNext = next->start;
@@ -151,8 +151,8 @@ ReadBytes(ilu_Transport t, ilu_bytes buf, ilu_cardinal bufLen,
     }
     return (next->len);
   }
-  DEBUG(CONNECTION_DEBUG,
-	(stderr, "ilubufxp.ReadBytes(%p): at EOM&F.\n", t));
+  ILU_NOTE(CONNECTION_DEBUG,
+	("ilubufxp.ReadBytes(%p): at EOM&F.\n", t));
   rpt->tr_eom = rpt->tr_eof = TRUE;
   ILU_CLER(*err);
   return 0;
@@ -164,7 +164,7 @@ Close(ilu_Transport t, ilu_integer * dfd,
 {
   TransportParms  tp = (TransportParms) t->tr_data;
   BuffList        bl = tp->buffs;
-  DEBUG(CONNECTION_DEBUG, (stderr, "ilubufxp.Close(%p)\n", t));
+  ILU_NOTE(CONNECTION_DEBUG, ("ilubufxp.Close(%p)\n", t));
   *dfd = 0;
   while (bl != NIL) {
     BuffList        tmp = bl;
@@ -184,7 +184,9 @@ static struct _ilu_TransportClass_s myClass = {
   CloseDFd,
   SetInputHandler,
   WaitForInput,
-  Interrupt,
+  InterruptInpWait,
+  AbleInpWait,
+  AbleInpWait,
   BeginMessage,
   EndMessage,
   SendWholeMessage,
@@ -193,44 +195,65 @@ static struct _ilu_TransportClass_s myClass = {
   Close
 };
 
-/* Main Invariant holds; L2 >= {t's conn's iomu} */
+/* Main Invariant holds; L2 >= {t's xmu, ymu} */
 ilu_Transport
 _ilu_BufferInputMessage(ilu_Transport t,
+			ilu_cardinal msgsize,
+			ilu_boolean byBytes,
 			ILU_ERRS((IoErrs)) * err)
 {
-  ilu_Transport   ans = (ilu_Transport) ilu_malloc(sizeof(*ans));
-  TransportParms  tp = (TransportParms) ilu_malloc(sizeof(tp));
+  ilu_Transport   ans = (ilu_Transport) ilu_MallocE(sizeof(*ans), err);
+  TransportParms  tp = NIL;
   BuffList       *tailp;
+  ilu_cardinal	  bytes_to_read;
   ilu_cardinal    nchunks = 0, nbytes = 0;
   ilu_TransportReport rpt = {FALSE, FALSE};
-  if (ans == NIL || tp == NIL) {
-    (void) ILU_ERR_CONS1(no_memory, err, nbytes,
-			 sizeof(*ans) + sizeof(*tp),
-			 6);
+  int             disabled;
+  if (ans)
+    tp = (TransportParms) ilu_MallocE(sizeof(*tp), err);
+  if (tp == NIL)
+    goto faild;
+  tp->buffs = NIL;
+  if (transport_boundaried(t) && byBytes) {
+    ILU_ERR_CONS1(internal, err, minor, ilu_im_tcBug, 0);
+    goto faild;
+  } else if (!transport_boundaried(t) && !byBytes) {
+    ILU_ERR_CONS1(internal, err, minor, ilu_im_tcNotBoundaried, 0);
     goto faild;
   }
-  tp->buffs = NIL;
   tailp = &tp->buffs;
-  while (!rpt.tr_eom) {
-    BuffList        c = (BuffList) ilu_malloc(sizeof(*c));
-    if (c == NIL) {
-      (void) ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*c), 6);
+  while ((!rpt.tr_eom) && (!byBytes || (nbytes < msgsize))) {
+    BuffList        c = (BuffList) ilu_MallocE(sizeof(*c), err);
+    if (c == NIL)
       goto faild;
-    }
     if (t->tr_inBuff == NIL || t->tr_inNext >= t->tr_inLimit)
-      if (!(*t->tr_class->tc_wait_for_input) (t, NIL, err))
+      if (!_ilu_TransportWaitForInputNoClose(t, NIL, err))
 	goto faild;
     c->start = 0;
     c->next = NIL;
     *tailp = c;
     tailp = &c->next;
-    c->len = transport_read_upto_bytes(t, c->bytes, CHUNKSIZE, &rpt,
+    if (byBytes)
+      bytes_to_read = MIN((msgsize - nbytes), CHUNKSIZE);
+    else
+      bytes_to_read = CHUNKSIZE;
+    c->len = transport_read_upto_bytes(t, c->bytes, bytes_to_read, &rpt,
 				       err);
     if (ILU_ERRNOK(*err))
       goto faild;
     nchunks += 1;
     nbytes += c->len;
   }
+  if (!tp->buffs) {
+    BuffList        c = (BuffList) ilu_MallocE(sizeof(*c), err);
+    if (c == NIL)
+      goto faild;
+    c->start = 0;
+    c->next = NIL;
+    *tailp = c;
+    tailp = &c->next;
+    c->len = 0;
+  }
   ans->tr_inBuff = tp->buffs->bytes;
   ans->tr_inNext = tp->buffs->start;
   ans->tr_inLimit = tp->buffs->start + tp->buffs->len;
@@ -238,12 +261,15 @@ _ilu_BufferInputMessage(ilu_Transport t,
   ans->tr_outNext = ans->tr_outLimit = 0;
   ans->tr_class = &myClass;
   ans->tr_data = tp;
-  if (!transport_end_message(t, FALSE, NIL, err))
-    goto faild;
-  DEBUG(CONNECTION_DEBUG,
-	(stderr,
-	 "_ilu_BufferInputMessage(%p) makes %p from %lu bytes in %lu chunks.\n",
-	 t, ans, (long unsigned) nbytes, (long unsigned) nchunks));
+  if (!byBytes) {
+    if (!transport_end_message(t, FALSE, NIL, err))
+      goto faild;
+  }
+  ILU_NOTE(CONNECTION_DEBUG,
+	   ("_ilu_BufferInputMessage(%p) makes %p from "
+	    "%lu bytes in %lu chunks.\n",
+	    t, ans, (long unsigned) nbytes,
+	    (long unsigned) nchunks));
   return ans;
 faild:
   if (tp != NIL) {
