@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iluerror.h,v 1.22 1996/07/09 21:54:50 mdavidso Exp $ */
+/* $Id: iluerror.h,v 1.23 1997/10/22 01:06:45 janssen Exp $ */
 /* Last edited by Mike Spreitzer February 1, 1996 11:54 am PST */
 
 #ifdef __cplusplus
@@ -215,7 +215,7 @@ automatically-generated procedures that effectively implement the arms
 of such type-switch statements.  The declaration of error type <id>
 expands to, among other things, a declaration like
 
-  extern ilu_ERRMEMP_T(<id>) ilu_ErrpQua_<id>(ilu_Error *e);
+  extern ILU_ERRMEMP_T(<id>) ilu_ErrpQua_<id>(ilu_Error *e);
 
 This procedure takes a non-null ilu_Error* (passing NIL is a checked
 runtime error) and returns a pointer to its members iff the error is
