@@ -14,28 +14,29 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iluerrs.h,v 1.61 1996/07/12 20:04:20 janssen Exp $ */
-/* Last edited by Mike Spreitzer April 2, 1996 10:37 pm PST */
+/* $Id: iluerrs.h,v 1.102 1997/11/13 23:13:41 spreitze Exp $ */
+/* Last edited by Mike Spreitzer November 13, 1997 11:28 am PST */
+
+#ifndef _ILUERRS_H_
+#define _ILUERRS_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef _ILUERRS_H_
-#define _ILUERRS_H_
-
 ILU_DECL_PARMLESS_ERR(success);
 /* A non-error that reserves a code for successful completion. */
 
 
 /*
- * First, the errors that can appear at the kernel interface.  Each
- * of these has a defined mapping into a CORBA 1.2 standard
- * exception; that mapping is documented in these declarations,
- * except where the mapping is to the exception of the same name.
- * Some, which are noted as such, aren't actually used --- but
- * appear in this list so that a prefix of this list corresponds in
- * number and position to the published list for CORBA.
+ * First, the errors that can appear (suitably mapped, of course) in
+ * the language mappings.  Each of these has a defined mapping into
+ * a CORBA 1.2 standard system exception; that mapping is documented
+ * in these declarations, except where the mapping is to the
+ * exception of the same name. Some, which are noted as such, aren't
+ * actually used --- but appear in this list so that a prefix of
+ * this list corresponds in number and position to the published
+ * list for CORBA.
  */
 
 ILU_DECL_PARMLESS_ERR(unknown);	/* not used */
@@ -62,6 +63,7 @@ typedef enum {
   ilu_bpm_sequenceLimit,	/* sequence too long for its limit */
   ilu_bpm_unionDiscSize,	/* invalid discriminant size */
   ilu_bpm_nil,			/* NIL passed where not allowed */
+  ilu_bpm_broken,		/* not NIL, but contents look bad */
   ilu_bpm_closed,		/* op invoked on closed something */
   ilu_bpm_small_buffer,		/* callee needs larger buffer */
   ilu_bpm_string_null_char,	/* octet 0 in `string' parm */
@@ -70,6 +72,47 @@ typedef enum {
    * A proc appropriate only for single-threaded runtimes was called
    * in a mult-threaded runtime --- or the other way around.
    */
+  ilu_bpm_prefix_type_violation,
+  /* type in pickle doesn't satisfy prefix type condition on pickle
+   * being marshalled or unmarshalled.
+   */
+
+  /*
+   * These errors may be raised by the runtime type system
+   */
+  ilu_bpm_not_byte_type,	/* expected byte type */
+  ilu_bpm_not_boolean_type,	/* expected boolean type */
+  ilu_bpm_not_character_type,	/* expected character type */
+  ilu_bpm_not_shortcharacter_type,/* expected shortcharacter type */
+  ilu_bpm_not_shortinteger_type,/* expected shortinteger type */
+  ilu_bpm_not_integer_type,	/* expected integer type */
+  ilu_bpm_not_longinteger_type,	/* expected longinteger type */
+  ilu_bpm_not_shortcardinal_type,/* expected shortcardinal type */
+  ilu_bpm_not_cardinal_type,	/* expected cardinal type */
+  ilu_bpm_not_longcardinal_type,/* expected longcardinal type */
+  ilu_bpm_not_shortreal_type,	/* expected shortreal type */
+  ilu_bpm_not_real_type,	/* expected real type */
+  ilu_bpm_not_longreal_type,	/* expected longreal type */
+  ilu_bpm_not_object_type,	/* expected object type */
+  ilu_bpm_not_optional_type,	/* expected optional type */
+  ilu_bpm_not_alias_type,	/* expected alias type */
+  ilu_bpm_not_union_type,	/* expected union type */
+  ilu_bpm_invalid_union_arm_index,	/* out of bounds */
+  ilu_bpm_not_sequence_type,	/* expected sequence type */
+  ilu_bpm_not_array_type,	/* expected array type */
+  ilu_bpm_not_record_type,	/* expected record type */
+  ilu_bpm_invalid_record_field_index,
+  ilu_bpm_not_enumeration_type,	/* expected enumeration type */
+  ilu_bpm_invalid_enum_element_index,
+  ilu_bpm_not_variant_type,	/* expected variant type */
+  ilu_bpm_invalid_variant_type,	/* variant type is badly formed */
+  ilu_bpm_not_string_type,	/* expected string type */
+  ilu_bpm_invalid_typekind,	/* ilu_Type with bad "kind" */
+  ilu_bpm_invalid_method_index,	/* out of bounds */
+  ilu_bpm_invalid_argument_index,	/* out of bounds */
+  ilu_bpm_invalid_exception_index,	/* out of bounds */
+  ilu_bpm_not_sibling,		/* non-sibling passed where sibling required */
+
   ilu_bpm_protocol_registered,
   /* attempt to register already-registered protocol */
   ilu_bpm_transport_registered,
@@ -79,8 +122,16 @@ typedef enum {
 
   ilu_bpm_bogus_raise,		/* Method tried to raise an
 				 * exception not in its RAISES list. */
-  ilu_bpm_some_raise		/* Method with empty exn list tried
+  ilu_bpm_some_raise,		/* Method with empty exn list tried
 				 * to raise an exn. */
+  ilu_bpm_short_char_codeset,	/* unsupported short character code set */
+  ilu_bpm_char_codeset,		/* unsupported character code set */
+  ilu_bpm_non_batching,		/* flush called on non-batching conn */
+  ilu_bpm_serialVsServer,	/* ilu_Serializer used on wrong server */
+  ilu_bpm_serialConcurrent,	/* bad case of ilu_Serializer used
+				 * in concurrent calls */
+  ilu_bpm_serialVsTransport	/* ilu_Serializer used with
+				 * unreliable transport */
 }               ilu_bad_param_Minor;
 
 ILU_DECL_ERR(bad_param)
@@ -108,7 +159,32 @@ typedef enum {
    */
   ilu_ilm_max_protocols,	/* too many protocols registered */
   ilu_ilm_max_transports,	/* too many transports registered */
-  ilu_ilm_max_identity_types	/* too many identity types registered */
+  ilu_ilm_max_identity_types,	/* too many identity types
+				 * registered */
+  ilu_ilm_refcnt,		/* refcount would overflow */
+  /*
+   * ILU only permits 32766 arms to a union
+   */
+  ilu_ilm_max_union_arms,	/* too many arms to a union */
+  /*
+   * Some reasonable operation cannot be performed because of a bug
+   * in the CORBA spec which ILU faithfully implements...
+   */
+  ilu_ilm_bad_type_for_protocol,
+  /* this protocol does support this type
+   */
+  ilu_ilm_sub_protocol,
+  /* a sub-protocol has been specified which is not supported
+   */
+  ilu_ilm_max_buffer_size,
+  /* attempt to use a buffer for a sequence too long for it */
+  ilu_ilm_unsupported_charset_encoding,
+  /* specified character set encoding not supported */
+  ilu_ilm_corba_iiop_unions
+  /*
+   * some valid OMG IDL union typecodes cannot be marshalled with
+   * IIOP 1.0 or 1.1
+   */
 
 }               ilu_imp_limit_Minor;
 
@@ -133,7 +209,9 @@ typedef enum {
 
   ilu_cfm_eof,			/* unexpected eof on connection */
   ilu_cfm_protocol_sync_lost,	/* unexpected bytes with no way to recover */
-  ilu_cfm_conn_lost		/* other or unknown reason */
+  ilu_cfm_tr_non_boundaried,	/* can't use this transport stack */
+  ilu_cfm_conn_lost,		/* other or unknown reason */
+  ilu_cfm_resource_mgmt		/* closed due to need for FD */
 }               ilu_comm_failure_Minor;
 
 ILU_DECL_ERR(comm_failure)
@@ -142,6 +220,7 @@ ILU_DECL_ERR(comm_failure)
 } ILU_END_DECL_ERR;
 
 typedef enum {
+  ilu_iom_unknown,		/* reason unknown or not below */
   ilu_iom_sbh,			/* malformed SBH */
   ilu_iom_ci,			/* malformed contact info */
   ilu_iom_pi,			/* malformed protocol info */
@@ -152,15 +231,26 @@ typedef enum {
   ilu_iom_ts,			/* transport-specific part invalid */
   ilu_iom_sid,			/* malformed server ID */
   ilu_iom_ih,			/* malformed instance handle */
-  ilu_iom_mstid,		/* malformed MSTID */
-  ilu_iom_nil,			/* NIL object found in invalid context */
+  ilu_iom_bad_mstid,		/* malformed MSTID */
+  ilu_iom_mstid_fail,		/* can't resolve MSTID */
+  ilu_iom_nil,			/* NIL object found in invalid
+				 * context */
   ilu_iom_bad_url_scheme,	/* invalid scheme tag */
   ilu_iom_tf,
   /* Transport class used in inappropriately filterly position. */
   ilu_iom_ior,			/* invalid IIOP IOR */
   ilu_iom_svr_closed,		/* server closed */
+  ilu_iom_type_nf,		/* exist no inst.s of req'd type */
   ilu_iom_inst_nf,		/* instance doesn't exist */
-  ilu_iom_wrong_type		/* instance doesn't have right type */
+  ilu_iom_bad_single,		/* can't exist due to singleton
+				 * restr. */
+  ilu_iom_wrong_type,		/* instance doesn't have right type */
+  ilu_iom_short_char_codeset,	/* unsupported short character code
+				 * set */
+  ilu_iom_char_codeset,		/* unsupported character code set */
+  ilu_iom_relocate_loop,	/* detection might be conservative */
+  ilu_iom_conc_serial		/* serializer used when contact info
+				 * is concurrent */
 
 }               ilu_inv_objref_Minor;
 
@@ -180,6 +270,7 @@ typedef enum {
   ilu_im_errno,			/* syscall raised unexpected errno */
   ilu_im_trBufSize,		/* tc_get_output_buffer couldn't */
   ilu_im_tInfoLen,		/* tinfo too long */
+  ilu_im_badTypeKind,		/* invalid ilu_TypeKind code for context */
   ilu_im_callFail,		/* an internal call failed */
   ilu_im_badLocks,		/* bad_locks maps to this case */
   ilu_im_brokenLocks,		/* broken_locks maps to this case */
@@ -194,16 +285,23 @@ typedef enum {
   ilu_im_tcNotBoundaried,	/* for b'd method of unb'd Trans'Class */
   ilu_im_tcReliable,		/* for unreliable call on reliable TC */
   ilu_im_tcBadBuff,		/* bad buffer given to Trans'Class proc */
+  ilu_im_protNonConcurrent,	/* DelayInterp called on non-concurrent prot */
   ilu_im_tcNoMsgHandle,		/* for tc_end_message */
   ilu_im_noHostName,		/* unable to get a name for this host */
   ilu_im_noHostIpAddr,		/* unable to get IP addr for this host */
   ilu_im_bufxpMisuse,		/* bufxp caller violated contract */
   ilu_im_typeMismatch,		/* two stubs with different ideas */
   ilu_im_typeIncomplete,	/* type not completely constructed */
+  ilu_im_typeDuplicated,	/* type multiply registered */
+  ilu_im_dupForkProc,		/* attempt to register ForkProc twice */
+  ilu_im_noForkProc,		/* no way to fork an internal thread */
   ilu_im_threading,		/* confusion on whether threaded */
   ilu_im_threadFork,		/* fork failed */
+  ilu_im_threadIDSize,		/* thread ID size too large for context */
+  ilu_im_threadAttribute,	/* problem setting/getting thread attr */
   ilu_im_tportRole,		/* incoming vs. outgoing transport err */
-  ilu_im_check			/* internal consistency check failed */
+  ilu_im_check,			/* internal consistency check failed */
+  ilu_im_badEnumValue		/* unexpected enumeration value */
 }               ilu_internal_Minor;
 
 ILU_DECL_ERR(internal)
@@ -215,6 +313,9 @@ typedef enum {
   ilu_mm_eom,			/* attempted read past end of msg */
   ilu_mm_alien_disc,		/* unmarshalling discriminator of
 				 * different server */
+  ilu_mm_bad_union_disc,	/* invalid union discriminant value */
+  ilu_mm_bad_typekind,		/* TypeCode received with invalid
+				 * type kind value */
   ilu_mm_wronglen,		/* fixed length array came in with
 				 * different length */
   ilu_mm_sequenceLimit,		/* attempt to read or write a
@@ -227,10 +328,16 @@ typedef enum {
 				 * fit position */
   ilu_mm_dgramLimit,		/* datagram (eg, UDP) size limit
 				 * exceeded */
+  ilu_mm_badVariant,		/* variant value of wrong class
+				 * received */
+  ilu_mm_badTypeName,		/* badly formed type name */
+  ilu_mm_protNoTypekind,	/* this typekind not supported by this protocol */
   ilu_mm_msgTypeUnknown,	/* invalid message type received */
-  ilu_mm_utf2Len		/* UTF2 encoded string's length
+  ilu_mm_utf2Len,		/* UTF2 encoded string's length
 				 * doesn't correspond to plain
 				 * string length */
+  ilu_mm_mst_unreg,		/* LSR can't make surrogate */
+  ilu_mm_url_quoted_char	/* Bad quoted hex char in URL form */
 }               ilu_marshal_Minor;
 
 ILU_DECL_ERR(marshal)
@@ -264,6 +371,8 @@ typedef enum {
   ilu_nrm_ENFILE,		/* system file table full */
   ilu_nrm_ENOBUFS,		/* insufficient buffer space avail */
   ilu_nrm_fds,			/* FD budget */
+  ilu_nrm_mux_sessions,		/* Sessions on a single mux channel */
+  ilu_nrm_mux_channels,		/* Channels available via mux in a single address space */
   ilu_nrm_mlreg			/* main loop registrations */
 }               ilu_no_resources_Minor;
 
@@ -275,7 +384,18 @@ ILU_DECL_ERR(no_resources)
 ILU_DECL_PARMLESS_ERR(no_response);	/* not used */
 ILU_DECL_PARMLESS_ERR(persist_store);	/* not used */
 ILU_DECL_PARMLESS_ERR(bad_inv_order);	/* not used */
-ILU_DECL_PARMLESS_ERR(transient);	/* not used */
+
+typedef enum {
+  ilu_tm_retry			/* stub should retry call (but
+				 * didn't, if app sees this) */
+}               ilu_transient_Minor;
+
+ILU_DECL_ERR(transient)
+{
+  ilu_transient_Minor minor;
+} ILU_END_DECL_ERR;
+
+
 ILU_DECL_PARMLESS_ERR(free_mem);	/* not used */
 ILU_DECL_PARMLESS_ERR(inv_ident);	/* not used */
 ILU_DECL_PARMLESS_ERR(inv_flag);	/* not used */
@@ -284,8 +404,15 @@ ILU_DECL_PARMLESS_ERR(bad_context);	/* not used */
 ILU_DECL_PARMLESS_ERR(obj_adapter);	/* not used */
 ILU_DECL_PARMLESS_ERR(data_conversion);	/* not used */
 
+ILU_DECL_PARMLESS_ERR(codeset_incompatible);	/* added in 2.1 revision */
+
 /* This is the end of the prefix that corresponds to CORBA's list. */
 
+ILU_DECL_PARMLESS_ERR(barrier);
+/*
+ * Raised to indicate an exception to a serialization guarantee.
+ */
+
 ILU_DECL_PARMLESS_ERR(bad_locks);
 /*
  * Raised when the calling thread is detected to have violated the
@@ -346,6 +473,26 @@ ILU_DECL_ERR(gss_security)
 
 } ILU_END_DECL_ERR;
 
+
+/* Next, the additional errors that can appear in the kernel MOPs. */
+
+typedef enum {
+  ilu_relocate_call,	/* just this call */
+  ilu_relocate_conn	/* use new conn. while it lasts */
+}               ilu_RelocateScope;
+
+extern ilu_string ilu_RelocateScope_Names[2];
+
+#define ilu_RelocateScope_Name(rs) ((rs) <= ilu_relocate_conn ? ilu_RelocateScope_Names[rs] : "(invalid ilu_RelocateScope)")
+
+ILU_DECL_ERR(relocate)
+{
+  ilu_RelocateScope	rel_scope;
+  ilu_ProtocolInfo	rel_pinfo;	/* owned by this err */
+  ilu_TransportInfo	rel_tinfo;	/* owned by this err */
+} ILU_END_DECL_ERR;
+
+
 /* Following are the internal errors of the new draft. */
 
 
@@ -425,8 +572,8 @@ ILU_END_DECL_ERR;
 
 ILU_DECL_ERR(ProtocolAlreadyRegistered) {
   char * name;
-  ilu_Protocol (*old_protocol) (void);
-  ilu_Protocol (*new_protocol) (void);
+  ilu_Protocol (*old_protocol) (ilu_ProtocolInfo, ilu_Error *);
+  ilu_Protocol (*new_protocol) (ilu_ProtocolInfo, ilu_Error *);
 }
 ILU_END_DECL_ERR;
 
@@ -533,12 +680,15 @@ ILU_DECL_PARMLESS_ERR(CantCondition);
     ILU_ERRLISTELT(bad_context)		\
     ILU_ERRLISTELT(obj_adapter)		\
     ILU_ERRLISTELT(data_conversion)	\
+    ILU_ERRLISTELT(codeset_incompatible)	\
+    ILU_ERRLISTELT(barrier)		\
     ILU_ERRLISTELT(bad_locks)		\
     ILU_ERRLISTELT(broken_locks)	\
     ILU_ERRLISTELT(interrupted)		\
     ILU_ERRLISTELT(gss_security)	\
     					\
     /* approved private types (not to cross kernel interface) */	\
+    ILU_ERRLISTELT(relocate)		\
     					\
     /* unresolved private types (not to cross kernel interface) */	\
     ILU_ERRLISTELT(KernelBroken)	\
@@ -592,8 +742,8 @@ ILU_PUBLIC unsigned long
  * to -1 and returns 0.  Calls ILU_HANDLED(*err).
  */
 
-#endif /* ndef _ILUERRS_H_ */
 #ifdef __cplusplus
 }
 #endif
 
+#endif /* ndef _ILUERRS_H_ */
