@@ -14,13 +14,15 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: inmem.c,v 1.27 1996/05/29 03:34:30 janssen Exp $ */
-/* Last edited by Mike Spreitzer March 12, 1996 9:38 am PST */
+/* $Id: inmem.c,v 1.33 1997/11/25 00:30:57 janssen Exp $ */
+/* Last edited by Mike Spreitzer November 13, 1997 2:26 pm PST */
 
-/* In-memory Transport
-   The in-memory transport is a boundaried, reliable transport
-   which buffers a request or reply.
-   Its transport info begins "inmem_".
+/*
+ * In-memory Transport.  The in-memory transport is a boundaried,
+ * reliable transport which buffers a single request and a reply
+ * sequence.  Its transport info begins "inmem_".  An LSR must
+ * animate an inmem_ port in the single-threaded style, even in
+ * multi-threaded runtimes.
  */
 
 /* place includes here */
@@ -30,9 +32,11 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include "transprt.h"
 
 /* place #defines here */
-#define BUFSIZE 4096 /* Stab in the dark.  Doubled if too small.  */
+#define BUFSIZE 4096 /* Stab in the dark.  Double if too small.  */
 
 typedef struct BufferList_s {
+  /* L1, L2 unconstrained */
+  
   ilu_bytes buffer;
   ilu_cardinal offset; /* valid data at [0, offset),
 			  unused space at [offset, size). */
@@ -42,57 +46,58 @@ typedef struct BufferList_s {
 
 /* Shared between the two transports at the "ends" of the connection */
 typedef struct SharedData_s {
-  /* L2 >= {callmu, iomu of outgoing transport's conn} */
+  /* L1, L2 unconstrained; synch provided by single-threadedness */
   
-  BufferList sd_request; /* next is NIL */
+  BufferList sd_request;	/* next is NIL */
 
   /* There may be multiple outstanding replies. */
   /* -Tail is NIL exactly when -Head is NIL. */
-  BufferList sd_repliesHead; /* Take off from */
-  BufferList sd_repliesTail; /* Put on to */
+  BufferList sd_repliesHead;	/* Take off from */
+  BufferList sd_repliesTail;	/* Put on to */
 
   /* Messages that have been read. */
   BufferList sd_reuseHead; 
   BufferList sd_reuseTail;
 
   /* Input handler of incoming side Transport */
-  ilu_TIH        *sd_tih;    /* tihSet => tih != NIL */
-  ilu_boolean     sd_tihSet; /* (if FALSE, sd_tih not meaningful.) */
+  ilu_TIH         sd_tih;	/* owned by client, not this struct */
 
-  ilu_cardinal sd_bufferSize; /* Used to allocate buffers.
+  ilu_cardinal sd_bufferSize;	/* Used to allocate buffers.
 			Possibly read from creator's tinfo.  */
   
 } *SharedData;
 
-
 typedef struct {
+  /* L1, L2 unconstrained */
+  
   /* Connection request handler. */
-  ilu_TIH        *md_tih;		/* !=NIL when req handler reg'd */
-  SharedData      md_sharedData;   /* NIL outside mutex. */
-  ilu_boolean     md_locked; /* stand-in for a mutex */
+  ilu_TIH         md_tih;		/* !=NIL when req handler reg'd */
+  SharedData      md_sharedData;	/* NIL outside mutex. */
+  ilu_boolean     md_locked;		/* stand-in for a mutex */
 }              *MooringParms;
+/* (md_tih) storage owned by client. */
 
 
 typedef struct {
-  MooringParms cd_mooringData; /* NIL until mooring is created */
-  ilu_TransportInfo cd_tinfo;   /* the tinfo passed at creator creation time */
+  /* L1, L2 unconstrained */
+  
+  MooringParms cd_mooringData;	/* NIL until mooring is created */
+  ilu_TransportInfo cd_tinfo;
+  /* tinfo passed at creator creation time */
 }              *CreatorParms;
 
 
 
 typedef struct {
+  /* L1, L2 unconstrained */
 
-  ilu_boolean     td_working; /* between Begin- and EndMessage */
-  ilu_boolean     td_input; /* vs. output.  Meaningful if working. */
+  ilu_boolean     td_working;	/* between Begin- and EndMessage */
+  ilu_boolean     td_input;	/* vs. output.  Meaningful if working. */
 
-  ilu_boolean     td_outgoing; /* vs. incoming. */
-
-#if 0
-  MooringParms    td_mooringData;  /* Address of mooring parms */
-#endif
+  ilu_boolean     td_outgoing;	/* vs. incoming. */
 
   SharedData      td_sharedData; /* shared with "other end" transport */
-  BufferList      td_current; /* NIL if !working */
+  BufferList      td_current;	/* NIL if !working */
 
 }              *TransportParms;
 
@@ -114,87 +119,72 @@ typedef struct {
 
 /*L1, L2 unconstrained*/
 
-/*Main Invariant holds; L2 no further constrained*/
-static          ilu_boolean
-_inmem_Interrupt(ilu_Transport self, ILU_ERRS((bad_param)) * err)
-{
-  return ILU_CLER(*err);
-}
-
 static          ilu_integer
 _inmem_CloseDFd(ilu_Transport self)
 {
   return (0);
 }
 
-
-/*Main Invariant holds; L2 disjoint {conn's iomu, callmu}*/
 static          ilu_boolean
 _inmem_SetInputHandler(ilu_Transport self,
-		     ilu_TransportInputHandler tih_proc,
-		     ilu_refany tih_rock,
-		     ILU_ERRS((no_memory, no_resources)) * err)
+		       ilu_TIH ptih,
+		       ILU_ERRS((no_memory, no_resources)) * err)
 {
   TransportParms  tp = (TransportParms) self->tr_data;
   SharedData      sd = tp->td_sharedData;
-  ilu_TIH        *tih = sd->sd_tih;
 
   if (tp->td_outgoing)
     /* Don't bother, it's just ReadExtraMsg */
     return ILU_CLER(*err);
-  sd->sd_tihSet = FALSE;
-  if (tih_proc != NULLFN) {
-    if (tih == NIL) {
-      tih = (ilu_TIH *) ilu_malloc(sizeof(*tih));
-      if (tih == NIL)
-	return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*tih), FALSE);
-      sd->sd_tih = tih;
-    }
-    tih->tih_proc = tih_proc;
-    tih->tih_rock = tih_rock;
-    sd->sd_tihSet = TRUE;
-  }
+  sd->sd_tih = ptih;
   return ILU_CLER(*err);
 }
 
-/*Main Invariant holds*/
-/*L2 >= {conn's iomu}*/
-
 /*
- * Should only be called when message is finished and waiting in
- * shared data space, and control has passed from message writer to
- * message reader.
+ * Should only be called on outgoing end, when message is finished
+ * and waiting in shared data space, and control has returned from
+ * message writer to message reader.
  */
 static          ilu_boolean
-_inmem_WaitForInput(ilu_Transport t, ilu_FineTime * limit,
-		  ILU_ERRS((interrupted)) * err)
+_inmem_WaitForInput(ilu_Transport t, int *disabled,
+		    ilu_FineTime * limit, ILU_ERRS(()) * err)
+{
+  return (*disabled = 0, ILU_CLER(*err));
+}
+
+static          ilu_boolean
+_inmem_Interrupt(ilu_Transport self, ILU_ERRS(()) * err)
 {
   return ILU_CLER(*err);
 }
 
-
-static BufferList
-  NewBufferList (ilu_cardinal size, ilu_bytes buffer, ILU_ERRS((IoErrs)) * err)
+static          ilu_boolean
+_inmem_AbleInpWait(ilu_Transport self, ILU_ERRS((bad_param)) * err)
 {
- BufferList b = (BufferList) ilu_malloc(sizeof(struct BufferList_s));
- if (b == NIL)
-   return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(struct BufferList_s), NIL);
- if (buffer == NIL)
-   {
-     b->buffer = (ilu_bytes) ilu_malloc( size );
-     if (b->buffer == NIL)
-       return ILU_ERR_CONS1(no_memory, err, nbytes, size, NIL);
-   }
- else
-   b->buffer = buffer;
- b->offset = 0;
- b->size = size;
- b->next = NIL;
- ILU_CLER(*err);
- return b;
+  return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_threading, FALSE);
 }
 
-  /*Main Invariant holds; L2 >= {conn's iomu}*/
+static          BufferList
+NewBufferList(ilu_cardinal size, ilu_bytes buffer,
+	      ILU_ERRS((IoErrs)) * err)
+{
+  BufferList      b = (BufferList) ilu_MallocE(sizeof(*b), err);
+  if (b == NIL)
+    return NIL;
+  if (buffer == NIL) {
+    b->buffer = (ilu_bytes) ilu_MallocE(size, err);
+    if (b->buffer == NIL) {
+      ilu_free(b);
+      return NIL;
+    }
+  } else
+    b->buffer = buffer;
+  b->offset = 0;
+  b->size = size;
+  b->next = NIL;
+  ILU_CLER(*err);
+  return b;
+}
 
 /*
  * The inmemory transport is boundaried.
@@ -209,8 +199,8 @@ _inmem_BeginMessage(ilu_Transport self,
   TransportParms tp = (TransportParms) self->tr_data;
   SharedData sd = tp->td_sharedData;
 
-  DEBUG(INMEM_DEBUG,
-	(stderr, "inmem(%p,%p): BeginMessage(%s).\n",
+  ILU_NOTE(INMEM_DEBUG,
+	("inmem(%p,%p): BeginMessage(%s).\n",
 	 self, sd, input_p ? "input" : "output"));
   
   /* Already began a message? */
@@ -223,9 +213,8 @@ _inmem_BeginMessage(ilu_Transport self,
     tp->td_input = TRUE;
     if (tp->td_outgoing) {
       if (sd->sd_repliesHead == NIL) {
-	DEBUG(INMEM_DEBUG,
-	      (stderr,
-	       "inmem(%p,%p): BeginMessage(outgoing input) found nothing.\n",
+	ILU_NOTE(INMEM_DEBUG,
+	      ("inmem(%p,%p): BeginMessage(outgoing input) found nothing.\n",
 	       self, sd));
 	tp->td_working = FALSE;
 	return ilu_rhrc_nothing;
@@ -236,9 +225,8 @@ _inmem_BeginMessage(ilu_Transport self,
 	sd->sd_repliesTail = NIL;
     } else {
       if (sd->sd_request == NIL) {
-	DEBUG(INMEM_DEBUG,
-	      (stderr,
-	       "inmem(%p,%p): BeginMessage(incoming input) found nothing.\n",
+	ILU_NOTE(INMEM_DEBUG,
+	      ("inmem(%p,%p): BeginMessage(incoming input) found nothing.\n",
 	       self, sd));
 	tp->td_working = FALSE;
 	return ilu_rhrc_nothing;
@@ -275,7 +263,8 @@ _inmem_BeginMessage(ilu_Transport self,
   return ilu_rhrc_ok;
 }
 
-/* Before: (does not hold mooring mutex)
+/**Main Invariant holds;
+   Before: (does not hold mooring mutex)
    After: if flush, holds mooring mutex */
 static          ilu_boolean
 _inmem_EndMessage(ilu_Transport self,
@@ -292,8 +281,8 @@ _inmem_EndMessage(ilu_Transport self,
   tp->td_working = FALSE;
   
   if (tp->td_input) {
-    DEBUG(INMEM_DEBUG,
-	  (stderr, "inmem(%p,%p): EndMessage(input)\n",
+    ILU_NOTE(INMEM_DEBUG,
+	  ("inmem(%p,%p): EndMessage(input)\n",
 	   self, sd));
     /* Finished reading a message, "throw it away" */
     if (sd->sd_reuseHead == NIL)
@@ -310,10 +299,10 @@ _inmem_EndMessage(ilu_Transport self,
   } else {
     /* Finished writing a message. */
     /* Remember how much was written */
-    DEBUG(INMEM_DEBUG,
-      (stderr, "inmem(%p,%p): EndMessage(%s, flush=%d, len=%lu)\n",
-       self, sd, "output", flush != FALSE,
-       (long unsigned) self->tr_outNext));
+    ILU_NOTE(INMEM_DEBUG,
+	  ("inmem(%p,%p): EndMessage(%s, flush=%d, len=%lu)\n",
+	   self, sd, "output", flush != FALSE,
+	   (long unsigned) self->tr_outNext));
     tp->td_current->offset = self->tr_outNext;
     if (tp->td_outgoing) {
       /* Store mesage as request */
@@ -322,8 +311,8 @@ _inmem_EndMessage(ilu_Transport self,
       /* signal ilu_im_broken? */
       sd->sd_request = tp->td_current;
       /* Call the input handler! */
-      if (sd->sd_tih->tih_proc != NULLFN)
-	(*sd->sd_tih->tih_proc) (sd->sd_tih->tih_rock);
+      if (sd->sd_tih)
+	(*sd->sd_tih->proc) (sd->sd_tih->rock);
       /* No return value. */
     } else {
       /* Store message as reply */
@@ -334,6 +323,7 @@ _inmem_EndMessage(ilu_Transport self,
 	sd->sd_repliesTail = sd->sd_repliesTail->next;
       }
     }
+    tp->td_current = NIL;
   }
   return ILU_CLER(*err);
 }
@@ -343,7 +333,7 @@ _inmem_EndMessage(ilu_Transport self,
  */
 static          ilu_boolean
 _inmem_SendWholeMessage(ilu_Transport self, ilu_Message * msgh,
-		      ilu_Error * err)
+		      ILU_ERRS((IoErrs)) * err)
 {
   return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcReliable,
 		       FALSE);
@@ -372,13 +362,14 @@ _inmem_WriteBytes(ilu_Transport self, ilu_bytes buf,
     return ILU_ERR_CONS1(internal, err, minor,
 			 ilu_im_bytesWithoutMsg, FALSE);
 
-  if ((self->tr_outNext + bufferSize) >= self->tr_outLimit) {
-    tp->td_current->size += MAX(tp->td_current->size, more);
-    self->tr_outLimit = tp->td_current->size;
-    b = (ilu_bytes) ilu_realloc(self->tr_outBuff, tp->td_current->size);
+  if ((self->tr_outNext + bufferSize + 16) >= self->tr_outLimit) {
+    ilu_cardinal    nusize;
+    nusize = tp->td_current->size + MAX(tp->td_current->size, more);
+    b = (ilu_bytes) ilu_realloc(self->tr_outBuff, nusize);
     if (b == NIL)
       return ILU_ERR_CONS1(no_memory, err, nbytes,
-			   tp->td_current->size, FALSE);
+			   nusize, FALSE);
+    self->tr_outLimit = tp->td_current->size = nusize;
     self->tr_outBuff = tp->td_current->buffer = b;
   }
   if (buf != NIL) {
@@ -417,24 +408,15 @@ _inmem_ReadBytes(ilu_Transport self,
   rpt->tr_eom = TRUE;
   ILU_CLER(*err);
   return 0;
-
-  /*
-   * "Does not read past message boundary (if boundaried), EOF, or end
-   * of given buffer."
-   */
-  /* if there's an EOF in the buffer, spec breaks.
-     (would there ever be?) */
-
 }
 
 
 /*L1, L2 unconstrained*/
+
 static void CloseSharedData(SharedData sd)
 {
   BufferList bl, next;
 
-  if (sd->sd_tih != NIL)
-    ilu_free(sd->sd_tih);
   if (sd->sd_request != NIL) {
     ilu_free(sd->sd_request->buffer);
     ilu_free(sd->sd_request);
@@ -456,22 +438,38 @@ static void CloseSharedData(SharedData sd)
   ilu_free(sd);
 }
 
-/*L1, L2 unconstrained*/
 static          ilu_boolean
 _inmem_CloseTransport(ilu_Transport self, ilu_integer * dfd,
-		    ILU_ERRS((internal)) * err)
+		      ILU_ERRS((bad_locks, broken_locks,
+				internal)) * err)
 {
   /*
    * Free anything that belongs to self. Outgoing transport
    * responsible for shared data.
    */
   TransportParms  tp = (TransportParms) self->tr_data;
-  DEBUG(INMEM_DEBUG,
-	(stderr, "inmem(%p,%p): Close\n",
-	 tp, tp->td_sharedData));
+  SharedData      sd = tp->td_sharedData;
+  ILU_NOTE(INMEM_DEBUG,
+	   ("inmem(%p,%p): Close\n", tp, sd));
   *dfd = 0;
   if (tp->td_outgoing)
     CloseSharedData(tp->td_sharedData);
+  else if (sd->sd_tih) {
+    ILU_ERRS((internal, bad_param, bad_locks, broken_locks)) lerr;
+    ilu_boolean     ans;
+    ilu_Closure     c = _ilu_ClosureFromTIH(sd->sd_tih);
+    ILU_NOTE(INMEM_DEBUG,
+	     ("inmem(%p,%p): Close => DoSoon(%p{%p(%p)})\n",
+	      tp, sd, c, c->proc, c->rock));
+    ans = ilu_DoSoon(c, &lerr);
+    ILU_ERR_SWITCH(lerr) {
+      ILU_SUCCESS_CASE 0;
+      ILU_ERR_CASE3(internal, bad_locks, broken_locks) * err = lerr;
+      ILU_ERR_ELSE    ilu_Check(FALSE, err);
+    } ILU_ERR_ENDSWITCH;
+    if (!ans)
+      return FALSE;
+  }
   if (tp->td_current != NIL) {
     ilu_free(tp->td_current->buffer);
     ilu_free(tp->td_current);
@@ -482,16 +480,6 @@ _inmem_CloseTransport(ilu_Transport self, ilu_integer * dfd,
   return ILU_CLER(*err);
 }
 
-/***********************************
-  Methods of the in-memory mooring
-
-  mo_set_req_handler
-  mo_wait_for_req
-  mo_accept_connection
-  mo_close
-  
-  ***********************************/
-
 /*
  * Set the transport class methods...
  * there is only one memClass.
@@ -504,6 +492,8 @@ static struct _ilu_TransportClass_s memClass = {
   _inmem_SetInputHandler,
   _inmem_WaitForInput,
   _inmem_Interrupt,
+  _inmem_AbleInpWait,
+  _inmem_AbleInpWait,
   _inmem_BeginMessage,
   _inmem_EndMessage,
   _inmem_SendWholeMessage,
@@ -512,24 +502,70 @@ static struct _ilu_TransportClass_s memClass = {
   _inmem_CloseTransport
 };
 
-/*L1, L2, Main unconstrained*/
+/***********************************
+  Methods of the in-memory mooring
+
+  mo_set_req_handler
+  mo_wait_for_req
+  mo_accept_connection
+  mo_close
+  
+  ***********************************/
+
+/*L1, L2 unconstrained*/
+
+static ilu_integer _inmem_MooringDFd(ilu_Mooring self, ilu_boolean add)
+{
+  return 0;
+}
+
+static          ilu_boolean
+_inmem_SetReqHandler(ilu_Mooring self,
+		     ilu_TIH tih,
+		     ILU_ERRS((no_memory, internal)) * err)
+{
+  MooringParms    mp = (MooringParms) self->mo_data;
+  if (!ilu_Check(mp->md_tih == NIL, err))
+    return FALSE;
+  mp->md_tih = tih;
+  return ILU_CLER(*err);
+
+}
+
+static          ilu_boolean
+_inmem_WaitForReq(ilu_Mooring self, int *disabled,
+		  ILU_ERRS(()) * err)
+{
+  *disabled = 0;
+  /* Should not be used for inmem. */
+  _ilu_Assert(0, "_inmem_WaitForReq");
+  return FALSE;
+}
+
+static          ilu_boolean
+_inmem_AbleReqWait(ilu_Mooring self,
+		  ILU_ERRS((bad_param)) * err)
+{
+  /* Should not be used for inmem. */
+  return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_threading, FALSE);
+}
 
-/* L1, L2 unconstrained */
 static          ilu_Transport
 _inmem_NewT(ILU_ERRS((IoErrs)) * err)
-     /* Used for outgoing and incoming transports
-	If buffer, allocate a SharedData. */
+     /* Used for outgoing and incoming transports. */
 {
   ilu_Transport   ans;
   TransportParms parms;
 
   /* Allocate. */
-  ans = (ilu_Transport) ilu_malloc(sizeof(*ans));
+  ans = (ilu_Transport) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
-  parms = (TransportParms) ilu_malloc(sizeof(*parms));
-  if (parms == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*parms), NIL);
+    return NIL;
+  parms = (TransportParms) ilu_MallocE(sizeof(*parms), err);
+  if (parms == NIL) {
+    ilu_free(ans);
+    return NIL;
+  }
 
   /* Initialize. */
   parms->td_working = FALSE; /* set in Begin/EndMsg */
@@ -550,34 +586,35 @@ _inmem_NewT(ILU_ERRS((IoErrs)) * err)
   return ans;
 }
 
-/*L1_sup < trmu*/
 static          ilu_Transport
 _inmem_AcceptClient(ilu_Mooring self, ilu_string * tinfo_out,
-		  ilu_integer *dfd, ilu_Passport pp, ILU_ERRS((IoErrs)) * err)
+		    ilu_integer * dfd, ilu_Passport pp,
+		    ILU_ERRS((IoErrs)) * err)
 {
-  /* Create incoming transport.
-     Called by connection request handler. */
+  /*
+   * Create incoming transport. Called by connection request
+   * handler.
+   */
 
   MooringParms    mp = (MooringParms) self->mo_data;
   TransportParms  tp;
   ilu_Transport   ans;
-  char buf[32];
+  char            buf[32];
 
   *dfd = 0;
   _ilu_Assert((int) mp->md_locked, "_inmem_AcceptClient");
   /* signal ilu_im_brokenLocks? */
-  
+
   if (tinfo_out != NIL) {
     sprintf(buf, "inmem(sd=%p)", mp->md_sharedData);
-    *tinfo_out = _ilu_Strdup(buf);
+    *tinfo_out = ilu_StrdupE(buf, err);
     if (*tinfo_out == NIL)
-      return ILU_ERR_CONS1(no_memory, err, nbytes, 1 + strlen(buf), NIL);
+      return NIL;
   }
-  
   ans = _inmem_NewT(err);
   if (ILU_ERRNOK(*err))
     return NIL;
-  
+
   tp = (TransportParms) ans->tr_data;
   tp->td_outgoing = FALSE;
 
@@ -585,86 +622,44 @@ _inmem_AcceptClient(ilu_Mooring self, ilu_string * tinfo_out,
   tp->td_sharedData = mp->md_sharedData;
   mp->md_sharedData = NIL;
   mp->md_locked = FALSE;
-  
-  DEBUG(INMEM_DEBUG,
-	(stderr,
-	 "inmem(%p): AcceptClient => transport %p shared data %p\n",
+
+  ILU_NOTE(INMEM_DEBUG,
+	("inmem(%p): AcceptClient => transport %p shared data %p\n",
 	 self, ans, tp->td_sharedData));
-  
+
   return ans;
 }
 
 static          ilu_boolean
 _inmem_CloseMooring(ilu_Mooring self, ilu_integer * dfd,
-		  ILU_ERRS((internal)) * err)
+		    ILU_ERRS((bad_locks, broken_locks,
+			      internal)) * err)
 {
   MooringParms    mp = (MooringParms) self->mo_data;
+  ilu_Closure     c = _ilu_ClosureFromTIH(mp->md_tih);
 
-  DEBUG(INMEM_DEBUG, (stderr, "inmem(%p): CloseMooring\n", self));
-
+  ILU_NOTE(INMEM_DEBUG,
+	   ("inmem(%p): CloseMooring; req handler=%p{%p(%p)}\n",
+	    self, c, (c ? c->proc : 0), (c ? c->rock : 0)));
+  if (c) {
+    ILU_ERRS((internal, bad_param, bad_locks, broken_locks)) lerr;
+    ilu_boolean     ans;
+    ans = ilu_DoSoon(c, &lerr);
+    ILU_ERR_SWITCH(lerr) {
+      ILU_SUCCESS_CASE 0;
+      ILU_ERR_CASE3(internal, bad_locks, broken_locks) * err = lerr;
+      ILU_ERR_ELSE    ilu_Check(FALSE, err);
+    } ILU_ERR_ENDSWITCH;
+    if (!ans)
+      return FALSE;
+  }
   *dfd = 0;
-  if (mp->md_tih != NIL)
-    ilu_free(mp->md_tih);
   ilu_free(mp);
   self->mo_data = NIL;
+  ilu_free(self);
   return ILU_CLER(*err);
 }
 
-/*L1.sup < trmu; L2 unconstrained*/
-static ilu_integer _inmem_MooringDFd(ilu_Mooring self, ilu_boolean add)
-{
-  return 0;
-}
-
-/*L1, L2 unconstrained*/
-static          ilu_boolean
-_inmem_SetReqHandler(ilu_Mooring self, ilu_Server s,
-		   ilu_TransportInputHandler tih_proc,
-		   ilu_refany tih_rock,
-		   ILU_ERRS((no_memory, imp_limit, no_resources,
-			     broken_locks, internal)) * err)
-{
-  /* Called only by single threaded RT? ***********************************/
-
-  MooringParms    mp = (MooringParms) self->mo_data;
-  ilu_TIH         *tih;
-  tih = (ilu_TIH *) ilu_malloc(sizeof(*tih));
-  _ilu_Assert(mp->md_tih == NIL, "_inmem_SetReqHandler");      /* signal ilu_im_ ??? */
-  if (tih == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*tih), FALSE);
-  tih->tih_proc = tih_proc;
-  tih->tih_rock = tih_rock;
-  mp->md_tih = tih;
-  return ILU_CLER(*err);
-  
-}
-
-/*Before: L1 = {s},
-          forall conn: (L2 >= {conn.iomu}) => (L2 >= {conn.callmu});
-  After: Main Invariant holds*/
-static          ilu_boolean
-_inmem_WaitForReq(ilu_Mooring self, ilu_Server s,
-		ILU_ERRS((interrupted, broken_locks)) * err)
-{
-  /*  (Instead of waiting on FD directly.)
-   * In a multi-threaded runtime, a thread calls this to wait for
-   * the next connection request.  The server mutex is released
-   * while this thread is blocked.  Because the server mutex is held
-   * at entry, another thread cannot close the Mooring until after
-   * the impl releases it.  Returns TRUE after a connection request
-   * arrives; may also return TRUE after the Mooring's been closed.
-   */
-
-  /* Should not be used for inmem.
-     (would be called when new port created?)
-   */
-
-  _ilu_Assert(0, "_inmem_WaitForReq");
-  /* signal something */
-  return FALSE;
-  
-}
-
 
 
 /****************************************************
@@ -678,7 +673,8 @@ _inmem_WaitForReq(ilu_Mooring self, ilu_Server s,
   ***************************************************/
 
 
-/* L1_sup < trmu; L2 unconstrained */
+/* L1, L2 unconstrained */
+
 static          ilu_integer
 _inmem_FdUsage(ilu_TransportCreator self, ilu_boolean mooring)
 {
@@ -686,9 +682,10 @@ _inmem_FdUsage(ilu_TransportCreator self, ilu_boolean mooring)
 }
 
 /* Main Invariant holds */
-static ilu_Transport
-  _inmem_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
-			 ilu_integer *dfd, ilu_Passport pp, ILU_ERRS((IoErrs)) * err)
+static          ilu_Transport
+_inmem_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
+		       ilu_integer * dfd, ilu_Passport pp,
+		       ILU_ERRS((IoErrs)) * err)
 {
   /* Creates outgoing transport, makes call to create incoming transport
      and link the two.  (Will allocate buffers even if !buffer.) */
@@ -708,39 +705,19 @@ static ilu_Transport
   
   /* Create the shared data space. */
     
-  sd = (SharedData) ilu_malloc(sizeof(struct SharedData_s));
+  sd = (SharedData) ilu_MallocE(sizeof(struct SharedData_s), err);
   if (sd == NIL)  {
     _inmem_CloseTransport(ans, dfd, &lerr); /* clean up */
     ILU_HANDLED(lerr);
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*sd), NIL);
+    return NIL;
   }
   sd->sd_bufferSize = BUFSIZE; /* could also be contained in tinfo? */
   sd->sd_tih = NIL;
-  sd->sd_tihSet = FALSE;
   sd->sd_reuseHead = sd->sd_reuseTail = NIL;
-
-#if 0
-/* MJS August 23, 1995.  Don't we want these initialized to NIL,
-   to represent the absence of a request and any replies? */
-
-  sd->sd_request = NewBufferList(sd->sd_bufferSize, NIL err);
-  if (ILU_ERRNOK(*err)) {
-    _inmem_CloseTransport(ans, &lerr); /* clean up */
-    return NIL;
-  }
-  sd->sd_repliesHead = NewBufferList(sd->sd_bufferSize, NIL, err);
-  if (ILU_ERRNOK(*err)) {
-    _inmem_CloseTransport(ans, &lerr); /* clean up */
-    return NIL;
-  }
-  sd->sd_repliesTail = sd->sd_repliesHead;
-#else
   sd->sd_repliesTail = sd->sd_repliesHead = sd->sd_request = NIL;
-#endif
   
-  DEBUG(INMEM_DEBUG,
-	(stderr,
-	 "inmem: Create outgoing transport %p sharedData %p\n",
+  ILU_NOTE(INMEM_DEBUG,
+	("inmem: Create outgoing transport %p sharedData %p\n",
 	 ans, sd));
   
   tp = (TransportParms) ans->tr_data;
@@ -757,7 +734,7 @@ static ilu_Transport
 	     Call the mooring's registered conn-request handler. */
 	  mp->md_locked = TRUE;
 	  mp->md_sharedData = sd;
-	  (*mp->md_tih->tih_proc) (mp->md_tih->tih_rock);
+	  (*mp->md_tih->proc) (mp->md_tih->rock);
 	  /* No return value.  Must call mo_accept_connection. */
 	}
     }
@@ -765,17 +742,17 @@ static ilu_Transport
   return ans;
 }
 
-/*L1, L2 unconstrained*/
 static struct _ilu_Mooring_s mooringProto = {
   _inmem_MooringDFd,
   _inmem_SetReqHandler,
   _inmem_WaitForReq,
+  _inmem_AbleReqWait,
+  _inmem_AbleReqWait,
   _inmem_AcceptClient,
   _inmem_CloseMooring,
   NIL				/* data */
 };
 
-/* L1_sup < trmu; L2 unconstrained */
 static ilu_Mooring 
 _inmem_CreateMooring(ilu_TransportCreator self,
 		     ilu_TransportInfo * tinfo_out,
@@ -794,13 +771,14 @@ _inmem_CreateMooring(ilu_TransportCreator self,
   _ilu_Assert(cp->cd_mooringData == NIL, "_inmem_CreateMooring");
   /* signal something? */
   
-  ans = (ilu_Mooring) ilu_malloc(sizeof(*ans));
+  ans = (ilu_Mooring) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL) {
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
+    return NIL;
   }
-  mp = (MooringParms) ilu_malloc(sizeof(*mp));
+  mp = (MooringParms) ilu_MallocE(sizeof(*mp), err);
   if (mp == NIL) {
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*mp), NIL);
+    ilu_free(ans);
+    return NIL;
   }
   mp->md_tih = NIL;
   mp->md_sharedData = NIL;
@@ -825,17 +803,19 @@ _inmem_CreateMooring(ilu_TransportCreator self,
   /* Convey to outgoing transports. */
   cp->cd_mooringData = mp;
   
-  DEBUG(INMEM_DEBUG, (stderr, "inmem: CreateMooring %p\n", ans));
+  ILU_NOTE(INMEM_DEBUG, ("inmem: CreateMooring %p\n", ans));
 
   return ans;
   
 }
 
 /* L1, L2 unconstrained */
+
 static void 
 _inmem_CloseCreator(ilu_TransportCreator self)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
+  ilu_free(cp->cd_tinfo);
   ilu_free(cp);
   ilu_free(self);
   return;
@@ -850,6 +830,8 @@ _inmem_CloseCreator(ilu_TransportCreator self)
 static struct _ilu_TransportCreator_s creatorProto = {
   TRUE,				/* boundaried */
   TRUE,				/* reliable */
+  0,				/* tcr_holds */
+  FALSE,			/* tcr_wantClose */
   _inmem_FdUsage,
   _inmem_CreateTransport,
   _inmem_CreateMooring,
@@ -857,19 +839,18 @@ static struct _ilu_TransportCreator_s creatorProto = {
   NIL				/* data */
 };
 
-/*L1_sup < trmu*/
 ilu_TransportCreator
 _ilu_inmem_TransportCreator(ilu_TransportInfo tinfo,
 			  ILU_ERRS((no_memory, inv_objref)) * err)
 {
   ilu_TransportCreator ans;
   CreatorParms    cp;
-  cp = (CreatorParms) ilu_malloc(sizeof(*cp));
+  cp = (CreatorParms) ilu_MallocE(sizeof(*cp), err);
   if (cp == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*cp), NIL);
-  ans = (ilu_TransportCreator) ilu_malloc(sizeof(*ans));
+    return NIL;
+  ans = (ilu_TransportCreator) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
+    return NIL;
   *ans = creatorProto;
   ans->tcr_data = cp;
   /* This will be set when the tCr creates a mooring and will not
@@ -887,7 +868,49 @@ _ilu_inmem_TransportCreator(ilu_TransportInfo tinfo,
   return (ans);
 }
 
-/* Main Invariant holds */
+/*L1, L2 unconstrained*/
+
+static          ilu_ReadHeaderResultCode
+_buffer_BeginMessage(ilu_Transport self,
+		     ilu_boolean input_p,
+		     ILU_ERRS((IoErrs)) * err)
+{
+  return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcNotBoundaried,
+		       ilu_rhrc_error);
+}
+
+static          ilu_boolean
+_buffer_EndMessage(ilu_Transport self,
+		   ilu_boolean flush,
+		   ilu_Message *msgh,
+		   ILU_ERRS((IoErrs)) * err)
+{
+  return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcNotBoundaried,
+		       FALSE);
+}
+
+/*
+ * Set the transport class methods...
+ * there is only one memClass.
+ */
+
+static struct _ilu_TransportClass_s bufMemClass = {
+  ilu_FALSE,			/* boundaried */
+  ilu_TRUE,			/* reliable */
+  _inmem_CloseDFd,
+  _inmem_SetInputHandler,
+  _inmem_WaitForInput,
+  _inmem_Interrupt,
+  _inmem_AbleInpWait,
+  _inmem_AbleInpWait,
+  _buffer_BeginMessage,
+  _buffer_EndMessage,
+  _inmem_SendWholeMessage,
+  _inmem_WriteBytes,
+  _inmem_ReadBytes,
+  _inmem_CloseTransport
+};
+
 ilu_Transport
 _ilu_BufferTransport_Create (ilu_cardinal size, ilu_bytes buffer,
 			     ILU_ERRS((IoErrs)) * err)
@@ -900,27 +923,26 @@ _ilu_BufferTransport_Create (ilu_cardinal size, ilu_bytes buffer,
   ilu_ReadHeaderResultCode rhrc;
 
   t = _inmem_NewT(err);
+  t->tr_class = &bufMemClass;
   if (ILU_ERRNOK(*err))
     return NIL;
   /* Exposed buffers are NIL until message begun. */
   
   /* Create the shared data space. */
     
-  sd = (SharedData) ilu_malloc(sizeof(struct SharedData_s));
+  sd = (SharedData) ilu_MallocE(sizeof(*sd), err);
   if (sd == NIL)  {
     _inmem_CloseTransport(t, &dfd, &lerr); /* clean up */
     ILU_HANDLED(lerr);
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*sd), NIL);
+    return NIL;
   }
   sd->sd_bufferSize = size; /* could also be contained in tinfo? */
   sd->sd_tih = NIL;
-  sd->sd_tihSet = FALSE;
   sd->sd_reuseHead = sd->sd_reuseTail = NIL;
   sd->sd_repliesTail = sd->sd_repliesHead = sd->sd_request = NIL;
   
-  DEBUG(INMEM_DEBUG,
-	(stderr,
-	 "_ilu_BufferTransport: Create outgoing inmem transport %p sharedData %p\n",
+  ILU_NOTE(INMEM_DEBUG,
+	("_ilu_BufferTransport: Create outgoing inmem transport %p sharedData %p\n",
 	 t, sd));
   
   tp = (TransportParms) t->tr_data;
@@ -929,16 +951,14 @@ _ilu_BufferTransport_Create (ilu_cardinal size, ilu_bytes buffer,
   tp->td_working = FALSE;
   tp->td_current = NIL;
 
-  if (buffer != NIL)
-    {
-      sd->sd_request = NewBufferList (size, buffer, err);
-      if (ILU_ERRNOK(*err))
-	{
-	  _inmem_CloseTransport (t, &dfd, &lerr);
-	  return (NIL);
-	}
-      sd->sd_request->offset = size;
+  if (buffer != NIL) {
+    sd->sd_request = NewBufferList(size, buffer, err);
+    if (ILU_ERRNOK(*err)) {
+      _inmem_CloseTransport(t, &dfd, &lerr);
+      return (NIL);
     }
+    sd->sd_request->offset = size;
+  }
 
   rhrc = _inmem_BeginMessage (t, (buffer != NIL), err);
   
@@ -956,38 +976,42 @@ _ilu_BufferTransport_Create (ilu_cardinal size, ilu_bytes buffer,
   return t;
 }
 
-/*L1, l2?*/
+/*L1, L2 unconstrained*/
 void
-_ilu_BufferTransport_Destroy (ilu_Transport self, ilu_cardinal * size,
-			      ilu_bytes * buffer, ILU_ERRS((IoErrs)) * err)
+_ilu_BufferTransport_Destroy(ilu_Transport self, ilu_cardinal * size,
+			     ilu_bytes * buffer,
+			     ILU_ERRS((IoErrs)) * err)
 {
-  TransportParms  tp;
-  ilu_boolean	input;
-  BufferList	current;
-  ilu_integer	dfd;
+  TransportParms  tp = (TransportParms) self->tr_data;
+  SharedData	sd = NIL;
+  ilu_boolean     input;
+  BufferList      current;
+  ilu_integer     dfd;
 
-  tp = (TransportParms) self->tr_data;
+  if (!tp->td_outgoing)
+    sd = ((TransportParms) self->tr_data)->td_sharedData;
   input = tp->td_input;
   current = tp->td_current;
 
-  _inmem_EndMessage(self, TRUE, NIL, err);
+  _inmem_EndMessage(self, TRUE, NIL, err);	/* XXX locking bug! */
   if (ILU_ERRNOK(*err))
     return;
 
-  if (current != NIL)
-    {
-      if (size != NIL)
-	*size = current->offset;
-      if (buffer != NIL) {
-	*buffer = current->buffer;
-	current->buffer = NIL;
-      }
+  if (current != NIL) {
+    if (size != NIL)
+      *size = current->offset;
+    if (buffer != NIL) {
+      *buffer = current->buffer;
+      current->buffer = NIL;
     }
+  }
 
-  /* we need to close this, as this transport is always td_outgoing == FALSE */
-  CloseSharedData(tp->td_sharedData);
-  tp->td_sharedData = NIL;
-  tp->td_current = NIL;
+  _inmem_CloseTransport(self, &dfd, err);
 
-  transport_close(self, &dfd, err);
+  /*
+   * We need to close this, as this transport is always (td_outgoing
+   * == FALSE).
+   */
+  if (sd != NIL)
+    CloseSharedData(sd);
 }
