@@ -14,12 +14,12 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: method.h,v 1.5 1994/01/20 05:02:17 janssen Exp $ */
+/* $Id: method.h,v 1.6 1997/06/12 00:59:25 janssen Exp $ */
 /* Last tweaked by Mike Spreitzer September 21, 1993 11:38 am PDT */
 
 #define method_id(method)			((method)->me_id)
 #define method_name(method)			((method)->me_name)
-#define method_stub_proc(method)		((method)->me_stubproc)
+#define method_stub_proc(method,lang)		((method)->me_stubprocs[(lang)])
 #define method_asynchronous(method)		((method)->me_asynchronous)
 #define method_functional(method)		((method)->me_cacheable)
 #define method_exception_count(method)		((method)->me_exceptionCount)
