@@ -14,10 +14,20 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: mooring.h,v 1.10 1995/09/19 23:01:06 spreitze Exp $ */
-/* Last edited by Mike Spreitzer September 19, 1995 12:42 pm PDT */
+/* $Id: mooring.h,v 1.15 1997/08/29 21:09:35 janssen Exp $ */
+/* Last edited by Mike Spreitzer November 20, 1996 9:16 am PST */
+
+
+#define mooring_accept(m,peerinfo,dfd,err)\
+	((*((m)->mo_accept_connection))(m,peerinfo,dfd,err)))
+
+#define mooring_data(m)			((m)->mo_data)
+#define mooring_close(m,dfd,err)	((*((m)->mo_close))(m,dfd,err))
+#define mooring_disableWait(m,err)	((*((m)->mo_disableWait))(m,err))
+#define mooring_enableWait(m,err)	((*((m)->mo_enableWait))(m,err))
+#define mooring_set_req_handler(m,tih,err)	((*((m)->mo_set_req_handler))((m),(tih),(err)))
+#define mooring_accept_connection(m,peerinfo,dfd,passport,err) ((*((m)->mo_accept_connection))((m),(peerinfo),(dfd),(passport),(err)))
+#define mooring_dfd(m,add)		((*((m)->mo_dfd))((m),(add)))
+
 
 
-#define mooring_accept(mooring,peerinfo,dfd,err)	((*((mooring)->mo_accept_connection))(mooring,peerinfo,dfd,err)))
-#define mooring_data(mooring)		((mooring)->mo_data)
-#define mooring_close(mooring,dfd,err)	((*((mooring)->mo_close))(mooring,dfd,err))
