@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: newtcp.c,v 1.59 1996/07/11 07:23:28 spreitze Exp $ */
-/* Last edited by Mike Spreitzer July 10, 1996 11:18 pm PDT */
+/* $Id: newtcp.c,v 1.75 1997/11/21 22:23:34 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 25, 1997 11:31 pm PDT */
 
 #include "iluntrnl.h"
 
@@ -26,12 +26,14 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #include "oscalls.h"
 
+/* L1, L2 unconstrained */
+
 #ifdef XEROX_FIREWALL_HANDLING
 #include "xerox_firewall.c"
 #define connect(a,b,c)	xerox_firewall_connect((a),(b),(c))
 #endif /* XEROX_FIREWALL_HANDLING */
 
-#if (defined(WIN32) || defined(WIN16))
+#if (defined(WIN32) || defined(WIN16) || defined( macintosh ))
 
 static int set_doer_non_blocking (int fd)
 {
@@ -77,6 +79,31 @@ static int set_doer_non_blocking (int fd)
 
 #endif
 
+/* now define macro for setting the CLOSE-ON-EXEC flag on a fd */
+#if (defined(_IS_POSIX) && !defined(WIN32))
+#include <fcntl.h>
+#include <unistd.h>
+static int set_fd_flag (int fd, int flag, ilu_boolean set)
+{
+  int             flags = fcntl(fd, F_GETFD, 0);
+  if (flags < 0)
+    return flags;
+  if (set)
+    flags |= flag;
+  else
+    flags &= ~flag;    
+  return fcntl(fd, F_SETFD, flags);
+}
+#define set_close_on_exec(fd)		set_fd_flag((fd), FD_CLOEXEC, ilu_TRUE)
+#else
+#ifdef WIN32
+/* return 0 to ignore errors because we expect this to fail on Windows 95 */
+#define set_close_on_exec(fd) (SetHandleInformation((HANDLE) fd, HANDLE_FLAG_INHERIT, 0), 0)
+#else
+#define set_close_on_exec(fd)		0
+#endif /* Win32 */
+#endif /* (defined(_IS_POSIX) && !defined(WIN32)) */
+
 #ifdef ILU_NEEDS_NETINET_TCP_H_FOR_NODELAY
 #include <netinet/tcp.h>
 #endif
@@ -93,7 +120,7 @@ typedef unsigned short ilutcpport_t;
 /* WIN port needs arg of htons() to be an unsigned short */
 
 typedef struct {
-  /* L1, L2, Main unconstrained */
+  /* L1, L2 unconstrained */
 
   struct sockaddr_in addr;
   /*
@@ -105,31 +132,82 @@ typedef struct {
 
   ilu_boolean     name_given;	/* did tinfo give name or address? */
   ilu_boolean     defaulted;	/* did tinfo give null name/address? */
+  ilu_cardinal    buffersize;	/* did tinfo give buffersize? */
 }              *CreatorParms;
 
 typedef struct {
-  /* L1, L2, Main unconstrained */
+  /* L1, L2 unconstrained */
 
-  int             fd;		/* listening socket */
-  ilu_boolean     buff_p;	/* buffering wanted */
-  ilu_TIH        *tih;		/* !=NIL when req handler reg'd */
+  int		fd;		/* listening socket */
+  ilu_cardinal	buffersize;	/* buffering wanted */
+  ilu_TIH       tih;		/* the reg'd req handler, if any */
 }              *MooringParms;
+/* (tih) owned by client, not mooring. */
 
 typedef struct {
-  /* L1, L2, Main unconstrained */
+  /* L1, L2 unconstrained */
 
-  int             fd;		/* I/O socket */
+  int             fd;		/* file descriptor for socket */
   ilu_cardinal    inSize;	/* size of input buffer */
-  ilu_boolean     inpReg;	/* got an inp handler reg'd? */
-  ilu_TIH        *tih;		/* inpReg => tih != NIL */
+  ilu_TIH         tih;		/* the reg'd inp handler, if any */
 
 }              *TCPParms;
-/* What goes in the data field of a TCP ilu_Transport */
+/*
+ * What goes in the data field of a TCP ilu_Transport. (tih) owned by
+ * client, not transport.
+ */
 
 #define BUFFERSIZE	4096
 #define DIRECT_THRESHOLD 1024
 
-/*L1, L2, Main unconstrained*/
+static ilu_cardinal DefaultBuffersize = BUFFERSIZE;
+
+/*********************************************************************/
+/*********************************************************************/
+/*********   Some debugging variables   ******************************/
+/*********************************************************************/
+/*********************************************************************/
+
+#ifdef ENABLE_DEBUGGING
+
+static ilu_cardinal _ilutcp_BytesSent = 0;
+static ilu_cardinal _ilutcp_BytesReceived = 0;
+static ilu_cardinal _ilutcp_MooringsCreated = 0;
+static ilu_cardinal _ilutcp_ConnectionsReceived = 0;
+static ilu_cardinal _ilutcp_ConnectionsCreated = 0;
+static ilu_cardinal _ilutcp_CurrentConnections = 0;
+static ilu_cardinal _ilutcp_MaxSimultaneousConnections = 0;
+
+void
+  ilu_tcp_InitializeStats (void)
+{
+  _ilutcp_BytesSent = 0;
+  _ilutcp_BytesReceived = 0;
+  _ilutcp_MooringsCreated = 0;
+  _ilutcp_ConnectionsReceived = 0;
+  _ilutcp_ConnectionsCreated = 0;
+  _ilutcp_MaxSimultaneousConnections = _ilutcp_CurrentConnections;  
+}
+
+void
+  ilu_tcp_GetStats (ilu_cardinal *bytes_sent,
+		    ilu_cardinal *bytes_received,
+		    ilu_cardinal *moorings_created,
+		    ilu_cardinal *connections_received,
+		    ilu_cardinal *connections_created,
+		    ilu_cardinal *current_connections,
+		    ilu_cardinal *max_connections)
+{
+  *bytes_sent = _ilutcp_BytesSent;
+  *bytes_received = _ilutcp_BytesReceived;
+  *moorings_created = _ilutcp_MooringsCreated;
+  *connections_received = _ilutcp_ConnectionsReceived;
+  *connections_created = _ilutcp_ConnectionsCreated;
+  *current_connections = _ilutcp_CurrentConnections;
+  *max_connections = _ilutcp_MaxSimultaneousConnections;
+}
+
+#endif /* ENABLE_DEBUGGING */
 
 /**********************************************************************
 ***********************************************************************
@@ -139,17 +217,25 @@ typedef struct {
 ***********************************************************************
 **********************************************************************/
 
-/*L1, L2 unconstrained*/
+ilu_cardinal
+  ilu_tcp_SetDefaultBuffersize (ilu_cardinal buffersize)
+{
+  ilu_cardinal oldbuffersize = DefaultBuffersize;
+  DefaultBuffersize = buffersize;
+  return oldbuffersize;
+}
 
+/*L1.sup < trmu; L2 unconstrained*/
 static          CreatorParms
 _tcp_InterpretInfo(ilu_string tinfo,
 		   ILU_ERRS((no_memory, inv_objref)) * err)
 {
   CreatorParms    cp;
   char            hostname[1000];
-  long unsigned   port;
+  long unsigned   port, buffersize;
 
-  if ((sscanf(tinfo, "tcp_%999[^_]_%lu", hostname, &port)) != 2)
+  if (((sscanf(tinfo, "tcp_%999[^_]_%lu_%lu", hostname, &port, &buffersize)) != 3) &&
+      (buffersize = DefaultBuffersize, sscanf(tinfo, "tcp_%999[^_]_%lu", hostname, &port) != 2))
     return (ILU_ERR_CONS1(inv_objref, err, minor, ilu_iom_ts, NIL));
   cp = (CreatorParms) ilu_MallocE(sizeof(*cp), err);
   if (cp == NIL)
@@ -161,6 +247,7 @@ _tcp_InterpretInfo(ilu_string tinfo,
   cp->name_given = (cp->addr.sin_addr.s_addr == -1);
   cp->defaulted = (cp->addr.sin_addr.s_addr == 0 ||
 		   strcmp(hostname, "localhost") == 0);
+  cp->buffersize = buffersize;
   if (cp->defaulted) {
     if (!ilu_EnterMutex(ilu_trmu, err))
       goto faild;
@@ -190,8 +277,8 @@ _tcp_InterpretInfo(ilu_string tinfo,
 	memcpy((ilu_string) & cp->addr.sin_addr, hp->h_addr,
 	       hp->h_length);
       if (cp->addr.sin_addr.s_addr == -1) {
-	DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	      (stderr, "tcp:  Invalid host name (%s).\n",
+	ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	      ("tcp:  Invalid host name (%s).\n",
 	       hostname));
 	ilu_free(cp);
 	return ILU_ERR_CONS1(inv_objref, err, minor, ilu_iom_ts, NIL);
@@ -210,15 +297,7 @@ faild:
   return NIL;
 }
 
-/*Main Invariant holds; L2 no forther constrained*/
-static          ilu_boolean
-_tcp_Interrupt(ilu_Transport self, ILU_ERRS((bad_param)) * err)
-{
-  TCPParms        p = (TCPParms) self->tr_data;
-  return _ilu_InterruptFD(p->fd, err);
-}
-
-/* L2, Main, unconstrained; L1_sup < trmu */
+/* L1, L2 unconstrained */
 
 static          ilu_integer
 _tcp_FdUsage(ilu_TransportCreator self, ilu_boolean mooring)
@@ -232,91 +311,63 @@ _tcp_CloseDFd(ilu_Transport self)
   return (1);
 }
 
-/*Main Invariant holds; L2 otherwise unconstrained*/
-static void 
-CallInpHandler(int fd, ilu_private rock)
-{
-  ilu_Transport   self = (ilu_Transport) rock;
-  TCPParms        p = (TCPParms) self->tr_data;
-  ilu_TIH        *tp = p->tih;
-  (*tp->tih_proc) (tp->tih_rock);
-  return;
-}
-
-/*tih => Main Invariant holds && L2 disjoint {conn's iomu, callmu}*/
+/* L1.sup < trmu; L2 >= {xmu, ymu} */
 static          ilu_boolean
-SetInputHandler(ilu_Transport self, ilu_TransportInputHandler tih,
-		ilu_refany tih_rock,
+SetInputHandler(ilu_Transport self, ilu_TIH tih,
 		ILU_ERRS((no_memory, internal, no_resources)) * err)
 {
   TCPParms        p = (TCPParms) self->tr_data;
-  ilu_TIH        *tp = p->tih;
-  if (p->inpReg)
+  if (p->tih)
     ilu_UnregisterInputSource(p->fd);
-  p->inpReg = FALSE;
-  if (tih != NULLFN) {
-    if (tp == NIL) {
-      tp = (ilu_TIH *) ilu_malloc(sizeof(*tp));
-      if (tp == NIL)
-	return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*tp), FALSE);
-      p->tih = tp;
-    }
-    tp->tih_proc = tih;
-    tp->tih_rock = tih_rock;
-    if (!ilu_RegisterInputSource(p->fd, CallInpHandler, self))
+  p->tih = NIL;
+  if (tih) {
+    if (!ilu_RegisterInputSource(p->fd, _ilu_InvokeTIH, tih))
       return ILU_ERR_CONS1(no_resources, err, minor, ilu_nrm_mlreg,
 			   FALSE);
-    p->inpReg = TRUE;
+    p->tih = tih;
   }
   return ILU_CLER(*err);
 }
 
-/*Main Invariant holds*/
-/*L2 >= {conn's iomu}*/
-
+/* Main Invariant holds; L2 >= {ymu} */
 static          ilu_boolean
-_tcp_WaitForInput(ilu_Transport t, ilu_FineTime * limit,
-		  ILU_ERRS((interrupted)) * err)
+_tcp_WaitForInput(ilu_Transport t, int *disabled, ilu_FineTime * limit,
+		  ILU_ERRS((broken_locks, interrupted)) * err)
 {
-  TCPParms p = (TCPParms) t->tr_data;
+  TCPParms        p = (TCPParms) t->tr_data;
   ilu_boolean     sure;
-  ILU_CLER(*err);
-  if (t->tr_inBuff != NIL && t->tr_inNext < t->tr_inLimit)
-    return ilu_TRUE;
+  if ((t->tr_inBuff != NIL) && (t->tr_inNext < t->tr_inLimit))
+    return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcInputSkipsBuff,
+			 FALSE);
+  *disabled = 0;		/* could do better here */
   _ilu_WaitForInputOnFD(p->fd, &sure, limit, err);
   return ILU_ERROK(*err);
 }
 
+/*Main Invariant holds; L2 no forther constrained*/
 static          ilu_boolean
-_tcp_HasHiddenInput(ilu_Transport self, ILU_ERRS((IoErrs)) * err)
-{
-  ILU_CLER(*err);
-  return FALSE;
-}
-
-static          ilu_boolean
-Write(ilu_Transport self, ilu_bytes buf, ilu_cardinal size,
-      ILU_ERRS((IoErrs)) * err)
+_tcp_InterruptST(ilu_Transport self, ILU_ERRS((bad_param)) * err)
 {
   TCPParms        p = (TCPParms) self->tr_data;
-  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	(stderr, "tcp(%p):  writing %lu bytes from %p to FD %d.\n",
-	 self, size, buf, p->fd));
-
-#ifdef ENABLE_DEBUGGING
-
-  if (((_ilu_DebugLevel & PACKET_DEBUG) != 0) &&
-      (size > 0))
-    _ilu_debug_DumpPacket(buf, size, "outgoing TCP");
-
-#endif /* ENABLE_DEBUGGING */
-
-  if (!_ilu_SockWrite(p->fd, buf, size, err))
-    return FALSE;
-
-  return TRUE;
+  return _ilu_InterruptFD(p->fd, err);
 }
 
+/* L1 >= {cmu}; L1.sup < trmu; L2 unconstrained */
+
+static          ilu_boolean
+_tcp_DisableWait(ilu_Transport self, ILU_ERRS((bad_param)) * err)
+{
+  return _ilu_DisableFDWaits(err);
+}
+
+static          ilu_boolean
+_tcp_EnableWait(ilu_Transport self, ILU_ERRS((bad_param)) * err)
+{
+  return _ilu_EnableFDWaits(err);
+}
+
+/*L1, L2 unconstrained*/
+
 static          ilu_ReadHeaderResultCode
 _tcp_BeginMessage(ilu_Transport self,
 		  ilu_boolean input_p,
@@ -344,6 +395,35 @@ _tcp_SendWholeMessage(ilu_Transport self, ilu_Message * msgh,
 		       ilu_FALSE);
 }
 
+/*Main Invariant holds; L2 >= {xmu}*/
+
+static          ilu_boolean
+Write(ilu_Transport self, ilu_bytes buf, ilu_cardinal size,
+      ILU_ERRS((IoErrs)) * err)
+{
+  TCPParms        p = (TCPParms) self->tr_data;
+  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	("tcp(%p):  writing %lu bytes from %p to FD %d.\n",
+	 self, size, buf, p->fd));
+
+#ifdef ENABLE_DEBUGGING
+
+  if (((ilu_DebugLevel & PACKET_DEBUG) != 0) &&
+      (size > 0))
+    _ilu_debug_DumpPacket(buf, size, "outgoing TCP");
+
+#endif /* ENABLE_DEBUGGING */
+
+  if (!_ilu_SockWrite(p->fd, buf, size, err))
+    return FALSE;
+
+#ifdef ENABLE_DEBUGGING
+  _ilutcp_BytesSent += size;
+#endif /* ENABLE_DEBUGGING */
+
+  return TRUE;
+}
+
 static          ilu_boolean
 _tcp_WriteBytes(ilu_Transport self, ilu_bytes b,
 		ilu_cardinal bufferSize,
@@ -351,12 +431,17 @@ _tcp_WriteBytes(ilu_Transport self, ilu_bytes b,
 		ILU_ERRS((IoErrs)) * err)
 {
   ilu_cardinal    rem = self->tr_outLimit - self->tr_outNext;
-  ilu_boolean direct;
 
-  direct = (bufferSize >= DIRECT_THRESHOLD || flush && bufferSize > rem
-	    || self->tr_outBuff == NIL);
+  if ((bufferSize >= DIRECT_THRESHOLD)
+      /* big enough that copy would take longer than write() system call */
 
-  if (direct) {
+      || (flush && (bufferSize > rem))
+      || (!flush && (bufferSize > (rem + self->tr_outLimit - 16)))
+      /* going to need two writes anyway, so avoid copy */
+
+      || (self->tr_outBuff == NIL)
+      /* no internal buffer to copy into, so can't copy */
+      ) {
     if (self->tr_outNext > 0) {
       if (!Write(self, self->tr_outBuff, self->tr_outNext, err))
 	return FALSE;
@@ -364,13 +449,14 @@ _tcp_WriteBytes(ilu_Transport self, ilu_bytes b,
     }
     if (!Write(self, b, bufferSize, err))
       return FALSE;
+    /* empty exposed buffer, so at least 16 bytes */
   } else {
     ilu_cardinal    l1 = MIN(rem, bufferSize);
     memcpy((void*)(self->tr_outBuff + self->tr_outNext), (void*)b, l1);
     b += l1;
     bufferSize -= l1;
     self->tr_outNext += l1;
-    if (flush || self->tr_outNext == self->tr_outLimit) {
+    if (flush || ((self->tr_outNext + 16) > self->tr_outLimit)) {
       if (!Write(self, self->tr_outBuff, self->tr_outNext, err))
 	return FALSE;
       self->tr_outNext = 0;
@@ -380,9 +466,14 @@ _tcp_WriteBytes(ilu_Transport self, ilu_bytes b,
       self->tr_outNext += bufferSize;
     }
   }
+  _ilu_Assert(((self->tr_outBuff == ILU_NIL)||
+	       ((self->tr_outLimit - self->tr_outNext) >= 16)),
+	      "_tcp_WriteBytes returns with fewer than 16 free bytes in buffer");
   return ILU_CLER(*err);
 }
 
+/*Main Invariant holds; L2 >= {xmu, ymu}*/
+
 static          ilu_cardinal
 _tcp_ReadBytes(ilu_Transport self,
 	       ilu_bytes buffer,
@@ -391,61 +482,55 @@ _tcp_ReadBytes(ilu_Transport self,
 	       ILU_ERRS((IoErrs)) * err)
 {
   TCPParms        p = (TCPParms) self->tr_data;
-  ilu_cardinal	  toread;
-  ilu_cardinal    lread;
-  ilu_bytes	  actbuff;
-  ilu_boolean     use_internal = ((buffer == NIL)
-				  || (len < DIRECT_THRESHOLD));
+  ilu_cardinal    lread, ans;
+  ilu_bytes       actbuff;
+  ilu_boolean     use_internal;
   rpt->tr_eom = rpt->tr_eof = FALSE;
   if (self->tr_inBuff != NIL && self->tr_inNext != self->tr_inLimit)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcInputSkipsBuff,
 			 0);
+  use_internal = (!buffer) || (self->tr_inBuff && (len < p->inSize));
   if (use_internal) {
     actbuff = self->tr_inBuff;
-    toread = p->inSize;
-    lread = _ilu_NbSockRead(p->fd, actbuff, toread, rpt, err);
+    lread = _ilu_NbSockRead(p->fd, actbuff, p->inSize, rpt, err);
     self->tr_inLimit = lread;
-    self->tr_inNext = 0;
+    if (ILU_ERROK(*err) && buffer && lread) {
+      ilu_cardinal    tomove = ans = MIN(lread, len);
+      memcpy((void *) buffer, (void *) self->tr_inBuff, tomove);
+      self->tr_inNext = tomove;
+    } else {
+      ans = lread;
+      self->tr_inNext = 0;
+    }
   } else {
     actbuff = buffer;
-    toread = len;
-    lread = _ilu_NbSockRead(p->fd, actbuff, toread, rpt, err);
+    ans = lread = _ilu_NbSockRead(p->fd, actbuff, len, rpt, err);
   }
-  if (ILU_ERRNOK(*err)) {
 
 #ifdef ENABLE_DEBUGGING
-    if (_ilu_DebugLevel & TCP_DEBUG)
+  if (ILU_ERRNOK(*err)) {
+    if (ilu_DebugLevel & TCP_DEBUG)
       ILU_ERRPRINTF("tcp(%p): ReadBytes got error %s from FD %d!\n",
 		    self, ILU_ERR_NAME(*err), p->fd);
-#endif /* ENABLE_DEBUGGING */
-
   } else {
-    DEBUG(TCP_DEBUG,
-	  (stderr,
-	   "tcp(%p): read %u bytes from FD %d, eof=%s\n",
+    ILU_NOTE(TCP_DEBUG,
+	  ("tcp(%p): read %u bytes from FD %d, eof=%s\n",
 	   self, lread, p->fd, rpt->tr_eof ? "T" : "F"));
-
-#ifdef ENABLE_DEBUGGING
-    if (((_ilu_DebugLevel & PACKET_DEBUG) != 0) &&
+    if (((ilu_DebugLevel & PACKET_DEBUG) != 0) &&
 	(lread > 0))
       _ilu_debug_DumpPacket(actbuff, lread, "incoming TCP");
-#endif /* ENABLE_DEBUGGING */
-
-    if (use_internal) {
-      toread = MIN(lread,len);
-      memcpy ((void *) buffer, (void *) self->tr_inBuff, toread);
-      self->tr_inNext += toread;
-      return toread;
-    }
+    _ilutcp_BytesReceived += lread;
   }
-  return lread;
+#endif				/* ENABLE_DEBUGGING */
+
+  return ans;
 }
 
 
 #if (defined (WIN32) || defined (WIN16))
-/* **************************************************** */
-/* returns 1 is the fd is ready for writing, else 0 */
 
+/* returns 1 if the fd is ready for writing, else 0 */
+/*L1, L2 unconstrained*/
 static int _tcp_writeable_fd(int fd) {
 
 	fd_set write_set;
@@ -459,8 +544,6 @@ static int _tcp_writeable_fd(int fd) {
 #endif
 
 
-/*L1, L2, Main unconstrained*/
-
 /*Main Invariant holds*/
 static ilu_boolean 
 _tcp_Connect(ilu_Transport self,
@@ -481,21 +564,21 @@ _tcp_Connect(ilu_Transport self,
 
   port = ntohs(sinaddr.sin_port);
   
-  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	(stderr, "tcp(%p):  connecting to host %s, port %u...\n",
+  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	("tcp(%p):  connecting to host %s, port %u...\n",
 	 self, inet_ntoa(sinaddr.sin_addr), port));
 
   if (OS_SOCKINV(fd = socket(AF_INET, SOCK_STREAM, 0))) {
     err1 = sockerrno;
-    DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	  (stderr, "tcp(%p):  socket call failed:  %s.\n",
+    ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	  ("tcp(%p):  socket call failed:  %s.\n",
 	   self, strerror(err1)));
     switch (err1) {
     case SOCKERRID(ACCES):		/* permission error */
       ILU_ERR_CONS0(no_permission, err, 0);
       break;
 
-#if !(defined(WIN32) || defined(WIN16))
+#if !(defined(WIN32) || defined(WIN16)|| defined( macintosh ))
     case SOCKERRID(NFILE):
       ILU_ERR_CONS1(no_resources, err, minor, ilu_nrm_ENFILE, 0);
       break;
@@ -554,8 +637,8 @@ _tcp_Connect(ilu_Transport self,
 #endif				/* IPPROTO_TCP */
 
     if (cp->defaulted) {
-      DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr, "tcp(%p):  Can't connect to defaulted host.\n",
+      ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	    ("tcp(%p):  Can't connect to defaulted host.\n",
 	     self));
       ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_networkAddr, 0);
       status = FALSE;
@@ -563,9 +646,8 @@ _tcp_Connect(ilu_Transport self,
       /* firewall code doesn't work with non-blocking connect */
     } else if (OS_SOCKERR(set_doer_non_blocking(fd))) {
       err1 = sockerrno;
-      DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr,
-	     "tcp(%p):  Failed to set socket (FD %d, host %s)"
+      ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	    ("tcp(%p):  Failed to set socket (FD %d, host %s)"
 	     " non-blocking, error \"%s\".\n",
 	     self, fd, inet_ntoa(sinaddr.sin_addr),
 	     strerror(err1)));
@@ -577,19 +659,23 @@ _tcp_Connect(ilu_Transport self,
       connres = connect(fd, (struct sockaddr *) & sinaddr,
 			sizeof(sinaddr));
       if (!OS_SOCKERR(connres)) {
-	DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	      (stderr,
-	       "tcp(%p):  connected to %s:%u on FD %d\n",
+	ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	      ("tcp(%p):  connected to %s:%u on FD %d\n",
 	       self, inet_ntoa(sinaddr.sin_addr), port, fd));
 	p->fd = fd;
 	status = TRUE;
+	if (OS_SOCKERR(set_close_on_exec(fd))) {
+	  int             err2 = sockerrno;
+	  ilu_DebugPrintf("tcp(%p):  Unable to set outgoing connection "
+			  "(FD %d) close-on-exec: %s (ignoring error)\n",
+			  self, fd, strerror(err2));
+	}
 #ifdef XEROX_FIREWALL_HANDLING
 	/* do non-blocking after the connect */
 	if (OS_SOCKERR(set_doer_non_blocking(fd))) {
 	  err1 = sockerrno;
-	  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-		(stderr,
-		 "tcp(%p):  Failed to set socket (FD %d, host %s)"
+	  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+		("tcp(%p):  Failed to set socket (FD %d, host %s)"
 		 " non-blocking.  Error \"%s\".\n",
 		 self, fd, inet_ntoa(sinaddr.sin_addr),
 		 strerror(err1)));
@@ -623,18 +709,16 @@ _tcp_Connect(ilu_Transport self,
 	    getpeername(fd, &peername, &pnlen) == 0
 #endif
 	  ) {
-	  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-		(stderr,
-		 "tcp(%p):  eventually connected to %s:%u on FD %d\n",
+	  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+		("tcp(%p):  eventually connected to %s:%u on FD %d\n",
 		 self, inet_ntoa(sinaddr.sin_addr), port, fd));
 	  p->fd = fd;
 	  status = TRUE;
 	} else {
-	  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-		(stderr,
-		 "tcp(%p): connect FD %d to %s:%u failed "
-		 "(no meaningful errno available).\n",
-		 self, fd, inet_ntoa(sinaddr.sin_addr), port));
+	  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+		("tcp(%p): connect FD %d to %s:%u failed "
+		 "(no meaningful errno available? errno = %d).\n",
+		 self, fd, inet_ntoa(sinaddr.sin_addr), port, sockerrno));
 	  OS_SOCKLOSE(fd);
 	  ILU_ERR_CONS1(comm_failure, err, minor,
 			ilu_cfm_connect_failed, 0);
@@ -642,8 +726,8 @@ _tcp_Connect(ilu_Transport self,
 	}
       } else {
 	int erri = sockerrno;
-	DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	      (stderr, "tcp(%p): connect FD %d to %s:%u failed,"
+	ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	      ("tcp(%p): connect FD %d to %s:%u failed,"
 	       " error %d (%s)\n",
 	       self, fd, inet_ntoa(sinaddr.sin_addr),
 	       port, erri, strerror(erri)));
@@ -686,6 +770,7 @@ _tcp_Connect(ilu_Transport self,
 }
 
 /*L1, L2 unconstrained */
+
 static ilu_boolean 
 _tcp_Close(ilu_Transport self, ilu_integer * dfd,
 	   ILU_ERRS((internal)) * err);
@@ -696,7 +781,9 @@ static struct _ilu_TransportClass_s tcpClass = {
   _tcp_CloseDFd,
   SetInputHandler,
   _tcp_WaitForInput,
-  _tcp_Interrupt,
+  _tcp_InterruptST,
+  _tcp_DisableWait,
+  _tcp_EnableWait,
   _tcp_BeginMessage,
   _tcp_EndMessage,
   _tcp_SendWholeMessage,
@@ -705,42 +792,39 @@ static struct _ilu_TransportClass_s tcpClass = {
   _tcp_Close
 };
 
-/*L1, L2 unconstrained */
 static          ilu_Transport
-NewT(ilu_boolean buffer, ILU_ERRS((IoErrs)) * err)
+NewT(ilu_cardinal buffersize, ILU_ERRS((IoErrs)) * err)
 {
   ilu_Transport   ans;
-  TCPParms parms;
-  ans = (ilu_Transport) ilu_malloc(sizeof(*ans));
+  TCPParms        parms;
+  ans = (ilu_Transport) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
-  parms = (TCPParms)ilu_malloc(sizeof(*parms));
+    return NIL;
+  parms = (TCPParms) ilu_MallocE(sizeof(*parms), err);
   if (parms == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*parms), NIL);
+    return NIL;
   parms->fd = -1;
-  parms->inSize = BUFFERSIZE;
-  parms->inpReg = FALSE;
+  parms->inSize = buffersize;
   parms->tih = NIL;
   ans->tr_class = &tcpClass;
   ans->tr_data = parms;
-  if (buffer) {
-    ans->tr_inBuff = (ilu_bytes) malloc(parms->inSize);
-    ans->tr_outBuff = (ilu_bytes) malloc(BUFFERSIZE);
+  if (buffersize > 0) {
+    ans->tr_inBuff = (ilu_bytes) ilu_MallocE(parms->inSize, err);
+    if (ans->tr_inBuff)
+      ans->tr_outBuff = (ilu_bytes) ilu_MallocE(buffersize, err);
     if (ans->tr_inBuff == NIL || ans->tr_outBuff == NIL) {
       ilu_Error       lerr;
       ilu_integer     cdfd;
       _tcp_Close(ans, &cdfd, &lerr);
       ILU_HANDLED(lerr);
-      return ILU_ERR_CONS1(no_memory, err, nbytes,
-	      (ans->tr_inBuff == NIL) ? parms->inSize : BUFFERSIZE,
-			   ans);
+      return NIL;
     }
     ans->tr_inLimit = 0;
   } else {
     ans->tr_inBuff = ans->tr_outBuff = NIL;
-    ans->tr_inLimit = BUFFERSIZE;	/* have no mercy on bugs */
+    ans->tr_inLimit = buffersize;	/* have no mercy on bugs */
   }
-  ans->tr_outLimit = BUFFERSIZE;
+  ans->tr_outLimit = buffersize;
   ans->tr_inNext = ans->tr_outNext = 0;
   ILU_CLER(*err);
   return ans;
@@ -755,12 +839,18 @@ static ilu_Transport
   CreatorParms    cp = (CreatorParms) self->tcr_data;
   ilu_Transport   ans;
   *dfd = 0;
-  ans = NewT(buffer, err);
+  ans = NewT(buffer ? cp->buffersize : 0, err);
   if (ILU_ERRNOK(*err))
     return NIL;
-  if (_tcp_Connect(ans, cp, err))
+  if (_tcp_Connect(ans, cp, err)) {
     *dfd = 1;
-  else {
+#ifdef ENABLE_DEBUGGING
+    _ilutcp_ConnectionsCreated += 1;
+    _ilutcp_CurrentConnections += 1;
+    if (_ilutcp_CurrentConnections > _ilutcp_MaxSimultaneousConnections)
+      _ilutcp_MaxSimultaneousConnections = _ilutcp_CurrentConnections;
+#endif
+  } else {
     ilu_Error       lerr;
     ilu_integer     cdfd;
     _tcp_Close(ans, &cdfd, &lerr);
@@ -770,29 +860,31 @@ static ilu_Transport
   return ans;
 }
 
-
-/*L1, L2 unconstrained*/
 static          ilu_boolean
 _tcp_Close(ilu_Transport self, ilu_integer * dfd,
 	   ILU_ERRS((internal)) * err)
 {
   TCPParms        parms;
+  int             fd;
   *dfd = 0;
-  if (self == NIL)
+  if (self == NIL || (parms = (TCPParms) self->tr_data) == NIL)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_broken, FALSE);
-  parms = (TCPParms) self->tr_data;
   ILU_CLER(*err);
-  if (parms->fd >= 0) {
-    ilu_UnregisterInputSource(parms->fd);
-    ilu_UnregisterOutputSource(parms->fd);
-    OS_SOCKLOSE(parms->fd);
+  if ((fd = parms->fd) >= 0) {
+    parms->fd = -1;
+    _ilu_FinishInputSource(fd);
+    _ilu_FinishOutputSource(fd);
+    OS_SOCKLOSE(fd);
     *dfd = 1;
   }
-  ilu_free(parms->tih);
   ilu_free(self->tr_inBuff);
   ilu_free(self->tr_outBuff);
   ilu_free(parms);
   ilu_free(self);
+#ifdef ENABLE_DEBUGGING
+  _ilutcp_CurrentConnections -= 1;
+#endif
+  
   return TRUE;
 }
 
@@ -804,20 +896,16 @@ _tcp_Close(ilu_Transport self, ilu_integer * dfd,
 ***********************************************************************
 **********************************************************************/
 
-/*L1, L2, Main unconstrained*/
+/*L1, L2 unconstrained*/
 
 static ilu_string
 _tcp_FormInfo(ilu_string hostname, ilutcpport_t port,
 	      ilu_boolean altfmt, ILU_ERRS((no_memory)) * err)
 {
-  ilu_string      retval;
   char            buf[1000];
   sprintf(buf, altfmt ? "TCP %s %u" : "tcp_%s_%u", hostname,
 	  ((unsigned) port) & 0xFFFF);
-  if ((retval = _ilu_Strdup(buf)) == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, strlen(buf) + 1, NIL);
-  ILU_CLER(*err);
-  return retval;
+  return ilu_StrdupE(buf, err);
 }
 
 /*L1_sup < trmu*/
@@ -835,12 +923,11 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
   int             mfd;
   *dfd = 0;
   if (self == NIL)
-    return (NIL);
-    mParms = (MooringParms) self->mo_data;
+    return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_duh, NIL);
+  mParms = (MooringParms) self->mo_data;
   mfd = mParms->fd;
-  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	(stderr,
-	 "tcp(%p): accepting connection request on FD %d.\n",
+  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	("tcp(%p): accepting connection request on FD %d.\n",
 	 self, mfd));
   addrlen = sizeof(sinaddr);
   if (OS_SOCKINV(ns = OS_ACCEPT(mfd, (struct sockaddr *) & sinaddr,
@@ -849,8 +936,8 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
     ilu_no_resources_Minor nrm;
     switch (theerr) {
     case SOCKERRID(WOULDBLOCK):
-      DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr, "tcp(%p): Spurrious call on accept(FD %d).\n",
+      ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	    ("tcp(%p): Spurrious call on accept(FD %d).\n",
 	     self, mfd));
       ILU_CLER(*err);
       return NIL;
@@ -860,22 +947,20 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
     case SOCKERRID(NOBUFS):
       nrm = ilu_nrm_ENOBUFS;
   raise_no_resources:
-      DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr,
-	     "tcp(%p): accept(FD %d) => errno %d, %s.\n",
+      ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	    ("tcp(%p): accept(FD %d) => errno %d, %s.\n",
 	     self, mfd, theerr, strerror(theerr)));
       return ILU_ERR_CONS1(no_resources, err, minor, nrm, NIL);
 #if (defined(WIN32) || defined(WIN16))
     case WSANOTINITIALISED:
-      DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr,
-	     "tcp(%p): Can't accept(FD %d) because WinSock not initialized.\n",
+      ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	    ("tcp(%p): Can't accept(FD %d) because WinSock not initialized.\n",
 	     self, mfd));
       goto bitch_errno;
 #endif
     default:
-      DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr, "tcp(%p): accept(FD %d) => errno %d.\n",
+      ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	    ("tcp(%p): accept(FD %d) => errno %d.\n",
 	     self, mfd, theerr));
   bitch_errno:
       return ILU_ERR_CONS1(internal, err, minor, ilu_im_errno, NIL);
@@ -884,12 +969,17 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
   *dfd = 1;
   if (OS_SOCKERR(set_doer_non_blocking(ns))) {
     int theerr = sockerrno;
-    DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	  (stderr,
-	   "tcp(%p):  Failed to set socket"
+    ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	  ("tcp(%p):  Failed to set incoming connection"
 	   " (FD %d) non-blocking.  Error \"%s\".\n",
 	   self, ns, strerror(theerr)));
   }
+  if (OS_SOCKERR(set_close_on_exec(ns))) {
+    int             err2 = sockerrno;
+    ilu_DebugPrintf("tcp(%p):  Unable to set incoming connection "
+		    "(FD %d) close-on-exec: %s (ignoring error)\n",
+		    self, ns, strerror(err2));
+  }
   /* See above comment on similar code below */
 #ifdef IPPROTO_TCP
 #ifdef TCP_NODELAY
@@ -913,15 +1003,20 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
       return NIL;
     }
   }
-  newT = NewT(mParms->buff_p, err);
+  newT = NewT(mParms->buffersize, err);
   if (newT == NIL)
     return NIL;
-  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	(stderr,
-	 "tcp(%p): new transport %p, on FD %d,"
+#ifdef ENABLE_DEBUGGING
+  ILU_NOTE((CONNECTION_DEBUG | TCP_DEBUG),
+	("tcp(%p): new transport %p, on FD %d,"
 	 " from IP host %s, port %u.\n",
 	 self, newT, ns, inet_ntoa(sinaddr.sin_addr),
 	 ntohs(sinaddr.sin_port)));
+  _ilutcp_ConnectionsReceived += 1;
+  _ilutcp_CurrentConnections += 1;
+  if (_ilutcp_CurrentConnections > _ilutcp_MaxSimultaneousConnections)
+    _ilutcp_MaxSimultaneousConnections = _ilutcp_CurrentConnections;
+#endif
   tParms = (TCPParms) newT->tr_data;
   tParms->fd = ns;
   return (newT);
@@ -933,80 +1028,87 @@ _tcp_CloseMooring(ilu_Mooring m, ilu_integer * dfd,
 {
   MooringParms    mp = (MooringParms) m->mo_data;
   int             fd = mp->fd, res, err1;
+  ILU_NOTE((EXPORT_DEBUG | TCP_DEBUG),
+	("tcp(%p): CloseMooring\n", m));
   if (fd >= 0) {
-    ilu_UnregisterInputSource(fd);
+    _ilu_FinishInputSource(fd);
     *dfd = 1;
     while (((res = OS_SOCKLOSE(fd)) == -1)
-	   && ((err1 = sockerrno) == SOCKERRID(INTR)));
+	   && ((err1 = sockerrno) == SOCKERRID(INTR)))
+      0;
     if (res < 0)
       return ILU_ERR_CONS1(internal, err, minor, ilu_im_errno, FALSE);
   } else
     *dfd = 0;
   if (mp != NIL) {
-    if (mp->tih != NIL)
-      ilu_free(mp->tih);
     ilu_free(mp);
     m->mo_data = NIL;
   }
+  ilu_free(m);
   return ILU_CLER(*err);
 }
 
-/*Main Invariant holds; L2 otherwise unconstrained*/
-static void
-CallReqHandler(int fd, ilu_private rock)
-{
-  ilu_TIH        *tp = (ilu_TIH *) rock;
-  (*tp->tih_proc) (tp->tih_rock);
-  return;
-}
-
 /*L1, L2 unconstrained*/
 static          ilu_boolean
-SetReqHandler(ilu_Mooring self, ilu_Server s,
-	      ilu_TransportInputHandler tih, ilu_refany tih_rock,
+SetReqHandler(ilu_Mooring self,
+	      ilu_TIH tih,
 	      ILU_ERRS((no_memory, imp_limit, no_resources,
 			broken_locks, internal)) * err)
 {
   MooringParms    mp = (MooringParms) self->mo_data;
-  ilu_TIH         *p = (ilu_TIH *) ilu_malloc(sizeof(*p));
   _ilu_Assert(mp->tih == NIL, "SetReqHandler");
-  if (p == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*p), FALSE);
-  p->tih_proc = tih;
-  p->tih_rock = tih_rock;
-  if (!ilu_RegisterInputSource(mp->fd, CallReqHandler, p))
+  if (!ilu_RegisterInputSource(mp->fd, _ilu_InvokeTIH, tih))
     return ILU_ERR_CONS1(no_resources, err, minor, ilu_nrm_mlreg, FALSE);
-  mp->tih = p;
+  mp->tih = tih;
   return ILU_CLER(*err);
 }
 
-/*Before: L1 = {s},
-          forall conn: (L2 >= {conn.iomu}) => (L2 >= {conn.callmu});
-  After: Main Invariant holds*/
+/*Main Invariant holds; L2 >= {ymu}*/
 static          ilu_boolean
-WaitForReq(ilu_Mooring self, ilu_Server s,
+WaitForReq(ilu_Mooring self, int *disabled,
 	   ILU_ERRS((interrupted, broken_locks)) * err)
 {
   MooringParms    mp = (MooringParms) self->mo_data;
   int             fd = mp->fd;
   ilu_boolean     sure;
-  if (!ilu_ExitMutex(s->sr_lock, TRUE, err))
-    return FALSE;
+  *disabled = 0;		/* could do better here */
   _ilu_WaitForInputOnFD(fd, &sure, NIL, err);
   return ILU_ERROK(*err);
 }
 
-/*L1.sup < trmu; L2 unconstrained*/
+/* L1 >= {cmu}; L1.sup < trmu; L2 >= {xmu} */
+
+static          ilu_boolean
+DisableReqWait(ilu_Mooring self,
+	       ILU_ERRS((broken_locks,
+			 bad_param,
+			 internal)) * err)
+{
+  return _ilu_DisableFDWaits(err);
+}
+
+static          ilu_boolean
+EnableReqWait(ilu_Mooring self,
+	       ILU_ERRS((broken_locks,
+			 bad_param,
+			 internal)) * err)
+{
+  return _ilu_EnableFDWaits(err);
+}
+
+/*L1, L2 unconstrained*/
+
 static ilu_integer MooringDFd(ilu_Mooring self, ilu_boolean add)
 {
   return 1;
 }
 
-/*L1, L2 unconstrained*/
 static struct _ilu_Mooring_s mooringProto = {
   MooringDFd,
   SetReqHandler,
   WaitForReq,
+  DisableReqWait,
+  EnableReqWait,
   _tcp_AcceptClient,
   _tcp_CloseMooring,
   NIL				/* data */
@@ -1043,11 +1145,11 @@ _tcp_CreateMooring(ilu_TransportCreator self,
 
   if (OS_SOCKINV(skt = socket(AF_INET, SOCK_STREAM, 0))) {
     err1 = sockerrno;
-    DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	  (stderr, "tcp(%p): create Mooring socket failed:  %s.\n",
+    ILU_NOTE((EXPORT_DEBUG | TCP_DEBUG),
+	  ("tcp(%p): create Mooring socket failed:  %s.\n",
 	   self, strerror(err1)));
     switch (err1) {
-#if !(defined(WIN32) || defined(WIN16))
+#if !(defined(WIN32) || defined(WIN16)|| defined( macintosh ))
     case SOCKERRID(ACCES):		/* permission error */
       ILU_ERR_CONS0(no_permission, err, 0);
       break;
@@ -1086,9 +1188,8 @@ _tcp_CreateMooring(ilu_TransportCreator self,
 			 sizeof(sinaddr))))
       break;
     err2 = sockerrno;
-    DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	  (stderr,
-	   "tcp(%p): bind to %08lx:%u failed:  %s.\n",
+    ILU_NOTE((EXPORT_DEBUG | TCP_DEBUG),
+	  ("tcp(%p): bind to %08lx:%u failed:  %s.\n",
 	   self, ntohl(sinaddr.sin_addr.s_addr), ntohs(sinaddr.sin_port),
 	   strerror(err2)));
     switch (err2) {
@@ -1139,8 +1240,8 @@ tryagain:
     if (OS_SOCKERR(getsockname(skt, (struct sockaddr *) & sinaddr,
 			       &namelen))) {
       int             err2 = sockerrno;
-      DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	 (stderr, "tcp(%p): getsockname failed:  %s.\n",
+      ILU_NOTE((EXPORT_DEBUG | TCP_DEBUG),
+	 ("tcp(%p): getsockname failed:  %s.\n",
 	  self, strerror(err2)));
       OS_SOCKLOSE(skt);
       *dfd = 0;
@@ -1167,9 +1268,8 @@ tryagain:
   }
   if (OS_SOCKERR(listen(skt, 4))) {
     int             err2 = sockerrno;
-    DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	  (stderr,
-	   "tcp(%p):  listen on port %u, FD %d failed:  %s.\n",
+    ILU_NOTE((EXPORT_DEBUG | TCP_DEBUG),
+	  ("tcp(%p):  listen on port %u, FD %d failed:  %s.\n",
 	   self, ntohs(sinaddr.sin_port), skt, strerror(err2)));
     OS_SOCKLOSE(skt);
     *dfd = 0;
@@ -1186,11 +1286,16 @@ tryagain:
     };
     goto failc;
   }
+  if (OS_SOCKERR(set_close_on_exec(skt))) {
+    int             err2 = sockerrno;
+    ilu_DebugPrintf("tcp(%p):  Unable to set listening socket "
+		    "(FD %d) close-on-exec: %s (ignoring error)\n",
+		    self, skt, strerror(err2));
+  }
   if (OS_SOCKERR(set_listener_non_blocking(skt))) {
     int             err2 = sockerrno;
-    DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	  (stderr,
-	   "tcp(%p):  Unable to set listening socket (FD %d) non-blocking: %s\n",
+    ILU_NOTE((EXPORT_DEBUG | TCP_DEBUG),
+	  ("tcp(%p):  Unable to set listening socket (FD %d) non-blocking: %s\n",
 	   self, skt, strerror(err2)));
     OS_SOCKLOSE(skt);
     *dfd = 0;
@@ -1207,9 +1312,9 @@ tryagain:
   ans->mo_data = mp;
   mp->tih = NIL;
   mp->fd = skt;
-  mp->buff_p = buffer;
-  DEBUG(TCP_DEBUG,
-	(stderr, "tcp(%p): created Mooring %p on FD %d, port %u.\n",
+  mp->buffersize = (buffer ? cp->buffersize : 0);
+  ILU_NOTE(TCP_DEBUG,
+	("tcp(%p): created Mooring %p on FD %d, port %u.\n",
 	 self, ans, skt, ntohs(sinaddr.sin_port)));
   if (tinfo_out != NIL) {
     ilu_string      hostname, tinfo;
@@ -1231,6 +1336,9 @@ tryagain:
     strcpy((*tinfo_out)[0], tinfo);
     ilu_free(tinfo);
   }
+#ifdef ENABLE_DEBUGGING
+  _ilutcp_MooringsCreated += 1;
+#endif
   return (ans);
 faild:
   if (skt >= 0)
@@ -1261,6 +1369,8 @@ _tcp_CloseCreator(ilu_TransportCreator self)
 static struct _ilu_TransportCreator_s creatorProto = {
   FALSE,			/* !boundaried */
   TRUE,				/* reliable */
+  0,				/* tcr_holds */
+  FALSE,			/* tcr_wantClose */
   _tcp_FdUsage,
   _tcp_CreateTransport,
   _tcp_CreateMooring,
@@ -1284,5 +1394,3 @@ _ilu_tcp_TransportCreator(ilu_TransportInfo tinfo,
   ans->tcr_data = cp;
   return (ans);
 }
-
-
