@@ -14,7 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* Last tweaked by Mike Spreitzer November 18, 1993 11:31 am PST */
+/* $Id: object.h,v 1.15 1997/11/08 02:58:21 janssen Exp $ */
+/* Last tweaked by Mike Spreitzer July 8, 1997 11:04 pm PDT */
 
 #define object_ih(o)			((o)->ob_ih)
 #define object_server(o)		((o)->ob_server)
@@ -26,10 +27,28 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #define object_lspos(o)			((o)->ob_lspos)
 #define object_holds(o)			((o)->ob_holds)
 #define object_intNoted(o)		((o)->ob_intNoted)
-#define object_notifying(o)		((o)->ob_collectibleInfo.ob_surrogate.ob_notifying)
-#define object_known(o)			((o)->ob_collectibleInfo.ob_surrogate.ob_known)
-#define object_gco(o)			((o)->ob_collectibleInfo.ob_true.ob_gco)
-#define object_lastRemote(o)		((o)->ob_collectibleInfo.ob_true.ob_lastRemote)
-#define object_gclist(o)		((o)->ob_collectibleInfo.ob_true.ob_gclist)
 #define object_is_true(o)		server_is_true(o->ob_server)
 #define object_collectible(o)		class_collectible((o)->ob_class)
+#define object_is_singleton(o)		((o)->ob_class->cl_singleton != NIL)
+#define object_singleton_info(o)	((o)->ob_class->cl_singleton)
+
+
+/* Use the following only if object_is_true(o) is FALSE!!! */
+
+#define object_notifying(o) \
+	((o)->ob_collectibleInfo.ob_surrogate.ob_notifying)
+
+#define object_known(o) \
+	((o)->ob_collectibleInfo.ob_surrogate.ob_known)
+
+
+/* Use the following only if object_is_true(o) is TRUE!!! */
+
+#define object_gco(o) \
+	((o)->ob_collectibleInfo.ob_true.ob_gco)
+
+#define object_lastRemote(o) \
+	((o)->ob_collectibleInfo.ob_true.ob_lastRemote)
+
+#define object_gclist(o) \
+	((o)->ob_collectibleInfo.ob_true.ob_gclist)
