@@ -15,7 +15,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: oscalls.h,v 1.9 1995/11/29 19:12:22 janssen Exp $ */
+/* $Id: oscalls.h,v 1.10 1997/03/06 03:00:03 janssen Exp $ */
 /* Antony A. Courtney, 16/7/93 */
 /* Last tweaked by Mike Spreitzer May 1, 1995 2:30 pm PDT */
 
@@ -26,6 +26,8 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #if (defined(WIN32) || defined (WIN16))
 #include "os/win.h"
+#elif defined( macintosh )
+#include "macos.h"
 #else
 #error "Whoa!  Don't have any way of making system calls!"
 #endif
