@@ -14,14 +14,16 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* Last tweaked by Mike Spreitzer November 9, 1993 5:43 pm PST */
+/* $Id: port.h,v 1.11 1997/04/08 02:07:10 janssen Exp $ */
+/* Last tweaked by Mike Spreitzer March 28, 1997 2:48 pm PST */
 
 #define port_mooring(port)		((port)->po_mooring)
 #define port_protocol(port)		((port)->po_protocol)
+#define port_protocol_data(port)	((port)->po_prot_data)
 #define port_server(port)		((port)->po_server)
 #define port_tinfo(port)		((port)->po_tinfo)
 #define port_pinfo(port)		((port)->po_pinfo)
 #define port_transport_creator(port)	((port)->po_tcr)
 #define port_closed(port)		((port)->po_closed != FALSE)
-#define port_next(port)			((port)->po_next)
+#define port_next(port)			((port)->po_links.pl_next)
 #define port_connections(port)		((port)->po_connHead.next)
