@@ -14,7 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: protocol.c,v 1.36 1996/06/11 02:17:38 janssen Exp $ */
+/* $Id: protocol.c,v 1.41 1997/09/10 22:14:18 janssen Exp $ */
+/* Last edited by Mike Spreitzer August 15, 1996 4:38 pm PDT */
 
 #define _POSIX_SOURCE
 
@@ -25,12 +26,15 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include "call.h"
 #include "type.h"
 
+#define MAX_PROTOCOLS	10	/* max number of protocols in registration table */
+
 struct protocols_s {
   /*L1, L2, Main unconstrained*/
   
   ilu_string name;
   /*For calling: L1_sup < prmu, L2, Main unconstrained*/
-  ilu_Protocol (*instantiator)(void);
+  ilu_Protocol (*instantiator)(ilu_ProtocolInfo /* pinfo */,
+			       ilu_Error * /* err */);
 };
 
 /*L1, L2, Main unconstrained*/
@@ -47,17 +51,21 @@ static struct protocols_s protocols[MAX_PROTOCOLS] = {
 #endif /* COURIER_PROTOCOL */
 #ifdef IIOP_PROTOCOL
   { "iiop", _ilu_IIOP_Protocol },
+  { "siiop", _ilu_IIOP_SerialProtocol },
 #endif /* IIOP_PROTOCOL */
-#ifdef W3MUX_PROTOCOL
-  { "w3mux", _ilu_w3mux_Protocol },
-#endif /* W3MUX_PROTOCOL */
+#ifdef W3NG_PROTOCOL
+  { "w3ng", _ilu_w3ng_Protocol },
+#endif /* W3NG_PROTOCOL */
+#ifdef JAVARMI_PROTOCOL
+  { "javarmi", _ilu_javarmi_Protocol },
+#endif /* JAVARMI_PROTOCOL */
 #ifdef HTTP_PROTOCOL
    { "http", _ilu_http_Protocol },
 #endif
   { NIL, NULLFN } };
 
 ILU_ERRS((ProtocolAlreadyRegistered, MaxCountExceeded))
-ilu_RegisterProtocol(char *name, ilu_Protocol(*new_protocol) (void),
+ilu_RegisterProtocol(char *name, ilu_Protocol(*new_protocol) (ilu_ProtocolInfo, ilu_Error *),
 		     ilu_boolean override)
 {
   int             i;
@@ -70,7 +78,7 @@ ilu_RegisterProtocol(char *name, ilu_Protocol(*new_protocol) (void),
       else
 	return ILU_ERR_CONS3(ProtocolAlreadyRegistered, &e,
 			     name, name,
-			  old_protocol, protocols[i].instantiator,
+			     old_protocol, protocols[i].instantiator,
 			     new_protocol, new_protocol, e);
     }
   }
@@ -114,9 +122,30 @@ static struct protocols_s *FindProtocol (ilu_string pinfo)
 ilu_Protocol _ilu_GetProtocolFromInfo (ilu_string pinfo)
 {
   struct protocols_s *p;
+  ilu_Error lerr;
 
   if ((p = FindProtocol(pinfo)) == NIL)
     return (NIL);
-  else
-    return ((*(p->instantiator))());
+  else {
+    ilu_Protocol pr = (*(p->instantiator))(pinfo, &lerr);
+    if (ILU_ERRNOK(lerr)) {
+      ilu_DebugPrintf("_ilu_GetProtocolFromInfo:  Can't find protocol with pinfo string <%p>!\n",
+		      pinfo);
+      ILU_HANDLED(lerr);
+      return NIL;
+    };
+    ILU_HANDLED(lerr);
+    return pr;
+  }
 }
+
+#ifndef ILU_DEFAULT_PROTOCOL_INFO
+#define ILU_DEFAULT_PROTOCOL_INFO "sunrpc"
+#endif
+
+ilu_ProtocolInfo
+  ilu_DefaultProtocolInfo (void)
+{
+  return (ilu_ProtocolInfo) ILU_DEFAULT_PROTOCOL_INFO ;
+}
+
