@@ -14,12 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: sbfile.c,v 1.36 1996/04/26 20:56:37 mdavidso Exp $ */
-/* Last edited by Mike Spreitzer March 8, 1996 9:08 am PST */
-
-#ifdef MACOS
-#pragma segment ilu
-#endif
+/* $Id: sbfile.c,v 1.43 1997/06/18 17:42:22 larner Exp $ */
+/* Last edited by Mike Spreitzer April 17, 1997 10:50 pm PDT */
 
 #include "iluntrnl.h"
 #include "object.h"
@@ -30,6 +26,8 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #if (defined(WIN32) || defined(WIN16))
 static char    *directory_delimiter = "\\";
+#elif defined( macintosh )
+static char    *directory_delimiter = "";
 #else
 static char    *directory_delimiter = "/";
 #endif
@@ -71,9 +69,9 @@ get_binding_file_name(char *buf, char *sid, char *ih)
   ilu_cardinal crc;
   static ilu_byte nullbyte = 0;
 
-  crc = _ilu_CRC32((ilu_bytes) sid, strlen(sid));
-  crc = _ilu_CRC32WithAccum(&nullbyte, 1, crc);
-  crc = _ilu_CRC32WithAccum((ilu_bytes) ih, strlen(ih), crc);
+  crc = ilu_CRC32((ilu_bytes) sid, strlen(sid));
+  crc = ilu_CRC32WithAccum(&nullbyte, 1, crc);
+  crc = ilu_CRC32WithAccum((ilu_bytes) ih, strlen(ih), crc);
   sprintf(buf, "%s%s%8.8lx", _ilu_get_binding_directory(),
 	  directory_delimiter, ((unsigned long) crc) & 0xFFFFFFFF);
 }
@@ -94,23 +92,20 @@ PASS(OPTIONAL(char *)) ilu_PublishObject (ilu_Object obj)
   char           *proof = NIL;
 
   if ((sbh = ilu_SBHOfObject(obj)) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "ilu_PublishObject:  object %p not exported.\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_PublishObject:  object %p not exported.\n",
 	   obj));
     return (NIL);
   } else {
     get_binding_file_name(buf, obj->ob_server->sr_id, obj->ob_ih);
     if (OS_ACCESS(buf, OS_ACCESS_R_OK) == 0
 	&& OS_ACCESS(buf, OS_ACCESS_W_OK) != 0) {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "ilu_PublishObject:  obj \"%s\" \"%s\" (%s) already published.\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("ilu_PublishObject:  obj \"%s\" \"%s\" (%s) already published.\n",
 	     obj->ob_server->sr_id, obj->ob_ih, buf));
     } else if ((f = fopen(buf, "w+")) == NIL) {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "ilu_PublishObject:  Can't open file %s for obj \"%s\" \"%s\".\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("ilu_PublishObject:  Can't open file %s for obj \"%s\" \"%s\".\n",
 	     buf, obj->ob_server->sr_id, obj->ob_ih));
     } else {
       proof = ilu_InventID();
@@ -120,9 +115,8 @@ PASS(OPTIONAL(char *)) ilu_PublishObject (ilu_Object obj)
 	       (OS_CHMOD_S_IWUSR | OS_CHMOD_S_IRUSR
 		| OS_CHMOD_S_IWGRP | OS_CHMOD_S_IRGRP
 		| OS_CHMOD_S_IWOTH | OS_CHMOD_S_IROTH));
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "ilu_PublishObject:  Published \"%s\" \"%s\" on %s, proof %s.\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("ilu_PublishObject:  Published \"%s\" \"%s\" on %s, proof %s.\n",
 	     obj->ob_server->sr_id, obj->ob_ih, buf, proof));
     }
   }
@@ -147,41 +141,36 @@ ilu_WithdrawObject(ilu_Object obj, PASS(char *) ownership_proof)
   ilu_boolean     result = ilu_FALSE;
 
   if ((sbh = ilu_SBHOfObject(obj)) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "ilu_WithdrawObject:  object %p not exported.\n", obj));
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_WithdrawObject:  object %p not exported.\n", obj));
   } else {
     get_binding_file_name(buf, obj->ob_server->sr_id, obj->ob_ih);
     if (OS_ACCESS(buf, OS_ACCESS_R_OK) == 0
 	&& OS_ACCESS(buf, OS_ACCESS_W_OK) != 0) {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	   "ilu_WithdrawObject:  Can't access registry file %s.\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("ilu_WithdrawObject:  Can't access registry file %s.\n",
 	     buf));
     } else if ((f = fopen(buf, "r+")) == NIL) {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "ilu_WithdrawObject:  Can't open registry file %s.\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("ilu_WithdrawObject:  Can't open registry file %s.\n",
 	     buf));
     } else {
       char            buf2[1000];
 
       if (fgets(buf2, sizeof(buf2), f) == NIL) {
-	DEBUG(BINDING_DEBUG,
-	      (stderr,
-	       "ilu_WithdrawObject:  Bad registry file %s.\n",
+	ILU_NOTE(BINDING_DEBUG,
+	      ("ilu_WithdrawObject:  Bad registry file %s.\n",
 	       buf));
 	fclose(f);
       } else if (strncmp(ownership_proof, buf2,
 			 strlen(ownership_proof)) != 0) {
-	DEBUG(BINDING_DEBUG,
-	      (stderr,
-	       "ilu_WithdrawObject:  no match on ownership proofs with proof %s.\n",
+	ILU_NOTE(BINDING_DEBUG,
+	      ("ilu_WithdrawObject:  no match on ownership proofs with proof %s.\n",
 	       ownership_proof));
 	fclose(f);
       } else {
-	OS_UNLINK(buf);
 	fclose(f);
+	OS_UNLINK(buf);
 	result = ilu_TRUE;
       }
     }
@@ -220,27 +209,24 @@ ilu_ReLookupObject(char *sid, char *ih,
   ilu_ConsiderSbhResult csr;
   ILU_ERRS((BadProtocolInfo, no_memory, inv_objref, internal)) lerr;
   ilu_boolean     ans;
-  ilu_Server      s2;
   *po = NIL;
   get_binding_file_name(buf, sid, ih);
   if (OS_ACCESS(buf, OS_ACCESS_R_OK) != 0) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	 "ilu_LookupObject:  No such object %s (\"%s\" \"%s\").\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_LookupObject:  No such object %s (\"%s\" \"%s\").\n",
 	   buf, sid, ih));
     return (ilu_FALSE);
   }
   if ((f = fopen(buf, "r")) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "ilu_LookupObject:  Can't open registry file %s.\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_LookupObject:  Can't open registry file %s.\n",
 	   buf));
     return (ilu_FALSE);
   }
   if (fgets(proof, sizeof(proof), f) == NIL
       || fgets(sbh, sizeof(sbh), f) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr, "ilu_LookupObject:  Bad registry file %s.\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_LookupObject:  Bad registry file %s.\n",
 	   buf));
     fclose(f);
     return (ilu_FALSE);
@@ -250,12 +236,11 @@ ilu_ReLookupObject(char *sid, char *ih,
   proof[strlen(proof) - 1] = '\0';
   sbh[strlen(sbh) - 1] = '\0';
 
-  DEBUG(BINDING_DEBUG,
-	(stderr,
-   "ilu_LookupObject:  found SBH=<%s>, pclass=<%s>, binding file=%s\n",
+  ILU_NOTE(BINDING_DEBUG,
+	("ilu_LookupObject:  found SBH=<%s>, pclass=<%s>, binding file=%s\n",
 	 sbh, pclass->cl_name, buf));
 
-  csr = ilu_ConsiderSBH(sbh, &s2, &lerr);
+  csr = ilu_ConsiderSBH(sbh, &lerr);
   switch (csr) {
   case ilucsr_err:
     ILU_HANDLED(lerr);
@@ -273,9 +258,8 @@ ilu_ReLookupObject(char *sid, char *ih,
   }
 
   if ((obj = ilu_ObjectOfSBH(sbh, pclass, &lerr)) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "ilu_LookupObject(%s/%s):  ObjectOfSBH(%s) raises %s"
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_LookupObject(%s/%s):  ObjectOfSBH(%s) raises %s"
 	   " from %s:%d\n",
 	   sid, ih, sbh, ILU_ERR_NAME(lerr),
 	   ilu_ErrorFile(&lerr), ilu_ErrorLine(&lerr)));
@@ -283,17 +267,16 @@ ilu_ReLookupObject(char *sid, char *ih,
     return (ilu_FALSE);
   }
   if (!ilu_IsSubObjectType(obj->ob_class, pclass)) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "ilu_LookupObject(%s/%s):  actual type %s does not include putative type %s\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_LookupObject(%s/%s):  actual type %s does not include putative type %s\n",
 	   sid, ih, obj->ob_class->cl_name, pclass->cl_name));
     ilu_ExitServer (object_server(obj), object_class(obj));
     return (ilu_FALSE);
   }
   *po = obj;
   if (ilu_TrueInstanceP(obj)) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr, "ilu_LookupObject:  Local object.\n"));
+    ILU_NOTE(BINDING_DEBUG,
+	  ("ilu_LookupObject:  Local object.\n"));
     return (ans);
   } else {
     ilu_boolean     status;
@@ -324,28 +307,25 @@ ilu_ReLookupObject(char *sid, char *ih,
       ilu_DHolds(obj, -1);
       return (ans);
     } else {
-      int             wasonly, bankit = 0;
+      int             wasonly;
       ilu_cardinal    nobj, nconn;
       ilu_cardinal    nL = 0;
       /* wasonly: 1 hash pair, no lspo. */
-      wasonly = _ilu_hash_PairsInTable(s->sr_objs) == 1;
+      wasonly = ilu_hash_PairsInTable(s->sr_objs) == 1;
       if (object_lspos(obj) != NIL) /* check every language */
 	for (nL = 0 ; wasonly && (nL < _ilu_NLanguages) ; nL++)
 	  wasonly = wasonly && object_lspo(obj,nL) == NIL;
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "ilu_LookupObject:  Bad ping of object %s\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("ilu_LookupObject:  Bad ping of object %s\n",
 	     sbh));
       (void) ilu_DeltaHolds(obj, -1);	/* BUG! should pass some errs */
       if (wasonly) {
 	nobj = ilu_NumObjsInServer(s);
 	nconn = ilu_NumIoingConnsOfServer(s);
-	if (bankit = (nobj == 0 && nconn == 0))
-	  ilu_PreBankServer(s);
+	if (nobj == 0 && nconn == 0)
+	  ilu_InnerBankServer(s);
       }
       ilu_ExitServer(s, pclass);
-      if (bankit)
-	ilu_BankServer(s);
       *po = NIL;
       return (ans);
     }
