@@ -14,7 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: sbilu.c,v 1.17 1996/07/10 00:00:20 janssen Exp $ */
+/* $Id: sbilu.c,v 1.29 1997/08/22 03:49:03 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 13, 1997 10:06 am PDT */
 
 /* Simple binding service, take 3.  (ILU version 2.0)
  */
@@ -22,6 +23,7 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include <stdio.h>
 #include <iluntrnl.h>
 #include <object.h>
+#include <server.h>
 
 #define SERVER_MSTID "ilu:ilu-simple-binding-version-1"
 #define SUNRPC_VERNO 2693483824
@@ -47,6 +49,7 @@ static ilu_Method enumerateMethod;
 static ilu_Class serverType = ILU_NIL;
 
 #include <stdlib.h>
+#include <string.h>
 #include "iluxport.h"
 
 /* Locate the binding realm, host, and port for this address space */
@@ -72,8 +75,8 @@ ilu_boolean ilu_GetSBServiceParms (char *realm_name, char *host, ilu_shortcardin
 	   ((temp = strtoul(tempbuf, ILU_NIL, 0)) == 0 ||
 	    temp > 0xFFFF)))
 	{
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "ilu_GetSBServiceParms:  can't parse value of ILU_BINDING_SERVICE:  \"%s\"\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("ilu_GetSBServiceParms:  can't parse value of ILU_BINDING_SERVICE:  \"%s\"\n",
 		 envval));
 	  return ilu_FALSE;
 	}
@@ -147,22 +150,22 @@ static void
 
   if (!ilu_EnterMutex(ilu_otmu, err))
     goto fail1;
-  BadSBH = ilu_DefineException("ilu-simpbind", "BadSBH", err);
+  BadSBH = ilu_DefineException("ilu-simpbind", "BadSBH", NIL, err);
   if (ILU_ERRNOK(*err))
     goto fail1;
-  AlreadyPublished = ilu_DefineException("ilu-simpbind", "AlreadyPublished", err);
+  AlreadyPublished = ilu_DefineException("ilu-simpbind", "AlreadyPublished", NIL, err);
   if (ILU_ERRNOK(*err))
     goto fail1;
-  MallocFailure = ilu_DefineException("ilu-simpbind", "MallocFailure", err);
+  MallocFailure = ilu_DefineException("ilu-simpbind", "MallocFailure", NIL, err);
   if (ILU_ERRNOK(*err))
     goto fail1;
-  NoTable = ilu_DefineException("ilu-simpbind", "NoTable", err);
+  NoTable = ilu_DefineException("ilu-simpbind", "NoTable", NIL, err);
   if (ILU_ERRNOK(*err))
     goto fail1;
-  NotPublished = ilu_DefineException("ilu-simpbind", "NotPublished", err);
+  NotPublished = ilu_DefineException("ilu-simpbind", "NotPublished", NIL, err);
   if (ILU_ERRNOK(*err))
     goto fail1;
-  BadProof = ilu_DefineException("ilu-simpbind", "BadProof", err);
+  BadProof = ilu_DefineException("ilu-simpbind", "BadProof", NIL, err);
   if (ILU_ERRNOK(*err))
     goto fail1;
   { ilu_string supers[] = { NULL};
@@ -185,12 +188,19 @@ static void
     exns[1] = AlreadyPublished;
     exns[2] = MallocFailure;
     publishMethod = ilu_DefineMethod(cl, 0,
-			 "Publish",	/*name*/
-			 1,	/*id*/
-			 0,	/*functional*/
-			 0,	/*asynch*/
-			 3,	/*n exns*/
-			 exns, err);
+				     "Publish",	/*name*/
+				     1,	/*id*/
+				     0,	/*functional*/
+				     0,	/*asynch*/
+				     3,	/*n exns*/
+				     exns,
+				     1, (char *) ilu_TypeID_ilu_CString,
+				     err);
+    if (ILU_ERRNOK(*err))
+      goto fail2;
+    (void) ilu_DefineMethodArg (publishMethod, 0,
+				"sbh", ilu_FALSE, ilu_In,
+				ILU_TYPEID_CONST_ilu_CString, err);
     if (ILU_ERRNOK(*err))
       goto fail2;
   }
@@ -200,12 +210,24 @@ static void
     exns[2] = BadProof;
     exns[3] = BadSBH;
     withdrawMethod = ilu_DefineMethod(cl, 1,
-			 "Withdraw",	/*name*/
-			 2,	/*id*/
-			 0,	/*functional*/
-			 0,	/*asynch*/
-			 4,	/*n exns*/
-			 exns, err);
+				      "Withdraw",	/*name*/
+				      2,	/*id*/
+				      0,	/*functional*/
+				      0,	/*asynch*/
+				      4,	/*n exns*/
+				      exns,
+				      2, (char *) ilu_TypeID_ilu_boolean,
+				      err);
+    if (ILU_ERRNOK(*err))
+      goto fail2;
+    (void) ilu_DefineMethodArg (withdrawMethod, 0,
+				"sbh", ilu_FALSE, ilu_In,
+				ILU_TYPEID_CONST_ilu_CString, err);
+    if (ILU_ERRNOK(*err))
+      goto fail2;
+    (void) ilu_DefineMethodArg (withdrawMethod, 1,
+				"cookie", ilu_FALSE, ilu_In,
+				ILU_TYPEID_CONST_ilu_CString, err);
     if (ILU_ERRNOK(*err))
       goto fail2;
   }
@@ -214,23 +236,42 @@ static void
     exns[1] = NotPublished;
     exns[2] = MallocFailure;
     lookupMethod = ilu_DefineMethod(cl, 2,
-			 "Lookup",	/*name*/
-			 3,	/*id*/
-			 0,	/*functional*/
-			 0,	/*asynch*/
-			 3,	/*n exns*/
-			 exns, err);
+				    "Lookup",	/*name*/
+				    3,	/*id*/
+				    0,	/*functional*/
+				    0,	/*asynch*/
+				    3,	/*n exns*/
+				    exns,
+				    2, (char *) ilu_TypeID_ilu_CString,
+				    err);
+    if (ILU_ERRNOK(*err))
+      goto fail2;
+    (void) ilu_DefineMethodArg (lookupMethod, 0,
+				"sid", ilu_FALSE, ilu_In,
+				ILU_TYPEID_CONST_ilu_CString, err);
+    if (ILU_ERRNOK(*err))
+      goto fail2;
+    (void) ilu_DefineMethodArg (lookupMethod, 1,
+				"ih", ilu_FALSE, ilu_In,
+				ILU_TYPEID_CONST_ilu_CString, err);
     if (ILU_ERRNOK(*err))
       goto fail2;
   }
   { ilu_Exception	*exns = NULL;
     enumerateMethod = ilu_DefineMethod(cl, 3,
-			 "Enumerate",	/*name*/
-			 4,	/*id*/
-			 0,	/*functional*/
-			 0,	/*asynch*/
-			 0,	/*n exns*/
-			 exns, err);
+				       "Enumerate",	/*name*/
+				       4,	/*id*/
+				       0,	/*functional*/
+				       0,	/*asynch*/
+				       0,	/*n exns*/
+				       exns,
+				       1, "IDL:ilu.parc.xerox.com/ilu_simpbind/StringBindingHandleList:1.0",
+				       err);
+    if (ILU_ERRNOK(*err))
+      goto fail2;
+    (void) ilu_DefineMethodArg (enumerateMethod, 0,
+				"pattern", ilu_FALSE, ilu_In,
+				ILU_TYPEID_CONST_ilu_CString, err);
     if (ILU_ERRNOK(*err))
       goto fail2;
   }
@@ -240,9 +281,8 @@ static void
   _sb = _ilu_FindSBServer(err); /* err. if nil */
   if (_sb == NIL)
     {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "%s:  Could not find simple binding service.\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("%s:  Could not find simple binding service.\n",
 	     "ilu_InitializeSimpleBinding"));
       serverType = ILU_NIL;
     }
@@ -272,8 +312,8 @@ static ilu_boolean
       InitializeSimpleBinding2(&err);
       if (ILU_ERRNOK(err))
 	{
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "InitializeSimpleBinding:  error %s initializing simple binding\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("InitializeSimpleBinding:  error %s initializing simple binding\n",
 		 ILU_ERR_NAME(err)));
 	  ILU_HANDLED(err);
 	  return ilu_FALSE;
@@ -288,8 +328,8 @@ static ilu_boolean
 
       if (ILU_ERRNOK(err))
 	{
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "InitializeSimpleBinding:  error %s copying instance handle of object instance %p.\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("InitializeSimpleBinding:  error %s copying instance handle of object instance %p.\n",
 		 ILU_ERR_NAME(err), kobj));
 	  ILU_HANDLED(err);
 	  return ilu_FALSE;
@@ -298,8 +338,8 @@ static ilu_boolean
       InitializeSimpleBinding2(&err);
       if (ILU_ERRNOK(err))
 	{
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "InitializeSimpleBinding:  error %s initializing simple binding\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("InitializeSimpleBinding:  error %s initializing simple binding\n",
 		 ILU_ERR_NAME(err)));
 	  ILU_HANDLED(err);
 	  ilu_free(ih);
@@ -310,8 +350,8 @@ static ilu_boolean
       ilu_free(ih);
       if (kobj2 != kobj)
 	{
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "InitializeSimpleBinding:  object changed while binding simple binding server!\n"));
+	  ILU_NOTE(BINDING_DEBUG,
+		("InitializeSimpleBinding:  object changed while binding simple binding server!\n"));
 	  return ilu_FALSE;
 	}
     }
@@ -339,40 +379,42 @@ _ilu_Publish_stub(ilu_Object kobj, ilu_string sbh, ilu_Error *err)
   /* assumes kobj doesn't change */
 
   if (kobj == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "_ilu_Publish_stub: nil simple binding service object\n"));
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Publish_stub: nil simple binding service object\n"));
     return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_nil, ILU_NIL);
   }
   if ((s = ilu_ServerOfObject(kobj)) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr, "%s: nil server for simple binding service object\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("%s: nil server for simple binding service object\n",
 	   "_ilu_Publish_stub"));
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_broken, ILU_NIL);
   }
   if (!ilu_StartCall(call, s, serverType, publishMethod, langIndex,
 		     ILU_NIL, &newconn, err))
     {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "_ilu_Publish_stub: ilu_StartCall failed with err <%s>\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("_ilu_Publish_stub: ilu_StartCall failed with err <%s>\n",
 	     ILU_ERR_NAME(*err)));
       return NIL;
     }
+  if (newconn != NIL) {
+    _ilu_HandOffNewConnection(newconn, err);
+    newconn = NIL;
+  };
+  if (ILU_ERRNOK(*err)) goto faild;
   ilu_AcquireMutex(s->sr_lock);
   reqSize += ilu_SizeOfObjectID(call, kobj, TRUE,
 			       ilu_ClassOfObject(kobj), err);
+  ilu_ReleaseMutex(s->sr_lock);
   if (ILU_ERRNOK(*err))
     goto faild;
-  ilu_ReleaseMutex(s->sr_lock);
   reqSize += ilu_SizeOfString(call, sbh, _ilu_SafeStrlen(sbh), 0, err);
   if (ILU_ERRNOK(*err))
     goto faild;
 
   if (!ilu_StartRequest(call, reqSize, err)) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-      "_ilu_Publish_stub: ilu_StartRequest failed with err <%s>\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Publish_stub: ilu_StartRequest failed with err <%s>\n",
 	   ILU_ERR_NAME(*err)));
     goto faild;
   }
@@ -385,16 +427,22 @@ _ilu_Publish_stub(ilu_Object kobj, ilu_string sbh, ilu_Error *err)
     goto faild;
 
   if (!ilu_FinishRequest(call, err)) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-     "_ilu_Publish_stub: ilu_FinishRequest failed with err <%s>\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Publish_stub: ilu_FinishRequest failed with err <%s>\n",
 	   ILU_ERR_NAME(*err)));
     goto faild;
   }
-  if ((perror = ilu_GetReply(call, &status_code, err)) == ilu_ProtocolException_Success)
+  perror = ilu_GetReply(call, &status_code, &newconn, err);
+  if (newconn != NIL) {
+    ILU_ERRS((bad_locks, broken_locks, internal)) herr;
+    _ilu_HandOffNewConnection(newconn, &herr);
+    ILU_MUST_BE_SUCCESS(herr);
+  }
+  if (perror == ilu_ProtocolException_Success)
     {
       ilu_cardinal len;
 
+      if (ILU_ERRNOK(*err)) goto faild;
       if (status_code == 0) {
 	retvalue = NIL;
 	ilu_InputString(call, &retvalue, &len, 0, err);
@@ -402,18 +450,18 @@ _ilu_Publish_stub(ilu_Object kobj, ilu_string sbh, ilu_Error *err)
 	  goto faild;
       } else { /* status_code != 0 => exception was signalled */
 	if (status_code > publishMethod->me_exceptionCount) {
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "_ilu_Publish_stub:  unexpected error signalled by server, status_code value of %lu\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("_ilu_Publish_stub:  unexpected error signalled by server, status_code value of %lu\n",
 		 status_code));
 	} else
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "_ilu_Publish_stub:  server signalled error %s\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("_ilu_Publish_stub:  server signalled error %s\n",
 		 (char *) publishMethod->me_exceptionVector[status_code - 1]));
       }
       ilu_ReplyRead(call, err);
     } else {
-      DEBUG(BINDING_DEBUG,
-	    (stderr, "_ilu_Publish_stub:  protocol error %s on GetReply\n", ilu_PEName(perror)));
+      ILU_NOTE(BINDING_DEBUG,
+	    ("_ilu_Publish_stub:  protocol error %s on GetReply\n", ilu_PEName(perror)));
       goto faild;
   }
 
@@ -438,32 +486,33 @@ static ilu_boolean
   /* assumes kobj doesn't change */
 
   if (kobj == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-	   "_ilu_Withdraw_stub: nil simple binding service object\n"));
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Withdraw_stub: nil simple binding service object\n"));
     return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_nil, ilu_FALSE);
   }
   if ((s = ilu_ServerOfObject(kobj)) == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr, "%s: nil server for simple binding service object\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("%s: nil server for simple binding service object\n",
 	   "_ilu_Withdraw_stub"));
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_broken, ilu_FALSE);
   }
   if (!ilu_StartCall(call, s, serverType, withdrawMethod, langIndex,
 		     ILU_NIL, &newconn, err))
     {
-      DEBUG(BINDING_DEBUG,
-	    (stderr,
-	     "_ilu_Withdraw_stub: ilu_StartCall failed with err <%s>\n",
+      ILU_NOTE(BINDING_DEBUG,
+	    ("_ilu_Withdraw_stub: ilu_StartCall failed with err <%s>\n",
 	     ILU_ERR_NAME(*err)));
       return ilu_FALSE;
     }
+  if (newconn != NIL)
+    _ilu_HandOffNewConnection(newconn, err);
+  if (ILU_ERRNOK(*err)) goto faild;
   ilu_AcquireMutex(s->sr_lock);
   reqSize += ilu_SizeOfObjectID(call, kobj, TRUE,
 			       ilu_ClassOfObject(kobj), err);
+  ilu_ReleaseMutex(s->sr_lock);
   if (ILU_ERRNOK(*err))
     goto faild;
-  ilu_ReleaseMutex(s->sr_lock);
   reqSize += ilu_SizeOfString(call, sbh, _ilu_SafeStrlen(sbh), 0, err);
   if (ILU_ERRNOK(*err))
     goto faild;
@@ -472,9 +521,8 @@ static ilu_boolean
     goto faild;
 
   if (!ilu_StartRequest(call, reqSize, err)) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-      "_ilu_Withdraw_stub: ilu_StartRequest failed with err <%s>\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Withdraw_stub: ilu_StartRequest failed with err <%s>\n",
 	   ILU_ERR_NAME(*err)));
     goto faild;
   }
@@ -490,34 +538,36 @@ static ilu_boolean
     goto faild;
 
   if (!ilu_FinishRequest(call, err)) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr,
-     "_ilu_Withdraw_stub: ilu_FinishRequest failed with err <%s>\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Withdraw_stub: ilu_FinishRequest failed with err <%s>\n",
 	   ILU_ERR_NAME(*err)));
     goto faild;
   }
-  if ((perror = ilu_GetReply(call, &status_code, err)) == ilu_ProtocolException_Success)
+  if ((perror = ilu_GetReply(call, &status_code, &newconn, err)) == ilu_ProtocolException_Success)
     {
       ilu_cardinal len;
 
+      if (newconn != NIL)
+	_ilu_HandOffNewConnection(newconn, err);
+      if (ILU_ERRNOK(*err)) goto faild;
       if (status_code == 0) {
 	ilu_InputBoolean(call, &retvalue, err);
 	if (ILU_ERRNOK(*err))
 	  goto faild;
       } else { /* status_code != 0 => exception was signalled */
 	if (status_code > withdrawMethod->me_exceptionCount) {
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "_ilu_Withdraw_stub:  unexpected error signalled by server, status_code value of %lu\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("_ilu_Withdraw_stub:  unexpected error signalled by server, status_code value of %lu\n",
 		 status_code));
 	} else
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "_ilu_Withdraw_stub:  server signalled error %s\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("_ilu_Withdraw_stub:  server signalled error %s\n",
 		 (char *) withdrawMethod->me_exceptionVector[status_code - 1]));
       }
       ilu_ReplyRead(call, err);
     } else {
-      DEBUG(BINDING_DEBUG,
-	    (stderr, "_ilu_Withdraw_stub:  protocol error %s on GetReply\n", ilu_PEName(perror)));
+      ILU_NOTE(BINDING_DEBUG,
+	    ("_ilu_Withdraw_stub:  protocol error %s on GetReply\n", ilu_PEName(perror)));
       goto faild;
   }
 
@@ -546,14 +596,14 @@ static ilu_string _ilu_Lookup_stub(ilu_Object kobj, ilu_string sid, ilu_string i
   /* assumes kobj doesn't change */
   
   if (kobj == NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr, "_ilu_Lookup_stub: nil simple binding service object\n"));
+    ILU_NOTE(BINDING_DEBUG,
+	  ("_ilu_Lookup_stub: nil simple binding service object\n"));
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_broken, ILU_NIL);
   }
   
   if ((s = object_server(kobj)) == ILU_NIL) {
-    DEBUG(BINDING_DEBUG,
-	  (stderr, "%s: nil server for simple binding service object\n",
+    ILU_NOTE(BINDING_DEBUG,
+	  ("%s: nil server for simple binding service object\n",
 	   "_ilu_Lookup_stub"));
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_broken, ILU_NIL);
   }
@@ -561,6 +611,9 @@ static ilu_string _ilu_Lookup_stub(ilu_Object kobj, ilu_string sid, ilu_string i
   if (!ilu_StartCall (call, s, serverType, lookupMethod, langIndex, ILU_NIL, &newconn, err))
     return ILU_NIL;
 
+  if (newconn != NIL)
+    _ilu_HandOffNewConnection(newconn, err);
+  if (ILU_ERRNOK(*err)) goto faild;
   ilu_AcquireMutex(s->sr_lock);
   reqSize = ilu_SizeOfObjectID(call,kobj,ilu_TRUE,ilu_ClassOfObject(kobj), err);
   ilu_ReleaseMutex(s->sr_lock);
@@ -586,9 +639,12 @@ static ilu_string _ilu_Lookup_stub(ilu_Object kobj, ilu_string sid, ilu_string i
     goto faild;
   if (! ilu_FinishRequest(call, err))
     goto faild;
-  if ((perror = ilu_GetReply(call, &scode, err)) == ilu_ProtocolException_Success)
+  if ((perror = ilu_GetReply(call, &scode, &newconn, err)) == ilu_ProtocolException_Success)
     {
       ilu_cardinal len;
+      if (newconn != NIL)
+	_ilu_HandOffNewConnection(newconn, err);
+      if (ILU_ERRNOK(*err)) goto faild;
       if (scode == 0)
 	{
 	  ilu_InputString (call, &retvalue, &len, 0, err);
@@ -597,18 +653,18 @@ static ilu_string _ilu_Lookup_stub(ilu_Object kobj, ilu_string sid, ilu_string i
 	}
       else if (scode > lookupMethod->me_exceptionCount)
 	{
-	  DEBUG(BINDING_DEBUG,
-		(stderr, "_ilu_Lookup_stub:  unexpected error signalled by server, scode value of %lu\n",
+	  ILU_NOTE(BINDING_DEBUG,
+		("_ilu_Lookup_stub:  unexpected error signalled by server, scode value of %lu\n",
 		 scode));
 	}
       else
-	DEBUG(BINDING_DEBUG,
-	      (stderr, "_ilu_Lookup_stub:  server signalled error %s\n",
+	ILU_NOTE(BINDING_DEBUG,
+	      ("_ilu_Lookup_stub:  server signalled error %s\n",
 	       (char *) lookupMethod->me_exceptionVector[scode - 1]));
       ilu_ReplyRead(call, err);
     } else {
-      DEBUG(BINDING_DEBUG,
-	    (stderr, "_ilu_Lookup_stub:  protocol error %s on GetReply\n", ilu_PEName(perror)));
+      ILU_NOTE(BINDING_DEBUG,
+	    ("_ilu_Lookup_stub:  protocol error %s on GetReply\n", ilu_PEName(perror)));
       goto faild;
     }
 
@@ -638,7 +694,8 @@ PASS(OPTIONAL(char *)) ilu_PublishObject (ilu_Object obj)
 /*  ILU_C_ENVIRONMENT       env; */
   char                   *proof;
   char                   *sbh;
-  char                   *mstid;
+  ilu_Server		 server;
+  ilu_Class		 type;
   ilu_Error err;
 
   if (!InitializeSimpleBinding(obj))
@@ -646,30 +703,30 @@ PASS(OPTIONAL(char *)) ilu_PublishObject (ilu_Object obj)
 
   /* Get sbh, mstid of object-to-be-published */
   if ((sbh = ilu_SBHOfObject(obj)) == NIL) {
-    DEBUG(BINDING_DEBUG | EXPORT_DEBUG,
-	  (stderr,
-	   "ilu_PublishObject:  object %p not exported.\n",
+    ILU_NOTE(BINDING_DEBUG | EXPORT_DEBUG,
+	  ("ilu_PublishObject:  object %p not exported.\n",
 	   obj));
     return (NIL);
   } else {
-    ilu_ReleaseMutex(ilu_cmu);  /* necessary for BeginCall */
+    server = object_server(obj);
+    type = object_class(obj);
+    ilu_DHolds(obj, +1);
+    ilu_ExitServer(server, type);
     proof = _ilu_Publish_stub(_sb, sbh, &err);
-    ilu_AcquireMutex(ilu_cmu);
     
     if (ILU_ERRNOK(err)) {
-      DEBUG(BINDING_DEBUG | EXPORT_DEBUG,
-	    (stderr,
-	     "ilu_PublishObject:  Publish failed on object \"%s\", error %s\n",
+      ILU_NOTE(BINDING_DEBUG | EXPORT_DEBUG,
+	    ("ilu_PublishObject:  Publish failed on object \"%s\", error %s\n",
 	     sbh, ILU_ERR_NAME(err)));
       ILU_HANDLED(err);
     }
-    ilu_ExitServer(object_server(obj), object_class(obj));
+    ilu_EnterServer(server, type);
+    ilu_DeltaHolds(obj, -1);
+    ilu_ExitServer(server, type);
     return (proof);
   }
 }
 
-
-
 /*before: Inside(s, cl)
   after:				 L1 disjoint {cmu, s};
   after: cl collectible		      => L1  not >=  {gcmu};
@@ -678,40 +735,39 @@ PASS(OPTIONAL(char *)) ilu_PublishObject (ilu_Object obj)
   (We don't really need to hold cmu for surrogate or non-collectible
    objects, but this is convenient because ilu_Enter/ExitServer can
    be used.)*/
-/* To make the stub call, cmu is released and reacquired.
-   (Out of order.  Exit obj's server after getting the sbh, instead?)
- */
 ilu_boolean
 ilu_WithdrawObject(ilu_Object obj, PASS(char *) ownership_proof)
 {
   char                   *sbh;
-  char                   *oid;
   ilu_boolean             result = ilu_FALSE;
+  ilu_Server		 server;
+  ilu_Class		 type;
   ilu_Error		 err;
 
   /* Get oid of published obj */
   if ((sbh = ilu_SBHOfObject(obj)) == NIL) {
-    DEBUG(BINDING_DEBUG | EXPORT_DEBUG,
-	  (stderr,
-	   "ilu_WithdrawObject:  object %p not exported.\n",
+    ILU_NOTE(BINDING_DEBUG | EXPORT_DEBUG,
+	  ("ilu_WithdrawObject:  object %p not exported.\n",
 	   obj));
     return (ilu_FALSE);
   } else {
-
-    ilu_ReleaseMutex(ilu_cmu);  /* necessary for BeginCall */ 
+    server = object_server(obj);
+    type = object_class(obj);
+    ilu_DHolds(obj, +1);
+    ilu_ExitServer(server, type);
     result = _ilu_Withdraw_stub(_sb, sbh, ownership_proof, &err);
-    ilu_AcquireMutex(ilu_cmu);
 
     if (ILU_ERRNOK(err)) {
-      DEBUG(BINDING_DEBUG | EXPORT_DEBUG,
-	    (stderr,
-	     "ilu_WithdrawObject:  Withdraw failed on object \"%s\", error %s\n",
+      ILU_NOTE(BINDING_DEBUG | EXPORT_DEBUG,
+	    ("ilu_WithdrawObject:  Withdraw failed on object \"%s\", error %s\n",
 	     sbh, ILU_ERR_NAME(err)));
       ILU_HANDLED(err);
       return ilu_FALSE;
     }
     ilu_free(ownership_proof);
-    ilu_ExitServer(object_server(obj), object_class(obj));
+    ilu_EnterServer(server, type);
+    ilu_DeltaHolds(obj, -1);
+    ilu_ExitServer(server, type);
     return result;
   }
 }
@@ -745,7 +801,6 @@ ilu_ReLookupObject(char *sid, char *ih,
   ilu_Object            obj;
   ilu_ConsiderSbhResult csr;
   ilu_boolean           ans;
-  ilu_Server            s2;
   ilu_Connection	newconn;
   ILU_ERRS((BadSBH, BadURL, BadContactInfo,
 	    BadTransportInfo, BadProtocolInfo)) err;
@@ -757,18 +812,16 @@ ilu_ReLookupObject(char *sid, char *ih,
   /* Get the sbh and mstid from the oid */
   sbh = _ilu_Lookup_stub(_sb, sid, ih, &err);
   if (sbh == NIL) {
-    DEBUG(BINDING_DEBUG | INCOMING_DEBUG,
-	  (stderr,
-	   "ilu_LookupObject:  Lookup failed on oid %s/%s\n",
+    ILU_NOTE(BINDING_DEBUG | INCOMING_DEBUG,
+	  ("ilu_LookupObject:  Lookup failed on oid %s/%s\n",
 	   sid, ih));
     return (ilu_FALSE); }
   
   /* Unchanged from the previous simpbind.c ... */
-  DEBUG(INCOMING_DEBUG,
-	(stderr,
-	 "ilu_LookupObject:  found SBH=<%s>\n", sbh));
+  ILU_NOTE(INCOMING_DEBUG,
+	("ilu_LookupObject:  found SBH=<%s>\n", sbh));
   
-  csr = ilu_ConsiderSBH(sbh, &s2, &err);
+  csr = ilu_ConsiderSBH(sbh, &err);
   switch (csr) {
   case ilucsr_err:
     ILU_HANDLED(err);
@@ -786,14 +839,16 @@ ilu_ReLookupObject(char *sid, char *ih,
   }
 
   if ((obj = ilu_ObjectOfSBH(sbh, pclass, &err)) == NIL) {
-    DEBUG(INCOMING_DEBUG,
-	  (stderr, "ilu_LookupObject:  Bad object info.\n"));
+    ILU_NOTE(INCOMING_DEBUG,
+	  ("ilu_LookupObject:  Bad object info (%s).\n", sbh));
+    ilu_free(sbh);
     return (ilu_FALSE);
   }
+  ilu_free(sbh);
   *po = obj;
   if (ilu_TrueInstanceP(obj)) {
-    DEBUG(INCOMING_DEBUG,
-	  (stderr, "ilu_LookupObject:  Local object.\n"));
+    ILU_NOTE(INCOMING_DEBUG,
+	  ("ilu_LookupObject:  Local object.\n"));
     return (ans);
   } else {
     ilu_boolean     status;
@@ -820,26 +875,23 @@ ilu_ReLookupObject(char *sid, char *ih,
       ilu_DHolds(obj, -1);
       return (ans);
     } else {
-      int             wasonly, nobj, nconn, bankit, nL = 0;
+      int             wasonly, nobj, nconn, nL = 0;
       /* wasonly: 1 hash pair, no lspo. */
-      wasonly = _ilu_hash_PairsInTable(s->sr_objs) == 1;
+      wasonly = ilu_hash_PairsInTable(s->sr_objs) == 1;
       if (object_lspos(obj) != NIL) /* check every language */
 	for (nL = 0 ; wasonly && (nL < _ilu_NLanguages) ; nL++)
 	  wasonly = wasonly && object_lspo(obj,nL) == NIL;
-      DEBUG(INCOMING_DEBUG,
-	    (stderr,
-	     "ilu_LookupObject:  Bad ping of object %s\n",
-	     sbh));
-      (void) ilu_DeltaHolds(obj, -1);	/* BUG! should pass some errs */
+      ILU_NOTE(INCOMING_DEBUG,
+	    ("ilu_LookupObject:  Bad ping of object %s %s\n",
+	     server_id(object_server(obj)), object_ih(obj)));
+      (void) ilu_DeltaHolds(obj, -1); /* XXX - BUG! should pass some errs */
       if (wasonly) {
 	nobj = ilu_NumObjsInServer(s);
 	nconn = ilu_NumIoingConnsOfServer(s);
-	if (bankit = (nobj == 0 && nconn == 0))
-	  ilu_PreBankServer(s);
+	if (nobj == 0 && nconn == 0)
+	  ilu_InnerBankServer(s);
       }
       ilu_ExitServer(s, pclass);
-      if (bankit)
-	ilu_BankServer(s);
       *po = NIL;
       return (ans);
     }
