@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: sunrpcrm.c,v 1.28 1996/06/06 19:18:23 spreitze Exp $ */
-/* Last edited by Mike Spreitzer June 6, 1996 10:37 am PDT */
+/* $Id: sunrpcrm.c,v 1.36 1997/11/13 23:13:41 spreitze Exp $ */
+/* Last edited by Mike Spreitzer November 13, 1997 2:43 pm PST */
 
 #include "iluntrnl.h"
 
@@ -31,31 +31,44 @@ typedef struct {
   ilu_TransportCreator lower;
 }              *CreatorParms;
 
-typedef struct sunrpcrmparms {
-  /* L1, L2, Main unconstrained */
+typedef struct {
+  /* L1 unconstrained */
 
+  /* L2 >= {ymu} */
   /* For input: */
+
   ilu_boolean     inWedged;	/* further input msgs forbidden? */
-  ilu_boolean     busyIn;	/* currently processing input msg? */
   ilu_boolean     lastIn;	/* current chunk is last? */
   ilu_boolean     firstIn;	/* next chunk is first? */
   ilu_cardinal    inLength;	/* of lower-level data in buffer */
   ilu_cardinal    inChunkRem;	/* how much more after inLength? */
   ilu_cardinal    inSize;	/* size of input buffer */
+  /**for read:  L2 includes xmu or ymu;
+     for write: L2    >=   {xmu,   ymu}*/
+  ilu_boolean     busyIn;	/* currently processing input msg? */
+  ilu_boolean     skipping;	/* prev msg not yet finished? */
   /*
-   * Unconsumed lower-level input bytes are at indices [tr_inNext,
-   * inLength) in tr_inBuff.  When busyIn, firstIn and lastIn are
-   * meaningful, and the unconsumed bytes of current chunk are: (a)
-   * at indices [tr_inNext, tr_inLimit), and then (b) inChunkRem
-   * bytes yet to be read from lower level.  tr_inLimit==inLength
-   * when inChunkRem>0.  inChunkRem==0 and tr_inNext==tr_inLimit
-   * when !busyIn && !inWedged.  inSize is the allocated size of
-   * tr_inBuff.  inWedged and busyIn are mutually exclusive.  When
-   * inWedged, no more messages may be input (we've lost synch with
-   * lower layer).
+   * ymu's invariant includes all the following.  Unconsumed
+   * lower-level input bytes are at indices [tr_inNext, inLength) in
+   * tr_inBuff.  "next" in def'n of (firstIn) means first chunk
+   * whose header hasn't been entirely processed by this layer.
+   * (skipping) means we're between messages as far as our caller is
+   * concerned, but we haven't finished skipping over all the input
+   * from the previous message.  When busyIn||skipping: firstIn and
+   * lastIn are meaningful, and the unconsumed bytes of the current
+   * chunk are: (a) at indices [tr_inNext, tr_inLimit), and then (b)
+   * inChunkRem bytes yet to be read from lower level. inChunkRem>0
+   * => tr_inLimit == inLength.  skipping => tr_inNext ==
+   * tr_inLimit.  inChunkRem==0 and tr_inNext==tr_inLimit when
+   * !busyIn && !skipping && !inWedged.  inSize is the allocated
+   * size of tr_inBuff.  busyIn, skipping, and inWedged are mutually
+   * exclusive.  When inWedged, no more messages may be input (we've
+   * lost synch with lower layer).
    */
 
+  /* L2 >= {xmu} */
   /* For output: */
+
   ilu_boolean     busyOut;	/* currently processing output msg? */
   ilu_cardinal    outStart;	/* loc o 1st payload byte of cur msg */
   ilu_cardinal    outSize;	/* size of output buffer */
@@ -68,24 +81,33 @@ typedef struct sunrpcrmparms {
    * outSize, which is always >= tr_outLimit + (busyOut?4:8).
    */
 
-  ilu_Transport   lower;
+  /* For general use: */
+  
   ilu_boolean     dirIsIn;
   /*
-   * At most one of busyIn, busyOut is true; when one is, dirIsIn
-   * indicates which one.
+   * xmu's invariant includes this: at most one of (busyIn),
+   * (busyOut) is true; when one is, (dirIsIn) indicates which one.
+   */
+
+  /* L2 unconstrained */
+
+  ilu_Transport   lower;
+  /*
+   * The unboundaried transport from which (self) is constructed. We
+   * don't want an exposed buffer in (lower).  lower's xmu = self's
+   * xmu; same for ymu.
    */
 }              *SUNRPCRMParms;
 /* What goes in the data field of a SUNRPCRM ilu_Transport. */
 
-/*L1, L2, Main unconstrained*/
+/*L1, L2 unconstrained*/
 
 #define BUFFERSIZE		(8192 + 8)
 #define DIRECT_THRESHOLD	1024
 
-/*L1 >= {connection's iomu}*/
 #define SUNRPCRMPARMS(a) ((SUNRPCRMParms)(a))
 
-/*L1_sup < trmu; L2 unconstrained*/
+/*L1.sup < trmu; L2 unconstrained*/
 static          CreatorParms
 _sunrpcrm_InterpretInfo(ilu_TransportInfo info,
 			ILU_ERRS((no_memory, inv_objref)) * err)
@@ -101,50 +123,68 @@ _sunrpcrm_InterpretInfo(ilu_TransportInfo info,
     return NIL;
   if (lower->tcr_boundaried || !lower->tcr_reliable)
     return ILU_ERR_CONS1(inv_objref, err, minor, ilu_iom_ts, NIL);
-  cp = (CreatorParms) ilu_malloc(sizeof(*cp));
+  cp = (CreatorParms) ilu_MallocE(sizeof(*cp), err);
   if (cp == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*cp), NIL);
+    return NIL;
   cp->lower = lower;
   return cp;
 }
 
-/*Main Invariant holds; L2 disjoint {conn's iomu, callmu}*/
+/* L1.sup < trmu; L2 >= {xmu, ymu} */
 static          ilu_boolean
-SetInputHandler(ilu_Transport self, ilu_TransportInputHandler tih,
-		ilu_refany tih_rock,
+SetInputHandler(ilu_Transport self, ilu_TIH tih,
 		ILU_ERRS((no_memory, internal, no_resources)) * err)
 {
   SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
-  if (tih != NULLFN && self->tr_inLimit < p->inLength) {
-    (*tih) (tih_rock);
-    ILU_CLER(*err);
-    return FALSE;
-  }
-  return ilu_SetTransportInputHandler(p->lower, tih, tih_rock, err);
+  if (tih &&
+      (p->inWedged || self->tr_inNext + 4 <= p->inLength
+       || p->busyIn && p->lastIn && p->inChunkRem == 0))
+    return (ILU_CLER(*err), FALSE);
+  return _ilu_SetTransportInputHandler(p->lower, tih, err);
 }
 
-/*Main Invariant holds; L2 not further constrained*/
-
+/* Main Invariant holds; L2 >= {ymu} */
 static          ilu_boolean
-_sunrpcrm_WaitForInput(ilu_Transport self, ilu_FineTime * limit,
-		       ILU_ERRS((interrupted)) * err)
+_sunrpcrm_WaitForInput(ilu_Transport self, int *disabled,
+		       ilu_FineTime * limit,
+		       ILU_ERRS((broken_locks, interrupted)) * err)
 {
   SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
-  if (self->tr_inNext + 4 <= p->inLength
-      || p->busyIn && (self->tr_inNext < self->tr_inLimit
-		       || p->lastIn && p->inChunkRem == 0))
-    return ILU_CLER(*err);
-  return (transport_wait_for_input(p->lower, limit, err));
+  if (p->inWedged || self->tr_inNext + 4 <= p->inLength
+      || p->busyIn && p->lastIn && p->inChunkRem == 0)
+    return (*disabled = 0, ILU_CLER(*err));
+  return (transport_wait_for_input(p->lower, disabled, limit, err));
+}
+
+/* L1.sup < trmu; L2 >= {xmu} */
+static          ilu_boolean
+_sunrpcrm_InterruptST(ilu_Transport self, ILU_ERRS((bad_param)) * err)
+{
+  SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
+  return (transport_interruptST(p->lower, err));
+}
+
+/* L1 >= {cmu}; L1.sup < trmu; L2 >= {xmu} */
+
+static          ilu_boolean
+_sunrpcrm_DisableWait(ilu_Transport self,
+		      ILU_ERRS((broken_locks, bad_param,
+				internal)) * err)
+{
+  SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
+  return (transport_disableWait(p->lower, err));
 }
 
 static          ilu_boolean
-_sunrpcrm_Interrupt(ilu_Transport self, ILU_ERRS((bad_param)) * err)
+_sunrpcrm_EnableWait(ilu_Transport self,
+		     ILU_ERRS((broken_locks, bad_param,
+			       internal)) * err)
 {
   SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
-  return (transport_interrupt(p->lower, err));
+  return (transport_enableWait(p->lower, err));
 }
 
-/* L2, Main, unconstrained; L1_sup < trmu */
+/* L1.sup < trmu; L2 unconstrained */
 
 static          ilu_integer
 _sunrpcrm_FdUsage(ilu_TransportCreator self, ilu_boolean mooring)
@@ -153,6 +193,7 @@ _sunrpcrm_FdUsage(ilu_TransportCreator self, ilu_boolean mooring)
   return (*cp->lower->tcr_dfd) (cp->lower, mooring);
 }
 
+/* L1.sup < trmu; L2 >= {xmu, ymu} */
 static          ilu_integer
 _sunrpcrm_CloseDFd(ilu_Transport self)
 {
@@ -160,53 +201,56 @@ _sunrpcrm_CloseDFd(ilu_Transport self)
   return ((*p->lower->tr_class->tc_closeDFd) (p->lower));
 }
 
-/*Main Invariant holds*/
-/*L2 >= {conn's iomu}*/
-
+/*Main Invariant holds; L2 >= {xmu, ymu}*/
 static          ilu_cardinal
 ReadWork(ilu_Transport self,
 	 ilu_bytes buffer,
 	 ilu_cardinal len,
 	 ilu_TransportReport * rpt,
+	 int *progress,
 	 ILU_ERRS((IoErrs)) * err);
 
+/* Main Invariant holds; L2 >= {xmu}; input => L2 >= {ymu} */
+
 static          ilu_ReadHeaderResultCode
 _sunrpcrm_BeginMessage(ilu_Transport self,
 		       ilu_boolean input_p,
 		       ILU_ERRS((IoErrs)) * err)
 {
   SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
-  DEBUG(SUNRPCRM_DEBUG,
-	(stderr, "sunrpcrm(%p): BeginMessage(%s)\n",
+  int             progress = 1;
+  ILU_NOTE(SUNRPCRM_DEBUG,
+	("sunrpcrm(%p): BeginMessage(%s)\n",
 	 self, input_p ? "input" : "output"));
   if (p->busyIn || p->busyOut)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_beginMessage,
 			 ilu_rhrc_error);
   p->dirIsIn = input_p;
   if (input_p) {
+    ilu_TransportReport rpt = {FALSE, FALSE};
     if (p->inWedged)
       return ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_conn_lost,
 			   ilu_rhrc_error);
+    if (p->skipping) {
+      while (progress && !rpt.tr_eom) {
+	(void) ReadWork(self, NIL, 0, &rpt, &progress, err);
+	self->tr_inNext = self->tr_inLimit;
+	if (ILU_ERRNOK(*err))
+	  return p->inWedged = TRUE, p->skipping = FALSE, ilu_rhrc_error;
+      }
+      if (!progress)
+	return ilu_rhrc_nothing;
+      p->skipping = FALSE;
+    }
     p->busyIn = p->firstIn = TRUE;
     p->lastIn = FALSE;
     if (self->tr_inNext == p->inLength) {
-      ilu_TransportReport rpt;
-      (void) ReadWork(self, NIL, 0, &rpt, err);
-      if (ILU_ERRNOK(*err)) {
-	p->inWedged = TRUE;
-	p->busyIn = FALSE;
-	return ilu_rhrc_error;
-      }
-      if (p->inLength < 4 && rpt.tr_eof) {
-	p->inWedged = TRUE;
-	p->busyIn = FALSE;
-	return ilu_rhrc_eof;
-      }
+      (void) ReadWork(self, NIL, 0, &rpt, &progress, err);
+      if (ILU_ERRNOK(*err))
+	return p->inWedged = TRUE, p->busyIn = FALSE, ilu_rhrc_error;
       if (p->inLength == 0) {
-	_ilu_Assert(p->inChunkRem == 0 && self->tr_inLimit == p->inLength,
-		    "sunrpcrm.c:BeginMessage");
-	p->busyIn = FALSE; 
-	return ilu_rhrc_nothing;
+	p->busyIn = FALSE;
+	return (rpt.tr_eof ? ilu_rhrc_eof : ilu_rhrc_nothing);
       }
     }
   } else {
@@ -218,6 +262,7 @@ _sunrpcrm_BeginMessage(ilu_Transport self,
   return ilu_rhrc_ok;
 }
 
+/*L1, L2 unconstrained*/
 #define FORMAT_HEADER(where,size,eom)		\
 (						\
  (where)[0] = (((size) >> 24) & 0xFF) | ((eom) ? 0x80 : 0),	\
@@ -236,9 +281,8 @@ _sunrpcrm_EndMessage(ilu_Transport self,
   if (p->busyOut) {
     ilu_cardinal    n1 = self->tr_outNext - p->outStart;
     ilu_boolean     ans = TRUE;
-    DEBUG(SUNRPCRM_DEBUG,
-	  (stderr,
-	   "sunrpcrm(%p): EndMessage(output, flush=%d, last chunkSize=%lu)\n",
+    ILU_NOTE(SUNRPCRM_DEBUG,
+	  ("sunrpcrm(%p): EndMessage(output, flush=%d, last chunkSize=%lu)\n",
 	   self, (flush != 0), (long unsigned) (n1)));
     FORMAT_HEADER(self->tr_outBuff + p->outStart - 4, n1, TRUE);
     p->busyOut = FALSE;
@@ -253,25 +297,25 @@ _sunrpcrm_EndMessage(ilu_Transport self,
   } else if (p->busyIn) {
     ilu_boolean     first = TRUE, drops = FALSE;
     ilu_TransportReport rpt = {FALSE, FALSE};
-    DEBUG(SUNRPCRM_DEBUG,
-	  (stderr, "sunrpcrm(%p): EndMessage(input)\n", self));
-    while (!((p->lastIn && p->inChunkRem == 0) || rpt.tr_eof)) {
+    ILU_NOTE(SUNRPCRM_DEBUG,
+	  ("sunrpcrm(%p): EndMessage(input)\n", self));
+    if (!(p->lastIn && p->inChunkRem == 0)) {
       ilu_cardinal    lread;
-      drops = (drops || p->inChunkRem != 0 ||
-	       self->tr_inLimit != self->tr_inNext);
+      int             progress;
+      drops = (p->inChunkRem || self->tr_inLimit != self->tr_inNext);
       self->tr_inNext = self->tr_inLimit;
-      if (p->inChunkRem > 0 || p->inLength < self->tr_inNext + 4) {
-	if (!(first || transport_wait_for_input(p->lower, NIL, err)))
-	  goto wedge;
-	first = FALSE;
-      }
-      lread = ReadWork(self, NIL, 0, &rpt, err);
+      lread = ReadWork(self, NIL, 0, &rpt, &progress, err);
       if (ILU_ERRNOK(*err))
 	goto wedge;
     }
-    drops = drops || self->tr_inLimit != self->tr_inNext;
+    drops = (drops || p->inChunkRem ||
+	     self->tr_inLimit != self->tr_inNext);
     self->tr_inNext = self->tr_inLimit;
     p->busyIn = FALSE;
+    if (p->lastIn && p->inChunkRem == 0)
+      p->skipping = FALSE;
+    else
+      p->skipping = TRUE;
     if (drops)
       return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcBytesDropped,
 			   FALSE);
@@ -284,6 +328,15 @@ wedge:
 			 FALSE);
 }
 
+/*Main Invariant holds; L2 >= {xmu}*/
+
+static          ilu_boolean
+_sunrpcrm_SendWholeMessage(ilu_Transport self, ilu_Message * msgh,
+			   ILU_ERRS((IoErrs)) * err)
+{
+  return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcReliable, FALSE);
+}
+
 static          ilu_boolean
 _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
 		     ilu_cardinal bufferSize,
@@ -305,25 +358,22 @@ _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
       FORMAT_HEADER(self->tr_outBuff + self->tr_outNext, bufferSize,
 		    FALSE);
       self->tr_outNext += 4;
-      DEBUG(SUNRPCRM_DEBUG,
-	    (stderr,
-	     "sunrpcrm(%p): Writing length %lu chunk, plus header for length %lu.\n",
+      ILU_NOTE(SUNRPCRM_DEBUG,
+	    ("sunrpcrm(%p): Writing length %lu chunk, plus header for length %lu.\n",
 	     self, l1, bufferSize));
     } else {
       /* current chunk empty --- hijack its header */
       FORMAT_HEADER(self->tr_outBuff + p->outStart - 4, bufferSize,
 		    FALSE);
-      DEBUG(SUNRPCRM_DEBUG,
-	    (stderr,
-	     "sunrpcrm(%p): Writing header for length %lu chunk.\n",
+      ILU_NOTE(SUNRPCRM_DEBUG,
+	    ("sunrpcrm(%p): Writing header for length %lu chunk.\n",
 	     self, bufferSize));
     }
     if (!transport_write_bytes(lt, self->tr_outBuff,
 			       self->tr_outNext, err))
       return FALSE;
-    DEBUG(SUNRPCRM_DEBUG,
-	  (stderr,
-	   "sunrpcrm(%p): Writing length %lu chunk body.\n",
+    ILU_NOTE(SUNRPCRM_DEBUG,
+	  ("sunrpcrm(%p): Writing length %lu chunk body.\n",
 	   self, bufferSize));
     if (!transport_write_bytes(lt, b, bufferSize, err))
       return FALSE;
@@ -342,9 +392,8 @@ _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
       /* Possible improvement: avoid writing 0-length chunk here. */
       FORMAT_HEADER(self->tr_outBuff + p->outStart - 4,
 		    self->tr_outNext - p->outStart, FALSE);
-      DEBUG(SUNRPCRM_DEBUG,
-	    (stderr,
-	     "sunrpcrm(%p): Writing length %lu chunk.\n",
+      ILU_NOTE(SUNRPCRM_DEBUG,
+	    ("sunrpcrm(%p): Writing length %lu chunk.\n",
 	     self, self->tr_outNext - p->outStart));
       if (!transport_write_bytes(lt, self->tr_outBuff,
 				 self->tr_outNext, err))
@@ -354,23 +403,20 @@ _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
       self->tr_outNext = 4 + bufferSize;
     }
   }
-  _ilu_Assert((self->tr_outLimit - self->tr_outNext)>=16,
-	      "_sunrpcrm_WriteBytes failed to provide at least 16 bytes in output buffer");
-  return ILU_CLER(*err);
-}
-
-static          ilu_boolean
-_sunrpcrm_SendWholeMessage(ilu_Transport self, ilu_Message * msgh,
-			   ILU_ERRS((IoErrs)) * err)
-{
-  return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcReliable, FALSE);
+  if (!ilu_Check((self->tr_outLimit - self->tr_outNext) >= 16,
+		 err))
+    return FALSE;
+  return TRUE;
 }
 
+/*L1, L2 unconstrained*/
 #define PARSE_HEADER(ptr,last,size)				\
 	(last = ((ptr)[0] & 0x80) != 0,				\
 	 size = ((((ptr)[0] & 0x7F) << 24) + ((ptr)[1] << 16)	\
 		 + ((ptr)[2] << 8) + (ptr)[3]))
 
+/*Main Invariant holds; L2 >= {xmu, ymu}*/
+
 static          ilu_cardinal
 _sunrpcrm_ReadBytes(ilu_Transport self,
 		    ilu_bytes buffer,
@@ -378,7 +424,12 @@ _sunrpcrm_ReadBytes(ilu_Transport self,
 		    ilu_TransportReport * rpt,
 		    ILU_ERRS((IoErrs)) * err)
 {
-  return ReadWork(self, buffer, len, rpt, err);
+  SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
+  int             progress;
+  if (!p->busyIn)
+    return ILU_ERR_CONS1(internal, err, minor, ilu_im_bytesWithoutMsg,
+			 0);
+  return ReadWork(self, buffer, len, rpt, &progress, err);
 }
 
 static          ilu_cardinal
@@ -386,84 +437,95 @@ ReadWork(ilu_Transport self,
 	 ilu_bytes buffer,
 	 ilu_cardinal len,
 	 ilu_TransportReport * rpt,
+	 int *progress,
 	 ILU_ERRS((IoErrs)) * err)
 {
   SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
   ilu_cardinal    rem = self->tr_inLimit - self->tr_inNext;
-  if (!p->busyIn)
-    return ILU_ERR_CONS1(internal, err, minor, ilu_im_bytesWithoutMsg,
-			 0);
   if (rem > 0)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcInputSkipsBuff,
 			 0);
-  rpt->tr_eom = rpt->tr_eof = FALSE;
-  if (p->lastIn && p->inChunkRem == 0) {
-    rpt->tr_eom = TRUE;
-    return 0;
-  }
+  rpt->tr_eom = rpt->tr_eof = *progress = 0;
   if (p->inChunkRem > 0) {
     /* Chunk not done; read more of it. */
-    ilu_cardinal    lread;
-    self->tr_inNext = 0;
-    lread = transport_read_upto_bytes(p->lower, self->tr_inBuff,
-				      p->inSize, rpt, err);
-    DEBUG(SUNRPCRM_DEBUG,
-	  (stderr,
-      "sunrpcrm(%p): Read %lu more bytes, out of %lu remaining.\n",
-	   self, lread, p->inChunkRem));
-    p->inLength = lread;
-    p->inChunkRem -= (self->tr_inLimit = MIN(lread, p->inChunkRem));
+    ilu_cardinal    lread, limit, oldrem = p->inChunkRem;
+    if (buffer) {
+      limit = MIN(len, p->inChunkRem);
+      lread = transport_read_upto_bytes(p->lower, buffer,
+					limit, rpt, err);
+      buffer += lread;
+      len -= lread;
+      self->tr_inNext = self->tr_inLimit = p->inLength = 0;
+      p->inChunkRem -= lread;
+    } else {
+      lread = transport_read_upto_bytes(p->lower, self->tr_inBuff,
+					p->inSize, rpt, err);
+      self->tr_inNext = 0;
+      p->inLength = lread;
+      p->inChunkRem -= (self->tr_inLimit = MIN(lread, p->inChunkRem));
+    }
+    ILU_NOTE(SUNRPCRM_DEBUG,
+	  ("sunrpcrm(%p): Read %lu more bytes, out of %lu remaining.\n",
+	   self, lread, oldrem));
+    *progress = (lread > 0);
     if (ILU_ERRNOK(*err))
       return (buffer ? 0 : lread);
     if (rpt->tr_eof && self->tr_inLimit == p->inLength) {
       rpt->tr_eom = TRUE;
-      DEBUG(SUNRPCRM_DEBUG,
-	    (stderr,
-	     (p->inChunkRem == 0 && p->lastIn)
-	     ? "sunrpcrm(%p): chunk ends cleanly after remainder.\n"
-	     : "sunrpcrm(%p): unexpected EOF in chunk remainder!\n",
-	     self));
-      return (buffer ? 0 : lread);
+      if (p->inChunkRem == 0 && p->lastIn) {
+	ILU_NOTE(SUNRPCRM_DEBUG,
+	      ("sunrpcrm(%p): stream ends cleanly after msg remainder.\n",
+	       self));
+      } else {
+	ILU_NOTE(SUNRPCRM_DEBUG,
+	      ("sunrpcrm(%p): stream ends mid-message!\n",
+	       self));
+	return ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_eof,
+			     0);
+      }
     } else
       rpt->tr_eof = FALSE;
-  } else {
-    /* !p->lastIn; read another chunk. */
+    if (buffer)
+      return lread;
+  } else if (!p->lastIn) {
+    /* process another chunk. */
     ilu_cardinal    csize, cbsize;
     if (self->tr_inNext + 4 <= p->inLength) {
       /* Chunk's header already in buffer. */
-      csize = 1;		/* a place to set breakpoints */
+      *progress = 1;
     } else {
       /* Shift header fragment to buffer start, then read more */
       ilu_cardinal    l1 = p->inLength - self->tr_inNext;
       ilu_cardinal    lread, l2 = p->inSize - l1;
-      memcpy((void *) self->tr_inBuff,
-	     (void *) (self->tr_inBuff + self->tr_inNext), l1);
+      if (self->tr_inNext)
+	memcpy((void *) self->tr_inBuff,
+	       (void *) (self->tr_inBuff + self->tr_inNext), l1);
       self->tr_inNext = self->tr_inLimit = 0;
       p->inLength = l1;
       lread = transport_read_upto_bytes(p->lower,
 				   (void *) (self->tr_inBuff + l1),
 					l2, rpt, err);
-      DEBUG(SUNRPCRM_DEBUG,
-	    (stderr,
-	     "sunrpcrm(%p): Read %lu more bytes, in addition to %lu previous, of next chunk (incl header).\n",
+      ILU_NOTE(SUNRPCRM_DEBUG,
+	    ("sunrpcrm(%p): Read %lu more bytes, in addition to %lu previous, of next chunk (incl header).\n",
 	     self, lread, l1));
+      *progress = (lread > 0);
       p->inLength = l1 + lread;
       if (ILU_ERRNOK(*err))
 	return 0;
       if (l1 + lread == 0 && rpt->tr_eof && p->firstIn) {
 	/* clean EOF */
-	DEBUG(SUNRPCRM_DEBUG,
-	      (stderr,
-	       "sunrpcrm(%p): No more messages incoming.\n", self));
+	ILU_NOTE(SUNRPCRM_DEBUG,
+	      ("sunrpcrm(%p): No more messages incoming.\n", self));
 	rpt->tr_eom = TRUE;
 	return 0;
       }
-      if (l1 + lread < 4 && rpt->tr_eof) {	/* EOF */
+      if (l1 + lread < 4 && rpt->tr_eof) {
+	/* EOF before end of next header */
 	rpt->tr_eom = TRUE;
-	DEBUG(SUNRPCRM_DEBUG,
-	      (stderr,
-	       "sunrpcrm(%p): EOF while reading header.\n", self));
-	return 0;
+	ILU_NOTE(SUNRPCRM_DEBUG,
+	      ("sunrpcrm(%p): EOF while reading header!\n", self));
+	return ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_eof,
+			     0);
       }
       rpt->tr_eof = FALSE;
       /* If the EOF is real, it'll happen again. */
@@ -475,14 +537,17 @@ ReadWork(ilu_Transport self,
       cbsize = MIN(p->inLength - self->tr_inNext, csize);
       self->tr_inLimit = self->tr_inNext + cbsize;
       p->inChunkRem = csize - cbsize;
-      DEBUG(SUNRPCRM_DEBUG,
-	    (stderr,
-	     "sunrpcrm(%p): Parse header of length %lu chunk (last=%d), %lu in buffer.\n",
+      ILU_NOTE(SUNRPCRM_DEBUG,
+	    ("sunrpcrm(%p): Parse header of length %lu chunk (last=%d), %lu in buffer.\n",
 	     self, csize, p->lastIn != 0, cbsize));
     }
   }
   /* Now use what's in tr_inBuff */
   rem = self->tr_inLimit - self->tr_inNext;
+  if (rem == 0 && p->lastIn && p->inChunkRem == 0) {
+    rpt->tr_eom = TRUE;
+    return 0;
+  }
   if (buffer == NIL)
     return rem;
   if (rem > 0) {
@@ -496,18 +561,16 @@ ReadWork(ilu_Transport self,
   return 0;
 }
 
-/*L1, L2, Main unconstrained*/
-
-/*L2 >= {conn's iomu}*/
+/*L1.sup < trmu; L2 >= {xmu, ymu}*/
 
 static          ilu_boolean
 _sunrpcrm_Close(ilu_Transport self, ilu_integer * dfd,
-		ILU_ERRS((internal)) * err)
+		ILU_ERRS((bad_locks, broken_locks, internal)) * err)
 {
   SUNRPCRMParms   p = SUNRPCRMPARMS(transport_data(self));
   ilu_Transport   lt = p->lower;
-  DEBUG(SUNRPCRM_DEBUG,
-	(stderr, "sunrpcrm(%p): Close(lower=%p)\n", self, lt));
+  ILU_NOTE(SUNRPCRM_DEBUG,
+	("sunrpcrm(%p): Close(lower=%p)\n", self, lt));
   ilu_free(self->tr_inBuff);
   ilu_free(self->tr_outBuff);
   ilu_free(p);
@@ -515,13 +578,17 @@ _sunrpcrm_Close(ilu_Transport self, ilu_integer * dfd,
   return transport_close(lt, dfd, err);
 }
 
+/*L1, L2 unconstrained*/
+
 static struct _ilu_TransportClass_s myclass = {
   ilu_TRUE,			/* boundaried */
   ilu_TRUE,			/* reliable */
   _sunrpcrm_CloseDFd,
   SetInputHandler,
   _sunrpcrm_WaitForInput,
-  _sunrpcrm_Interrupt,
+  _sunrpcrm_InterruptST,
+  _sunrpcrm_DisableWait,
+  _sunrpcrm_EnableWait,
   _sunrpcrm_BeginMessage,
   _sunrpcrm_EndMessage,
   _sunrpcrm_SendWholeMessage,
@@ -530,35 +597,55 @@ static struct _ilu_TransportClass_s myclass = {
   _sunrpcrm_Close
 };
 
+/*L1.sup < trmu; L2 >= result's {xmu, ymu}*/
 static          ilu_Transport
-NewTrans(ilu_Transport lower, ILU_ERRS((no_memory)) * err)
+NewTrans(ilu_Transport lower, ilu_integer * dfd,
+	 ILU_ERRS((no_memory)) * err)
 {
-  ilu_Transport   ans;
-  SUNRPCRMParms   parms = (SUNRPCRMParms) ilu_malloc(sizeof(*parms));
+  ilu_Transport   ans = NIL;
+  SUNRPCRMParms   parms = NIL;
+  parms = (SUNRPCRMParms) ilu_MallocE(sizeof(*parms), err);
   if (parms == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*parms), NIL);
-  parms->busyIn = parms->busyOut = parms->inWedged = FALSE;
+    return NIL;
+  parms->busyIn = parms->busyOut = FALSE;
+  parms->inWedged = parms->skipping = FALSE;
   parms->inLength = parms->inChunkRem = 0;
   parms->inSize = parms->outSize = BUFFERSIZE;
   parms->lower = lower;
-  ans = (ilu_Transport) ilu_malloc(sizeof(*ans));
+  ans = (ilu_Transport) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
-  ans->tr_inBuff = ilu_malloc(parms->inSize);
-  ans->tr_outBuff = ilu_malloc(parms->outSize);
+    goto fale;
+  ans->tr_outBuff = NIL;
+  ans->tr_inBuff = ilu_MallocE(parms->inSize, err);
+  if (ans->tr_inBuff)
+    ans->tr_outBuff = ilu_MallocE(parms->outSize, err);
   if (ans->tr_inBuff == NIL || ans->tr_outBuff == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
+    goto fale;
   ans->tr_inNext = ans->tr_inLimit = 0;
   ans->tr_outNext = ans->tr_outLimit = 0;
   ans->tr_class = &myclass;
   ans->tr_data = parms;
-  DEBUG(SUNRPCRM_DEBUG,
-	(stderr, "sunrpcrm:NewTrans(lower=%p) = %p\n",
+  ILU_NOTE(SUNRPCRM_DEBUG,
+	("sunrpcrm:NewTrans(lower=%p) = %p\n",
 	 lower, ans));
   ILU_CLER(*err);
   return ans;
+fale:{
+    ILU_ERRS((bad_locks, broken_locks, internal)) cerr;
+    ilu_integer     cdfd = 0;
+    if (ans) {
+      ilu_free(ans->tr_inBuff);
+      ilu_free(ans->tr_outBuff);
+    }
+    ilu_free(parms);
+    transport_close(lower, &cdfd, &cerr);
+    ILU_HANDLED(cerr);
+    *dfd += cdfd;
+    return NIL;
+  }
 }
 
+/* Main Invariant holds */
 static          ilu_Transport
 _sunrpcrm_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
 			  ilu_integer *dfd, ilu_Passport pp,
@@ -566,47 +653,65 @@ _sunrpcrm_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
   ilu_Transport   lower;
-  lower = (*cp->lower->tcr_createTransport) (cp->lower, FALSE, dfd, pp, err);
+  lower = (*cp->lower->tcr_createTransport) (cp->lower, FALSE, dfd, pp,
+					     err);
   if (ILU_ERRNOK(*err))
     return NIL;
   if (lower->tr_class->tc_boundaried || !transport_reliable(lower))
     /* He promised! (We checked earlier) */
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcBug, NIL);
-  return NewTrans(lower, err);
+  /* pretend to acquire result's xmu, ymu */
+  return NewTrans(lower, dfd, err);
+  /* pretend to release result's xmu, ymu */
 }
 
-/*L1.sup < trmu; L2 unconstrained*/
+/*L1.sup < trmu; L2 >= {xmu, ymu}*/
 static ilu_integer MooringDFd(ilu_Mooring self, ilu_boolean add)
 {
   ilu_Mooring     lm = (ilu_Mooring) self->mo_data;
   return ((*lm->mo_dfd) (lm, add));
 }
 
-/*L1, L2 unconstrained*/
+/*L1.sup < trmu; L2 >= {xmu, ymu}*/
 static          ilu_boolean
-SetReqHandler(ilu_Mooring self, ilu_Server s,
-	      ilu_TransportInputHandler tih, ilu_refany tih_rock,
+SetReqHandler(ilu_Mooring self,
+	      ilu_TIH tih,
 	      ILU_ERRS((no_memory, imp_limit, no_resources,
 			broken_locks, internal)) * err)
 {
   ilu_Mooring     lm = (ilu_Mooring) self->mo_data;
-  return ((*lm->mo_set_req_handler) (lm, s, tih, tih_rock, err));
+  return ((*lm->mo_set_req_handler) (lm, tih, err));
 }
 
-/*Before: L1 = {s},
-          forall conn: (L2 >= {conn.iomu}) => (L2 >= {conn.callmu});
-  After: Main Invariant holds*/
+/* Main Invariant holds; L2 >= {ymu} */
 static          ilu_boolean
-WaitForReq(ilu_Mooring self, ilu_Server s,
-	   ILU_ERRS((broken_locks)) * err)
+WaitForReq(ilu_Mooring self, int *disabled,
+	   ILU_ERRS((interrupted, broken_locks)) * err)
 {
   ilu_Mooring     lm = (ilu_Mooring) self->mo_data;
-  return ((*lm->mo_wait_for_req) (lm, s, err));
+  return ((*lm->mo_wait_for_req) (lm, disabled, err));
 }
 
-/*L1, L2, Main unconstrained*/
+/* L1 >= {cmu}; L1.sup < trmu; L2 >= {xmu} */
+
+static          ilu_boolean
+DisableReqWait(ilu_Mooring self,
+	       ILU_ERRS((broken_locks, bad_param, internal)) * err)
+{
+  ilu_Mooring     lm = (ilu_Mooring) self->mo_data;
+  return ((*lm->mo_disableWait) (lm, err));
+}
+
+static          ilu_boolean
+EnableReqWait(ilu_Mooring self,
+	       ILU_ERRS((broken_locks, bad_param, internal)) * err)
+{
+  ilu_Mooring     lm = (ilu_Mooring) self->mo_data;
+  return ((*lm->mo_enableWait) (lm, err));
+}
+
+/* Main Invariant holds; L2 >= self's {xmu, ymu} */
 
-/*L1_sup < trmu*/
 static          ilu_Transport
 _sunrpcrm_AcceptClient(ilu_Mooring self, ilu_string * tinfo_out,
 		       ilu_integer *dfd, ilu_Passport pp,
@@ -614,28 +719,34 @@ _sunrpcrm_AcceptClient(ilu_Mooring self, ilu_string * tinfo_out,
 {
   ilu_Mooring     lm = (ilu_Mooring) self->mo_data;
   ilu_Transport   lower;
-  ilu_string      subtinfo;
+  ilu_string      subtinfo = NIL;
+  ilu_Transport   ans = NIL;
   lower = ((*lm->mo_accept_connection)
 	   (lm, tinfo_out ? &subtinfo : NIL, dfd, pp, err));
   if (ILU_ERRNOK(*err) || lower == NIL)
-    return NIL;
+    goto dun;
   if (tinfo_out) {
-    *tinfo_out = _ilu_Strcat3("sunrpcrm", " over ", subtinfo);
-    if (*tinfo_out == NIL)
-      return ILU_ERR_CONS1(no_memory, err, nbytes,
-			   10 + strlen(subtinfo), NIL);
+    *tinfo_out = ilu_Strcat3E("sunrpcrm", " over ", subtinfo, err);
+    if (ILU_ERRNOK(*err))
+      goto dun;
   }
-  return NewTrans(lower, err);
+  ans = NewTrans(lower, dfd, err);
+dun:
+  if (subtinfo)
+    ilu_free(subtinfo);
+  return ans;
 }
 
+/*L1.sup < trmu; L2 >= {xmu, ymu}*/
 static          ilu_boolean
-_sunrpcrm_CloseMooring(ilu_Mooring self, ilu_integer *dfd,
-		       ILU_ERRS((internal)) * err)
+_sunrpcrm_CloseMooring(ilu_Mooring self, ilu_integer * dfd,
+		       ILU_ERRS((bad_locks, broken_locks,
+				 internal)) * err)
 {
   ilu_Mooring     lower = (ilu_Mooring) self->mo_data;
-  DEBUG(SUNRPCRM_DEBUG,
-	(stderr, "sunrpcrm(%p): CloseMooring(lower=%p)\n",
-	 self, lower));
+  ILU_NOTE(SUNRPCRM_DEBUG,
+	   ("sunrpcrm(%p): CloseMooring(lower=%p)\n",
+	    self, lower));
   if (!(*lower->mo_close) (lower, dfd, err))
     return FALSE;
   ilu_free(self);
@@ -643,22 +754,25 @@ _sunrpcrm_CloseMooring(ilu_Mooring self, ilu_integer *dfd,
 }
 
 /*L1, L2 unconstrained*/
+
 static struct _ilu_Mooring_s mooringProto = {
   MooringDFd,
   SetReqHandler,
   WaitForReq,
+  DisableReqWait,
+  EnableReqWait,
   _sunrpcrm_AcceptClient,
   _sunrpcrm_CloseMooring,
   NIL				/* data */
 };
 
-/*L1_sup < trmu*/
+/*L1.sup < trmu*/
 static          ilu_Mooring
 _sunrpcrm_CreateMooring(ilu_TransportCreator self,
 			ilu_TransportInfo * tinfo_out,
 			ilu_boolean buffer,
 			ilu_integer *dfd,
-			ilu_Passport pp,	/* unused in this transport */
+			ilu_Passport pp,	/* unused here */
 			ILU_ERRS((no_memory)) * err)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
@@ -668,9 +782,9 @@ _sunrpcrm_CreateMooring(ilu_TransportCreator self,
 	   (cp->lower, tinfo_out ? &subtinfo : NIL, FALSE, dfd, pp, err));
   if (ILU_ERRNOK(*err))
     return NIL;
-  ans = (ilu_Mooring) ilu_malloc(sizeof(*ans));
+  ans = (ilu_Mooring) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
-    return (ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL));
+    return NIL;
   if (tinfo_out) {
     *tinfo_out = _ilu_ConcatTinfo ("sunrpcrm", subtinfo, err);
     if (ILU_ERRNOK(*err))
@@ -678,16 +792,14 @@ _sunrpcrm_CreateMooring(ilu_TransportCreator self,
     else
       ilu_free(subtinfo);
   }
-  DEBUG(SUNRPCRM_DEBUG,
-	(stderr, "sunrpcrm(%p): CreateMooring(lower=%p)\n",
+  ILU_NOTE(SUNRPCRM_DEBUG,
+	("sunrpcrm(%p): CreateMooring(lower=%p)\n",
 	 ans, lower));
   *ans = mooringProto;
   ans->mo_data = lower;
   return (ans);
 }
 
-/*L1, L2 unconstrained*/
-
 static void _sunrpcrm_CloseCreator(ilu_TransportCreator self)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
@@ -700,6 +812,8 @@ static void _sunrpcrm_CloseCreator(ilu_TransportCreator self)
 static struct _ilu_TransportCreator_s myCreatorProto = {
   TRUE,				/* boundaried */
   TRUE,				/* reliable */
+  0,				/* tcr_holds */
+  FALSE,			/* tcr_wantClose */
   _sunrpcrm_FdUsage,
   _sunrpcrm_CreateTransport,
   _sunrpcrm_CreateMooring,
@@ -707,7 +821,7 @@ static struct _ilu_TransportCreator_s myCreatorProto = {
   NIL				/* data */
 };
 
-/*L1_sup < trmu*/
+/*L1.sup < trmu*/
 ilu_TransportCreator
 _ilu_sunrpcrm_TransportCreator(ilu_TransportInfo tinfo,
 			       ILU_ERRS((no_memory,
@@ -718,9 +832,9 @@ _ilu_sunrpcrm_TransportCreator(ilu_TransportInfo tinfo,
   cp = _sunrpcrm_InterpretInfo(tinfo, err);
   if (ILU_ERRNOK(*err))
     return NIL;
-  ans = (ilu_TransportCreator) ilu_malloc(sizeof(*ans));
+  ans = (ilu_TransportCreator) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
+    return NIL;
   *ans = myCreatorProto;
   ans->tcr_data = cp;
   ILU_CLER(*err);
