@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: threads.c,v 1.22 1996/07/11 16:35:14 spreitze Exp $ */
-/* Last edited by Mike Spreitzer July 11, 1996 9:17 am PDT */
+/* $Id: threads.c,v 1.38 1997/11/17 18:46:37 spreitze Exp $ */
+/* Last edited by Mike Spreitzer November 17, 1997 9:22 am PST */
 
 /* This code was kindly donated by Digital Creations, Inc. */
 
@@ -45,7 +45,7 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #ifdef ENABLE_DEBUGGING
 DWORD ShowLastError(char* pc_file, int i_line) {
 	DWORD d_result = GetLastError();
-	DEBUG(THREAD_DEBUG, (stderr, "GetLastError() = 0x%x in %s, line %d\n", d_result, pc_file, i_line));
+	ILU_NOTE(THREAD_DEBUG, ("GetLastError() = 0x%x in %s, line %d\n", d_result, pc_file, i_line));
 	return d_result;
 }
 #endif
@@ -297,11 +297,11 @@ static void Lock_Mutex_Work(DECLARE_MUTEX(*pm))
 {
   char            fm[sizeof(*pm) * 2 + 1];
   FmtBytes(fm, (unsigned char *) pm, sizeof(*pm));
-  DEBUG(LOCK_DEBUG, (stderr, "Lock_Mutex(*%p = {%s}) entered.\n",
+  ILU_NOTE(LOCK_DEBUG, ("Lock_Mutex(*%p = {%s}) entered.\n",
 		     pm, fm));
   LOCK_MUTEX(*pm);
   FmtBytes(fm, (unsigned char *) pm, sizeof(*pm));
-  DEBUG(LOCK_DEBUG, (stderr, "Lock_Mutex(*%p = {%s}) finished.\n",
+  ILU_NOTE(LOCK_DEBUG, ("Lock_Mutex(*%p = {%s}) finished.\n",
 		     pm, fm));
 }
 
@@ -310,7 +310,7 @@ static void Unlock_Mutex_Work(DECLARE_MUTEX(*pm))
   char            fm[sizeof(*pm) * 2 + 1];
   UNLOCK_MUTEX(*pm);
   FmtBytes(fm, (unsigned char *) pm, sizeof(*pm));
-  DEBUG(LOCK_DEBUG, (stderr, "Unlock_Mutex(*%p = {%s}) finished.\n",
+  ILU_NOTE(LOCK_DEBUG, ("Unlock_Mutex(*%p = {%s}) finished.\n",
 		     pm, fm));
 }
 
@@ -353,59 +353,66 @@ typedef struct
 /* ----------------------------------------------- */
 /* create a mutex                                  */
 
-static ilu_Mutex
-  ilukt_LT_mcreate(ilu_string d1, ilu_string d2)
+static          ilu_Mutex
+ilukt_LT_mcreate(ilu_string d1, ilu_string d2)
 {
-  ilukt_Mutex *new_mutex = (ilukt_Mutex *)ilu_malloc(sizeof(ilukt_Mutex));
+  ilukt_Mutex    *new_mutex;
+  new_mutex = (ilukt_Mutex *) ilu_malloc(sizeof(ilukt_Mutex));
 
-  DEBUG(LOCK_DEBUG, (stderr, "ilukt_LT_mcreate('%s','%s')\n", d1, d2));
+  ILU_NOTE(LOCK_DEBUG, ("ilukt_LT_mcreate('%s','%s')\n", d1, d2));
 
   if (!new_mutex)
-    /* couldn't allocate space, so fail out */
-    {
-      return NIL;
-    }
-
-  if (!INIT_MUTEX(new_mutex->mutex))
-    /* mutex initialization failed for some reason, so clean up and fail out */
-    {
+    /* Couldn't allocate space, so fail out. */
+    return NIL;
+  if (!INIT_MUTEX(new_mutex->mutex)) {
+    /*
+     * Mutex initialization failed for some reason, so clean up and
+     * fail out.
+     */
+    ilu_free(new_mutex);
+    return NIL;
+  }
+  if (d1 != NIL) {
+    new_mutex->d1 = _ilu_Strdup(d1);
+    if (new_mutex->d1 == NIL) {
+      DESTROY_MUTEX(new_mutex->mutex);
       ilu_free(new_mutex);
       return NIL;
     }
-
-  if (d1 != NIL)
-    {
-      new_mutex->d1 = _ilu_Strdup(d1);
-      if (new_mutex->d1 == NIL)
-	{
-	  DESTROY_MUTEX(new_mutex->mutex);
-	  ilu_free(new_mutex);
-	  return NIL;
-	}
+  } else
+    new_mutex->d1 = NIL;
+  if (d2 != NIL) {
+    new_mutex->d2 = _ilu_Strdup(d2);
+    if (new_mutex->d2 == NIL) {
+      DESTROY_MUTEX(new_mutex->mutex);
+      if (new_mutex->d1 != NIL) {
+	ilu_free(new_mutex->d1);
+      }
+      ilu_free(new_mutex);
+      return NIL;
     }
-  else new_mutex->d1 = NIL;
-
-  if (d2 != NIL)
-    {
-      new_mutex->d2 = _ilu_Strdup(d2);
-      if (new_mutex->d2 == NIL)
-	{
-	  DESTROY_MUTEX(new_mutex->mutex);
-	  if (new_mutex->d1 != NIL)
-	    {
-	      ilu_free(new_mutex->d1);
-	    }
-	  ilu_free(new_mutex);
-	  return NIL;
-	}
-    }
-  else new_mutex->d2 = NIL;
+  } else
+    new_mutex->d2 = NIL;
 
   new_mutex->locked = ilu_FALSE;
 
-  DEBUG(LOCK_DEBUG, (stderr, "ilukt_LT_mcreate succeeded: %p\n", new_mutex));
+  ILU_NOTE(LOCK_DEBUG,
+	("ilukt_LT_mcreate succeeded: %p\n", new_mutex));
 
-  return (ilu_Mutex) new_mutex;
+  return new_mutex;
+}
+
+static void 
+ilukt_LT_mdestroy(ilu_Mutex m,
+		  ILU_ERRS((bad_param, bad_locks)) * err)
+{
+  CAST_MUTEX(m, mutex_obj);
+  ASSERT_VALID_MUTEX(mutex_obj, err);
+  ilu_free(mutex_obj->d1);
+  ilu_free(mutex_obj->d2);
+  DESTROY_MUTEX(mutex_obj->mutex);
+  ilu_free(mutex_obj);
+  ILU_CLER(*err);
 }
 
 
@@ -444,10 +451,10 @@ static void
   current_thread = GET_CURRENT_THREAD();
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG) {
+  if (ilu_DebugLevel & LOCK_DEBUG) {
     char           *ftt = FmtThread(GET_CURRENT_THREAD());
-    DEBUG(LOCK_DEBUG,
-	  (stderr, "(thread %s) waiting for (mutex [%s%s])\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("(thread %s) waiting for (mutex [%s %s])\n",
 	   ftt, mutex_obj->d1, mutex_obj->d2));
     FreeThreadFmt(ftt);
   }
@@ -464,10 +471,10 @@ static void
   ILU_LOCK_MUTEX(mutex_obj->mutex);
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG) {
+  if (ilu_DebugLevel & LOCK_DEBUG) {
     char           *ftt = FmtThread(GET_CURRENT_THREAD());
-    DEBUG(LOCK_DEBUG,
-	  (stderr, "(thread %s) acquired (mutex [%s%s])\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("(thread %s) acquired (mutex [%s %s])\n",
 	   ftt, mutex_obj->d1, mutex_obj->d2));
     FreeThreadFmt(ftt);
   }
@@ -513,10 +520,10 @@ static void
   current_thread = GET_CURRENT_THREAD();
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG) {
+  if (ilu_DebugLevel & LOCK_DEBUG) {
     char           *ftt = FmtThread(GET_CURRENT_THREAD());
-    DEBUG(LOCK_DEBUG,
-	  (stderr, "(thread %s) releasing (mutex [%s%s])\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("(thread %s) releasing (mutex [%s %s])\n",
 	   ftt, mutex_obj->d1, mutex_obj->d2));
     FreeThreadFmt(ftt);
   }
@@ -546,7 +553,7 @@ static ilu_Condition
   ilukt_Condition *new_condition;
   new_condition = (ilukt_Condition *)ilu_malloc(sizeof(ilukt_Condition));
 
-  DEBUG(LOCK_DEBUG, (stderr, "ilukt_LT_ccreate called\n"));
+  ILU_NOTE(LOCK_DEBUG, ("ilukt_LT_ccreate called\n"));
 
   if (new_condition == NIL)
     return NIL;
@@ -585,7 +592,7 @@ static ilu_Condition
     }
   else new_condition->d2 = NIL;
 
-  DEBUG(LOCK_DEBUG, (stderr, "ilukt_LT_ccreate succeeded: %p\n", new_condition));
+  ILU_NOTE(LOCK_DEBUG, ("ilukt_LT_ccreate succeeded: %p\n", new_condition));
 
   return new_condition;
 }
@@ -625,10 +632,10 @@ static void
   CAST_CONDITION(c, condition_obj);
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG) {
+  if (ilu_DebugLevel & LOCK_DEBUG) {
     char           *ftt = FmtThread(GET_CURRENT_THREAD());
-    DEBUG(LOCK_DEBUG,
-	  (stderr, "(thread %s) notifying (condition [%s%s])\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("(thread %s) notifying (condition [%s %s])\n",
 	   ftt,
 	   (condition_obj->d1 ? condition_obj->d1 : ""),
 	   (condition_obj->d2 ? condition_obj->d2 : "")));
@@ -654,8 +661,8 @@ static void
 {
   CAST_CONDITION(c, condition_obj);
 
-  DEBUG(LOCK_DEBUG,
-	(stderr, "ilukt_LT_cdestroy called: (condition = %p)\n",
+  ILU_NOTE(LOCK_DEBUG,
+	("ilukt_LT_cdestroy called: (condition = %p)\n",
 	 condition_obj));
 
   ASSERT_VALID_CONDITION(condition_obj, err);
@@ -683,76 +690,80 @@ static void
 /* Waits on the condition                          */
 
 static void
-  ilukt_LT_wait(ilu_Condition c, ilu_Mutex m, ilu_Mutex m2, ILU_ERRS((bad_param, bad_locks)) *err)
+ilukt_LT_wait(ilu_Condition c, ilu_Mutex m, ilu_Mutex m2,
+	      ILU_ERRS((bad_param, bad_locks)) * err)
 {
   CAST_CONDITION(c, condition_obj);
   CAST_MUTEX(m, mutex_obj);
+  CAST_MUTEX(m2, mutex_obj2);
   DECLARE_THREAD(current_thread);
-  ilukt_Mutex    *mutex_obj2 = (ilukt_Mutex *) m2;
   ilu_string      ftt = no_ftt;
   ilu_string      d1 = (condition_obj->d1 ? condition_obj->d1 : "");
   ilu_string      d2 = (condition_obj->d2 ? condition_obj->d2 : "");
   current_thread = GET_CURRENT_THREAD();
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG)
-      ftt = FmtThread(current_thread);
-#endif /* ENABLE_DEBUGGING */
+  if (ilu_DebugLevel & LOCK_DEBUG)
+    ftt = FmtThread(current_thread);
+#endif				/* ENABLE_DEBUGGING */
 
   ASSERT_VALID_CONDITION(condition_obj, err);
   ASSERT_VALID_MUTEX(mutex_obj, err);
   ASSERT_VALID_MUTEX(mutex_obj2, err);
-    
+
   ASSERT_HOLD_MUTEX(mutex_obj, err);
   if (mutex_obj != mutex_obj2) {
     ASSERT_HOLD_MUTEX(mutex_obj2, err);
 
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-    "(thread %s) unlocking 2nd mutex [%s%s] for condition [%s%s]\n",
-	   ftt, mutex_obj2->d1, mutex_obj2->d2, d1, d2));
+    ILU_NOTE(LOCK_DEBUG,
+	     ("(thread %s) waiting on condition [%s %s]"
+	      " wrt mutexes [%s %s] & [%s %s].\n",
+	      ftt, d1, d2, mutex_obj2->d1, mutex_obj2->d2,
+	      mutex_obj->d1, mutex_obj->d2));
 
     ILU_UNLOCK_MUTEX(mutex_obj2->mutex);
+  } else {
+    ILU_NOTE(LOCK_DEBUG,
+	     ("(thread %s) waiting on condition [%s %s]"
+	      " wrt mutex [%s %s]\n",
+	      ftt, d1, d2, mutex_obj->d1, mutex_obj->d2));
   }
-  
-  DEBUG(LOCK_DEBUG,
-	(stderr,
-    "(thread %s) waiting on (condition [%s%s]) wrt mutex [%s%s]\n",
-	 ftt, d1, d2, mutex_obj->d1, mutex_obj->d2));
 
   CONDITION_WAIT(condition_obj->condition, mutex_obj->mutex);
-  
+
   if (mutex_obj == mutex_obj2) {
     mutex_obj->owner_thread = current_thread;
     mutex_obj->locked = ilu_TRUE;
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-	   "(thread %s) resumed from wait on cond [%s%s] wrt mutex[%s%s]\n",
-	   ftt, d1, d2, mutex_obj->d1, mutex_obj->d2));
+    ILU_NOTE(LOCK_DEBUG,
+	     ("(thread %s) resumed from wait on cond [%s %s]"
+	      " wrt mutex [%s %s].\n",
+	      ftt, d1, d2, mutex_obj->d1, mutex_obj->d2));
   } else {
     ILU_UNLOCK_MUTEX(mutex_obj->mutex);
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-	   "(thread %s) resuming from wait on cond [%s%s], relocking mutex[%s%s]\n",
-	   ftt, d1, d2, mutex_obj2->d1, mutex_obj2->d2));
+    ILU_NOTE(LOCK_DEBUG,
+	     ("(thread %s) resuming from wait on cond [%s %s]"
+	  " wrt mutexes [%s %s] & [%s %s]; relocking lower one.\n",
+	      ftt, d1, d2, mutex_obj2->d1, mutex_obj2->d2,
+	      mutex_obj->d1, mutex_obj->d2));
     ILU_LOCK_MUTEX(mutex_obj2->mutex);
     mutex_obj2->owner_thread = current_thread;
     mutex_obj2->locked = ilu_TRUE;
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-	   "(thread %s) resuming from wait on cond [%s%s], relocking mutex[%s%s]\n",
-	   ftt, d1, d2, mutex_obj->d1, mutex_obj->d2));
+    ILU_NOTE(LOCK_DEBUG,
+	     ("(thread %s) resuming from wait on cond [%s %s]"
+	 " wrt mutexes [%s %s] & [%s %s]; relocking higher one.\n",
+	      ftt, d1, d2, mutex_obj2->d1, mutex_obj2->d2,
+	      mutex_obj->d1, mutex_obj->d2));
     ILU_LOCK_MUTEX(mutex_obj->mutex);
     mutex_obj->owner_thread = current_thread;
     mutex_obj->locked = ilu_TRUE;
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-	   "(thread %s) resumed from wait on cond [%s%s] wrt mutexes [%s%s] and [%s%s]\n",
-	   ftt, d1, d2, mutex_obj->d1, mutex_obj->d2,
-	   mutex_obj2->d1, mutex_obj2->d2));
+    ILU_NOTE(LOCK_DEBUG,
+	     ("(thread %s) resumed from wait on cond [%s %s]"
+	      " wrt mutexes [%s %s] & [%s %s].\n",
+	      ftt, d1, d2, mutex_obj2->d1, mutex_obj2->d2,
+	      mutex_obj->d1, mutex_obj->d2));
   }
-  
-  if (_ilu_DebugLevel & LOCK_DEBUG)
+
+  if (ilu_DebugLevel & LOCK_DEBUG)
     FreeThreadFmt(ftt);
 
   ILU_CLER(*err);
@@ -771,6 +782,7 @@ static ilu_LockTech
     ilukt_LT_acquire,
     ilukt_LT_hold,
     ilukt_LT_release,
+    ilukt_LT_mdestroy,
     ilukt_LT_ccreate,
     ilukt_LT_cuncons,
     ilukt_LT_notify,
@@ -783,27 +795,33 @@ static ilu_LockTech
 /* read wait                                       */
 
 static void
-  ilukt_WT_read_wait(int fd, ilu_boolean *sure, ilu_FineTime *limit,
-                     ILU_ERRS((interrupted)) *err)
+ilukt_WT_read_wait(int fd, int auxfd, ilu_boolean * sure,
+		   ilu_FineTime * limit, ILU_ERRS((interrupted)) * err)
 {
   fd_set          read_set, exn_set;
-  int             stat;
+  int             stat, maxfd = fd;
   struct timeval  time, *time_p;
   ilu_string      ftt = no_ftt;
   
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG)
+  if (ilu_DebugLevel & LOCK_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
 #endif /* ENABLE_DEBUGGING */
 
-  DEBUG(LOCK_DEBUG,
-	(stderr, "ilukt_WT_read_wait(%d, thr=%s) entered.\n",
+  ILU_NOTE(LOCK_DEBUG,
+	("ilukt_WT_read_wait(%d, thr=%s) entered.\n",
 	 fd, ftt));
 
   FD_ZERO(&read_set);
   FD_SET(fd, &read_set);
   FD_ZERO(&exn_set);
   FD_SET(fd, &exn_set);
+  if (auxfd >= 0) {
+    FD_SET(auxfd, &read_set);
+    FD_SET(auxfd, &exn_set);
+    if (auxfd > fd)
+      maxfd = auxfd;
+  }
 
   if (limit != 0) {
     time.tv_sec = limit->ft_s;
@@ -813,15 +831,18 @@ static void
     time_p = NULL;
   }
 
-  if ((stat = select(fd + 1, &read_set, NULL, &exn_set, time_p)) > 0) {
+  if ((stat = select(maxfd + 1, &read_set, NULL, &exn_set, time_p)) > 0) {
     *sure = ilu_TRUE;
 
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-	"ilukt_WT_read_wait(%d, thr=%s) got input=%s && exn=%s.\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("ilukt_WT_read_wait(%d, thr=%s) got input=%s, exn=%s"
+	   " (aux input=%s, exn=%s).\n",
 	   fd, ftt,
 	   (FD_ISSET(fd, &read_set) ? "T" : "F"),
-	   (FD_ISSET(fd, &exn_set) ? "T" : "F")));
+	   (FD_ISSET(fd, &exn_set) ? "T" : "F"),
+	   (auxfd >= 0 && FD_ISSET(auxfd, &read_set) ? "T" : "F"),
+	   (auxfd >= 0 && FD_ISSET(auxfd, &exn_set) ? "T" : "F")
+	   ));
 
     ILU_CLER(*err);
 
@@ -831,44 +852,29 @@ static void
     switch (theerr) {
     case SOCKERRID(INTR):
       *sure = ilu_FALSE;
-      DEBUG(LOCK_DEBUG,
-	    (stderr,
-	     "ilukt_WT_read_wait(%d, thr=%s) interrupted.\n",
+      ILU_NOTE(LOCK_DEBUG,
+	    ("ilukt_WT_read_wait(%d, thr=%s) interrupted.\n",
 	     fd, ftt));
       ILU_ERR_CONS1(interrupted, err, ilu_interruptSet, 0, 0);
       break;
-#ifdef WIN32
-    case SOCKERRID(NOTSOCK):
-#else
-    case SOCKERRID(BADF):
-#endif
-      /* FD closed in another thread */
-      *sure = TRUE;
-      DEBUG(LOCK_DEBUG,
-	    (stderr,
-	     "ilukt_WT_read_wait(%d, thr=%s) assuming FD closed.\n",
-	     fd, ftt));
-      ILU_CLER(*err);
-      break;
     default:
       ASSERT(FALSE, buf,
 	     (buf,
 	  "ilukt_WT_read_wait(%d) got select failure, errno=%d=%s",
-	      fd, errno, strerror(errno)));
+	      fd, theerr, strerror(theerr)));
     }
 
   } else {
     *sure = ilu_FALSE;
 
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-	   "ilukt_WT_read_wait(%d, thr=%s) timed out.\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("ilukt_WT_read_wait(%d, thr=%s) timed out.\n",
 	   fd, ftt));
 
     ILU_CLER(*err);
   }
 
-  if (_ilu_DebugLevel & LOCK_DEBUG)
+  if (ilu_DebugLevel & LOCK_DEBUG)
     FreeThreadFmt(ftt);
 
   return;
@@ -879,26 +885,34 @@ static void
 /* write wait                                      */
 
 static void
-  ilukt_WT_write_wait(int fd, ilu_boolean *sure, ilu_FineTime *limit, ILU_ERRS((interrupted)) *err)
+ilukt_WT_write_wait(int fd, int auxfd, ilu_boolean * sure,
+	       ilu_FineTime * limit, ILU_ERRS((interrupted)) * err)
 {
-  fd_set          write_set, exn_set;
-  int             stat;
+  fd_set          read_set, write_set, exn_set;
+  int             stat, maxfd = fd;
   struct timeval  time, *time_p;
   ilu_string      ftt = no_ftt;
   
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & LOCK_DEBUG)
+  if (ilu_DebugLevel & LOCK_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
 #endif /* ENABLE_DEBUGGING */
 
-  DEBUG(LOCK_DEBUG,
-	(stderr, "ilukt_WT_write_wait(%d, thr=%s) entered.\n",
+  ILU_NOTE(LOCK_DEBUG,
+	("ilukt_WT_write_wait(%d, thr=%s) entered.\n",
 	 fd, ftt));
 
+  FD_ZERO(&read_set);
   FD_ZERO(&write_set);
   FD_SET(fd, &write_set);
   FD_ZERO(&exn_set);
   FD_SET(fd, &exn_set);
+  if (auxfd >= 0) {
+    FD_SET(auxfd, &read_set);
+    FD_SET(auxfd, &exn_set);
+    if (auxfd > fd)
+      maxfd = auxfd;
+  }
 
   if (limit != 0) {
     time.tv_sec = limit->ft_s;
@@ -908,60 +922,49 @@ static void
     time_p = NULL;
   }
 
-  if ((stat = select(fd + 1, NULL, &write_set, &exn_set, time_p)) > 0) {
+  stat = select(maxfd + 1, &read_set, &write_set, &exn_set, time_p);
+  if (stat > 0) {
     *sure = ilu_TRUE;
 
-    DEBUG(LOCK_DEBUG,
-	  (stderr,
-      "ilukt_WT_write_wait(%d, thr=%s) got output=%s && exn=%s.\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("ilukt_WT_write_wait(%d, thr=%s) got output=%s && exn=%s"
+	   " (and aux input=%s, exn=%s).\n",
 	   fd, ftt,
 	   (FD_ISSET(fd, &write_set) ? "T" : "F"),
-	   (FD_ISSET(fd, &exn_set) ? "T" : "F")
+	   (FD_ISSET(fd, &exn_set) ? "T" : "F"),
+	   (auxfd >= 0 && FD_ISSET(auxfd, &read_set) ? "T" : "F"),
+	   (auxfd >= 0 && FD_ISSET(auxfd, &exn_set) ? "T" : "F")
 	   ));
 
     ILU_CLER(*err);
   } else if (stat == -1) {	/* an error occured; hopefully an
 				 * interrupt! */
-    int             theerr = errno;
+    int             theerr = sockerrno;
     switch (theerr) {
     case SOCKERRID(INTR):
       *sure = ilu_FALSE;
-      DEBUG(LOCK_DEBUG,
-	    (stderr,
-	     "ilukt_WT_write_wait(%d, thr=%s) interrupted.\n",
+      ILU_NOTE(LOCK_DEBUG,
+	    ("ilukt_WT_write_wait(%d, thr=%s) interrupted.\n",
 	     fd, ftt));
       ILU_ERR_CONS1(interrupted, err, ilu_interruptSet, 0, 0);
       break;
-#ifdef WIN32
-    case SOCKERRID(NOTSOCK):
-#else
-    case SOCKERRID(BADF):
-#endif
-      /* FD closed in another thread */
-      *sure = TRUE;
-      DEBUG(LOCK_DEBUG,
-	    (stderr,
-	   "ilukt_WT_write_wait(%d, thr=%s) assuming FD closed.\n",
-	     fd, ftt));
-      ILU_CLER(*err);
-      break;
     default:
-      ASSERT((errno == SOCKERRID(INTR)), buf,
+      ASSERT((theerr == SOCKERRID(INTR)), buf,
 	(buf, "ilukt_WT_write_wait(%d):select failed, errno=%d=%s",
-	 fd, errno, strerror(errno)));
+	 fd, theerr, strerror(theerr)));
     }
 
   } else {
     *sure = ilu_FALSE;
 
-    DEBUG(LOCK_DEBUG,
-	  (stderr, "ilukt_WT_write_wait(%d, thr=%s) timed out.\n",
+    ILU_NOTE(LOCK_DEBUG,
+	  ("ilukt_WT_write_wait(%d, thr=%s) timed out.\n",
 	   fd, ftt));
 
     ILU_CLER(*err);
   }
 
-  if (_ilu_DebugLevel & LOCK_DEBUG)
+  if (ilu_DebugLevel & LOCK_DEBUG)
     FreeThreadFmt(ftt);
 
   return;
@@ -980,14 +983,14 @@ static ilu_WaitTech
 /* ----------------------------------------------- */
 /* Alarm struct                                    */
 
-typedef struct
-{
+typedef struct {
   DECLARE_MUTEX(alarm_mutex);
   DECLARE_CONDITION(alarm_condition);
-  ilu_FineTime fire_time;
-  void (*proc)(ilu_private rock);
-  ilu_private rock;
-} ilukt_Alarm;
+  ilu_FineTime    fire_time;
+  void            (*proc) (ilu_private rock);
+  ilu_private     rock;
+  ilu_boolean     going;
+}               ilukt_Alarm;
 
 
 /* ----------------------------------------------- */
@@ -1003,6 +1006,8 @@ typedef struct
 /* ----------------------------------------------- */
 /* function that basically runs the thread's procedure */
 
+DECLARE_THREADKEY(_ilukt_implicit_args_key);
+
 #ifndef WIN32
 static void *
   run_function(void *func_and_args)
@@ -1012,20 +1017,18 @@ static void *
   proc_args = (function_and_argument *) func_and_args;
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & THREAD_DEBUG)
+  if (ilu_DebugLevel & THREAD_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
 #endif
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "run_function(fn = %p, arg = %p, thr = %s) starting.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("run_function(fn = %p, arg = %p, thr = %s) starting.\n",
 	 proc_args->proc, proc_args->arg, ftt));
 
   proc_args->proc(proc_args->arg);
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "run_function(fn = %p, arg = %p, thr = %s) finishing.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("run_function(fn = %p, arg = %p, thr = %s) finishing.\n",
 	 proc_args->proc, proc_args->arg, ftt));
 
   ilu_free(proc_args);
@@ -1034,32 +1037,37 @@ static void *
   return NULL;
 }
 #else
+static void (*_ilukt_win32_attribute_finalizer)(void *) = 0;
+
 static void 
   run_function(void *func_and_args)
 {
   function_and_argument *proc_args;
   ilu_string      ftt = no_ftt;
+  void *val;
+
   proc_args = (function_and_argument *)func_and_args;
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & THREAD_DEBUG)
+  if (ilu_DebugLevel & THREAD_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
 #endif
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "run_function(fn = %p, arg = %p, thr = %s) starting.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("run_function(fn = %p, arg = %p, thr = %s) starting.\n",
 	 proc_args->proc, proc_args->arg, ftt));
 
   proc_args->proc(proc_args->arg);
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "run_function(fn = %p, arg = %p, thr = %s) finishing.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("run_function(fn = %p, arg = %p, thr = %s) finishing.\n",
 	 proc_args->proc, proc_args->arg, ftt));
 
   ilu_free(proc_args);
   FreeThreadFmt(ftt);
+  if (((val = TlsGetValue(_ilukt_implicit_args_key)) != ILU_NIL) &&
+      (_ilukt_win32_attribute_finalizer != (void (*)(void *)) 0))
+    (*_ilukt_win32_attribute_finalizer)(val);
 }
 #endif
 
@@ -1072,36 +1080,34 @@ static void *
 #else
 static void
 #endif
-  ilukt_ML_run_alarm(void *alarm_in_disguise)
+ilukt_ML_run_alarm(void *alarm_in_disguise)
 {
   ilukt_Alarm    *alarm = (ilukt_Alarm *) alarm_in_disguise;
   ilu_string      ftt = no_ftt;
 #ifdef ILU_SOLARIS2_THREADS
   timestruc_t     set_time;
 #endif
-#ifdef ILU_POSIX_THREADS
+#if (defined(ILU_POSIX_THREADS) || defined(ILU_DCE_THREADS))
   struct timespec set_time;
   int             additional_info;
 #endif
   int             err;
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & THREAD_DEBUG)
+  if (ilu_DebugLevel & THREAD_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_run_alarm(%p): started on thread %s.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_run_alarm(%p): started on thread %s.\n",
 	 alarm, ftt));
 #endif
 
   ILU_LOCK_MUTEX(alarm->alarm_mutex);
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_run_alarm(%p): mutex locked.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_run_alarm(%p): mutex locked.\n",
 	 alarm));
 
-  while (1) {
+  while (alarm->going) {
     if (alarm->fire_time.ft_s != 0) {
 
 #ifdef ILU_WIN32_THREADS
@@ -1114,25 +1120,23 @@ static void
 	relative_time_in_msec = delta.ft_s * 1000
 	  + ilu_rescale(delta.ft_t, ilu_FineTimeRate, 1000);
 
-	DEBUG(THREAD_DEBUG,
-	      (stderr,
-	"ilukt_ML_run_alarm(%p): waiting for (%ld) milliseconds.\n",
+	ILU_NOTE(THREAD_DEBUG,
+	      ("ilukt_ML_run_alarm(%p): waiting for (%ld) milliseconds.\n",
 	       alarm, relative_time_in_msec));
 
 	err = CONDITION_TIMEDWAIT(alarm->alarm_condition,
 				  alarm->alarm_mutex,
 				  relative_time_in_msec);
       } else
-	err = 0;
+	err = WAIT_TIMEOUT;
 
 #else
       set_time.tv_sec = alarm->fire_time.ft_s;
       set_time.tv_nsec = ilu_rescale(alarm->fire_time.ft_t,
 				     ilu_FineTimeRate, 1000000000);
 
-      DEBUG(THREAD_DEBUG,
-	    (stderr,
-	     "ilukt_ML_run_alarm(%p): waiting for t=%ld.%09ld.\n",
+      ILU_NOTE(THREAD_DEBUG,
+	    ("ilukt_ML_run_alarm(%p): waiting for t=%ld.%09ld.\n",
 	     alarm, set_time.tv_sec, set_time.tv_nsec));
 
       err = CONDITION_TIMEDWAIT(alarm->alarm_condition,
@@ -1146,7 +1150,7 @@ static void
 #ifdef ILU_SOLARIS2_THREADS
 	  err == ETIME
 #endif
-#ifdef ILU_POSIX_THREADS
+#if (defined(ILU_POSIX_THREADS) || defined(ILU_DCE_THREADS))
 	  err == ETIME		/* on Solaris 2 */
 	  || err == ETIMEDOUT	/* on Linux, AIX */
 #endif
@@ -1159,18 +1163,16 @@ static void
 	alarm->fire_time.ft_s = 0;
 	ILU_UNLOCK_MUTEX(alarm->alarm_mutex);
 
-	DEBUG(THREAD_DEBUG,
-	      (stderr,
-	       "ilukt_ML_run_alarm(%p): firing! (in thread %s)\n",
+	ILU_NOTE(THREAD_DEBUG,
+	      ("ilukt_ML_run_alarm(%p): firing! (in thread %s)\n",
 	       alarm, ftt));
 
 	alarm->proc(alarm->rock);
 
 	ILU_LOCK_MUTEX(alarm->alarm_mutex);
 
-	DEBUG(THREAD_DEBUG,
-	      (stderr,
-	       "ilukt_ML_run_alarm(%p): mutex re-locked, by thread %s.\n",
+	ILU_NOTE(THREAD_DEBUG,
+	      ("ilukt_ML_run_alarm(%p): mutex re-locked, by thread %s.\n",
 	       alarm, ftt));
 
       } else {
@@ -1183,9 +1185,8 @@ static void
 	 * variable was signalled (i.e. the alarm was reset), so
 	 * we'll just loop back to the top and reset the time
 	 */
-	DEBUG(THREAD_DEBUG,
-	      (stderr,
-	       "ilukt_ML_run_alarm(%p): woken early; rethinking.\n",
+	ILU_NOTE(THREAD_DEBUG,
+	      ("ilukt_ML_run_alarm(%p): woken early; rethinking.\n",
 	       alarm));
       }
     } else {
@@ -1193,13 +1194,22 @@ static void
        * The alarm must've been unset, so wait until something else
        * happens.
        */
-      DEBUG(THREAD_DEBUG,
-	    (stderr,
-	  "ilukt_ML_run_alarm(%p): waiting for alarm to be set.\n",
+      ILU_NOTE(THREAD_DEBUG,
+	    ("ilukt_ML_run_alarm(%p): waiting for alarm to be set.\n",
 	     alarm));
       CONDITION_WAIT(alarm->alarm_condition, alarm->alarm_mutex);
     }
   }
+  if (!alarm->going) {
+    DESTROY_CONDITION(alarm->alarm_condition);
+    ILU_UNLOCK_MUTEX(alarm->alarm_mutex);
+    DESTROY_MUTEX(alarm->alarm_mutex);
+    ilu_free(alarm);
+  }
+#ifndef WIN32
+  return NIL;
+#else
+#endif
 }
 
 
@@ -1213,58 +1223,55 @@ static ilu_refany  ilukt_ML_create_alarm(void)
   DECLARE_THREAD(thread_id);
 
   if (new_alarm == NIL) {
-    DEBUG(THREAD_DEBUG,
-	  (stderr,
-      "ilukt_ML_create_alarm:  can't allocate space for alarm\n"));
+    ILU_NOTE(THREAD_DEBUG,
+	  ("ilukt_ML_create_alarm:  can't allocate space for alarm\n"));
     return NIL;			/* couldn't allocate space, so
 				 * return */
   }
-
   memset(new_alarm, 0, sizeof(*new_alarm));
 
   if (!INIT_CONDITION(new_alarm->alarm_condition)) {
     ilu_free(new_alarm);
-    DEBUG(THREAD_DEBUG,
-	  (stderr, "ilukt_ML_create_alarm:  "
+    ILU_NOTE(THREAD_DEBUG,
+	  ("ilukt_ML_create_alarm:  "
 	   "something went wrong with initialization of condition variable\n"));
     return NIL;
   }
   if (!INIT_MUTEX(new_alarm->alarm_mutex)) {
-    DEBUG(THREAD_DEBUG,
-	  (stderr, "ilukt_ML_create_alarm:  "
+    ILU_NOTE(THREAD_DEBUG,
+	  ("ilukt_ML_create_alarm:  "
 	   "something went wrong with mutex initialization,"
 	   " so clean up and fail\n"));
     DESTROY_CONDITION(new_alarm->alarm_condition);
     ilu_free(new_alarm);
     return NIL;
   }
+  new_alarm->going = TRUE;
   status = DISPATCH_THREAD(ilukt_ML_run_alarm, (void *) new_alarm,
 			   thread_id);
   if (status != 0) {
-    DEBUG(THREAD_DEBUG,
-	  (stderr, "ilukt_ML_create_alarm:  "
+    ILU_NOTE(THREAD_DEBUG,
+	  ("ilukt_ML_create_alarm:  "
 	   "couldn't spawn a new thread (errno %d)\n",
 	   status));
-    goto givup;
+    DESTROY_CONDITION(new_alarm->alarm_condition);
+    DESTROY_MUTEX(new_alarm->alarm_mutex);
+    ilu_free(new_alarm);
+    return NIL;
   }
   status = DETACH_THREAD(thread_id);
   if (status != 0) {
-    DEBUG(THREAD_DEBUG,
-	  (stderr, "ilukt_ML_create_alarm:  "
+    ILU_NOTE(THREAD_DEBUG,
+	  ("ilukt_ML_create_alarm:  "
 	   "couldn't detach new thread (errno %d)\n",
 	   status));
-    goto givup;
+    new_alarm->going = FALSE;
+    return NIL;
   }
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_create_alarm:  new alarm %p\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_create_alarm:  new alarm %p\n",
 	 new_alarm));
   return new_alarm;
-givup:
-  DESTROY_CONDITION(new_alarm->alarm_condition);
-  DESTROY_MUTEX(new_alarm->alarm_mutex);
-  ilu_free(new_alarm);
-  return NIL;
 }
 
 
@@ -1278,20 +1285,20 @@ static void
   ilu_string      ftt = no_ftt;
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & THREAD_DEBUG)
+  if (ilu_DebugLevel & THREAD_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
 #endif
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_set_alarm(%p): called by thread %s.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_set_alarm(%p): called by thread %s.\n",
 	 alarm, ftt));
 
+  _ilu_Assert(alarm->going, "ilukt_ML_set_alarm vs. going 1");
   ILU_LOCK_MUTEX(alarm->alarm_mutex);
+  _ilu_Assert(alarm->going, "ilukt_ML_set_alarm vs. going 2");
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_set_alarm(%p): mutex locked by thread %s\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_set_alarm(%p): mutex locked by thread %s\n",
 	 alarm, ftt));
 
   alarm->fire_time = t;
@@ -1301,9 +1308,8 @@ static void
   CONDITION_BROADCAST(alarm->alarm_condition);
   ILU_UNLOCK_MUTEX(alarm->alarm_mutex);
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_set_alarm(%p): signalled and unlocked by thread %s.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_set_alarm(%p): signalled and unlocked by thread %s.\n",
 	 alarm, ftt));
 
   FreeThreadFmt(ftt);
@@ -1321,20 +1327,20 @@ static void
   ilu_string      ftt = no_ftt;
 
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & THREAD_DEBUG)
+  if (ilu_DebugLevel & THREAD_DEBUG)
     ftt = FmtThread(GET_CURRENT_THREAD());
 #endif
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_unset_alarm(%p): called by thread %s.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_unset_alarm(%p): called by thread %s.\n",
 	 alarm, ftt));
 
+  _ilu_Assert(alarm->going, "ilukt_ML_unset_alarm vs. going 1");
   ILU_LOCK_MUTEX(alarm->alarm_mutex);
+  _ilu_Assert(alarm->going, "ilukt_ML_unset_alarm vs. going 2");
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_unset_alarm(%p): mutex locked by thread %s.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_unset_alarm(%p): mutex locked by thread %s.\n",
 	 alarm, ftt));
 
   alarm->fire_time.ft_s = 0;
@@ -1342,9 +1348,48 @@ static void
   CONDITION_BROADCAST(alarm->alarm_condition);
   ILU_UNLOCK_MUTEX(alarm->alarm_mutex);
 
-  DEBUG(THREAD_DEBUG,
-	(stderr,
-	 "ilukt_ML_unset_alarm(%p): cond. signalled and mutex unlocked by thread %s.\n",
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_unset_alarm(%p): cond. signalled and mutex unlocked by thread %s.\n",
+	 alarm, ftt));
+
+  FreeThreadFmt(ftt);
+  return;
+}
+
+
+/* ----------------------------------------------- */
+/* Destroy an alarm                                */
+
+static void
+  ilukt_ML_destroy_alarm(ilu_refany alarm_in_disguise)
+{
+  ilukt_Alarm    *alarm = (ilukt_Alarm *) alarm_in_disguise;
+  ilu_string      ftt = no_ftt;
+
+#ifdef ENABLE_DEBUGGING
+  if (ilu_DebugLevel & THREAD_DEBUG)
+    ftt = FmtThread(GET_CURRENT_THREAD());
+#endif
+
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_destroy_alarm(%p): called by thread %s.\n",
+	 alarm, ftt));
+
+  _ilu_Assert(alarm->going, "ilukt_ML_destroy_alarm vs. going 1");
+  ILU_LOCK_MUTEX(alarm->alarm_mutex);
+  _ilu_Assert(alarm->going, "ilukt_ML_destroy_alarm vs. going 2");
+
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_destroy_alarm(%p): mutex locked by thread %s.\n",
+	 alarm, ftt));
+
+  alarm->going = 0;
+
+  CONDITION_BROADCAST(alarm->alarm_condition);
+  ILU_UNLOCK_MUTEX(alarm->alarm_mutex);
+
+  ILU_NOTE(THREAD_DEBUG,
+	("ilukt_ML_destroy_alarm(%p): cond. signalled and mutex unlocked by thread %s.\n",
 	 alarm, ftt));
 
   FreeThreadFmt(ftt);
@@ -1360,9 +1405,115 @@ static ilu_MainLoop
     NULL, NULL, NULL, NULL, NULL, NULL, 
     ilukt_ML_create_alarm,
     ilukt_ML_set_alarm,
-    ilukt_ML_unset_alarm };
+    ilukt_ML_unset_alarm,
+    ilukt_ML_destroy_alarm };
 
 
+/* ----------------------------------------------- */
+/* per-thread state variables                      */
+
+static void *
+  ilukt_GetPerThreadArgs (void)
+{
+  void *a = ILU_NIL;
+
+#if (defined(ILU_POSIX_THREADS))
+  return ((void *) pthread_getspecific(_ilukt_implicit_args_key));
+#elif (defined(ILU_SOLARIS2_THREADS))
+  if (thr_getspecific(_ilukt_implicit_args_key, &a) < 0)
+    return ILU_NIL;
+  else
+    return a;
+#elif (defined(ILU_WIN32_THREADS))
+  return ((void *) TlsGetValue(_ilukt_implicit_args_key));
+#endif
+}
+
+static void
+  ilukt_SetPerThreadArgs (const void * args,
+			  ILU_ERRS((no_memory, internal)) *err)
+{
+#if (defined(ILU_POSIX_THREADS) || defined(ILU_SOLARIS2_THREADS))
+#if (defined(ILU_POSIX_THREADS))
+  if (pthread_setspecific(_ilukt_implicit_args_key, args) < 0)
+#elif (defined(ILU_SOLARIS2_THREADS))
+  if (thr_setspecific(_ilukt_implicit_args_key, (void *) args) < 0)
+#endif
+  {
+    int this_error = errno;
+    switch (this_error) {
+    case ENOMEM:
+      ILU_ERR_CONS1(no_memory, err, nbytes, 0, 0);
+      return;
+    default:
+      ILU_ERR_CONS1(internal, err, minor, ilu_im_threadAttribute, 0);
+      return;
+    }
+  } else {
+    ILU_CLER(*err);
+  }
+#elif (defined(ILU_WIN32_THREADS))
+  if (!TlsSetValue(_ilukt_implicit_args_key, (LPVOID) args))
+    ILU_ERR_CONS1(internal, err, minor, ilu_im_threadAttribute, 0);
+  else
+    ILU_CLER(*err);
+#endif /* (defined(ILU_POSIX_THREADS) || defined(ILU_SOLARIS2_THREADS)) */
+}
+
+ilu_boolean
+  ilu_OSThreads_GetPerThreadDataTech (void (*destructor) (void *),
+				      void *(**get_attributes)(void),
+				      void (**set_attributes)(const void *, ILU_ERRS((no_memory, internal)) *),
+				      ILU_ERRS((no_memory, internal)) *err)
+{
+  static ilu_boolean initialized = ilu_FALSE;
+
+  if (!initialized) {
+#if (defined(ILU_POSIX_THREADS) || defined(ILU_SOLARIS2_THREADS))
+#if defined(ILU_POSIX_THREADS)
+    if (pthread_key_create(&_ilukt_implicit_args_key, destructor) < 0)
+#else /* defined(ILU_POSIX_THREADS) */
+    if (thr_keycreate(&_ilukt_implicit_args_key, destructor) < 0)
+#endif /* defined(ILU_POSIX_THREADS) */
+      {
+	int this_error = errno;
+	switch (this_error) {
+	case ENOMEM:
+	  ILU_ERR_CONS1(no_memory, err, nbytes, 0, 0);
+	  return ilu_FALSE;
+	default:
+	  ILU_ERR_CONS1(internal, err, minor, ilu_im_threadAttribute, 0);
+	  return ilu_FALSE;
+	}
+      };
+#elif defined(ILU_WIN32_THREADS)
+    if ((_ilukt_implicit_args_key = TlsAlloc()) == 0xFFFFFFFF) {
+      ILU_ERR_CONS1(internal, err, minor, ilu_im_threadAttribute, 0);
+      return ilu_FALSE;
+    };
+    _ilukt_win32_attribute_finalizer = destructor;
+#endif
+    initialized = ilu_TRUE;
+  };
+  *get_attributes = ilukt_GetPerThreadArgs;
+  *set_attributes = ilukt_SetPerThreadArgs;
+  ILU_CLER(*err);
+  return ilu_TRUE;
+}
+
+/* ----------------------------------------------- */
+/* Accessors for the data structures               */
+
+void
+  ilu_OSThreads_GetTech (ilu_WaitTech **waittech,
+			 ilu_LockTech **locktech,
+			 ilu_MainLoop **mainloop)
+{
+  *waittech = &_ilu_threaded_wt;
+  *locktech = &_ilu_threaded_lt;
+  *mainloop = &_ilu_threaded_ml;
+}
+
 /* ----------------------------------------------- */
 /* ilu_InitializeOSThreading                   */
 
@@ -1382,11 +1533,15 @@ ilu_boolean
     ILU_HANDLED(*err);
     return (ilu_FALSE);
   }
+  ilu_SetForkTech(&ilu_OSForkNewThread, err);
+  if (ILU_ERRNOK(*err)) {
+    ILU_HANDLED(*err);
+    return (ilu_FALSE);
+  }
   ilu_SetMainLoop(&_ilu_threaded_ml);
   return TRUE;
 }
 
-
 /* ----------------------------------------------- */
 /* ilu_OSForkNewThread                               */
 
@@ -1407,26 +1562,24 @@ ilu_boolean
 
   status = DISPATCH_THREAD(run_function, func_args, thread_id);
   if (status != 0) {
-    DEBUG(THREAD_DEBUG,
-	  (stderr,
-	   "OS-fork(%p, %p) returns error code %d\n",
+    ILU_NOTE(THREAD_DEBUG,
+	  ("OS-fork(%p, %p) returns error code %d\n",
 	   proc, arg, status));
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_threadFork,
 			 ilu_FALSE);
   }
 #ifdef ENABLE_DEBUGGING
-  if (_ilu_DebugLevel & THREAD_DEBUG) {
+  if (ilu_DebugLevel & THREAD_DEBUG) {
     ilu_string      ftt = FmtThread(thread_id);
-    DEBUG(THREAD_DEBUG, (stderr, "ilu_OSForkNewThread(%p, %p) = %s\n",
+    ILU_NOTE(THREAD_DEBUG, ("ilu_OSForkNewThread(%p, %p) = %s\n",
 			 proc, arg, ftt));
     FreeThreadFmt(ftt);
   }
 #endif
   status = DETACH_THREAD(thread_id);
   if (status != 0) {
-    DEBUG(THREAD_DEBUG,
-	  (stderr,
-	   "OS-detach(new thread) returns error code %d\n",
+    ILU_NOTE(THREAD_DEBUG,
+	  ("OS-detach(new thread) returns error code %d\n",
 	   status));
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_threadFork,
 			 ilu_FALSE);
@@ -1435,6 +1588,26 @@ ilu_boolean
 }
 
 
+#if 0
+/* ----------------------------------------------- */
+/* ilu_OSThreadID                                  */
+
+ilu_refany
+  ilu_OSThreadID (ILU_ERRS((no_memory, no_resources,
+			    internal)) *err)
+{
+  DECLARE_THREAD(current_thread);
+  if (sizeof(current_thread) > sizeof(ilu_refany)) {
+    ILU_NOTE(THREAD_DEBUG,
+	     ("Can't use ILU kernel threads with this thread system implementation."));
+    return ILU_ERR_CONS1(internal, err, minor, ilu_im_threadIDSize, ILU_NIL);
+  } else {
+    ILU_CLER(*err);
+    return ((ilu_refany) GET_CURRENT_THREAD());
+  }
+}
+#endif
+
 /* ----------------------------------------------- */
 /* end of file                                     */
 /* ----------------------------------------------- */
