@@ -17,15 +17,11 @@
  * IF XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH
  * DAMAGES.
  */
-/* $Id: type.c,v 1.68 1996/07/02 03:48:36 janssen Exp $ */
-/* Last edited by Mike Spreitzer March 8, 1996 9:08 am PST */
+/* $Id: type.c,v 1.86 1997/08/22 06:37:45 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 10, 1997 3:26 pm PDT */
 
 #define _POSIX_SOURCE
 
-#ifdef MACOS
-#pragma segment ilu
-#endif
-
 #include "iluntrnl.h"
 
 #include "object.h"
@@ -35,25 +31,33 @@
 
 #include <limits.h>		/* for ULONG_MAX */
 
+#define ILU_TYPEID_CONST_root_obj  "ilu:root-object-type"
+
 /* L1, L2, Main unconstrained */
 
 static void _ilu_HandleIsA (ilu_Call call);
 
+static struct _ilu_MethodArg_s RegisterGCInterestArgs[1] = { 
+  {"object", ILU_TYPEID_CONST_root_obj, ilu_In, ilu_FALSE }};
+
+static struct _ilu_MethodArg_s IsAArgs[1] = {
+  {"typeid", ILU_TYPEID_CONST_ilu_CString, ilu_In, ilu_FALSE }};  
+
 static struct _ilu_Method_s _ilu_InternalMethods[] = {
 
   {"ILUGetTypes", 0xFF83,
-     TRUE, FALSE, NIL, 0, _ilu_HandleGetTypes},
+     TRUE, FALSE, TRUE, 0, 0, NIL, NIL, ILU_TYPEID_CONST_ilu_CString, { _ilu_HandleGetTypes, 0 }},
 
   {"ILURegisterGCInterest", 0xFF81,
-     FALSE, FALSE, NIL, 0, _ilu_HandleGCInterestRegistration},
+     FALSE, FALSE, FALSE, 0, 1, NIL, RegisterGCInterestArgs, NIL, { _ilu_HandleGCInterestRegistration, 0 }},
   {"ILUUnregisterGCInterest", 0xFF82,
-     FALSE, FALSE, NIL, 0, _ilu_HandleGCInterestDeregistration},
+     FALSE, FALSE, FALSE, 0, 1, NIL, RegisterGCInterestArgs, NIL, { _ilu_HandleGCInterestDeregistration, 0 }},
 
   {"ILUPing", 0xFF84,
-     FALSE, FALSE, NIL, 0, _ilu_HandlePing},
+     FALSE, FALSE, FALSE, 0, 0, NIL, NIL, NIL, { _ilu_HandlePing, 0 }},
 
   {"-is-a", 0xFF85,		/* for CORBA support */
-     FALSE, FALSE, NIL, 0, _ilu_HandleIsA}
+     FALSE, FALSE, FALSE, 0, 1, NIL, IsAArgs, ILU_TYPEID_CONST_ilu_boolean, { _ilu_HandleIsA, 0 }}
 };
 
 ilu_Method      _ilu_GetTypesMethod = &_ilu_InternalMethods[0];
@@ -68,10 +72,11 @@ static const    nInternal = sizeof(_ilu_InternalMethods)
 struct _ilu_Class_s _ilu_rootClass_s = {
   "ilu.Object",			/* name */
   "version 2",			/* brand */
-  "ilu:root-object-type",	/* unique_id */
+  ILU_TYPEID_CONST_root_obj,	/* unique ID */
   NIL,				/* singleton? */
   TRUE,				/* collectible? */
-  NIL,				/* authentication */
+  "internal root type for"	/* doc string */
+    "all ILU objects",
   _ilu_InternalMethods,		/* methods */
   sizeof(_ilu_InternalMethods) / sizeof(struct _ilu_Method_s),
   /* method_count */
@@ -86,7 +91,7 @@ const ilu_Class _ilu_rootClass = &_ilu_rootClass_s;
 const ilu_Class ilu_rootClass = &_ilu_rootClass_s;
 
 static struct _ilu_Method_s nometh = {NIL, ULONG_MAX, ilu_FALSE,
-ilu_FALSE, NIL, ULONG_MAX, NULLFN};
+ilu_FALSE, ilu_FALSE, 0, 0, NIL, NIL, NIL, NULLFN};
 
 static int optstrcmp(const char *s1, const char *s2)
 {
@@ -129,12 +134,6 @@ static HashTable UnlinkedClasses = NIL;
  * for id to be registered.
  */
 
-static HashTable ifcErrors = NIL;
-/* ifc name -> error table */
-
-static HashTable corbaErrors = NIL;
-/* corba rep -> NIL */
-
 
 static void 
 _ilu_RegisterClass(ilu_Class class)
@@ -144,6 +143,8 @@ _ilu_RegisterClass(ilu_Class class)
   register ilu_cardinal index;
   ilu_Vector      v;
 
+  static ilu_boolean initialized = ilu_FALSE;
+
   if (class != NIL AND((ilu_cardinal) (class->cl_singleton)) == 1) {
     ILU_ERRPRINTF("\
 Fatal ILU error:  Stubs for type \"%s\" were generated\n\
@@ -152,31 +153,33 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
     exit(1);
   }
 
-  DEBUG(OBJECT_DEBUG,
-	(stderr, "_ilu_RegisterClass:  Registering object type %s, id=%s, ilu_Class=%p.\n",
+  ILU_NOTE(OBJECT_DEBUG,
+	("_ilu_RegisterClass:  Registering object type %s, id=%s, ilu_Class=%p.\n",
 	 class->cl_name, class->cl_unique_id, class));
   
-  if (ClassNameTable == NIL)
-    ClassNameTable = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
-					    _ilu_hash_HashString,
-					    _ilu_hash_StringCompare);
-  if (ClassIDTable == NIL)
-    ClassIDTable = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
-					  _ilu_hash_HashString,
-					  _ilu_hash_StringCompare);
-  if (UnlinkedClasses == NIL)
-    UnlinkedClasses = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
-					     _ilu_hash_HashString,
-					     _ilu_hash_StringCompare);
+  if (!initialized) {
+    ClassNameTable = ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
+					   ilu_hash_HashString,
+					   ilu_hash_StringCompare);
+    ClassIDTable = ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
+					 ilu_hash_HashString,
+					 ilu_hash_StringCompare);
+    UnlinkedClasses = ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
+					    ilu_hash_HashString,
+					    ilu_hash_StringCompare);
+    initialized = ilu_TRUE;
+    /* make sure gc callback class is registered */
+    _ilu_RegisterClass(_ilu_GcCallbackClass);
+  }
 
-  if (_ilu_hash_FindInTable(ClassNameTable, class_name(class)) == NIL)
-    _ilu_Assert((int) _ilu_hash_AddToTable(ClassNameTable, class_name(class),
+  if (ilu_hash_FindInTable(ClassNameTable, class_name(class)) == NIL)
+    _ilu_Assert((int) ilu_hash_AddToTable(ClassNameTable, class_name(class),
 				     class),
 		"RegisterClass AddToTable ClassNameTable");
-  new = NIL == _ilu_hash_FindInTable(ClassIDTable,
+  new = NIL == ilu_hash_FindInTable(ClassIDTable,
 				      class_unique_id(class));
   if (new)
-    _ilu_Assert((int) _ilu_hash_AddToTable(ClassIDTable,
+    _ilu_Assert((int) ilu_hash_AddToTable(ClassIDTable,
 				   class_unique_id(class), class),
 		"RegisterClass AddToTable ClassIDTable");
 
@@ -185,7 +188,7 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
    * loaded into a running program
    */
   if (new && UnknownTypeIDs != NIL
-  && _ilu_hash_FindInTable(UnknownTypeIDs, class_unique_id(class))
+  && ilu_hash_FindInTable(UnknownTypeIDs, class_unique_id(class))
       != NIL) {		/* This shouldn't happen! */
     ASSERT(0, buf,
 	   (buf, "%s %s %s previously considered unknown.\n",
@@ -198,14 +201,14 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
    */
   for (index = 0; index < class_superclass_count(class); index++) {
     if (class_superclass_id(class, index) != NIL) {
-      p = (ilu_Class) _ilu_hash_FindInTable(ClassIDTable,
+      p = (ilu_Class) ilu_hash_FindInTable(ClassIDTable,
 			       class_superclass_id(class, index));
       if (p == NIL) {
-	v = (ilu_Vector) _ilu_hash_FindInTable(UnlinkedClasses,
+	v = (ilu_Vector) ilu_hash_FindInTable(UnlinkedClasses,
 			       class_superclass_id(class, index));
 	if (v == NIL) {
 	  v = _ilu_vector_new(1);
-	  _ilu_Assert((int) _ilu_hash_AddToTable(UnlinkedClasses,
+	  _ilu_Assert((int) ilu_hash_AddToTable(UnlinkedClasses,
 				class_superclass_id(class, index),
 					   v),
 		      "RegisterClass AddToTable UnlinkedClasses");
@@ -220,14 +223,14 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
    * now link those classes which have been waiting for this
    * superclass
    */
-  v = (ilu_Vector) _ilu_hash_FindInTable(UnlinkedClasses,
+  v = (ilu_Vector) ilu_hash_FindInTable(UnlinkedClasses,
 					 class_unique_id(class));
   if (v != NIL) {
     ilu_cardinal    size = _ilu_vector_size(v);
     ilu_Class      *elements = (ilu_Class *) _ilu_vector_elements(v);
     register ilu_cardinal j;
 
-    _ilu_Assert(_ilu_hash_RemoveFromTable(UnlinkedClasses,
+    _ilu_Assert(ilu_hash_RemoveFromTable(UnlinkedClasses,
 					  class_unique_id(class))
 		== v,
 		"RegisterClass RemoveFromTable UnlinkedClasses");
@@ -251,7 +254,7 @@ static ilu_Class ObjectTypeFromID(ilu_string id)
   if (strcmp(id, _ilu_rootClass->cl_unique_id) == 0)
     return _ilu_rootClass;
   else if (ClassIDTable != NIL)
-    return ((ilu_Class) _ilu_hash_FindInTable(ClassIDTable, id));
+    return ((ilu_Class) ilu_hash_FindInTable(ClassIDTable, id));
   else
     return NIL;
 }
@@ -263,7 +266,7 @@ ilu_DefineObjectType(ilu_string cl_name,
 		     ilu_string cl_singleton,
 		     ilu_boolean cl_optional,
 		     ilu_boolean cl_collectible,
-		     ilu_string cl_authentication,
+		     ilu_string cl_doc_string,
 		     ilu_cardinal cl_method_count,
 		     ilu_cardinal cl_scls_count,
 		     ilu_string cl_scls_ids[],
@@ -277,33 +280,37 @@ ilu_DefineObjectType(ilu_string cl_name,
     if (ans == NIL)
       return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), NIL);
     ans->cl_name = _ilu_Strdup(cl_name);
-    ans->cl_brand = _ilu_Strdup(cl_brand);
+    if (cl_brand != NIL && cl_brand[0] == 0)
+      ans->cl_brand = NIL;
+    else
+      ans->cl_brand = _ilu_Strdup(cl_brand);
     ans->cl_unique_id = _ilu_Strdup(cl_unique_id);
     ans->cl_singleton = _ilu_Strdup(cl_singleton);
     ans->cl_collectible = cl_collectible;
     ans->cl_optional = cl_optional;
-    ans->cl_authentication = _ilu_Strdup(cl_authentication);
+    ans->cl_doc_string = _ilu_Strdup(cl_doc_string);
     ans->cl_method_count = cl_method_count;
-    ans->cl_methods = (ilu_Method) ilu_malloc(cl_method_count * msize);
+    ans->cl_methods = (ilu_Method) (msize ? ilu_malloc(cl_method_count * msize) : NIL);
     for (i = 0; i < cl_method_count; i++)
       ans->cl_methods[i] = nometh;
     ans->cl_scls_count = cl_scls_count;
-    ans->cl_scls_ids = ((ilu_string *)
-		   ilu_malloc(cl_scls_count * sizeof(ilu_string)));
+    ans->cl_scls_ids = ((ilu_string *) (cl_scls_count ?
+		   ilu_malloc(cl_scls_count * sizeof(ilu_string)) : NIL));
     for (i = 0; i < cl_scls_count; i++)
       ans->cl_scls_ids[i] = _ilu_Strdup(cl_scls_ids[i]);
-    ans->cl_sclses = ((ilu_Class *)
-		    ilu_malloc(cl_scls_count * sizeof(ilu_Class)));
+    ans->cl_sclses = ((ilu_Class *) (cl_scls_count ?
+		    ilu_malloc(cl_scls_count * sizeof(ilu_Class)) : NIL));
     for (i = 0; i < cl_scls_count; i++)
       ans->cl_sclses[i] = NIL;
     _ilu_RegisterClass(ans);
     ILU_CLER(*err);
     return ans;
   } else {
+    if (cl_brand != NIL && cl_brand[0] == 0)
+      cl_brand = NIL; /* ensure NIL matches "" */
     if (strcmp(cl_name, ans->cl_name) ||
 	optstrcmp(cl_brand, ans->cl_brand) ||
-	optstrcmp(cl_singleton, ans->cl_singleton) ||
-	optstrcmp(cl_authentication, ans->cl_authentication))
+	optstrcmp(cl_singleton, ans->cl_singleton))
       goto no;
     if ((cl_collectible == 0) != (ans->cl_collectible == 0))
       goto no;
@@ -321,40 +328,42 @@ no:
 }
 
 ilu_Exception
-ilu_DefineException(char *i, char *e,
+ilu_DefineException(char *i, char *e, char *typeid,
 		    ILU_ERRS((internal, no_memory)) * err)
 {
-  HashTable       errtab;
   ilu_Exception   ans;
-  if (ifcErrors == NIL)
-    ifcErrors = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
-				       _ilu_hash_HashString,
-				       _ilu_hash_StringCompare);
-  if (corbaErrors == NIL)
-    corbaErrors = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
-					 _ilu_hash_HashString,
-					 _ilu_hash_StringCompare);
+  char buf[1024];
+  char *key;
+  ilu_cardinal keylen;
+  static HashTable exceptions = NIL; /* exception_id_string => C ptr */
+
   if (i == NIL)
-    errtab = corbaErrors;
+    key = e;
   else {
-    errtab = (HashTable) _ilu_hash_FindInTable(ifcErrors, i);
-    if (errtab == NIL) {
-      errtab = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
-				      _ilu_hash_HashString,
-				      _ilu_hash_StringCompare);
-      _ilu_Assert((int) _ilu_hash_AddToTable(ifcErrors, i, errtab),
-		  "ifcErrors");
-    }
+    keylen = strlen(i) + strlen(e) + 7;
+    if (keylen > sizeof(buf)) {
+      key = ilu_MallocE(keylen, err);
+      if (ILU_ERRNOK(*err)) return NIL;
+    } else
+      key = buf;
+    sprintf (key, "ilu:%s.%s", i, e);
   }
-  ans = (ilu_Exception) _ilu_hash_FindInTable(errtab, e);
+
+  if (exceptions == NIL)
+    exceptions = ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
+					ilu_hash_HashString,
+					ilu_hash_StringCompare);
+
+  ans = (ilu_Exception) ilu_hash_FindInTable(exceptions, key);
   if (ans == NIL) {
-    if (i == NIL)
-      ans = _ilu_Strdup(e);
-    else
-      ans = _ilu_Strcat5("ilu:", i, ".", e, "");
-    _ilu_Assert((int) _ilu_hash_AddToTable(errtab, e, ans), "errtab");
+    ans = ilu_must_malloc(strlen(key) + 2 + _ilu_SafeStrlen(typeid));
+    strcpy (ans, key);
+    strcpy (ans + strlen(ans) + 1, (typeid == NIL) ? "" : typeid);
+    _ilu_Assert((int) ilu_hash_AddToTable(exceptions, ans, ans), "errtab");
   }
   ILU_CLER(*err);
+  if ((key != e) && (key != buf))
+    ilu_free(key);
   return ans;
 }
 
@@ -367,26 +376,67 @@ ilu_DefineMethod(ilu_Class c,
 		 ilu_boolean me_asynchronous,
 		 ilu_cardinal me_exceptionCount,
 		 ilu_Exception *me_exceptionVector,
+		 ilu_cardinal me_nargs,
+		 ilu_string me_rettype,
 		 ILU_ERRS((internal, no_memory)) *err)
 {
   ilu_Method      m = c->cl_methods + i;
   ilu_cardinal    j;
   if (m->me_name == NIL) {
-    m->me_name = _ilu_Strdup(me_name);
+
+    m->me_name = ilu_StrdupE(me_name, err);
+    if (ILU_ERRNOK(*err)) goto free0;
+
+    if (me_exceptionCount > 0) {
+      m->me_exceptionVector = (ilu_Exception *)
+	ilu_MallocE(me_exceptionCount * sizeof(ilu_Exception), err);
+      if (ILU_ERRNOK(*err)) goto free1;
+    } else
+      m->me_exceptionVector = NIL;
+
+    if (me_rettype != NIL) {
+      m->me_returnType = ilu_StrdupE(me_rettype, err);
+      if (ILU_ERRNOK(*err)) goto free2;
+    } else
+      m->me_returnType = NIL;
+
+    if (me_nargs > 0) {
+      m->me_argVector = (ilu_MethodArg)
+	ilu_MallocE(me_nargs * sizeof(ilu_MethodArg_s), err);
+      if (ILU_ERRNOK(*err)) goto free3;
+      memset((void *) m->me_argVector, 0,
+	     me_nargs * sizeof(ilu_MethodArg_s));
+    } else
+      m->me_argVector = NIL;
+
     m->me_id = me_id;
     m->me_cacheable = me_cacheable;
     m->me_asynchronous = me_asynchronous;
+    m->me_return_vals = (me_rettype != NIL);
     m->me_exceptionCount = me_exceptionCount;
-    m->me_exceptionVector = ((ilu_Exception *)
-	    ilu_malloc(me_exceptionCount * sizeof(ilu_Exception)));
+    m->me_argCount = me_nargs;
+    
     for (j = 0; j < me_exceptionCount; j++) {
-      if (me_exceptionVector[j] == NIL)
+      if (me_exceptionVector[j] == NIL) {
 	ILU_ERR_CONS1(internal, err, minor, ilu_im_typeIncomplete,
 		      FALSE);
+	goto free4;
+      };
       m->me_exceptionVector[j] = me_exceptionVector[j];
     }
     ILU_CLER(*err);
     return m;
+
+  free4:
+    ilu_free(m->me_argVector);
+  free3:
+    ilu_free(m->me_returnType);
+  free2:
+    ilu_free(m->me_exceptionVector);
+  free1:
+    ilu_free(m->me_name);
+  free0:
+    return NIL;
   } else {
     if (strcmp(me_name, m->me_name) ||
 	me_id != m->me_id ||
@@ -405,6 +455,41 @@ no:
   }
 }
 
+ilu_boolean
+  ilu_DefineMethodArg(ilu_Method m,		/* m */
+		      ilu_cardinal i,		/* index of arg */
+		      ilu_string name,		/* arg name, RETAIN */
+		      ilu_boolean sibling,	/* sibling */
+		      ilu_ArgDirection dir,	/* in, out, or inout */
+		      ilu_string type,		/* type ID, RETAIN */
+		      ILU_ERRS((internal, no_memory)) *err)
+{
+  /*
+   * Defines the index'th arg of method m.
+   */
+
+  if (m == NIL)
+    return (ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_nil, ilu_FALSE));
+  if (i >= m->me_argCount)
+    return (ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_invalid_argument_index, ilu_FALSE));
+  m->me_argVector[i].ma_name = ilu_StrdupE(name, err);
+  if (ILU_ERRNOK(*err)) goto free0;
+  m->me_argVector[i].ma_type = ilu_StrdupE(type, err);
+  if (ILU_ERRNOK(*err)) goto free1;
+  m->me_argVector[i].ma_dir = dir;
+  m->me_return_vals = (m->me_return_vals || (dir == ilu_Out) || (dir == ilu_InOut));
+  m->me_argVector[i].ma_sibling = sibling;
+  ILU_CLER(*err);
+  return ilu_TRUE;
+
+ free2:
+  ilu_free(m->me_argVector[i].ma_type);
+ free1:
+  ilu_free(m->me_argVector[i].ma_name);
+ free0:
+  return ilu_FALSE;
+}
+
 ilu_boolean
 ilu_ObjectTypeDefined(ilu_Class t,
 		      ILU_ERRS((internal / typeIncomplete)) * err)
@@ -422,6 +507,20 @@ no:
 		       FALSE);
 }
 
+ilu_string
+  ilu_TypeOfException (ilu_Exception e)
+{
+  ilu_string p;
+
+  if (e == NIL)
+    return NIL;
+  p = e + (strlen(e) + 1);
+  if (*p == 0)
+    return NIL;
+  else
+    return p;
+}
+
 ilu_boolean
   ilu_DataOfClass (ilu_Class c,
 		   char **name,
@@ -460,7 +559,7 @@ ilu_boolean
 		    ilu_boolean *asynchronous,
 		    ilu_cardinal *ecount,
 		    ilu_Exception **evec,
-		    ilu_StubProc *stubproc)
+		    ilu_StubProc **stubproc)
 {
 #define MERETURN(p,f)	if ((p) != NIL) *(p) = (f)
 
@@ -470,7 +569,7 @@ ilu_boolean
   MERETURN(asynchronous,m->me_asynchronous);
   MERETURN(ecount,m->me_exceptionCount);
   MERETURN(evec,m->me_exceptionVector);
-  MERETURN(stubproc,m->me_stubproc);
+  MERETURN(stubproc,m->me_stubprocs);
 
 #undef MERETURN
 
@@ -494,7 +593,7 @@ PrintClassEntry(ilu_refany entry, ilu_refany junk)
 void _ilu_EnumerateClasses (void (*proc) (ilu_Class c, ilu_refany rock), ilu_refany rock)
 {
   _ilu_AcquireMutex(ilu_otmu);
-  _ilu_hash_TableEnumerate (ClassNameTable, (void (*) (ilu_refany, ilu_refany)) proc, rock);
+  ilu_hash_TableEnumerate (ClassNameTable, (void (*) (ilu_refany, ilu_refany)) proc, rock);
   _ilu_ReleaseMutex(ilu_otmu);
   return;
 }
@@ -502,12 +601,6 @@ void _ilu_EnumerateClasses (void (*proc) (ilu_Class c, ilu_refany rock), ilu_ref
 ilu_Class 
 ilu_GetGcCallbackClass(void)
 {
-  _ilu_AcquireMutex(ilu_otmu);
-  if (ClassIDTable == NIL ||
-      _ilu_hash_FindInTable(ClassIDTable, _ilu_GcCallbackClass->cl_unique_id)
-      == NIL)
-    _ilu_RegisterClass(_ilu_GcCallbackClass);
-  _ilu_ReleaseMutex(ilu_otmu);
   return _ilu_GcCallbackClass;
 }
 
@@ -519,10 +612,10 @@ ilu_FindClassFromName(ilu_string classname)
 
 #ifdef ENABLE_DEBUGGING
 
-  if ((_ilu_DebugLevel & OBJECT_DEBUG) == OBJECT_DEBUG) {
+  if ((ilu_DebugLevel & OBJECT_DEBUG) == OBJECT_DEBUG) {
     ILU_ERRPRINTF("ilu_FindClassFromName:  classname is %s, class table is %p:\n", classname,
 	    ClassNameTable);
-    _ilu_hash_TableEnumerate(ClassNameTable, (void (*) (ilu_refany, ilu_refany)) PrintClassEntry, NIL);
+    ilu_hash_TableEnumerate(ClassNameTable, (void (*) (ilu_refany, ilu_refany)) PrintClassEntry, NIL);
   }
 
 #endif /* ENABLE_DEBUGGING */
@@ -530,7 +623,7 @@ ilu_FindClassFromName(ilu_string classname)
   if (strcmp(classname, _ilu_rootClass->cl_name) == 0)
     c = _ilu_rootClass;
   else if (ClassNameTable != NIL)
-    c = (ilu_Class) _ilu_hash_FindInTable(ClassNameTable, classname);
+    c = (ilu_Class) ilu_hash_FindInTable(ClassNameTable, classname);
   else
     c = NIL;
   _ilu_ReleaseMutex(ilu_otmu);
@@ -545,10 +638,10 @@ ilu_FindClassFromID(ilu_string ID)
 
 #ifdef ENABLE_DEBUGGING
 
-  if ((_ilu_DebugLevel & OBJECT_DEBUG) == OBJECT_DEBUG) {
+  if ((ilu_DebugLevel & OBJECT_DEBUG) == OBJECT_DEBUG) {
     ILU_ERRPRINTF("ilu_FindClassFromID:  id is %s, class table is %p:\n", ID,
 	    ClassIDTable);
-    _ilu_hash_TableEnumerate(ClassIDTable, (void (*) (ilu_refany, ilu_refany)) PrintClassEntry, NIL);
+    ilu_hash_TableEnumerate(ClassIDTable, (void (*) (ilu_refany, ilu_refany)) PrintClassEntry, NIL);
   }
 
 #endif /* ENABLE_DEBUGGING */
@@ -556,7 +649,7 @@ ilu_FindClassFromID(ilu_string ID)
   if (strcmp(ID, _ilu_rootClass->cl_unique_id) == 0)
     c = _ilu_rootClass;
   else if (ClassIDTable != NIL)
-    c = (ilu_Class) _ilu_hash_FindInTable(ClassIDTable, ID);
+    c = (ilu_Class) ilu_hash_FindInTable(ClassIDTable, ID);
   else
     c = NIL;
   _ilu_ReleaseMutex(ilu_otmu);
@@ -657,8 +750,8 @@ AddTypeName(ilu_Class type, iluBuffer buf)
 }
 
 /* L1_sup < otmu */
-static ilu_string 
-ClassToString(ilu_Class class)
+ilu_string 
+_ilu_ClassToString(ilu_Class class)
 {
   struct _iluBuffer_s buf;
   buf.allocated = BUF_INCREMENT;
@@ -714,7 +807,7 @@ _ilu_FindMSKA(ilu_string tid)
   ID_DAG          n;
   ilu_Class       ans;
   _ilu_AcquireMutex(ilu_otmu);
-  n = (ID_DAG) _ilu_hash_FindInTable(UnknownTypeIDs, tid);
+  n = (ID_DAG) ilu_hash_FindInTable(UnknownTypeIDs, tid);
   if (n != NIL && n->computed)
     ans = n->mska;
   else
@@ -736,8 +829,8 @@ ClassNamesToID_DAG(ilu_string * names)
   char           *pl, *pr, *pc, *pe, cr;
   ilu_integer     len;
 
-  DEBUG(OBJECT_DEBUG,
-  (stderr, "(ClassNamesToID_DAG):  Called with \"%s\"\n", *names));
+  ILU_NOTE(OBJECT_DEBUG,
+  ("(ClassNamesToID_DAG):  Called with \"%s\"\n", *names));
 
   if (*names == NIL OR ** names == '\0')
     return (NIL);
@@ -755,14 +848,14 @@ ClassNamesToID_DAG(ilu_string * names)
   pe = (pc < pe) ? pc : pe;
   cr = *pe;
   *pe = '\0';
-  n = (ID_DAG) _ilu_hash_FindInTable(UnknownTypeIDs, *names);
+  n = (ID_DAG) ilu_hash_FindInTable(UnknownTypeIDs, *names);
   if (n == NIL) {
     ilu_string      id = _ilu_Strdup(*names);
     n = (ID_DAG) ilu_malloc(sizeof(ID_DAG_s));
-    _ilu_Assert((int) _ilu_hash_AddToTable(UnknownTypeIDs, id, n),
+    _ilu_Assert((int) ilu_hash_AddToTable(UnknownTypeIDs, id, n),
 		"ClassNamesToID_DAG AddToTable UnknownTypeIDs");
     n->id = id;
-    n->it = (ilu_Class) _ilu_hash_FindInTable(ClassIDTable, id);
+    n->it = (ilu_Class) ilu_hash_FindInTable(ClassIDTable, id);
     n->supers = NIL;
     n->mska = n->it;
     n->computed = n->saak = n->mska != NIL;
@@ -848,8 +941,8 @@ static void
 FindMostSpecificType(ID_DAG t)
 {
   ID_DAG_List     p;
-  DEBUG(OBJECT_DEBUG,
-	(stderr, "(FindMostSpecificType):  at %s, %s visited.\n",
+  ILU_NOTE(OBJECT_DEBUG,
+	("(FindMostSpecificType):  at %s, %s visited.\n",
 	 t->id, (t->visited == FALSE) ? "not" : ""));
   if (t->visited == TRUE)
     return;
@@ -868,8 +961,8 @@ FindMostSpecificType(ID_DAG t)
 }
 
 /* L1_sup < otmu */
-static ilu_Class 
-StringToClass(ilu_string s)
+ilu_Class 
+  _ilu_StringToClass(ilu_string s)
 {
   ID_DAG          t;
   char           *q = s;
@@ -879,17 +972,17 @@ StringToClass(ilu_string s)
     return (NIL);
   _ilu_AcquireMutex(ilu_otmu);
   if (UnknownTypeIDs == NIL)
-    UnknownTypeIDs = (ilu_refany) _ilu_hash_MakeNewTable(
+    UnknownTypeIDs = (ilu_refany) ilu_hash_MakeNewTable(
 					      CLASS_HASHTABLESIZE,
-			     (ilu_hashfnptr) _ilu_hash_HashString,
-			 (ilu_compfnptr) _ilu_hash_StringCompare);
+			     (ilu_hashfnptr) ilu_hash_HashString,
+			 (ilu_compfnptr) ilu_hash_StringCompare);
   t = ClassNamesToID_DAG(&q);
   if (t == NIL)
     return (NIL);
-  DEBUG(OBJECT_DEBUG,
-  (stderr, "(StringToClass):  Converted names <%s> to DAG.\n", s));
+  ILU_NOTE(OBJECT_DEBUG,
+  ("(_ilu_StringToClass):  Converted names <%s> to DAG.\n", s));
   if (t->computed) {
-    DEBUG(OBJECT_DEBUG, (stderr, "(StringToClass):  Old problem.\n"));
+    ILU_NOTE(OBJECT_DEBUG, ("(_ilu_StringToClass):  Old problem.\n"));
   } else {
     ClearVisited(t);
     mska_head.anext = mska_head.aprev = &mska_head;
@@ -904,13 +997,12 @@ StringToClass(ilu_string s)
   ans = t->mska;
   _ilu_ReleaseMutex(ilu_otmu);
   if (ans == NIL) {
-    DEBUG(OBJECT_DEBUG,
-	  (stderr, 
-	   "(StringToClass):  %s ancestors of type <%s> are known!",
-	   t->saak ? "Multiple" : "No", s));
+    ILU_NOTE(OBJECT_DEBUG,
+	     ("(_ilu_StringToClass):  %s ancestors of type <%s> are known!",
+	      t->saak ? "Multiple" : "No", s));
   } else {
-    DEBUG(OBJECT_DEBUG,
-	  (stderr, "(StringToClass):  Found class %s.\n",
+    ILU_NOTE(OBJECT_DEBUG,
+	  ("(_ilu_StringToClass):  Found class %s.\n",
 	   class_name(ans)));
   }
   return (ans);
@@ -932,7 +1024,7 @@ ObtainTypes(ilu_Object o, ilu_string * types, ilu_cardinal * typeslen,
   ilu_Server      s = object_server(o);
   ilu_Connection  newconn;
   ilu_boolean     ans;
-  DEBUG(OBJECT_DEBUG, (stderr, "_ilu_FindClassViaRPC:  object %p...\n",
+  ILU_NOTE(OBJECT_DEBUG, ("_ilu_FindClassViaRPC:  object %p...\n",
 		       o));
   ans = ilu_StartCall(call, s, _ilu_rootClass, _ilu_GetTypesMethod, 0,
 		      NIL, &newconn, err);
@@ -940,9 +1032,9 @@ ObtainTypes(ilu_Object o, ilu_string * types, ilu_cardinal * typeslen,
     _ilu_HandOffNewConnection(newconn, err);
   if (ILU_ERRNOK(*err))
     return FALSE;
-  _ilu_AcquireMutex(server_lock(s));
+  _ilu_AcquireServerMutex(s);
   reqSize = ilu_SizeOfObjectID(call, o, TRUE, _ilu_rootClass, err);
-  _ilu_ReleaseMutex(server_lock(s));
+  _ilu_ReleaseServerMutex(s);
   if (ILU_ERRNOK(*err))
     goto faild;
   if (!ilu_StartRequest(call, reqSize, err))
@@ -953,8 +1045,10 @@ ObtainTypes(ilu_Object o, ilu_string * types, ilu_cardinal * typeslen,
     goto faild;
   if (!ilu_FinishRequest(call, err))
     goto faild;
-  internal = ilu_GetReply(call, &estatus, err);
-  if (internal == ilu_ProtocolException_Not)
+  internal = ilu_GetReply(call, &estatus, &newconn, err);
+  if (newconn != NIL)
+    _ilu_HandOffNewConnection(newconn, err);
+  if (ILU_ERRNOK(*err))
     goto faild;
   if (internal != ilu_ProtocolException_Success) {
     (void) ILU_ERR_CONS1(internal, err, minor, ilu_im_callFail, 0);
@@ -990,9 +1084,8 @@ _ilu_FindClassViaRPC(ilu_Object o)
 		      no_resources, IoErrs)) lerr;
 
   if (class_singleton(pclass)) {
-    DEBUG(OBJECT_DEBUG,
-	  (stderr,
-	 "%s %s is singleton, not attempting GetTypes RPC call.\n",
+    ILU_NOTE(OBJECT_DEBUG,
+	  ("%s %s is singleton, not attempting GetTypes RPC call.\n",
 	   "_ilu_FindClassViaRPC:  pclass", class_name(pclass)));
     return (NIL);
   }
@@ -1000,19 +1093,19 @@ _ilu_FindClassViaRPC(ilu_Object o)
   _ilu_Assert(!server_is_true(object_server(o)), "_ilu_FindClassViaRPC: called on true object");
   
   if (!ObtainTypes(o, &types, &typeslen, &lerr)) {
-    DEBUG(OBJECT_DEBUG,
-	  (stderr, "_ilu_FindClassViaRPC:  no types.\n"));
-    DEBUG(OBJECT_DEBUG,
-	  (stderr, "GetTypes call raised %s from %s:%d\n",
+    ILU_NOTE(OBJECT_DEBUG,
+	  ("_ilu_FindClassViaRPC:  no types.\n"));
+    ILU_NOTE(OBJECT_DEBUG,
+	  ("GetTypes call raised %s from %s:%d\n",
 	   ILU_ERR_NAME(lerr), ilu_ErrorFile(&lerr),
 	   ilu_ErrorLine(&lerr)));
   } else
-    DEBUG(OBJECT_DEBUG,
-	  (stderr, "_ilu_FindClassViaRPC:  typestring is <%s>...\n",
+    ILU_NOTE(OBJECT_DEBUG,
+	  ("_ilu_FindClassViaRPC:  typestring is <%s>...\n",
 	   types));
 
   if (types != NIL) {
-    c = StringToClass(types);
+    c = _ilu_StringToClass(types);
 #ifdef ENABLE_DEBUGGING
     if (c == NIL) {
       ilu_DebugPrintf("ILU Error:  Unable to determine the object type of object <%s/%s>.\n",
@@ -1024,8 +1117,8 @@ _ilu_FindClassViaRPC(ilu_Object o)
 #endif
     FREETOKEN(types);
   }
-  DEBUG(OBJECT_DEBUG,
-	(stderr, "_ilu_FindClassViaRPC:  class is \"%s\".\n",
+  ILU_NOTE(OBJECT_DEBUG,
+	("_ilu_FindClassViaRPC:  class is \"%s\".\n",
 	 (c == NIL) ? "*unknown*" : class_name(c)));
   return (c);
 }
@@ -1049,7 +1142,7 @@ _ilu_HandleGetTypes(ilu_Call call)
   if (!ilu_RequestRead(call, &lerr))
     goto dun;
   if (disc != NIL && object_class(disc) != NIL)
-    names = ClassToString(object_class(disc));
+    names = _ilu_ClassToString(object_class(disc));
   if (names == NIL) {
     ilu_cardinal asize =
       ilu_BeginSizingException (call, - (ilu_integer) ilu_ProtocolException_GarbageArguments, &lerr);
