@@ -14,11 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: udp.c,v 1.55 1996/05/04 04:54:12 janssen Exp $ */
-/* Last edited by Mike Spreitzer February 19, 1996 11:31 pm PST */
+/* $Id: udp.c,v 1.60 1997/04/26 07:30:00 spreitze Exp $ */
+/* Last edited by Mike Spreitzer April 25, 1997 11:32 pm PDT */
 
 #include "iluntrnl.h"
 
+#if 0
+
 #include "transprt.h"
 #include "mooring.h"
 
@@ -47,7 +49,7 @@ typedef struct {
   /* L1 >= {trmu} */
 
   HashTable       perpeers;	/* INetSockAddr* -> UDPParms */
-  int             reading;
+  int             reading;	/* num _ilu_WaitForInputOnFD active */
 }              *Sender;
 
 /* L1 >= {trmu} */
@@ -79,6 +81,7 @@ typedef struct {
   int             fd;
   ilu_TIH        *tih;		/* !=NIL when req handler reg'd */
 }              *MooringParms;
+/* (tih) owned by client, not mooring. */
 
 typedef struct udpparms {
   /* L1, L2 unconstrained */
@@ -87,7 +90,7 @@ typedef struct udpparms {
   INetSockAddr    addr;		/* the other side */
   ilu_boolean     busyIn, busyOut;	/* msgs open? */
   int             fd;
-  ilu_boolean     inInUsed;	/* meaningful iff incoming */
+  ilu_boolean     inIsUsed;	/* meaningful iff incoming */
 
   /* The remaining members are significant only iff !incoming */
 
@@ -96,7 +99,7 @@ typedef struct udpparms {
   /* L1 >= {trmu} */
 
   ilu_boolean     isopen;	/* TRUE 'till close */
-  int             waits;	/* num WaitForInput active */
+  int             waits;	/* num _ilu_WaitForInputOnFD active */
   int             irq;		/* abort outer this many */
   MsgList         queue;	/* incoming messages */
   ilu_Condition   change;	/* ... in queue */
@@ -108,17 +111,16 @@ typedef struct udpparms {
  * !busyOut, there is no exposed output buffer.
  * 
  * There is a new incoming Transport for each packet received at the
- * Mooring.  inInUsed is meaningful for incoming transports with
+ * Mooring.  inIsUsed is meaningful for incoming transports with
  * !busyIn; it says whether the message is waiting or already
- * consumed.  When !inInUsed && !busyIn, tr_inNext == tr_inLimit ==
+ * consumed.  When !inIsUsed && !busyIn, tr_inNext == tr_inLimit ==
  * length of the message, which is already stored in the exposed
- * buffer.  When inInUsed && !busyIn, the message has been freed and
+ * buffer.  When inIsUsed && !busyIn, the message has been freed and
  * tr_inBuff == NIL.
  * 
  * All the outgoing Transports of a Sender write to and read from that
- * Sender's FD.  Being between creation and tc_close adds 1 to
- * /refcnt/.  Waiting on input also contributes 1 to /refcnt/.  When
- * /refcnt/ goes to 0, we ilu_free(self).  tr_inBuff == NIL iff
+ * Sender's FD.  An outgoing transport is freed when it's closed and
+ * (waits == 0).  A Sender is never freed.  tr_inBuff == NIL iff
  * !busyIn.
  */
 
@@ -232,8 +234,8 @@ InterpretInfo(ilu_string info,
 	memcpy((ilu_string) & cp->addr.sin_addr, hp->h_addr,
 	       hp->h_length);
       if (cp->addr.sin_addr.s_addr == -1) {
-	DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	      (stderr, "udp:  Invalid host name (%s).\n",
+	ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	      ("udp:  Invalid host name (%s).\n",
 	       hostname));
 	ilu_free(cp);
 	return ILU_ERR_CONS1(inv_objref, err, minor, ilu_iom_ts, NIL);
@@ -258,8 +260,8 @@ static int Socket(ILU_ERRS((IoErrs)) * err)
   int             skt;
   if (OS_SOCKINV(skt = socket(AF_INET, SOCK_DGRAM, 0))) {
     int             theerr = sockerrno;
-    DEBUG((EXPORT_DEBUG | UDP_DEBUG),
-	  (stderr, "udp: create socket failed:  %s.\n",
+    ILU_NOTE((EXPORT_DEBUG | UDP_DEBUG),
+	  ("udp: create socket failed:  %s.\n",
 	   strerror(theerr)));
     switch (theerr) {
     case SOCKERRID(ACCES):		/* permission error */
@@ -307,10 +309,10 @@ static Sender NewSender(ilu_integer *dfd, ILU_ERRS((IoErrs)) * err)
   Sender          ans = ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
     return NIL;
-  ans->reading = FALSE;
-  ans->perpeers = _ilu_hash_MakeNewTable(5, HashAddr, CmpAddr);
-  DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	(stderr, "udp:  Creating new outgoing socket...\n"));
+  ans->reading = 0;
+  ans->perpeers = ilu_hash_MakeNewTable(5, HashAddr, CmpAddr);
+  ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	("udp:  Creating new outgoing socket...\n"));
 
   *dfd = 0;
   fd = Socket(err);
@@ -328,16 +330,16 @@ static Sender NewSender(ilu_integer *dfd, ILU_ERRS((IoErrs)) * err)
 #endif				/* SO_USELOOPBACK */
 
   if (set_non_blocking(fd) != 0) {
-    DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	  (stderr, "udp:  Failed to set FD %d non-blocking.\n",
+    ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	  ("udp:  Failed to set FD %d non-blocking.\n",
 	   fd));
     return ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_nonblock, NIL);
   }
-  DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	(stderr, "udp:  Created new outgoing socket, FD = %d\n",
+  ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	("udp:  Created new outgoing socket, FD = %d\n",
 	 fd));
   ans->fd = fd;
-  _ilu_Assert((int) _ilu_hash_AddToTable(senders, &ans->fd, ans),
+  _ilu_Assert((int) ilu_hash_AddToTable(senders, &ans->fd, ans),
 	      "UDP NewSender");
   return ans;
 }
@@ -348,7 +350,7 @@ static ilu_boolean
 {
   Sender          sender = (Sender) entry_data;
   INetSockAddr   *paddr = (INetSockAddr *) rock;
-  return (_ilu_hash_FindInTable(sender->perpeers, paddr) == NIL);
+  return (ilu_hash_FindInTable(sender->perpeers, paddr) == NIL);
 }
 
 /*L1, L2 unconstrained*/
@@ -379,20 +381,19 @@ static ilu_integer CloseDFd(ilu_Transport tr)
   return 0;
 }
 
-/*Main Invariant holds; L2 disjoint {conn's iomu, callmu}*/
+/* tih => (Main Invariant holds; L2 disjoint {conn's iomu, callmu}) */
 static          ilu_boolean
 SetInputHandler(ilu_Transport tr,
-		ilu_TransportInputHandler tih,
-		ilu_refany tih_rock,
+		ilu_TIH tih,
 		ILU_ERRS((no_memory, internal, no_resources)) * err)
 {
   UDPParms        parms = (UDPParms) transport_data(tr);
   if (!parms->incoming)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_tportRole, FALSE);
-  if (tih != NULLFN)
-    (*tih) (tih_rock);
+  if (tih)
+    (*tih->proc) (tih->rock);
   ILU_CLER(*err);
-  return (tih == NULLFN);
+  return (!tih);
 }
 
 /*mxamu = ilu_trmu*/
@@ -485,7 +486,7 @@ WaitForInput(ilu_Transport tr, ilu_FineTime * limit,
   if (!ilu_EnterMutex(ilu_trmu, err))
     return FALSE;
   while (parms->queue == NIL) {
-    if (sender->reading && threaded) {
+    if ((sender->reading > 0) && threaded) {
       ilu_boolean     irq = FALSE;
       if (limit != NIL) {
 	ita.ita_cc = parms->change;
@@ -543,7 +544,7 @@ WaitForInput(ilu_Transport tr, ilu_FineTime * limit,
       fromlen = sizeof(from);
       res = recvfrom(sender->fd, msg, msglen, 0, &from, &fromlen);
       if (res >= 0) {
-	rcpt = _ilu_hash_FindInTable(sender->perpeers, &from);
+	rcpt = ilu_hash_FindInTable(sender->perpeers, &from);
 	if (rcpt != NIL) {
 	  msl = ilu_MallocE(sizeof(*msl), err);
 	  if (msl == NIL)
@@ -557,9 +558,8 @@ WaitForInput(ilu_Transport tr, ilu_FineTime * limit,
 	      goto dun;
 	} else {
 	  INetSockAddr   *x = (INetSockAddr *) & from;
-	  DEBUG(UDP_DEBUG,
-		(stderr,
-		 "udp.c: received reply on FD %d from unknown peer 0x%08x:%u\n",
+	  ILU_NOTE(UDP_DEBUG,
+		("udp.c: received reply on FD %d from unknown peer 0x%08x:%u\n",
 		 sender->fd, x->sin_addr.s_addr, x->sin_port));
 	  ilu_free(msg);
 	}
@@ -599,8 +599,8 @@ BeginMessage(ilu_Transport tr,
 	     ILU_ERRS((IoErrs)) * err)
 {
   UDPParms        parms = (UDPParms) tr->tr_data;
-  DEBUG(UDP_DEBUG,
-	(stderr, "udp(%p): BeginMessage(%s)\n",
+  ILU_NOTE(UDP_DEBUG,
+	("udp(%p): BeginMessage(%s)\n",
 	 tr, input ? "input" : "output"));
   if (parms->busyIn || parms->busyOut)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_beginMessage,
@@ -625,7 +625,7 @@ BeginMessage(ilu_Transport tr,
       if (!ilu_ExitMutex(ilu_trmu, TRUE, err))
 	return ilu_rhrc_error;
       return ilu_rhrc_ok;
-    } else if (parms->inInUsed) {
+    } else if (parms->inIsUsed) {
       ILU_CLER(*err);
       return ilu_rhrc_eof;
     } else {
@@ -657,9 +657,8 @@ SendWholeMessage(ilu_Transport tr,
   ilu_bytes       buffer = msgh->msg_base;
   ilu_cardinal    count = msgh->msg_len;
   ilu_integer     sent;
-  DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	(stderr,
-	 "udp(%p):  (re-)sending %lu bytes from %p to fd %d.\n",
+  ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	("udp(%p):  (re-)sending %lu bytes from %p to fd %d.\n",
 	 tr, msgh->msg_len, msgh->msg_base, parms->fd));
   sent = sendto(parms->fd, (char *) buffer, (int) count, 0,
 		(struct sockaddr *) & parms->addr,
@@ -679,8 +678,8 @@ EndMessage(ilu_Transport tr,
 	   ILU_ERRS((IoErrs)) * err)
 {
   UDPParms        parms = (UDPParms) tr->tr_data;
-  DEBUG(UDP_DEBUG,
-	(stderr, "udp(%p): EndMessage(%s)\n",
+  ILU_NOTE(UDP_DEBUG,
+	("udp(%p): EndMessage(%s)\n",
 	 tr, (parms->busyIn ? "input"
 	      : parms->busyOut ? "output"
 	      : "no dir!")));
@@ -689,7 +688,7 @@ EndMessage(ilu_Transport tr,
     tr->tr_inNext = tr->tr_inLimit = 0;
     ilu_free(tr->tr_inBuff);
     tr->tr_inBuff = NIL;
-    parms->inInUsed = TRUE;
+    parms->inIsUsed = TRUE;
     return ILU_CLER(*err);
   } else if (parms->busyOut) {
     parms->busyOut = FALSE;
@@ -700,7 +699,7 @@ EndMessage(ilu_Transport tr,
 
 #ifdef ENABLE_DEBUGGING
 
-    if ((_ilu_DebugLevel & PACKET_DEBUG) != 0)
+    if ((ilu_DebugLevel & PACKET_DEBUG) != 0)
       _ilu_debug_DumpPacket(msgh->msg_base, msgh->msg_len, "outgoing UDP");
 
 #endif /* ENABLE_DEBUGGING */
@@ -743,12 +742,12 @@ Close(ilu_Transport tr, ilu_integer * dfd,
   *dfd = 0;
   ILU_CLER(*err);
   if (parms->incoming) {
-    DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	  (stderr, "udp(%p):  Close incoming.\n", tr));
+    ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	  ("udp(%p):  Close incoming.\n", tr));
     return TFree(tr);
   }
-  DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	(stderr, "udp(%p):  Close outgoing (waits=%d).\n", tr,
+  ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	("udp(%p):  Close outgoing (waits=%d).\n", tr,
 	 parms->waits));
   parms->isopen = FALSE;
   if (parms->waits == 0)
@@ -785,8 +784,8 @@ CreateTransport(ilu_TransportCreator tcr,
   UDPParms        tParms;
   *dfd = 0;
   if (cp->defaulted) {
-    DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	  (stderr, "udp(%p):  Don't contact defaulted host.\n",
+    ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	  ("udp(%p):  Don't contact defaulted host.\n",
 	   tcr));
     return ILU_ERR_CONS1(inv_objref, err, minor, ilu_iom_ts, NIL);
   }
@@ -807,24 +806,24 @@ CreateTransport(ilu_TransportCreator tcr,
   if (!ilu_EnterMutex(ilu_trmu, err))
     return FALSE;
   if (senders == NIL) {
-    senders = _ilu_hash_MakeNewTable(1, HashFD, CmpFD);
+    senders = ilu_hash_MakeNewTable(1, HashFD, CmpFD);
     if (senders == NIL)
       return ILU_ERR_CONS1(no_memory, err, nbytes, 0, NIL);
   }
-  tParms->sender = _ilu_hash_FindViaProc(senders, UseSenderP, &cp->addr);
+  tParms->sender = ilu_hash_FindViaProc(senders, UseSenderP, &cp->addr);
   if (tParms->sender == NIL)
     tParms->sender = NewSender(dfd, err);
   if (tParms->sender != NIL) {
     tParms->addr = cp->addr;
     tParms->isopen = TRUE;
-    tParms->waits = tParms->waits = 0;
+    tParms->waits = 0;
     tParms->queue = NIL;
     if (ilu_CanCondition())
       tParms->change = ilu_CreateCondition("a UDP", "transport", err);
     else
       tParms->change = NIL;
     ILU_MUST_BE_SUCCESS(*err);
-    _ilu_Assert((int) _ilu_hash_AddToTable(tParms->sender->perpeers,
+    _ilu_Assert((int) ilu_hash_AddToTable(tParms->sender->perpeers,
 				     &tParms->addr, tParms),
 		"UDP CreateTransport hash_AddToTable");
   }
@@ -847,9 +846,8 @@ CreateTransport(ilu_TransportCreator tcr,
   tParms->incoming = FALSE;
   tParms->busyIn = tParms->busyOut = FALSE;
   tParms->fd = tParms->sender->fd;
-  DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	(stderr,
-	 "udp(%p):  New outgoing transport, via FD %d, to %s:%u\n",
+  ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	("udp(%p):  New outgoing transport, via FD %d, to %s:%u\n",
 	 ans, tParms->fd, inet_ntoa(cp->addr.sin_addr),
 	 ntohs(cp->addr.sin_port)));
   return ans;
@@ -914,9 +912,8 @@ AcceptClient(ilu_Mooring m, ilu_string * tinfo_out,
 		  (struct sockaddr *) & from, &fromlen);
   if (ilen < 0) {
     int             theerr = sockerrno;
-    DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	  (stderr,
-	   "udp(%p): recvfrom addr %s port %u failed, errno=%d.\n",
+    ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	  ("udp(%p): recvfrom addr %s port %u failed, errno=%d.\n",
 	   m, inet_ntoa(from.sin_addr),
 	   ntohs(mParms->addr.sin_port), theerr));
     (void) ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_connect_failed,
@@ -941,7 +938,7 @@ AcceptClient(ilu_Mooring m, ilu_string * tinfo_out,
   }
   tParms->incoming = TRUE;
   tParms->addr = from;
-  tParms->busyIn = tParms->busyOut = tParms->inInUsed = FALSE;
+  tParms->busyIn = tParms->busyOut = tParms->inIsUsed = FALSE;
   tParms->fd = mParms->fd;
   tParms->sender = NIL;
   tParms->isopen = tParms->waits = tParms->irq = 0;
@@ -958,12 +955,11 @@ AcceptClient(ilu_Mooring m, ilu_string * tinfo_out,
   ans->tr_toN = ilu_FineTime_FromDouble(_udp_Timeout_toN);
   ans->tr_tto = ilu_FineTime_FromDouble(_udp_Timeout_tto);
 
-  DEBUG((CONNECTION_DEBUG | UDP_DEBUG),
-	(stderr,
-	 "udp(%p): Accept request of len %d from %s:%u on port %u, FD %d, mooring %p.\n",
+  ILU_NOTE((CONNECTION_DEBUG | UDP_DEBUG),
+	("udp(%p): Accept request of len %d from %s:%u on port %u, FD %d, mooring %p.\n",
 	 ans, ilen, inet_ntoa(from.sin_addr), ntohs(from.sin_port),
 	 ntohs(mParms->addr.sin_port), mParms->fd, m));
-  if ((_ilu_DebugLevel & PACKET_DEBUG) != 0)
+  if ((ilu_DebugLevel & PACKET_DEBUG) != 0)
     _ilu_debug_DumpPacket(ibuf, ilen, "incoming UDP");
   return (ans);
 faild:
@@ -985,9 +981,8 @@ CloseMooring(ilu_Mooring m, ilu_integer * dfd,
 {
   MooringParms    mp = (MooringParms) m->mo_data;
   int             res;
-  DEBUG((EXPORT_DEBUG | UDP_DEBUG),
-	(stderr,
-	 "udp(%p): Close mooring on FD %d, addr %s, port %u.\n",
+  ILU_NOTE((EXPORT_DEBUG | UDP_DEBUG),
+	("udp(%p): Close mooring on FD %d, addr %s, port %u.\n",
 	 m, mp->fd, inet_ntoa(mp->addr.sin_addr),
 	 ntohs(mp->addr.sin_port)));
   ilu_UnregisterInputSource(mp->fd);
@@ -997,21 +992,12 @@ CloseMooring(ilu_Mooring m, ilu_integer * dfd,
 	 (buf, "udp_CloseMooring: res=%d, errno=%s", res,
 	  strerror(sockerrno)));
   if (mp->tih != NIL)
-    (ilu_free(mp->tih), mp->tih = NIL);
+    mp->tih = NIL;
   ilu_free(mp);
   ilu_free(m);
   return ILU_CLER(*err);
 }
 
-/*Main Invariant holds; L2 otherwise unconstrained*/
-static void
-CallReqHandler(int fd, ilu_private rock)
-{
-  ilu_TIH        *tp = (ilu_TIH *) rock;
-  (*tp->tih_proc) (tp->tih_rock);
-  return;
-}
-
 /*L1.sup < trmu; L2 unconstrained*/
 static ilu_integer MooringDFd(ilu_Mooring self, ilu_boolean add)
 {
@@ -1022,20 +1008,15 @@ static ilu_integer MooringDFd(ilu_Mooring self, ilu_boolean add)
 
 static          ilu_boolean
 SetReqHandler(ilu_Mooring m, ilu_Server s,
-	      ilu_TransportInputHandler tih, ilu_refany tih_rock,
+	      ilu_TIH tih,
 	      ILU_ERRS((no_memory, imp_limit, no_resources,
 			broken_locks, internal)) * err)
 {
   MooringParms    mp = (MooringParms) m->mo_data;
-  ilu_TIH         *p = (ilu_TIH *) ilu_malloc(sizeof(*p));
   _ilu_Assert(mp->tih == NIL, "udp:SetReqHandler");
-  if (p == NIL)
-    return ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*p), FALSE);
-  p->tih_proc = tih;
-  p->tih_rock = tih_rock;
-  if (!ilu_RegisterInputSource(mp->fd, CallReqHandler, p))
+  if (!ilu_RegisterInputSource(mp->fd, _ilu_InvokeTIH, tih))
     return ILU_ERR_CONS1(no_resources, err, minor, ilu_nrm_mlreg, FALSE);
-  mp->tih = p;
+  mp->tih = tih;
   return ILU_CLER(*err);
 }
 
@@ -1097,8 +1078,8 @@ CreateMooring(ilu_TransportCreator tcr, ilu_TransportInfo * tinfo_out,
     if (!OS_SOCKERR(bind(skt, (struct sockaddr *) & mysin, sizeof(mysin))))
       break;
     err1 = sockerrno;
-    DEBUG((EXPORT_DEBUG | UDP_DEBUG),
-	  (stderr, "udp(%p): bind to port %lx:%u failed:  %s.\n",
+    ILU_NOTE((EXPORT_DEBUG | UDP_DEBUG),
+	  ("udp(%p): bind to port %lx:%u failed:  %s.\n",
 	   tcr, ntohl(mysin.sin_addr.s_addr),
 	   ntohs(cParms->addr.sin_port), strerror(err1)));
     if (err1 == SOCKERRID(ADDRNOTAVAIL) && !loopedback) {
@@ -1122,8 +1103,8 @@ tryagain:
     namelen = sizeof(mysin);
     if (OS_SOCKERR(getsockname(skt, (struct sockaddr *) & mysin,
 			       &namelen))) {
-      DEBUG((EXPORT_DEBUG | UDP_DEBUG),
-	    (stderr, "udp: getsockname failed:  %s.\n",
+      ILU_NOTE((EXPORT_DEBUG | UDP_DEBUG),
+	    ("udp: getsockname failed:  %s.\n",
 	     strerror(sockerrno)));
       OS_SOCKLOSE(skt);
       *dfd = 0;
@@ -1158,8 +1139,8 @@ tryagain:
     (void) ILU_ERR_CONS1(no_memory, err, nbytes, sizeof(*ans), 6);
     goto faild;
   }
-  DEBUG((EXPORT_DEBUG | UDP_DEBUG),
-	(stderr, "udp(%p): new mooring on FD %d, addr %s, port %u.\n",
+  ILU_NOTE((EXPORT_DEBUG | UDP_DEBUG),
+	("udp(%p): new mooring on FD %d, addr %s, port %u.\n",
 	 ans, skt, inet_ntoa(mysin.sin_addr), ntohs(mysin.sin_port)));
   *ans = mooringProto;
   ans->mo_data = mParms;
@@ -1185,6 +1166,8 @@ faild:
 static struct _ilu_TransportCreator_s creatorProto = {
   TRUE,				/* boundaried */
   FALSE,			/* reliable */
+  0,				/* tcr_holds */
+  FALSE,			/* tcr_wantClose */
   FdUsage,
   CreateTransport,
   CreateMooring,
@@ -1213,3 +1196,13 @@ _ilu_udp_TransportCreator(ilu_TransportInfo tinfo,
   return (tcr);
 }
 
+#else
+
+ilu_TransportCreator
+_ilu_udp_TransportCreator(ilu_TransportInfo tinfo,
+			  ILU_ERRS((no_memory, inv_objref)) * err)
+{
+  return 0;
+}
+
+#endif
