@@ -14,7 +14,7 @@
 # CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 # XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #
-# $Id: ilu.py,v 1.14 1996/06/18 03:27:16 head Exp $
+# $Id: ilu.py,v 1.32 1997/11/14 00:02:34 janssen Exp $
 #
 
 # Exceptions
@@ -32,6 +32,7 @@ from iluRt import		\
 
 # Methods
 from iluRt import		\
+    SetCalloutExceptionHandler,	\
     SetDebugLevel,		\
     SetDebugLevelViaString,	\
     CreateServer,		\
@@ -39,6 +40,9 @@ from iluRt import		\
     ObjectOfSBH,		\
     LookupObject,		\
     ParseSBH,			\
+    FormSBH,			\
+    FindObject,			\
+    RegisterCustomSurrogate,	\
     IOROfObject,		\
     CallerIdentity,		\
     RegisterInputHandler,	\
@@ -46,11 +50,28 @@ from iluRt import		\
     RunMainLoop,		\
     ExitMainLoop,		\
     SetMainLoop,		\
+    DoSoon,			\
     LongReal,			\
     FineTime,			\
     FineTime_Now,		\
     CreateAlarm,                \
-    ThreadedOperation
+    CreatePipeline,		\
+    SetPipeline,		\
+    GetPipeline,		\
+    SetSerializer,		\
+    GetSerializer,		\
+    SetPassport,		\
+    GetPassport,		\
+    CreatePassport,		\
+    Pickle,			\
+    Typecode,			\
+    ThreadedOperation,		\
+    SetFDBudget,		\
+    GetFDBudget,		\
+    TCPStatistics,		\
+    TCPDefaultBufferSize,	\
+    CORBA_Object,		\
+    __types__
 
 def UnregisterInputHandler (fd):
     RegisterInputHandler (fd, None)
@@ -61,5 +82,6 @@ def TypeName(cl):
 def TypeID(cl):
     return cl._IluClass.id()
 
-def FormSBH(objectID, contactInfo):
-    return objectID + '@' + contactInfo
+def AutoImport(path=(), verbose=None):
+	import iluimport
+	iluimport.install(path, verbose)
