@@ -14,7 +14,7 @@
 # CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 # XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #
-# $Id: ilu_tk.py,v 1.11 1996/05/15 18:36:58 janssen Exp $
+# $Id: ilu_tk.py,v 1.12 1997/04/29 01:17:57 larner Exp $
 #
 
 import ilu
@@ -117,13 +117,14 @@ def _deleteHandler(fd, mask):
     global _fdMap
     if _fdMap.has_key(fd):
 	    pair = _fdMap[fd]
+	    iohc = pair.which(mask)
 	    pair.unsetMember(mask)
 	    if pair.mask() == 0:
 		    del _fdMap[fd]
 		    ilutkinter.deletefilehandler(fd)
 	    else:
 		    ilutkinter.createfilehandler(pair, pair.mask(), _callhandler)
-	    return 1
+	    return iohc
     else:
 	    return None
 
