@@ -14,8 +14,11 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: ilualobject.h,v 1.2 1994/09/13 19:38:24 severson Exp $
+$Id: ilualobject.h,v 1.4 1996/07/09 23:02:52 larner Exp $
 */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* ILU Alarm Object implementation */
 
@@ -32,3 +35,7 @@ extern PyTypeObject	Ilual_Type;
 #define ilual_Check(op)	((op)->ob_type == &Ilual_Type)
 
 extern PyObject *	ilual_New(void);
+
+#ifdef __cplusplus
+}
+#endif
