@@ -14,8 +14,11 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: ilucaobject.h,v 1.4 1996/02/12 21:53:31 janssen Exp $
+$Id: ilucaobject.h,v 1.6 1996/07/09 23:03:54 larner Exp $
 */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* ILU Call Object implementation */
 
@@ -32,3 +35,7 @@ extern PyTypeObject	Iluca_Type;
 #define iluca_Check(op)	((op)->ob_type == &Iluca_Type)
 
 extern PyObject *	iluca_FromCall(ilu_Call call);
+
+#ifdef __cplusplus
+}
+#endif
