@@ -14,9 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: ilusvobject.h,v 1.2 1994/09/13 19:38:24 severson Exp $
+$Id: ilusvobject.h,v 1.4 1996/07/09 23:06:43 larner Exp $
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* ILU Server Object implementation */
 
 typedef struct
@@ -30,3 +34,7 @@ extern PyTypeObject	Ilusv_Type;
 #define ilusv_Check(op)	((op)->ob_type == &Ilusv_Type)
 
 extern PyObject *	ilusv_FromServer(ilu_Server kserver);
+
+#ifdef __cplusplus
+}
+#endif
