@@ -14,8 +14,11 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: iohcobject.h,v 1.2 1994/09/13 19:38:24 severson Exp $
+$Id: iohcobject.h,v 1.5 1996/07/09 23:10:25 larner Exp $
 */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* IO Handler Closure Object implementation */
 
@@ -31,5 +34,9 @@ extern PyTypeObject	Iohc_Type;
 
 #define iohc_Check(op)	((op)->ob_type == &Iohc_Type)
 
-extern PyObject *	iohc_New(int fileno, ilu_IOHandler handler,
+extern PyObject *	iohc_New(int fd, ilu_IOHandler handler,
 				ilu_private rock);
+
+#ifdef __cplusplus
+}
+#endif
