@@ -14,21 +14,21 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: python.h,v 1.19 1996/07/09 22:57:51 janssen Exp $
+$Id: python.h,v 1.22 1997/10/25 02:28:36 janssen Exp $
 */
 
 #ifdef __cplusplus
 #undef ILU_NIL
 #define ILU_NIL 0
-#define BOOLEAN(x) (x ? ilu_TRUE : ilu_FALSE)
+#define BOOLEAN(x) ((x) ? ilu_TRUE : ilu_FALSE)
 #else
-#define BOOLEAN(x) x
+#define BOOLEAN(x) (x)
 #endif
 
 /* Imports from Python */
 
 #define HAVE_PROTOTYPES 1	/* We assume an ANSI C compiler */
-#define HAVE_STDARG_PROTOTYPES	/* Again, ANSI C */
+#define HAVE_STDARG_PROTOTYPES 1	/* Again, ANSI C */
 
 #include "pythonversion.h"
 
@@ -53,9 +53,7 @@ typedef struct methodlist PyMethodDef;
 #endif /* (ILUPYTHON_MAJOR_VERSION == 1 && ILUPYTHON_MINOR_VERSION >= 2) */
 
 /* The following are for convenience. */
-#define PyCallable_Check(obj) \
-	(PyFunction_Check(obj) || PyCFunction_Check(obj) || PyMethod_Check(obj))
-
 extern char * _ilupython_formErrDescription (char * /* buf into which to print the error */, ilu_Error *);
 extern PyObject *_ilupython_GeneralError;
 extern ilu_cardinal _ilupython_LangIndex;
+extern void _ilupython_handleCalloutException (char *, ilu_Error *);
