@@ -14,8 +14,8 @@
  ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
  ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  */
-/* $Id: cheaders.c,v 1.82 1996/03/12 03:29:50 janssen Exp $ */
-/* Last edited by Mike Spreitzer January 10, 1996 1:33 am PST */
+/* $Id: cheaders.c,v 1.116 1997/11/21 03:24:15 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 1, 1997 2:07 pm PST */
 
 
 #include "cstubber.h"
@@ -28,6 +28,7 @@
   struct double_s {
     Type discriminant_type;
     Context context;
+    Argument default_arm;
     unsigned long id;
   };
 
@@ -138,9 +139,12 @@ static void declareException (Exception e, Context context)
 	  c_simple_name(e->name));
 }
 
-void listArgumentTypes (Argument arg, Context context)
+void listArgumentTypes(refany elt, refany rock)
 {
-  fprintf (context->file, ", %s", c_parameter_type (arg->type, arg->direction));
+  Argument        arg = (Argument) elt;
+  Context         context = (Context) rock;
+  fprintf(context->file, ", %s",
+	  c_parameter_type(arg->type, arg->direction));
 }
 
 
@@ -149,31 +153,34 @@ static void listException (Exception e, Context c)
   fprintf (c->file, " %s", exception_name(e));
 }
 
-static void mkClientPrototype (Procedure m, Context c)
+static void mkGenericFunction (refany elt, refany rock)
 {
-  fprintf (c->file, "%s", c_return_type(m->returnType));
-  if (list_size(m->exceptions) > 0)
-    {
-      fprintf (c->file, "  /* exceptions: ");
-      list_enumerate (m->exceptions, (iluparser_EnumProc) listException, c);
-      fprintf (c->file, " */");
-    }
-  fprintf (c->file, "\n");
+  Procedure       m = (Procedure) elt;
+  Context         c = (Context) rock;
+  fprintf(c->file, "extern %s", c_return_type(m->returnType));
+  if (list_size(m->exceptions) > 0) {
+    fprintf(c->file, "  /* exceptions: ");
+    list_enumerate(m->exceptions, (iluparser_EnumProc) listException, c);
+    fprintf(c->file, " */");
+  }
+  fprintf(c->file, "\n");
 
-  fprintf (c->file, "  %s_%s (%s", c_type_name (c->class), c_simple_name (m->name), c_type_name (c->class));
-  list_enumerate (m->arguments, (iluparser_EnumProc) listArgumentTypes, c);
-  fprintf (c->file, ", ILU_C_ENVIRONMENT *);\n\n");
+  fprintf(c->file, "  %s_%s (%s", c_type_name(c->class),
+	  c_simple_name(m->name), c_type_name(c->class));
+  list_enumerate(m->arguments, listArgumentTypes, c);
+  fprintf(c->file, ", ILU_C_ENVIRONMENT *);\n\n");
 }
 
-static void mkServerPrototype (Procedure m, Context c)
+static void mkServerPrototype(refany elt, refany rock)
 {
-  fprintf (c->file, "%s\n  server_%s_%s (%s", c_return_type (m->returnType),
-	   c_type_name (c->class), c_simple_name (m->name), c_type_name (c->class));
-  list_enumerate (m->arguments, (iluparser_EnumProc) listArgumentTypes, c);
-  fprintf (c->file, ", ILU_C_ENVIRONMENT *);\n");
+  Procedure       m = (Procedure) elt;
+  Context         c = (Context) rock;
+  fprintf(c->file, "extern ");
+  generateProcDecl(m, c, FALSE, "server_", "");
+  fprintf(c->file, ";\n\n");
 }
 
-static void generateClientPrototypes (Type t, Context context)
+static void generateGenericFunctions (Type t, Context context)
 {
   Class c;
 
@@ -181,10 +188,19 @@ static void generateClientPrototypes (Type t, Context context)
       type_basic_type(t) != object_Type ||
       (c = class_object(t)) == NULL)
     return;
-  list_enumerate(c->methods, (iluparser_EnumProc) mkClientPrototype, context);
+  list_enumerate(c->methods, mkGenericFunction, context);
 }
 
-static void generateServerPrototypes (Type t, Context context)
+static void declClientStub (refany elt, refany rock)
+{
+  Procedure       m = (Procedure) elt;
+  Context         c = (Context) rock;
+  fprintf(c->file, "extern ");
+  generateProcDecl(m, c, TRUE, "_", "__clientstub");
+  fprintf(c->file, ";\n");
+}
+
+static void declClientStubs (Type t, Context context)
 {
   Class c;
 
@@ -192,8 +208,19 @@ static void generateServerPrototypes (Type t, Context context)
       type_basic_type(t) != object_Type ||
       (c = class_object(t)) == NULL)
     return;
-  list_enumerate(c->superclasses, (iluparser_EnumProc) generateServerPrototypes, context);
-  list_enumerate(c->methods, (iluparser_EnumProc) mkServerPrototype, context);
+  list_enumerate(c->methods, declClientStub, context);
+}
+
+static void generateServerPrototypes(refany elt, refany rock)
+{
+  Type            t = (Type) elt;
+  Context         context = (Context) rock;
+  Class           c;
+  if (t == NULL || type_ur_kind(t) != object_Type ||
+      (c = class_object(t)) == NULL)
+    return;
+  list_enumerate(c->superclasses, generateServerPrototypes, context);
+  list_enumerate(c->methods, mkServerPrototype, context);
 }
 
 static void declareClassType (Type t, Context context)
@@ -214,7 +241,7 @@ static void declareCClass(refany elt, refany rock)
   Context         context = (Context) rock;
   Class           c;
   char *	  tn;
-  int             i, pad;
+  int             pad;
 
   if (t == NULL ||
       type_basic_type(t) != object_Type ||
@@ -235,24 +262,40 @@ static void declareCClass(refany elt, refany rock)
 	  "  %s__CreateFromSBH (char * /* sbh */, ILU_C_ENVIRONMENT * /* env */);\n\n",
 	  tn);
 
-  fprintf (context->file, "/* for true module only: */\n");
-  fprintf(context->file,
-	  "%s\n  %s__CreateTrue (ilu_string /* instance-handle */, ilu_Server /* server */, void * /* user data */);\n",
-	  tn, tn);
-  fprintf (context->file, "/* only for use within an object table's `ot_object_of_ih' method */\n");
-  fprintf(context->file,
-	  "%s\n  %s__OTCreateTrue (ilu_string /* instance-handle */, ilu_Server /* server */, void * /* user data */);\n",
-	  tn, tn);
+  fprintf (context->file, "/* For true module only: */\n\n");
   fprintf(context->file,
 	  "void\n  %s__SetUserData (%s /* self */, void * /* userData */);\n",
 	  tn, tn);
   fprintf(context->file, "void *\n  %s__GetUserData (%s /* self */);\n\n", tn, tn);
-  fprintf (context->file, "/*------- methods ------------------------------------------------------*/\n\n");
-  generateClientPrototypes(t, context);
-  fprintf (context->file, "\n\n");
+  fprintf(context->file, "extern %s\n", tn);
+  fprintf(context->file,
+	  "  %s__CreateTrue (ilu_string /* instance-handle */,",
+	  tn);
+  fprintf(context->file,
+	  "\n\tILU_C_Server /* server */, void * /* user data */);\n");
+  fprintf(context->file, "/* ... using default %s class */\n\n", tn);
+  fprintf(context->file, "extern %s\n", tn);
+  fprintf(context->file,
+	  "  %s__OTCreateTrue (ilu_string /* instance-handle */,",
+	  tn);
+  fprintf(context->file,
+	"\n\tILU_C_Server /* server */, void * /* user data */);\n");
+  fprintf(context->file, "/* only for use within an object table's `ot_object_of_ih' method */\n\n");
+  fprintf(context->file, "extern void %s__SetDefaultClassFinalization",
+	  tn);
+  fprintf(context->file, "(ILU_C_FinalizationProc f);\n\n");
+  fprintf(context->file, "extern ");
+  generateClassMakerDecl(t, context);
+  fprintf(context->file, ";\n\n");
+  fprintf (context->file, "/*------- generic functions --------------------------------------------*/\n\n");
+  generateGenericFunctions(t, context);
+  fprintf (context->file, "\n");
+  fprintf (context->file, "/*------- methods of the default class -------*/\n\n");
+  generateServerPrototypes(t, context);
+  fprintf(context->file, "\n\n");
 }
 
-void declareCClassJunk (Type t, Context context)
+static void declareCClassJunk (Type t, Context context)
 {
   char           *tn;
   int             i;
@@ -267,20 +310,8 @@ void declareCClassJunk (Type t, Context context)
   context->class = t;
 
   fprintf(context->file, "\n");
-  fprintf(context->file, "#ifdef _%s_TRUE_STUB_CODE\n",
-	  c_interface_name(context->interface));
-  generateServerPrototypes(t, context);
-  fprintf(context->file, "#endif /* def _%s_TRUE_STUB_CODE */\n\n",
-	  c_interface_name(context->interface));
   fprintf(context->file, "extern ilu_Class _%s__ILUType;\n", tn);
-  fprintf(context->file, "struct _%s__MethodBlock_s {\n", tn);
-  fprintf(context->file, " ilu_Class c;\n");
-  if ((i = list_size(c->methods)) > 0)
-    fprintf(context->file, " void (*methods[%u])(void);\n", i);
-  fprintf(context->file, "};\n");
-  fprintf(context->file,
-     "extern struct _%s__MethodBlock_s _%s__SurrogateMethodBlock;",
-	  tn, tn);
+  declClientStubs(t, context);
 }
 
 static boolean matchPointer (refany p1, refany p2)
@@ -300,11 +331,56 @@ static void generateRecordDeclaration (Type t, Context context)
   fprintf (context->file, "/* record \"%s\" */\n", c_type_name(t));
 
   fprintf (context->file, "struct %s {\n", c_type_name (t));
-  list_enumerate (type_description (t)->structuredDes.record, (iluparser_EnumProc) generateRecordField, context);
+  list_enumerate (type_description (t)->structuredDes.record.fields, (iluparser_EnumProc) generateRecordField, context);
   fprintf (context->file, "};\n");
 }
 
-static void generateUnionField (Argument a, Context context)
+static void listUnionDiscriminatorValues (Argument a, struct double_s *s)
+{
+  char *str;
+
+  if (a == s->default_arm)
+    fprintf (s->context->file, "DEFAULT");
+  else if (list_size(a->values) == 0)
+    fprintf (s->context->file, "%lu", s->id++);
+  else
+    {
+      ConstantValue val;
+      int i, limit;
+
+      for (i = 0, limit = list_size(a->values);  i < limit;  i++) {
+	val = list_ref(a->values, i);
+	switch (val->type)
+	  {
+	  case integer_Type:
+	  case shortinteger_Type:
+	  case cardinal_Type:
+	  case shortcardinal_Type:
+	  case byte_Type:
+	    fprintf (s->context->file, "%s%s%lu",
+		     (i == 0) ? "" : ", ",
+		     (val->val.i.sign < 0) ? "-" : "", val->val.i.value);
+	    break;
+	  case shortcharacter_Type:
+	    str = c_string(val->val.s);
+	    fprintf (s->context->file, "%s%s_%s",
+		     (i == 0) ? "" : ", ",
+		     c_interface_name(s->discriminant_type->interface), str);
+	    free(str);
+	    break;
+	  case boolean_Type:
+	    fprintf (s->context->file, "%silu_%s",
+		     (i == 0) ? "" : ", ",
+		     val->val.b ? "TRUE" : "FALSE");
+	    break;
+	  default:
+	    error ("illegal discriminator value\n");
+	  }
+      }
+    }
+}
+
+static void generateUnionField (Argument a, struct double_s *s)
 {
   char *name;
 
@@ -312,56 +388,28 @@ static void generateUnionField (Argument a, Context context)
     name = (char *) c_simple_name (a->name);
   else
     name = (char *) c_string (type_name (a->type));
-  fprintf (context->file, "\t\t%s %s;\n", c_type_name (a->type), name);
-}
-
-static void generateUnionTypeDiscriminator (Argument a, struct double_s *s)
-{
-  fprintf (s->context->file, "#define %s__%s ", c_type_name (s->context->class), c_string (type_name (a->type)));
-  if (list_size(a->values) == 0)
-    fprintf (s->context->file, "%lu\n", s->id++);
-  else
-    {
-      ConstantValue val = list_car(a->values);
-      switch (val->type)
-	{
-	case integer_Type:
-	case shortinteger_Type:
-	case cardinal_Type:
-	case shortcardinal_Type:
-	case byte_Type:
-	  fprintf (s->context->file, "%s%lu\n", (val->val.i.sign < 0) ? "-" : "", val->val.i.value);
-	  break;
-	case shortcharacter_Type:
-	  fprintf (s->context->file, "%s_%s\n", c_interface_name(s->discriminant_type->interface), val->val.s);
-	  break;
-	case boolean_Type:
-	  fprintf (s->context->file, "ilu_%s\n", val->val.b ? "TRUE" : "FALSE");
-	  break;
-	default:
-	  error ("illegal discriminator value\n");
-	}
-    }
+  fprintf (s->context->file, "\t\t%s %s;\t/* ", c_type_name (a->type), name);
+  listUnionDiscriminatorValues (a, s);
+  fprintf (s->context->file, " */\n");
 }
 
 static void generateUnionDeclaration (Type t, Context context)
 {
-  Type d = type_description (t)->structuredDes.uniond.discriminator_type;
-  list e = type_description (t)->structuredDes.uniond.types;
+  Type d = type_description(t)->structuredDes.uniond.discriminator_type;
+  list e = type_description(t)->structuredDes.uniond.types;
   struct double_s s;
   
-  s.discriminant_type = ur_type(type_description (t)->structuredDes.uniond.discriminator_type);
+  s.discriminant_type = ur_type(type_description(t)->structuredDes.uniond.discriminator_type);
   s.id = 0;
+  s.default_arm = type_description(t)->structuredDes.uniond.default_arm;
   s.context = context;
   context->class = t;
 
-  fprintf (context->file, "/* union \"%s\" */\n", c_type_name(t));
-
-  list_enumerate (e, (iluparser_EnumProc) generateUnionTypeDiscriminator, &s);
+  fprintf (context->file, "/* union \"%s\" */\n\n", c_type_name(t));
 
   fprintf (context->file, "struct _%s_union {\n", c_type_name (t));
   fprintf (context->file, "\t%s _d;\n\tunion {\n", c_type_name (d));
-  list_enumerate (e, (iluparser_EnumProc) generateUnionField, context);
+  list_enumerate (e, (iluparser_EnumProc) generateUnionField, &s);
   fprintf (context->file, "\t} _u;\n};\n");
 }
 
@@ -394,19 +442,20 @@ static void generateEnumerationTypedefDeclaration (Type t, Context context)
   fprintf (context->file, "} %s;\n", c_type_name (t));
 }
 
-static void generateSequenceDeclaration (Type t, Context context)
+static void generateSequenceDeclaration(Type t, Context context)
 {
-  TypeDescription d = type_description (t);
+  TypeDescription d = type_description(t);
+  TypeKind        tk = type_ur_kind(d->structuredDes.sequence.type);
 
-  if (type_basic_type (d->structuredDes.sequence.type) == shortcharacter_Type ||
-      type_basic_type (d->structuredDes.sequence.type) == character_Type)
+  if (tk == shortcharacter_Type || tk == character_Type)
     return;
-  else {
-    fprintf (context->file, "/* sequence \"%s\" */\n", c_type_name(t));
-    fprintf (context->file, "struct _%s__sequence {\n unsigned long _maximum;\n", c_type_name (t));
-    fprintf (context->file, " unsigned long _length;\n %s *_buffer;\n};\n",
-	     c_type_name (d->structuredDes.sequence.type));
-  }
+  fprintf(context->file, "/* sequence \"%s\" */\n", c_type_name(t));
+  fprintf(context->file, "struct _%s__sequence {\n", c_type_name(t));
+  fprintf(context->file, " unsigned long _maximum;\n");
+  fprintf(context->file, " unsigned long _length;\n");
+  fprintf(context->file, " %s *_buffer;\n};\n",
+	  c_type_name(d->structuredDes.sequence.type));
+
 }
 
 static void AliasMethod (Procedure p, Context context)
@@ -435,7 +484,7 @@ static void generateAliasSupport (Type t, Context context)
   /* in general, nothing to be done, except in the case of objects and sequences */
   Type ut;
 
-  if (type_basic_type(t) != alias_Type)
+  if (type_kind(t) != alias_Type)
     return;
 
   ut = ur_type(t);
@@ -450,9 +499,13 @@ static void generateAliasSupport (Type t, Context context)
       fprintf (context->file, "#define %s__CreateTrue %s__CreateTrue\n", c_type_name(t), c_type_name(ut));
       fprintf (context->file, "#define %s__OTCreateTrue %s__OTCreateTrue\n", c_type_name(t), c_type_name(ut));
       fprintf (context->file, "#define %s__CreateFromSBH %s__CreateFromSBH\n", c_type_name(t), c_type_name(ut));
+      fprintf (context->file, "#define %s__CreateFromURL %s__CreateFromURL\n", c_type_name(t), c_type_name(ut));
       fprintf (context->file, "#define %s__SetUserData %s__SetUserData\n", c_type_name(t), c_type_name(ut));
       fprintf (context->file, "#define %s__GetUserData %s__GetUserData\n", c_type_name(t), c_type_name(ut));
-      fprintf (context->file, "#define %s__MSType _%s__ILUType\n", c_type_name(t), c_type_name(ut));
+      fprintf (context->file, "#define _%s__ILUType _%s__ILUType\n", c_type_name(t), c_type_name(ut));
+      fprintf (context->file, "#define %s__MSType _%s__ILUType\n", c_type_name(t),
+	       (TypeIsJustAlias(t) ? c_type_name(ut) : c_type_name(t)));
+      fprintf (context->file, "#define %s__Free %s__Free\n", c_type_name(t), c_type_name(ut));
       AliasMethodsOfClass (t, context);
       break;
 
@@ -472,10 +525,12 @@ static void generateAliasSupport (Type t, Context context)
   
 	s.discriminant_type = ur_type(type_description(t)->structuredDes.uniond.discriminator_type);
 	s.id = 0;
+	s.default_arm = type_description(t)->structuredDes.uniond.default_arm;
 	s.context = context;
-
+/*
 	list_enumerate (type_description(t)->structuredDes.uniond.types,
 			(iluparser_EnumProc) generateUnionTypeDiscriminator, &s);
+*/
       }
       break;
 
@@ -487,9 +542,10 @@ static void generateAliasSupport (Type t, Context context)
   {
     char *a = c_type_name(t);
     char *b = c_type_name(ut);
-    fprintf (context->file, "#define CORBA_sequence_%s_bufalloc CORBA_sequence_%s_bufalloc\n",
+    fprintf (context->file, "#define CORBA_sequence_%s_allocbuf CORBA_sequence_%s_allocbuf\n",
 	     (t->builtIn && strncmp(a, "CORBA_", 6) == 0) ? a + 6 : a,
-	     (ut->builtIn && strncmp(b, "CORBA_", 6) == 0) ? b + 6 : b);
+	     (TypeIsString(ut) ? "string" :
+	      (ut->builtIn && strncmp(b, "CORBA_", 6) == 0) ? b + 6 : b));
   }
 }
 
@@ -497,7 +553,7 @@ static void generateBufAlloc (Type t, Context c)
 {
   char *s = c_type_name(t);
 
-  fprintf (c->file, "%s *\n  CORBA_sequence_%s_bufalloc (CORBA_unsigned_long /* count */);\n",
+  fprintf (c->file, "%s *\n  CORBA_sequence_%s_allocbuf (CORBA_unsigned_long /* count */);\n",
 	   s, (t->builtIn && strncmp(s, "CORBA_", 6) == 0) ? s + 6 : s);
 }
 
@@ -511,7 +567,7 @@ static void declareType (Type t, Context context)
   if (t->importInterfaceName != NULL)
     return;	/* defined in some other interface */
 
-  switch (type_basic_type (t)) {
+  switch (type_kind(t)) {
   case void_Type:
   case integer_Type:
   case cardinal_Type:
@@ -555,6 +611,10 @@ static void declareType (Type t, Context context)
     /* already done */
     break;
 
+  case pickle_Type:
+    /* already done */
+    break;
+
   default:
     fatal ("Error: Can't cope with declaration of type %s yet.\n", c_type_name (t));
   }
@@ -576,11 +636,12 @@ static void OutputDim (long d, Context context)
   fprintf (context->file, "[%lu]", d);
 }
 
-static void generateArrayTypedef (Type t, Context context)
+static void generateArrayTypedef (Type t, Context context, boolean slice)
 {
   TypeDescription d = type_description (t);
+  Type et = ur_type(d->structuredDes.array.type);
   fprintf (context->file, "typedef ");
-  switch (type_ur_kind(d->structuredDes.array.type)) {
+  switch (type_kind(et)) {
   case void_Type:
   case integer_Type:
   case cardinal_Type:
@@ -599,48 +660,64 @@ static void generateArrayTypedef (Type t, Context context)
   case object_Type:
   case optional_Type:
   case enumeration_Type:
-    fprintf (context->file, "%s", c_type_name(d->structuredDes.array.type));
+    fprintf (context->file, "%s", c_type_name(et));
     break;
 
+#ifdef ADD_VARIANT_SUPPORT
+  case pickle_Type:
+    fprintf (context->file, "CORBA_any");
+    break;
+#endif
+
   case record_Type:
-    fprintf (context->file, "struct %s", c_type_name(d->structuredDes.array.type));
+    fprintf (context->file, "struct %s", c_type_name(et));
     break;
 
   case union_Type:
-    fprintf (context->file, "struct _%s_union", c_type_name(d->structuredDes.array.type));
+    fprintf (context->file, "struct _%s_union", c_type_name(et));
     break;
 
   case sequence_Type:
     {
-      TypeKind t = type_ur_kind(type_description(d->structuredDes.array.type)->structuredDes.sequence.type);
+      Type eet = type_description(et)->structuredDes.sequence.type;
+      TypeKind        t = type_ur_kind(eet);
       if (t == shortcharacter_Type)
 	fprintf (context->file, "CORBA_char *");
       else if (t == character_Type)
 	fprintf (context->file, "ilu_character *");
       else
-	fprintf (context->file, "struct _%s__sequence", c_type_name(d->structuredDes.array.type));
+	fprintf (context->file, "struct _%s__sequence", c_type_name(et));
       break;
     }
 
   default:
-    fatal ("Error: Can't cope with declaration of type %s yet.\n", c_type_name (d->structuredDes.array.type));
+    fatal("Error: Can't cope with declaration of type %s yet.\n",
+	  c_type_name(et));
   }
 
-  fprintf (context->file, " %s", c_type_name (t));
-  list_enumerate (d->structuredDes.array.dimensions, (iluparser_EnumProc) OutputDim, context);
+  fprintf (context->file, " %s%s", c_type_name (t), (slice) ? "_slice" : "");
+  if (slice)
+    list_enumerate (list_cdr(d->structuredDes.array.dimensions), (iluparser_EnumProc) OutputDim, context);
+  else
+    list_enumerate (d->structuredDes.array.dimensions, (iluparser_EnumProc) OutputDim, context);
   fprintf (context->file, ";\n");
 }
 
-static void generateSequenceTypedef (Type t, Context context)
+static void generateSequenceTypedef(Type t, Context context)
 {
-  TypeDescription d = type_description (t);
+  TypeDescription d = type_description(t);
+  Type            et = d->structuredDes.sequence.type;
+  TypeKind        etk = type_ur_kind(et);
 
-  if (type_basic_type (d->structuredDes.sequence.type) == shortcharacter_Type)
-    fprintf (context->file, "typedef CORBA_char * %s;\n", c_type_name (t));
-  else if (type_basic_type (d->structuredDes.sequence.type) == character_Type)
-    fprintf (context->file, "typedef ilu_character * %s;\n", c_type_name (t));
+  if (etk == shortcharacter_Type)
+    fprintf(context->file, "typedef CORBA_char * %s;\n",
+	    c_type_name(t));
+  else if (etk == character_Type)
+    fprintf(context->file, "typedef ilu_character * %s;\n",
+	    c_type_name(t));
   else {
-    fprintf (context->file, "typedef struct _%s__sequence %s;\n", c_type_name(t), c_type_name(t));
+    fprintf(context->file, "typedef struct _%s__sequence %s;\n",
+	    c_type_name(t), c_type_name(t));
   }
 }
 
@@ -649,32 +726,15 @@ static void generateRecordTypedef (Type t, Context context)
   fprintf (context->file, "typedef struct %s %s;\n", c_type_name (t), c_type_name (t));
 }
 
-static void generateOptionalTypedef (Type t, Context context)
+static void generateOptionalTypedef(Type t, Context context)
 {
-  Type            t2 = type_description(t)->structuredDes.optional;
-  TypeKind        tp = type_ur_kind(t2);
-
-  fprintf (context->file, "typedef ");
-  switch (tp)
-    {
-    case sequence_Type:
-      {
-	if (TypeIsString(ur_type(t2)) || TypeIsWString(ur_type(t2)))
-	  fprintf (context->file, "%s", c_type_name(t2));
-	else
-	  fprintf (context->file, "%s *", c_type_name(t2));
-	break;
-      }
-    case optional_Type:
-    case array_Type:
-    case object_Type:
-      fprintf (context->file, "%s", c_type_name(t2));
-      break;
-    default:
-      fprintf (context->file, "%s *", c_type_name(t2));
-      break;
-    }
-  fprintf (context->file, " %s;\n", c_type_name(t));
+  Type            t2 = ur_type(type_description(t)->structuredDes.optional);
+  while (type_kind(t2) == optional_Type)
+    t2 = ur_type(type_description(t2)->structuredDes.optional);
+  fprintf(context->file, "typedef %s %s%s;\n",
+	  c_type_name(t2),
+	  TypeIsPointer(t2) ? "" : "*",
+	  c_type_name(t));
 }
 
 static void generateAliasTypedef (Type t, Context context)
@@ -682,6 +742,15 @@ static void generateAliasTypedef (Type t, Context context)
   fprintf (context->file, "typedef %s %s;\n", c_type_name(ur_type(t)), c_type_name(t));
 }
 
+static void generateVariantTypedef (Type t, Context c)
+{
+  if (type_kind(t) == pickle_Type)
+    fprintf(c->file, "typedef CORBA_any %s;\n", c_type_name(t));
+  else
+    fprintf (c->file, "typedef struct { CORBA_TypeCode _type; void * _value; } %s;\n",
+	     c_type_name(t));
+}
+
 static void typedefType (Type t, Context context)
 {
   if (t->builtIn)
@@ -738,11 +807,17 @@ static void typedefType (Type t, Context context)
     generateOptionalTypedef (t, context);
     break;
   case array_Type:
-    generateArrayTypedef (t, context);
+    generateArrayTypedef (t, context, FALSE);
+    generateArrayTypedef (t, context, TRUE);
     break;
   case record_Type:
     generateRecordTypedef (t, context);
     break;
+#ifdef ADD_VARIANT_SUPPORT
+  case pickle_Type:
+    generateVariantTypedef (t, context);
+    break;
+#endif
   case object_Type:
     break;
   case union_Type:
@@ -775,7 +850,8 @@ static void generateSeqProto (Type t, Context c)
   tn = c_type_name (t);
   st = c_parameter_type (seq, In);
   rtn = c_type_name (seq);
-  nr = (bt == record_Type OR (bt == sequence_Type AND NOT TypeIsString(ur_type(seq))) OR bt == union_Type);
+  nr = (bt == record_Type OR bt == union_Type OR bt == pickle_Type OR
+   (bt == sequence_Type AND NOT TypeIsEitherString(ur_type(seq))));
   if (bt == shortcharacter_Type || bt == character_Type)
     {
       /* these are a bit different */
@@ -791,41 +867,41 @@ static void generateSeqProto (Type t, Context c)
       fprintf (c->file, "void %s_Every (%s *h, void (*f)(%s%s, void *), void *);\n", tn, tn, st, nr ? "" : "*");
       fprintf (c->file, "void %s_Append (%s *h, %s item);\n", tn, tn, st);
       fprintf (c->file, "void %s_Push (%s *h, %s item);\n", tn, tn, st);
-      fprintf (c->file, "void %s_Pop (%s *h, %s item);\n", tn, tn, st);
+      fprintf (c->file, "void %s_Pop (%s *h, %s %sitem);\n", tn, tn, st, (nr) ? "" : "*");
       fprintf (c->file, "CORBA_unsigned_long %s_Length (%s *);\n", tn, tn);
       fprintf (c->file, "%s * %s_Nth (%s *, CORBA_unsigned_long);\n", rtn, tn, tn);
-      fprintf (c->file, "%s %s_Create (CORBA_unsigned_long sz, %s %sp);\n", tn, tn, st, (nr) ? "" : "*");
-      fprintf (c->file, "void %s_Init (%s *s, CORBA_unsigned_long sz, %s %sp);\n", tn, tn, st, (nr) ? "" : "*");
+      fprintf (c->file, "%s * %s_Create (CORBA_unsigned_long /* size */, %s %s /* init val */);\n", tn, tn, st, (nr) ? "" : "*");
+      fprintf (c->file, "void %s_Init (%s * /* seq */, CORBA_unsigned_long /* size */, %s %s /* init val */);\n", tn, tn, st, (nr) ? "" : "*");
     }
 }
 
 static void generateNormalIncludes (Context c)
 {
-  fprintf (c->file, "#ifndef __ilu_c_h_\n");
-  fprintf (c->file, "#include \"iluchdrs.h\"\n");
-  fprintf (c->file, "#endif\n");
+  fprintf (c->file, "#include \"ilucstub.h\"\n");
 }
 
-static void generateInputPrototype (Type type, enum PrimitiveTypes t, Context context)
+static void 
+generateInputPrototype(Type type, TypeKind t, Context context)
 {
-  char *ret = (char *) c_return_type (type);
-  char *parm = c_parameter_type (type, InOut);
-
+  string          parm = c_parameter_type(type, InOut);
+  string          rettype = c_role_type(type, role_InpRet, FALSE);
   context->class = type;
-
-  fprintf (context->file, "extern %s _%s__Input (ilu_Call, %s, ilu_Error *);\n",
-	   (t == array_Type OR TypeIsString(type) OR t == optional_Type) ? ret : parm,
-	   c_type_name(type), parm);
+  fprintf(context->file,
+	  "extern %s _%s__Input (ilu_Call, %s, ilu_Error *);\n",
+	  rettype, c_type_name(type), parm);
 }
 
 static void generateFreePrototype (Type type, enum PrimitiveTypes t, Context context)
 {
-  fprintf (context->file, "extern void %s__Free (%s);\n", c_type_name(type), c_parameter_type (type, Out));
+  fprintf(context->file, "extern void %s__Free (%s);\n",
+	  c_type_name(type),
+	  c_role_type(type, role_Exn, FALSE));
 }
 
 static void generateOutputPrototype (Type type, enum PrimitiveTypes t, Context context)
 {
-  fprintf (context->file, "extern void _%s__Output (ilu_Call, %s, ilu_Error *);\n", c_type_name(type), c_parameter_type (type, In));
+  fprintf (context->file, "extern void _%s__Output (ilu_Call, %s, ilu_Error *);\n", c_type_name(type),
+	   c_parameter_type (type, In));
 }
 
 static void generateSizeOfPrototype (Type type, enum PrimitiveTypes t, Context context)
@@ -837,34 +913,58 @@ static void generateSizeOfPrototype (Type type, enum PrimitiveTypes t, Context c
 static void generateAllocHeader (Type type, enum PrimitiveTypes t, Context context)
 {
   char *s = c_type_name(type);
-  char *p = (type_kind(type) == array_Type) ? c_return_type(type) : c_parameter_type(type, Out);
+  char *p = (type_kind(type) == array_Type) ? c_return_type(type) : c_parameter_type(type, InOut);
 
-  fprintf (context->file, "%s %s__alloc ();\n", p, s);
+  fprintf (context->file, "%s %s__alloc (void);\n", p, s);
+}
+
+static void generateIoFnsHeader (Type t, Context c)
+{
+  if (t->builtIn || (t->importInterfaceName != NULL))
+    return;
+
+  if (TypeIsJustAlias(t))
+    fprintf (c->file, "#define _%s__IoFns _%s__IoFns\n",
+	     c_type_name(t), c_type_name(ur_type(t)));
+
+  else if (TypeIsString(t))
+    fprintf (c->file, "#define _%s__IoFns _ilu_CString__IoFns\n", c_type_name(t));
+
+  else if (type_ur_kind(t) == pickle_Type)
+    fprintf (c->file, "#define _%s__IoFns _CORBA_any__IoFns\n", c_type_name(t));
+
+  else
+    fprintf (c->file, "extern struct _ILU_C_IoFnsRegistration_s _%s__IoFns;\n",
+	     c_type_name(t));
+
+  fprintf (c->file, "#define TC_%s (& _%s__IoFns)\n",
+	   c_type_name(t), c_type_name(t));
 }
 
 static void generateTypeIoProto (Type type, Context context)
 {
-  enum PrimitiveTypes t = type_basic_type (type);
+  TypeKind        t = type_basic_type(type);
 
+  generateIoFnsHeader (type, context);
   if (type->supertype != NULL
-      OR type->interface != context->interface
-      OR type->importInterfaceName != NULL)
+      || type->interface != context->interface
+      || type->importInterfaceName != NULL)
     return;
 
-  if (type->importInterfaceName == NULL AND
-      (t == union_Type OR
-       t == record_Type OR
-       t == sequence_Type OR
-       t == array_Type OR
-       t == optional_Type))
-    {
-      generateOutputPrototype (type, t, context);
-      generateSizeOfPrototype (type, t, context);
-      generateInputPrototype (type, t, context);
-      generateFreePrototype (type, t, context);
-    }
+  if (type->importInterfaceName == NULL &&
+      (t == union_Type ||
+       t == record_Type ||
+       (t == sequence_Type && (!TypeIsString(type))) ||
+       t == array_Type ||
+       t == optional_Type)) {
+    generateOutputPrototype(type, t, context);
+    generateSizeOfPrototype(type, t, context);
+    generateInputPrototype(type, t, context);
+  }
+  if (HasFreeRoutine(type))
+    generateFreePrototype(type, t, context);
   if (HasAllocRoutine(type))
-    generateAllocHeader (type, t, context);
+    generateAllocHeader(type, t, context);
 }
 
 static Interface SortContextInterface = NULL;
@@ -881,6 +981,8 @@ static void sortMethodTypes (Procedure m, list sorted)
   list_enumerate (m->arguments, (iluparser_EnumProc) sortArgTypes, sorted);
 }
 
+static boolean sortingForTypedef;
+
 static void sortTypesForDeclaration (Type type, list sorted)
 {
   enum PrimitiveTypes t;
@@ -915,7 +1017,8 @@ static void sortTypesForDeclaration (Type type, list sorted)
       switch (t) {
 
       case record_Type:
-	list_enumerate (type_description (type)->structuredDes.record, (iluparser_EnumProc) sortArgTypes, sorted);
+	if (!sortingForTypedef)
+	  list_enumerate (type_description (type)->structuredDes.record.fields, (iluparser_EnumProc) sortArgTypes, sorted);
 	break;
 
       case alias_Type:
@@ -960,6 +1063,10 @@ static void sortTypesForDeclaration (Type type, list sorted)
   }
 }
 
+static void printSortedTypes (Type t, void * junk)
+{
+  fprintf (stderr, "%s\n", type_name(t));
+}
 
 void generateCHeaders (Interface interface, FILE *file)
 {
@@ -973,6 +1080,9 @@ void generateCHeaders (Interface interface, FILE *file)
   context.class = NULL;
   fprintf (file, "#ifndef __%s_h_\n", c_interface_name (interface));
   fprintf (file, "#define __%s_h_\n\n", c_interface_name (interface));
+
+  fprintf (file, "#ifdef __cplusplus\nextern \"C\" {\n#endif\n\n");
+
   generateNecessaryIncludes (&context);
   generateNormalIncludes (&context);
   fprintf (file, "\n");
@@ -1004,12 +1114,6 @@ void generateCHeaders (Interface interface, FILE *file)
   fprintf (file, "extern void %s__InitializeServer(void);\n\n\n", c_interface_name(interface));
 
 
-  if (list_size(interface->constants) > 0)
-    { fprintf (file, "/*======================================================================*/\n");
-      fprintf (file, "/*=========================== constants ================================*/\n\n");
-      list_enumerate (interface->constants, (iluparser_EnumProc) declareConstant, &context);
-      fprintf (file, "\n\n");
-    }
   if (list_size(interface->classes) > 0)
     { fprintf (file, "/*======================================================================*/\n");
       fprintf (file, "/*=================== object type declarations =========================*/\n\n");
@@ -1018,20 +1122,38 @@ void generateCHeaders (Interface interface, FILE *file)
     }
 
   SortContextInterface = interface;
+  sortingForTypedef = 1;
   list_enumerate (interface->types, (iluparser_EnumProc) sortTypesForDeclaration, sorted);
-/*
+  /*
   list_enumerate (sorted, (iluparser_EnumProc) printSortedTypes, NULL);
+  */
   fprintf (file, "\n");
-*/
-  if (list_size(sorted) > 0)
-    { fprintf (file, "/*======================================================================*/\n");
-      fprintf (file, "/*========================= typedefs ===================================*/\n\n");
-      list_enumerate (sorted, (iluparser_EnumProc) typedefType, &context);
-      fprintf (file, "\n\n");
+  if (list_size(sorted) > 0) {
+    fprintf (file, "/*======================================================================*/\n");
+    fprintf (file, "/*========================= typedefs ===================================*/\n\n");
+    list_enumerate (sorted, (iluparser_EnumProc) typedefType, &context);
+    fprintf (file, "\n\n");
+    list_clear(sorted, FALSE);
+  };
 
-      fprintf (file, "/*======================================================================*/\n");
-      fprintf (file, "/*=============== non-object type declarations =========================*/\n\n");
-      list_enumerate (sorted, (iluparser_EnumProc) declareType, &context);
+  SortContextInterface = interface;
+  sortingForTypedef = 0;
+  list_enumerate (interface->types, (iluparser_EnumProc) sortTypesForDeclaration, sorted);
+  /*
+  list_enumerate (sorted, (iluparser_EnumProc) printSortedTypes, NULL);
+  */
+  fprintf (file, "\n");
+  if (list_size(sorted) > 0) {
+    fprintf (file, "/*======================================================================*/\n");
+    fprintf (file, "/*=============== non-object type declarations =========================*/\n\n");
+    list_enumerate (sorted, (iluparser_EnumProc) declareType, &context);
+    fprintf (file, "\n\n");
+  }
+
+  if (list_size(interface->constants) > 0)
+    { fprintf (file, "/*======================================================================*/\n");
+      fprintf (file, "/*=========================== constants ================================*/\n\n");
+      list_enumerate (interface->constants, (iluparser_EnumProc) declareConstant, &context);
       fprintf (file, "\n\n");
     }
 
@@ -1063,6 +1185,7 @@ void generateCHeaders (Interface interface, FILE *file)
   list_enumerate (interface->classes, (iluparser_EnumProc) declareCClassJunk, &context);
   fprintf (file, "\n");
   fprintf (file, "extern void _%s__GeneralInitialization(void);\n\n", c_interface_name(interface));
+  fprintf (file, "#ifdef __cplusplus\n}\n#endif\n\n");
   fprintf (file, "#endif /* ifndef __%s_h_ */\n", c_interface_name(interface));
 }
 
