@@ -14,7 +14,8 @@
  ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
  ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  */
-/* $Id: client.c,v 1.77 1996/06/21 01:12:56 janssen Exp $ */
+/* $Id: client.c,v 1.95 1997/11/08 02:59:26 janssen Exp $ */
+/* Last edited by Mike Spreitzer March 17, 1997 12:51 pm PST */
 
 #include "cstubber.h" 
 
@@ -60,17 +61,18 @@ static void listArgument (refany elt, refany rock)
 	  c_argument_name(arg));
 }
 
-void generateProcHeader (Procedure m, Context c, boolean external)
+void
+generateProcDecl(Procedure m, Context c, boolean urname,
+		 char *name_prefix, char *name_suffix)
 {
-  fprintf(c->file, "%s%s %s%s_%s (%s _handle",
-	  external ? "" : "static ",
+  fprintf(c->file, "%s %s%s_%s%s (%s _handle",
 	  ((type_basic_type(m->returnType) == void_Type)
 	   ? "void" : c_return_type(m->returnType)),
-	  external ? "" : "_",
-	  c_type_name(c->class),
-	  c_simple_name(m->name), c_type_name(c->class));
+	  name_prefix, c_type_name(urname ? m->object : c->class),
+	  c_simple_name(m->name), name_suffix,
+	  c_type_name(c->class));
   list_enumerate(m->arguments, listArgument, c);
-  fprintf(c->file, ", ILU_C_ENVIRONMENT *_status)\n{\n");
+  fprintf(c->file, ", ILU_C_ENVIRONMENT *_status)");
 }
 
 int methodNdxOf (Procedure p, Type t)
@@ -128,6 +130,7 @@ static void outputArgument (Argument arg, Context context)
 	  (t != sequence_Type AND
 	   t != union_Type AND
 	   t != array_Type AND
+	   t != pickle_Type AND
 	   t != record_Type))
 	{
 	  *b = '*';
@@ -156,17 +159,18 @@ static void listArg (Context c, char *op, Type t, char *name)
 {
   TypeKind        tk = type_ur_kind(t);
   fprintf(c->file, ",\n                       %s", name);
-  if (!(ur_type(t)->builtIn OR
-	TypeIsEitherString(t) OR
-	(type_ur_kind(t) == object_Type) OR
-	(type_ur_kind(t) == enumeration_Type)))
-    fprintf(c->file, ", _%s__%s", c_type_name(ur_type(t)), op);
+
   if (tk == object_Type)
-    fprintf(c->file, ", _%s__ILUType", c_type_name(ur_type(t)));
-  if (TypeIsEitherString(t))
+    fprintf (c->file, ", _%s__ILUType", c_type_name(ur_type(t)));
+  else if (TypeIsEitherString(t))
     fprintf(c->file, ", (ilu_cardinal) 0x%lx",
 	    (unsigned long) (type_description(t)->
 			     structuredDes.sequence.limit));
+  else if (type_kind(t) == pickle_Type)
+    fprintf(c->file, ", _CORBA_any__%s", op);
+  else if ((tk != enumeration_Type) &&
+	   (!(ur_type(t)->builtIn)))
+    fprintf(c->file, ", _%s__%s", c_type_name(ur_type(t)), op);
 }
 
 static void listSizeOfArgs (refany elt, refany rock)
@@ -230,16 +234,17 @@ void codeType (Type t, char **p, Role role)
 	a = 'z';
     }
     break;
+  case optional_Type:	a = 's'; break;
   case array_Type:
     a = (role == role_Out && VariableLength(t) ||
 	 role == role_Return) ? 'y' : 'z';
     break;
   case record_Type:
+  case pickle_Type:
   case union_Type:
     a = ((role == role_Out || role == role_Return) &&
 	 VariableLength(t)) ? 'y' : 'z';
     break;
-  case optional_Type:	a = 'z'; break;
   case void_Type:     a = '*'; break;
   default:     fatal ("Error: Bad parameter type %s.\n", c_type_name (t));
   }
@@ -267,20 +272,101 @@ static void codeOutputArg (refany elt, refany rock)
     codeType (a->type, p, adRole[a->direction]);
 }
 
-static char *genParmDescription (Procedure m)
+static cardinal declareParmDescriptionBlock (Procedure m, Context c)
 {
-  int len = list_size(m->arguments) * 2 + 3;
-  char *p, *q;
+  cardinal	size = 0, i;
+  Argument	a;
 
-  p = (char *) malloc(len);
-  q = p;
-  list_enumerate (m->arguments, codeInputArg, &q);
-  *q++ = ':';
-  if (m->returnType != NULL && type_ur_kind(m->returnType) != void_Type)
-    codeType (m->returnType, &q, role_Return);
-  list_enumerate (m->arguments, codeOutputArg, &q);
-  *q = 0;
-  return p;
+  size = list_size(m->arguments) + ((m->returnType == NULL) ? 0 : 1);
+
+  if (size > 0)
+    fprintf (c->file, "  _ILU_C_ParmDesc __parmsBlock[%u];\n", size);
+  return size;
+}
+
+static boolean IsScalar (Type type)
+{
+  enum PrimitiveTypes t;
+
+  if (type == NULL)
+    return (False);
+  t = type_ur_kind (type);
+  return  ((t == byte_Type) ||
+	   (t == boolean_Type) ||
+	   (t == shortinteger_Type) ||
+	   (t == integer_Type) ||
+	   (t == longinteger_Type) ||
+	   (t == shortcardinal_Type) ||
+	   (t == cardinal_Type) ||
+	   (t == longcardinal_Type) ||
+	   (t == shortreal_Type) ||
+	   (t == real_Type) ||
+	   (t == longreal_Type) ||
+	   (t == shortcharacter_Type) ||
+	   (t == character_Type));
+}
+
+static void initializeParmDescriptionBlock (Procedure m, Context c)
+{
+  cardinal	i;
+  cardinal	element;
+  Argument	a;
+
+  element = 0;
+  if (m->returnType != NULL) {
+      fprintf (c->file, "  __parmsBlock[%u].parm_in  = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_out = 1;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_assignment   = %u;\n", element,
+	       ((VariableLength(m->returnType) &&
+		 (!TypeIsEitherString(m->returnType)) &&
+		 (type_ur_kind(m->returnType) != pickle_Type))
+		|| (type_ur_kind(m->returnType) == array_Type)
+		|| (type_ur_kind(m->returnType) == object_Type)) ? 1 : 0);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_dereference  = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_type = %s;\n", element, c_typecode_name(m->returnType));
+      fprintf (c->file, "  __parmsBlock[%u].parm_val  = (void *) &_retvalue;\n", element);
+      element++;
+  };
+  /* do input args */
+  for (i = 0;  i < list_size(m->arguments);  i++) {
+    a = (Argument) list_ref(m->arguments, i);
+    if (a->direction == In) {
+      fprintf (c->file, "  __parmsBlock[%u].parm_in  = 1;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_out = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_assignment   = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_dereference  = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_type = %s;\n", element, c_typecode_name(a->type));
+      fprintf (c->file, "  __parmsBlock[%u].parm_val  = (void *) %s%s;\n", element,
+	       (IsScalar(a->type) ||
+		(type_ur_kind(a->type) == enumeration_Type)) ? "&" : "", c_argument_name(a));
+      element++;
+    } else if (a->direction == InOut) {
+      fprintf (c->file, "  __parmsBlock[%u].parm_in  = 1;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_out = 1;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_assignment   = %u;\n", element,
+	       (type_ur_kind(a->type) == object_Type) ? 1 : 0);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_dereference  = %u;\n", element,
+	       ((type_ur_kind(a->type) == optional_Type) ||
+		(type_ur_kind(a->type) == object_Type) ||
+		(TypeIsEitherString(a->type))) ? 1 : 0);
+      fprintf (c->file, "  __parmsBlock[%u].parm_type = %s;\n", element, c_typecode_name(a->type));
+      fprintf (c->file, "  __parmsBlock[%u].parm_val  = (void *) %s;\n", element,
+	       c_argument_name(a));
+      element++;
+    } else if (a->direction == Out) {
+      fprintf (c->file, "  __parmsBlock[%u].parm_in  = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_out = 1;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_assignment   = %u;\n", element,
+	       ((VariableLength(a->type) &&
+		 (!TypeIsEitherString(a->type)) &&
+		 (type_ur_kind(a->type) != pickle_Type))
+		|| (type_ur_kind(a->type) == object_Type)) ? 1 : 0);
+      fprintf (c->file, "  __parmsBlock[%u].parm_needs_dereference  = 0;\n", element);
+      fprintf (c->file, "  __parmsBlock[%u].parm_type = %s;\n", element, c_typecode_name(a->type));
+      fprintf (c->file, "  __parmsBlock[%u].parm_val  = (void *) %s;\n", element, c_argument_name(a));
+      element++;
+    }
+  }
 }
 
 static void addExceptionInputFn (refany elt, refany rock)
@@ -301,7 +387,7 @@ static void addExceptionInputFn (refany elt, refany rock)
     fprintf(f, "sizeof(%s), ", c_type_name(ut));
 
   if (tk == object_Type)
-    fprintf(f, "_%s__ILUType, ", c_type_name(ut));
+    fprintf(f, "&_%s__ILUType, ", c_type_name(ut));
   else
     fprintf(f, "ILU_NIL, ");
 
@@ -352,22 +438,34 @@ static void addExceptionInputFn (refany elt, refany rock)
     break;
 
   case enumeration_Type:
-    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputEnum, 0");
+    fprintf(f, "0, 0, (ILU_C_InputFn) _ILU_C_Enumeration__Input, 0");
     break;
 
   case object_Type:
     fprintf(f, "0, 0, 0, 0");
     break;
 
+  case sequence_Type:
+    if (TypeIsString(ut))
+      fprintf(f, "0, 0, (ILU_C_InputFn) _ilu_CString__Input, (ILU_C_FreeFn) ilu_CString__Free");
+    else
+      fprintf(f, "0, 0, (ILU_C_InputFn) _%s__Input, (ILU_C_FreeFn) %s__Free", c_type_name(ut), c_type_name(ut));
+    break;
+
+#ifdef ADD_VARIANT_SUPPORT
+  case pickle_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) _CORBA_any__Input, (ILU_C_FreeFn) CORBA_any__Free");
+    break;
+#endif
+
   case array_Type:
   case record_Type:
   case union_Type:
-  case sequence_Type:
   case optional_Type:
     fprintf(f, "0, 0, (ILU_C_InputFn) _%s__Input, ", c_type_name(ut));
-    if (HasFreeRoutine(ut))
-      fprintf(f, "(ILU_C_FreeFn) %s__Free", c_type_name(ut));
-    else
+    if (NeedsFree(ut)) {
+	fprintf(f, "(ILU_C_FreeFn) %s__Free", c_type_name(ut));
+    } else
       fprintf(f, "0");
     break;
 
@@ -392,9 +490,27 @@ static void declareExceptionValInputFns (list exceptions, Context context)
   fprintf(context->file, "};\n\n");
 }
 
-static void generateMethodCode (Procedure m, Context context)
+static void generateSiblingChecks (Argument a, Context context)
 {
+  if ((type_ur_kind(a->type) == object_Type) &&
+      a->sibling &&
+      ((a->direction == In) || (a->direction == InOut))) {
+    fprintf (context->file, "  if (!_ILU_C_CheckSibling(_handle, %s, _status)) goto errout;\n",
+	     c_argument_name(a));
+  }
+}
+
+static boolean findSibling (Argument a, refany junk)
+{
+  return (a->sibling);
+}
+
+static void generateMethodCode (refany elt, refany rock)
+{
+  Procedure       m = (Procedure) elt;
+  Context         context = (Context) rock;
   char           *p;
+  cardinal	  parms_size;
   TypeKind        t = type_basic_type(ur_type(m->returnType));
 
   if (methodInList(c_simple_name(m->name))) {
@@ -403,7 +519,9 @@ static void generateMethodCode (Procedure m, Context context)
   }
   addMethodToList(c_simple_name(m->name));
 
-  generateProcHeader(m, context, FALSE);
+  generateProcDecl(m, context, TRUE, "_", "__clientstub");
+  fprintf(context->file, "\n{\n");
+  
 
   /* Declare var to hold return value, if any. */
   declareCallerReturnValue(m->returnType, context);
@@ -413,6 +531,11 @@ static void generateMethodCode (Procedure m, Context context)
    */
   declareExceptionValInputFns(m->exceptions, context);
 
+  /*
+   * Declare parameter description block
+   */
+  parms_size = declareParmDescriptionBlock(m, context);
+
   /*
    * If method is functional, see if it is cached,     and if so,
    * return the cached value.
@@ -427,36 +550,38 @@ static void generateMethodCode (Procedure m, Context context)
    * necessary.
    */
 
-  /* generate call to _ILU_C_GenericCall */
+  /* Check object types for sibling-ness */
+  list_enumerate(m->arguments, (iluparser_EnumProc) generateSiblingChecks, context);
+
+  /* Initialize vector of information for the C runtime */
+  initializeParmDescriptionBlock(m, context);
+
+  /* generate call to _ILU_C_VectorCall */
 
   fprintf(context->file,
-	  "  _ILU_C_GenericCall (_%s__ILUType, &_%s__ILUType->cl_methods[%d],\n",
+	  "  _ILU_C_VectorCall (_%s__ILUType, &_%s__ILUType->cl_methods[%d],\n",
 	  c_type_name(m->object), c_type_name(m->object),
 	  methodNdxOf(m, m->object));
-  p = genParmDescription(m);
   fprintf(context->file,
-	  "                       %s, _handle, _status, \"%s\"",
-	  list_size(m->exceptions) < 1 ? "ILU_NIL" : "_evec", p);
-  free(p);
-  list_enumerate(m->arguments, listSizeOfArgs, context);
-  list_enumerate(m->arguments, listOutputArgs, context);
-  if (t != void_Type) {
-    listArg(context, "Input", m->returnType, "&_retvalue");
-  }
-  list_enumerate(m->arguments, listInputArgs, context);
-  fprintf(context->file, "\n                      );\n");
+	  "                     %s, _handle, _status, %u, %s);\n",
+	  list_size(m->exceptions) < 1 ? "ILU_NIL" : "_evec",
+	  parms_size, (parms_size > 0) ? "__parmsBlock" : "ILU_NIL");
 
   /*
    * If we returned an error, release the storage allocated for an
    * array.
    */
 
+  if (list_find(m->arguments, (iluparser_FindProc) findSibling, context) != NULL)
+    fprintf (context->file, " errout:\n");
   if (t == array_Type) {
     fprintf(context->file, "  if (!ILU_C_SUCCESSFUL(_status))\n");
     fprintf(context->file, "    ilu_free(_retvalue);\n");
   }
   if (t != void_Type)
     fprintf(context->file, "  return _retvalue;\n");
+  else
+    fprintf(context->file, "  return;\n");
   fprintf(context->file, "}\n\n");
 }
 
@@ -501,69 +626,59 @@ static void generateClsPtrs (refany elt, refany rock)
 	  c_type_name(ur_type(t)));
 }
 
-static void generateMiMethodPtrs(Type t, Context c)
+unsigned int countSuperClasses (Type t)
 {
-#if 0
-  if (class_object(t)->superclasses != NULL)
-    list_enumerate(class_object(t)->superclasses, generateMiMethodPtrs,
-		   c);
-  if (methodInList(c_type_name(t)))
-    return;
-  addMethodToList(c_type_name(t));
-#endif
-  if (OriginalInterface(t) != c->interface)
-    return;
-  generatePrototypes(t, c);
-  fprintf(c->file,
-       "struct _%s__MethodBlock_s _%s__SurrogateMethodBlock = {\n",
-	  c_type_name(t), c_type_name(t));
-  fprintf(c->file, "  ILU_NIL /* will be set to _%s__ILUType */",
-	  c_type_name(t));
-  if (list_size(class_object(t)->methods) > 0) {
-    fprintf(c->file, ",\n  {\n");
-    list_enumerate(class_object(t)->methods, generateMethodPtrs, c);
-    fprintf(c->file, "  }\n");
+  if (class_object(t)->superclasses == NULL)
+    return 1;
+  else {
+    int i;
+    unsigned int j = 1;
+    for (i = 0;  i < list_size(class_object(t)->superclasses);  i++) {
+      j += countSuperClasses((Type) list_ref(class_object(t)->superclasses, i));
+    }
+    return j;
   }
-  fprintf(c->file, "};\n\n");
 }
 
 static void generateClassTable (Type t, Context c)
 {
   char           *tn = c_type_name(t);
+  unsigned int    nsc;
 
-  clearMethodList();
-  generateMiMethodPtrs(t, c);
-  fprintf(c->file,
-     "static _ILU_C_MethodBlock *_%s__SurrogateTypeVector[] = {\n",
-	  tn);
-
-  /* This class's data must be first in the list. */
-  /* Some runtime machinery depends on this. */
-
-  fprintf(c->file,
-	  "  (_ILU_C_MethodBlock *) &_%s__SurrogateMethodBlock,\n",
-	  tn);
-  if (class_object(t)->superclasses != NULL)
-    list_enumerate(class_object(t)->superclasses, generateClsPtrs, c);
-  fprintf(c->file, "  ILU_NIL\n};\n\n");
-}
-
-static void surrMethBlkInitPerClass(refany elt, refany rock)
-{
-  Type            t = (Type) elt;
-  Context         c = (Context) rock;
   if (OriginalInterface(t) != c->interface)
     return;
-  fprintf(c->file, "  _%s__SurrogateMethodBlock.c = _%s__ILUType;\n",
-	  c_type_name(t), c_type_name(t));
-}
-
-static void generateMethods (    Type class,     Context context)
-{
-  if  (!class)
-    return;
-
-  list_enumerate(class_object(class)->methods,  (iluparser_EnumProc) generateMethodCode, context);
+#if 1
+  fprintf(c->file,
+	  "static ILU_C_Class _%s__SurrogateClass = 0;\n",
+	  tn);
+#else
+  nsc = countSuperClasses(t);
+  clearMethodList();
+  generatePrototypes(t, c);
+  if (list_size(class_object(t)->methods) > 0) {
+    fprintf(c->file,
+	  "static _ILU_C_Method _%s__SurrogateMethodBlock[] = {\n",
+	    c_type_name(t));
+    list_enumerate(class_object(t)->methods, generateMethodPtrs, c);
+    fprintf(c->file, "};\n\n");
+  }
+  fprintf(c->file,
+  "static _ILU_C_DispatchTableSection _%s__DispatchTable[%u] = {\n",
+	  tn, nsc + 1);
+  /* This class's data must be first in the list. */
+  /* Some runtime machinery depends on this. */
+  fprintf(c->file, "  {ILU_NIL /* will be set to _%s__ILUType */,\n",
+	  c_type_name(t));
+  fprintf(c->file,
+   "   ILU_NIL /* will be set to _%s__SurrogateMethodBlock */},\n",
+	  c_type_name(t));
+  fprintf(c->file, "  {ILU_NIL, ILU_NIL},};\n\n");
+  fprintf(c->file,
+	  "static _ILU_C_Class_struct _%s__SurrogateClass[1] = {\n",
+	  tn);
+  fprintf(c->file, "  {_%s__DispatchTable, 0}};\n\n",
+	  tn);
+#endif
 }
 
 static void generateClassCode(refany elt, refany rock)
@@ -573,20 +688,53 @@ static void generateClassCode(refany elt, refany rock)
   context->class = type;
   MethodRecordID = 0;
   clearMethodList();
-  generateMethods(type, context);
+  list_enumerate(class_object(type)->methods, generateMethodCode,
+		 context);
   generateClassTable(type, context);
   generateInstanceFromSBH(type, context);
 }
 
+static void ParmPerMethod(refany elt, refany rock)
+{
+  Procedure       m = (Procedure) elt;
+  Context         context = (Context) rock;
+  fprintf(context->file, "    _%s__clientstub,\n",
+	  c_procedure_name(m));
+  return;
+}
+
+static void MethodParmsPerType(refany elt, refany rock)
+{
+  Type            class = (Type) elt, ut;
+  Context         context = (Context) rock;
+  Class           od;
+  ut = ur_type(class);
+  od = class_object(ut);
+  if (methodInList(c_type_name(ut)))
+    return;
+  addMethodToList(c_type_name(ut));
+  list_enumerate(od->methods, ParmPerMethod, context);
+  if (class_object(ut)->superclasses != NULL)
+    list_enumerate(class_object(ut)->superclasses,
+		   MethodParmsPerType, context);
+}
+
 static void RegisterSurrogateTypes(refany elt, refany rock)
 {
   Type            t = (Type) elt;
   Context         c = (Context) rock;
-  if (OriginalInterface(t) == c->interface) {
-    fprintf(c->file,
-	    "  _ILU_C_RegisterSurrogateCType (_%s__ILUType, _%s__SurrogateTypeVector);\n",
-	    c_type_name(t), c_type_name(t));
-  }
+  if (OriginalInterface(t) != c->interface)
+    return;
+  fprintf(c->file, "  _%s__SurrogateClass = %s__MakeClass(\n",
+  c_type_name(t), c_type_name(t));
+  clearMethodList();
+  MethodParmsPerType(t, c);
+  fprintf(c->file, "    0 /* no finalization */\n");
+  fprintf(c->file, "    );\n");
+  fprintf(c->file, "  _ILU_C_RegisterSurrogateCType (_%s__ILUType,\n",
+	  c_type_name(t));
+  fprintf(c->file, "\t_%s__SurrogateClass);\n",
+	  c_type_name(t));
 }
 
 static void InitializeImportedInterfaces(refany elt, refany rock)
@@ -633,7 +781,6 @@ void generateClientCode(Interface interface, FILE *file)
 		 &context);
   fprintf(file, "  _%s__GeneralInitialization();\n",
 	  c_interface_name(interface));
-  list_enumerate(interface->classes, surrMethBlkInitPerClass, &context);
   list_enumerate(interface->classes, RegisterSurrogateTypes, &context);
   fprintf(file, "  return;\n");
   fprintf(file, "}\n");
