@@ -14,7 +14,7 @@
 ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: common.c,v 1.58 1996/06/20 17:57:25 spreitze Exp $ */
+/* $Id: common.c,v 1.73 1997/10/08 00:25:17 janssen Exp $ */
 /* Last edited by Mike Spreitzer June 20, 1996 10:25 am PDT */
  
 
@@ -61,9 +61,10 @@ boolean VariableLength (Type type)
   TypeKind t = type_ur_kind(type);
   TypeDescription d = type_description(type);
 
-  return (t == sequence_Type ||
+  return (t == sequence_Type || (t == pickle_Type) ||
+	  (t == object_Type) ||
 	  (t == array_Type && VariableLength(d->structuredDes.array.type)) ||
-	  (t == record_Type && HasVarField(d->structuredDes.record)) ||
+	  (t == record_Type && HasVarField(d->structuredDes.record.fields)) ||
 	  (t == union_Type && HasVarField(d->structuredDes.uniond.types)));
 }
 
@@ -89,30 +90,13 @@ boolean IsPipeObject(
 	  && type_ur_kind (type) == pipe_Type);
 }
 
-boolean PassedByRef (Type type)
-{
-    enum PrimitiveTypes t;
-
-    if (type == NULL)
-        return (False);
-    t = type_ur_kind (type);
-    return  (t == record_Type
-	  || t == union_Type
-	  || t == array_Type
-	  || t == object_Type
-	  || t == pipe_Type
-	  || t == sequence_Type
-	  || t == longcardinal_Type
-	  || t == longinteger_Type);
-}
-
 boolean TypeIsNonObjectStruct(Type type)
 {
   TypeKind        t;
   if (type == NULL)
     return (False);
   t = type_ur_kind(type);
-  return (t == record_Type || t == union_Type);
+  return (t == record_Type || t == union_Type || t == pickle_Type);
 }
 
 boolean TypeIsArray(Type type)
@@ -128,7 +112,8 @@ boolean TypeIsString(Type type)
   return (type_kind(u) == sequence_Type &&
 	  (type_ur_kind(type_description(u)
 			->structuredDes.sequence.type)
-	   == shortcharacter_Type));
+	   == shortcharacter_Type) &&
+	  (strcmp(type_uid(type), type_uid(iluparser_CString_Type)) == 0));
 }
 
 boolean TypeIsWString(Type type)
@@ -165,14 +150,53 @@ boolean TypeIsPointer(Type type)
   return (type_ur_kind(type) == object_Type || TypeIsEitherString(type));
 }
 
-boolean HasFreeRoutine (Type type)
+boolean TypeIsJustAlias (Type type)
 {
+  return (type_kind(type) == alias_Type &&
+	  (strcmp(type_uid(type), type_uid(type->supertype)) == 0));
+}
+
+static boolean ArgHasAllocatedValue (Argument field,
+				     void *unused)
+{
+  return NeedsFree(field->type);
+}
+
+boolean NeedsFree (Type type)
+{
+  /* if the type contains any pointers to malloc'ed storage, it should
+   * have a free routine to free that storage.  The free routine doesn't
+   * free the value itself (which might be stack-allocated).
+   */
+
   TypeKind        t = type_ur_kind(type);
 
-  return (type->importInterfaceName == NULL AND
-	  (t == union_Type || t == record_Type ||
-	   t == optional_Type || t == object_Type ||
-	   t == sequence_Type || t == array_Type));
+  /* none of the primitive types have free routines;
+   * check the constructed types.
+   */
+  if (t == sequence_Type || t == optional_Type || t == object_Type || t == pickle_Type)
+    return TRUE;	/* always *may* have malloc'ed storage */
+  else if (t == record_Type)	/* check field types here */
+    return (list_find(type_description(type)->structuredDes.record.fields,
+		      (iluparser_FindProc) ArgHasAllocatedValue, NULL) != NULL);
+  else if (t == union_Type)
+    return (list_find(type_description(type)->structuredDes.uniond.types,
+		     (iluparser_FindProc) ArgHasAllocatedValue, NULL) != NULL);
+  else if (t == array_Type)
+    return NeedsFree(type_description(type)->structuredDes.array.type);
+  else
+    /* primitive types don't have allocated values */
+    return FALSE;
+}
+
+boolean HasFreeRoutine(Type type)
+{
+  if (TypeIsString(type))
+    return FALSE;	/* handle with ilu_CString__Free() */
+  else if (type_kind(type) == pickle_Type)
+    return FALSE;	/* handle with CORBA_any__Free() */
+  else
+    return NeedsFree(type);
 }
 
 boolean HasAllocRoutine (Type type)
@@ -196,7 +220,7 @@ void SizeValue (Context context, Type type, string name)
   if (t == integer_Type)
     fprintf(f, "ilu_SizeOfInteger(_call, %s, _err)", name);
   else if (t == enumeration_Type)
-    fprintf(f, "ilu_SizeOfEnum(_call, (ilu_shortcardinal) %s, _err)",
+    fprintf(f, "ilu_SizeOfEnum(_call, (ilu_shortcardinal) %s, (ilu_Type)ILU_NIL,  _err)",
 	    name);
   else if (t == cardinal_Type)
     fprintf(f, "ilu_SizeOfCardinal(_call, %s, _err)", name);
@@ -225,16 +249,17 @@ void SizeValue (Context context, Type type, string name)
   else if (t == alias_Type)
     SizeValue(context, ur_type(type), name);
   else if (TypeIsString(type))
-    fprintf(f, "_ilu_CString__SizeOf(_call, %s, _err)",
-	    name);
+    fprintf(f, "_ilu_CString__SizeOf(_call, %s, _err)", name);
+  else if (t == pickle_Type)
+    fprintf(f, "_CORBA_any__SizeOf(_call, &%s, _err)", name);
   else if (t == record_Type ||
 	   t == optional_Type ||
 	   t == union_Type ||
 	   t == sequence_Type ||
 	   t == array_Type)
-    fprintf(f, "_%s__SizeOf (_call, %s%s, _err)",
+    fprintf(f, "_%s__SizeOf (_call, %s%s,_err)",
 	    c_type_name(type),
-	    (t == record_Type || t == union_Type ||
+	    (t == record_Type || t == union_Type || t == pickle_Type ||
 	     NonStringSequence(type)) ? "&" : "",
 	    name);
   else if (t == object_Type)
@@ -259,7 +284,7 @@ void MarshallValue (Context context, Type type, string name, int indent)
   if (t == integer_Type)
     fprintf(f, "ilu_OutputInteger (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n", name);
   else if (t == enumeration_Type)
-    fprintf(f, "ilu_OutputEnum (_call, (ilu_shortcardinal) %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
+    fprintf(f, "ilu_OutputEnum (_call, (ilu_shortcardinal) %s, (ilu_Type)ILU_NIL, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
 	    name);
   else if (t == cardinal_Type)
     fprintf(f, "ilu_OutputCardinal (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n", name);
@@ -288,14 +313,16 @@ void MarshallValue (Context context, Type type, string name, int indent)
   else if (t == alias_Type)
     MarshallValue(context, ur_type(type), name, 0);
   else if (TypeIsString(type))
-    fprintf(f, "_ilu_CString__Output (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
+    fprintf(f, "_ilu_CString__Output (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n", name);
+  else if (t == pickle_Type)
+    fprintf(f, "_CORBA_any__Output (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
 	    name);
   else if (t == record_Type OR
 	   t == union_Type OR
 	   t == sequence_Type OR
 	   t == optional_Type OR
 	   t == array_Type)
-    fprintf(f, "_%s__Output(_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
+    fprintf(f, "_%s__Output(_call, %s,_err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
 	    c_type_name(type), name);
   else if (t == object_Type)
     fprintf(f,
@@ -319,6 +346,7 @@ void encodeArgument(Argument arg, Context context)
   if (arg->direction == InOut) {
     if ( (t != array_Type) && 
 	(t != sequence_Type) && 
+	(t != pickle_Type) &&
 	(t != record_Type) && 
 	(t != union_Type)) {
       strcat (b, "*");
@@ -372,22 +400,25 @@ void UnmarshallValue (Context	 context,
   else if (t == enumeration_Type) {
     fprintf(f, "{\n%*.*silu_shortcardinal _xxx;\n",
 	    indent + 2, indent + 2, "");
-    fprintf(f, "%*.*silu_InputEnum (_call, &_xxx, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
+    fprintf(f, "%*.*silu_InputEnum (_call, &_xxx, (ilu_Type)ILU_NIL, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
 	    indent + 2, indent + 2, "");
     fprintf(f, "%*.*s*%s = (%s) _xxx;\n%*.*s};\n",
 	    indent + 2, indent + 2, "",
 	    val, c_type_name(type), indent, indent, "");
   } else if (TypeIsString(type)) {
-    fprintf(f,
-	"(void) _ilu_CString__Input (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
-    /* cast because some of sign fuckups */
-	    val, c_type_name(type));
+    fprintf(f,"(void) _ilu_CString__Input (_call, %s, _err); "
+	    "if (ILU_ERRNOK(*_err)) goto marshalError;\n", val);
+  } else if (t == pickle_Type) {
+    fprintf(f, "(void) _CORBA_any__Input "
+	    "(_call, (CORBA_any *) %s, _err); "
+	    "if (ILU_ERRNOK(*_err)) goto marshalError;\n", val);
   } else if (t == array_Type ||
 	     t == sequence_Type ||
 	     t == optional_Type ||
+	     t == pickle_Type ||
 	     t == record_Type ||
 	     t == union_Type) {
-    fprintf(f, "(void) _%s__Input (_call, %s, _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
+    fprintf(f, "(void) _%s__Input (_call, %s,  _err); if (ILU_ERRNOK(*_err)) goto marshalError;\n",
 	    c_type_name(ur_type(type)), val);
   } else if (t == object_Type)
     fprintf(f,
@@ -447,4 +478,3 @@ void generateNecessaryIncludes(
   list_enumerate (context->interface->imports, (void (*)(refany, refany)) listIncludes, InterfacesToInclude);
   list_enumerate (InterfacesToInclude, (void (*)(refany, refany)) PrintInclude, context);
 }
-
