@@ -14,8 +14,8 @@
 ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: cstubber.h,v 1.25 1996/05/29 19:04:39 janssen Exp $ */
-/* Last edited by Mike Spreitzer April 2, 1996 1:06 pm PST */
+/* $Id: cstubber.h,v 1.33 1997/08/25 22:33:31 janssen Exp $ */
+/* Last edited by Mike Spreitzer March 17, 1997 12:47 pm PST */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -71,6 +71,7 @@ string          c_type_name(Type);
 string          c_parameter_type(Type, ArgDirection);
 string          c_role_type(Type type, Role role, boolean temp);
 string          c_return_type(Type);
+string		c_typecode_name(Type);
 string          c_argument_name(Argument);
 string          c_constant_name(Constant);
 string          c_exception_name(Exception);
@@ -84,11 +85,13 @@ boolean         TypeIsArray(Type);
 boolean         TypeIsString(Type);
 boolean         TypeIsWString(Type);
 boolean         TypeIsEitherString(Type);
+boolean		TypeIsUnbrandedVariant(Type);	/* CORBA any */
 boolean         TypeIsNonObjectStruct(Type);
 boolean         VariableLength(Type);
 boolean         NonStringSequence(Type);
 boolean         Sliced(Type type, Role role);
-boolean         HasFreeRoutine(Type);
+boolean         HasFreeRoutine(Type);	/* has generated free routine */
+boolean		NeedsFree(Type);	/* contains malloc'ed storage */
 boolean         BlockingCall(Procedure);
 void            fatal(char *,...);
 void            error(char *,...);
@@ -109,7 +112,6 @@ boolean         IsCacheableMethod(Procedure m);
 boolean         PassedByRef(Type type);
 void            encodeArgument(Argument arg, Context context);
 
-void            generateProcHeader(Procedure m, Context c, boolean external);
 int             methodNdxOf(Procedure p, Type t);
 
 boolean HasAllocRoutine(Type type);
@@ -119,6 +121,22 @@ extern void     codeType(Type, char **, Role);
 extern char    *interface_header_name(char *pc_interfacename);
 extern char    *InterfaceHeaderTranslationFile;
 
+extern unsigned int countSuperClasses(Type t);
+/* Count includes (t) itself. */
+
+extern void
+generateProcDecl(Procedure m, Context c, boolean urname,
+		 char *name_prefix, char *name_suffix);
+/*
+ * Writes the "<return type> <class_name>_<meth_name>(<args>)" part
+ * of the C mapping of the given method.  If (urname), the named
+ * class is the one that introduced the method; otherwise, it's
+ * (c->class).
+ */
+
+extern void generateClassMakerDecl(Type class, Context context);
+/* Writes "ILU_C_Class <name(class)>__MakeClass(...)". */
+
 extern boolean	TypeIsPointer(Type type);
 /*
  * Returns true if the C mapping of the given type is a pointer.
@@ -126,6 +144,12 @@ extern boolean	TypeIsPointer(Type type);
  * is not a pointer (but implicitly converts to one when needed).
  */
 
+extern boolean	TypeIsJustAlias (Type);
+/*
+ * Returns true if Type is just an alias; that is, if the alias type
+ * and the original type have the same type ID.
+ */
+
 extern int      Indirectness(Type type, Role role, boolean temp, int ss);
 /*
  * How many asterisks appear in this position in table 14.19?  Note
@@ -176,11 +200,11 @@ look like this:
 
 void	<c_type_name(t)>__Free(<c_role_type(type, role_Exn, FALSE)> ref);
 
-void	_<c_type_name(t)>__Output(ilu_Call c, <c_parameter_type(t,In)> v,
+void	_<c_type_name(t)>__Output(ilu_Call c, <c_parameter_type(t,InOut)> v,
 				  ilu_Error *e);
 
 ilu_cardinal
-	_<c_type_name(t)>__SizeOf(ilu_Call c, <c_parameter_type(t, In)> v,
+	_<c_type_name(t)>__SizeOf(ilu_Call c, <c_parameter_type(t, InOut)> v,
 				  ilu_Error *e);
 
 <c_role_type(t, role_InpRet, FALSE)>
@@ -205,6 +229,9 @@ DataType	role_In		role_InOut	role_Out	Return
 scalar		type		type*		type*		type
 (temp)		type		type		type		type
 
+enum		type		type*		type*		type
+(temp)		type		type		type		type
+
 optional	type		type*		type*		type
 (temp)		type		type		type		type
 
@@ -226,6 +253,9 @@ union, var	union*		union*		union**		union*
 string		string		string*		string*		string
 (temp)		string		string		string		string
 
+wstring		wstring		wstring*	wstring*	wstring
+(temp)		wstring		wstring		wstring		wstring
+
 sequence	descr*		descr*		descr**		descr*
 (temp)		descr		descr		descr*		descr*
 
@@ -240,7 +270,8 @@ There are three more ILU-specific columns, for exception parameters, plain value
 Datatype	role_Exn	role_Val	role_InpRet
 --------	-------		-----		--------
 array		array*		array		slice*
-string		string*		string		string
+string		string*		string		string*
+wstring		wstring*	wstring		wstring*
 other		type*		type		type*
 
 ********************************/
