@@ -14,7 +14,7 @@
  ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
  ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  */
-/* $Id: names.c,v 1.27 1996/04/03 05:33:55 spreitze Exp $ */
+/* $Id: names.c,v 1.32 1997/09/29 16:54:04 larner Exp $ */
 /* Last edited by Mike Spreitzer April 2, 1996 3:18 pm PST */
 
 #include "cstubber.h"
@@ -23,7 +23,9 @@
 #define UNDERSCORE '_'
 #define EOS '\0'
 
-#define Prefix "ilu--prefix-idlAttribute-"
+#define OMGIDLAttributePrefix "ilu--prefix-idlAttribute-"
+#define OMGIDLExceptionNamePrefix "ilu--prefix-idlExceptionType-"
+#define OMGIDLExceptionNamePrefixLen strlen(OMGIDLExceptionNamePrefix)
 
 enum syntype {
   synonym_Method, synonym_Type, synonym_Exception, synonym_Constant, synonym_Interface, synonym_Invalid
@@ -157,8 +159,8 @@ static void do_C_fixups (buf)
   /* replace hyphen with underscore */
 
   for (p = buf; *p != '\0'; p++) {
-    if (strncmp (p, Prefix, strlen (Prefix)) == 0) {
-      strcpy (p, p + strlen (Prefix));
+    if (strncmp (p, OMGIDLAttributePrefix, strlen (OMGIDLAttributePrefix)) == 0) {
+      strcpy (p, p + strlen (OMGIDLAttributePrefix));
     }
     if (*p == '-')
       *p = '_';
@@ -227,8 +229,8 @@ static void get_builtin_name (char *buf, Type t)
     case byte_Type:
       p = "CORBA_octet";
       break;
-    default:
-      p = "??";
+    case pickle_Type:
+      p = "CORBA_any";
       break;
     }
   strcpy (buf, p);
@@ -248,8 +250,12 @@ string c_type_name (t)
 	  sprintf( buf, "%s_%s", c_string (t -> importInterfaceName), name_lang_name (t -> name, "import"));
 	else if (t->builtIn)
 	  get_builtin_name (buf, t);
-	else
-	  sprintf( buf, "%s_%s", c_interface_name (t -> interface), name_base_name (t -> name));
+	else {
+	  char *basename = name_base_name(t->name);
+	  if (strncmp(basename, OMGIDLExceptionNamePrefix, OMGIDLExceptionNamePrefixLen) == 0)
+	    basename = basename + OMGIDLExceptionNamePrefixLen;
+	  sprintf( buf, "%s_%s", c_interface_name (t -> interface), basename);
+	}
 	do_C_fixups (buf);
 	n = buf;
       }
@@ -259,6 +265,22 @@ string c_type_name (t)
   return (n);
 }
 
+string c_typecode_name (Type type)
+{
+  string n;
+
+  if ((n = name_lang_name (type->name, "C-typecode")) == NULL) {
+    static char buf[ 500 ];
+    char *tname;
+
+    tname = c_type_name(type);
+    sprintf (buf, "TC_%s", tname);
+    name_set_lang_name(type->name, "C-typecode", buf);
+    n = buf;
+  }
+  return n;
+}
+
 string c_return_type (Type type)
 {
   return c_role_type(type, role_Return, FALSE);
@@ -335,8 +357,10 @@ int Indirectness(Type type, Role role, boolean temp, int ss)
   if (role == role_InpRet) {
     if (tk == array_Type)
       return ss;
+/*
     if (TypeIsEitherString(ut))
       return 0;
+*/
     return 1;
   }
   switch (tk) {
@@ -361,6 +385,7 @@ int Indirectness(Type type, Role role, boolean temp, int ss)
   case enumeration_Type:
     return (role == role_Out || role == role_InOut);
   case union_Type:
+  case pickle_Type:
   case record_Type:
     if (role == role_In || role == role_InOut)
       return 1;
