@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1991-1995 Xerox Corporation.  All Rights Reserved.  
+Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.  
 
 Unlimited use, reproduction, and distribution of this software is
 permitted.  Any copy of this software must include both the above
@@ -13,21 +13,11 @@ PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
-*/
-/* Last edited by Mike Spreitzer December 8, 1995 10:34 am PST */
 
-/*
-$Id: code.c,v 1.65 1996/02/14 01:47:17 janssen Exp $
+$Id: code.c,v 1.71 1997/10/08 00:26:33 janssen Exp $
 */
 
-#ifdef MACOS
-#pragma segment cplcode
-#endif
-
-#ifdef WIN32
-// dll - added to get declaration for exit(int)
 #include <stdlib.h>
-#endif
 
 #include "cplusplus.h"
 
@@ -86,12 +76,17 @@ static void DefineException(refany elt, refany rock)
   fprintf(context->file, "  exnstr.%s = ilu::DefineException",
 	  cplusplus_simple_name(e->name));
   if (e->corba_rep_id == NULL)
-    fprintf(context->file, "(\"%s\", \"%s\");\n",
+    fprintf(context->file, "(\"%s\", \"%s\", ",
 	    interface_name(e->interface),
 	    exception_name(e));
   else
-    fprintf(context->file, "(NULL, \"%s\");\n",
+    fprintf(context->file, "(NULL, \"%s\", ",
 	    e->corba_rep_id);
+  if (e->type == NULL)
+    fprintf(context->file, "NULL");
+  else
+    fprintf(context->file, "\"%s\"", type_uid(e->type));
+  fprintf(context->file, ");\n");
   return;
 }
 
@@ -190,7 +185,7 @@ void generate_send_exception (Context context)
 	  "eCode > 0",
 	  cplusplus_interface_name(context->interface));
   fprintf(context->file,
-       "  _argSize = ilu::BeginSizingException(_call, eCode-1);\n");
+       "  _argSize = ilu::BeginSizingException(_call, eCode);\n");
   fprintf(context->file, "  if (stat->returnCode == NULL)\n");
   fprintf(context->file, "    0;\t/* can't happen */\n");
   list_enumerate(context->interface->exceptions, SizeException, context);
@@ -286,6 +281,39 @@ struct double_s {
   Argument default_arm;
 };
 
+
+static char* KernelTypeKindName (Type type)
+{
+  switch (type_kind(type))
+    {
+      case byte_Type: return("ilu_byte_tk");
+      case boolean_Type: return("ilu_boolean_tk");
+      case character_Type: return("ilu_character_tk");
+      case shortcharacter_Type: return("ilu_shortcharacter_tk");
+      case shortinteger_Type: return("ilu_shortinteger_tk");
+      case integer_Type: return("ilu_integer_tk");
+      case longinteger_Type: return("ilu_longinteger_tk");
+      case shortcardinal_Type: return("ilu_shortcardinal_tk");
+      case cardinal_Type: return("ilu_cardinal_tk");
+      case longcardinal_Type: return("ilu_longcardinal_tk");
+      case real_Type: return("ilu_real_tk");
+      case shortreal_Type: return("ilu_shortreal_tk");
+      case longreal_Type: return("ilu_longreal_tk");
+      case object_Type: return("ilu_object_tk");
+      case optional_Type: return("ilu_optional_tk");
+      case alias_Type: return("ilu_alias_tk");
+      case pickle_Type: return("ilu_pickle_tk");
+      case union_Type: return("ilu_union_tk");
+      case sequence_Type: return("ilu_sequence_tk");
+      case record_Type: return("ilu_record_tk");
+      case array_Type: return("ilu_array_tk");
+      case enumeration_Type: return("ilu_enumeration_tk");
+      case pipe_Type: return("ilu_enumeration_tk");
+      default: return NULL;
+    }
+}
+
+
 static void OutputRecordArg (refany elt, refany rock)
 {
   Argument        arg = (Argument) elt;
@@ -401,8 +429,8 @@ static void generate_output_code (Type type, enum PrimitiveTypes t, Context cont
     s.default_arm = d->structuredDes.uniond.default_arm;
 
     MarshO(context,
-	    "ilu::OutputUnion (_call, (ilu_Cardinal) _val->discriminator, sizeof(%s))",
-	    cplusplus_type_name(s.t));
+	    "ilu::OutputUnion (_call, (ilu_Cardinal) _val->discriminator, %s)",
+	     KernelTypeKindName(s.t));
     fprintf(context->file, "\tswitch (_val->discriminator) {\n");
     list_enumerate(d->structuredDes.uniond.types, OutputUnionType, &s);
     if (s.default_arm != NULL)
@@ -419,7 +447,7 @@ static void generate_output_code (Type type, enum PrimitiveTypes t, Context cont
     fprintf(context->file, "\t}\n\tilu::EndUnion(_call);\n");
   } else if (t == record_Type) {
     MarshO(context, "ilu::OutputRecord (_call)");
-    list_enumerate(d->structuredDes.record, OutputRecordArg, context);
+    list_enumerate(d->structuredDes.record.fields, OutputRecordArg, context);
     fprintf(context->file, "\tilu::EndRecord (_call);\n");
   }
   /* else if (t == object_Type) done elsewhere */
@@ -533,7 +561,7 @@ static void generate_input_code (Type type, enum PrimitiveTypes t, Context conte
       s.default_arm = d->structuredDes.uniond.default_arm;
 
       fprintf (context->file, "\tilu_Cardinal discriminator;\n");
-      fprintf (context->file, "\tilu::InputUnion (_call, &discriminator, sizeof(%s));\n", cplusplus_type_name(s.t));
+      fprintf (context->file, "\tilu::InputUnion (_call, &discriminator, %s);\n", KernelTypeKindName(s.t));
       fprintf (context->file, "\tif (_ref != NULL) _val = _ref; else _val = (%s) malloc(sizeof(%s));\n",
 	       ret, name);
       fprintf (context->file, "\tswitch (discriminator) {\n");
@@ -556,7 +584,7 @@ static void generate_input_code (Type type, enum PrimitiveTypes t, Context conte
     {
       fprintf (context->file, "\tilu::InputRecord(_call);\n");
       fprintf (context->file, "\tif (_ref != NULL) _val = _ref; else _val = (%s) malloc(sizeof(%s));\n", ret, name);
-      list_enumerate (d->structuredDes.record, (iluparser_EnumProc) InputRecordArg, context);
+      list_enumerate (d->structuredDes.record.fields, (iluparser_EnumProc) InputRecordArg, context);
       fprintf (context->file, "\tilu::EndRecord(_call);\n");
     }
   /* else if (t == object_Type) done elsewhere */
@@ -663,8 +691,8 @@ generate_sizeof_code(Type type, enum PrimitiveTypes t,
 
     fprintf(context->file, "\tilu_Cardinal _dSize, _argSize = 0;\n");
     MarshS(context,
-       "ilu::SizeOfUnion (_call, _val->discriminator, sizeof(%s))",
-	   cplusplus_type_name(s.t));
+       "ilu::SizeOfUnion (_call, _val->discriminator, %s)",
+	   KernelTypeKindName(s.t));
     fprintf(context->file, "\tswitch (_val->discriminator) {\n");
     list_enumerate(d->structuredDes.uniond.types, SizeOfUnionType, &s);
     if (s.default_arm != NULL)
@@ -684,7 +712,7 @@ generate_sizeof_code(Type type, enum PrimitiveTypes t,
   } else if (t == record_Type) {
     fprintf(context->file, "\tilu_Cardinal _dSize, _argSize = 0;\n");
     fprintf(context->file, "\t_argSize = ilu::SizeOfRecord (_call);\n");
-    list_enumerate(d->structuredDes.record, SizeRecordArg, context);
+    list_enumerate(d->structuredDes.record.fields, SizeRecordArg, context);
     fprintf(context->file, "\tilu::EndRecord(_call);\n");
     fprintf(context->file, "\treturn (_argSize);\n");
     fprintf(context->file, "faild:\n");
@@ -788,7 +816,7 @@ static void generateFreeCode (Type type, enum PrimitiveTypes t, Context context)
     }
   else if (t == record_Type)
     {
-      list_enumerate (d->structuredDes.record, (iluparser_EnumProc) FreeRecordArg, context);
+      list_enumerate (d->structuredDes.record.fields, (iluparser_EnumProc) FreeRecordArg, context);
     }
   else if (t == optional_Type)
     {
@@ -850,6 +878,7 @@ static void GenMethDef (refany elt, refany rock)
   Procedure       m = (Procedure) elt;
   Context         context = (Context) rock;
   unsigned        nexns = 0;
+  unsigned	  nargs = list_size(m->arguments);
   fprintf(context->file, "    {\t//for method %s\n", procedure_name(m));
   if (m->exceptions != NULL && (nexns = list_size(m->exceptions)) > 0) {
     fprintf(context->file, "      ilu_Exception exns[%lu];\n",
@@ -867,7 +896,12 @@ static void GenMethDef (refany elt, refany rock)
   fprintf(context->file, "\t%s,\t//asynchronous\n",
 	  m->asynch ? "ilu_kernelTRUE" : "ilu_kernelFALSE");
   fprintf(context->file, "\t%u,\t//num. exns\n", nexns);
-  fprintf(context->file, "\texns);\t//the exceptions\n");
+  fprintf(context->file, "\texns,\t//the exceptions\n");
+  fprintf(context->file, "\t%u,\t//num. args\n", nargs);
+  fprintf(context->file, "\t%s%s%s);\t//return type\n",
+	  (m->returnType == NULL) ? "" : "\"",
+	  (m->returnType == NULL) ? "(ilu_CString) 0" : type_uid(m->returnType),
+	  (m->returnType == NULL) ? "" : "\"");
   fprintf(context->file, "    }\t//end method %s\n", procedure_name(m));
   MethodIndex += 1;
 }
@@ -988,7 +1022,7 @@ static void generate_registration_code (Interface interface, Context context)
 	  interface_name);
 
   fprintf(context->file,
-	  "static class _%s_RegistrationClass _%s_RegistrationInstance;\n\n",
+	  "#ifndef macintosh\nstatic class _%s_RegistrationClass _%s_RegistrationInstance;\n#endif\n\n",
 	  interface_name, interface_name);
 
   fprintf(context->file, "void %s__Initialize(void)\n{\n",
