@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: common.c,v 1.45 1996/06/21 16:15:16 spreitze Exp $
+$Id: common.c,v 1.46 1997/10/08 00:26:33 janssen Exp $
 */
 /* Last edited by Mike Spreitzer June 21, 1996 9:14 am PDT */
 
@@ -107,7 +107,7 @@ boolean HasFreeRoutine (Type type)
 
   if (type_ur_kind(type) == record_Type)
     {
-      return (list_find (type_description(type)->structuredDes.record,
+      return (list_find (type_description(type)->structuredDes.record.fields,
 			 (iluparser_FindProc) FindComplexType, NULL) != NULL);
     }
   else if (type_ur_kind(type) == sequence_Type)
@@ -611,7 +611,7 @@ static void GenerateIncludesForType (Type t, Context context)
     }
   else if (type_basic_type(t) == record_Type)
     {
-      list_enumerate (type_description(t)->structuredDes.record, (iluparser_EnumProc) GenerateIncludeForArgument, context);
+      list_enumerate (type_description(t)->structuredDes.record.fields, (iluparser_EnumProc) GenerateIncludeForArgument, context);
     }
   else if (type_basic_type(t) == union_Type)
     {
