@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: headers.c,v 1.50 1996/05/29 19:00:59 janssen Exp $
+$Id: headers.c,v 1.51 1997/10/08 00:26:33 janssen Exp $
 */
 
 #include <stdlib.h>
@@ -91,7 +91,7 @@ static void sort_types_for_declaration (Type type, list sorted)
 	{
 	case record_Type:
 	      
-	  list_enumerate(type_description(type)->structuredDes.record, (iluparser_EnumProc) SortArgTypes, sorted);
+	  list_enumerate(type_description(type)->structuredDes.record.fields, (iluparser_EnumProc) SortArgTypes, sorted);
 	  break;
 	      
 	case union_Type:
@@ -147,7 +147,7 @@ static void GenerateRecordField (Argument a, Context context)
 static void GenerateRecordDeclaration (Type t, Context context)
 {
   fprintf (context->file, "struct _%s_record {\n", cplusplus_type_name(t));
-  list_enumerate (type_description(t)->structuredDes.record, (iluparser_EnumProc) GenerateRecordField, context);
+  list_enumerate (type_description(t)->structuredDes.record.fields, (iluparser_EnumProc) GenerateRecordField, context);
   fprintf (context->file, "};\n");
 }
 
