@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: server.c,v 1.32 1996/07/11 21:09:40 spreitze Exp $
+$Id: server.c,v 1.34 1997/06/12 00:57:22 janssen Exp $
 */
 /* Last edited by Mike Spreitzer July 11, 1996 2:09 pm PDT */
 
@@ -240,7 +240,7 @@ static void GenerateCalleeStub (Procedure m, Context context)
 
 static void InitializeStubPointer (Procedure method, Context context)
 {
-  fprintf (context->file, "\tilu_SetMethodStubProc(MethodRecord_%s_%s, (ilu_StubProc) _%s_%s_stub);\n",
+  fprintf (context->file, "\tilu_SetMethodStubProc(MethodRecord_%s_%s, (ilu_StubProc) _%s_%s_stub, ilu::CppLangIdx());\n",
 	   cplusplus_type_name(method->object), cplusplus_simple_name(method->name),
 	   cplusplus_type_name(method->object), cplusplus_simple_name(method->name));
 }
@@ -283,7 +283,7 @@ static void generate_registration_code (Interface interface, Context context)
 	  interface_name);
 
   fprintf(context->file,
-	  "static class _%s_InitializeServerClass _%s_InitializationInstance;\n\n",
+	  "#ifndef macintosh\nstatic class _%s_InitializeServerClass _%s_InitializationInstance;\n#endif\n\n",
 	  interface_name, interface_name);
 
   fprintf(context->file, "void %s__InitializeServer(void)\n{\n",
