@@ -14,16 +14,33 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: iluptype.h,v 1.55 1996/07/03 02:19:24 janssen Exp $
+$Id: iluptype.h,v 1.76 1997/11/08 02:56:04 janssen Exp $
 */
-/* Last edited by Mike Spreitzer November 10, 1994 4:27 pm PST */
+/* Last edited by Mike Spreitzer January 30, 1997 12:02 pm PST */
 
 #if (defined(WIN32)||defined(WIN16))
 #include <iluwin.h>
+#elif defined( macintosh )
+#include	<ilumac.h>
 #else
 #include <iluconf.h>
 #endif
 
+/* define dllexport to support building DLLs on Win32 */
+#if defined(WIN32)
+#if defined(ILU_BUILDING_PARSER)
+#define ILU_PARSER_PUBLIC __declspec(dllexport) extern
+#define ILU_PARSER_PUBLIC_CLASS  class __declspec(dllexport)
+#else
+#define ILU_PARSER_PUBLIC __declspec(dllimport) extern
+#define ILU_PARSER_PUBLIC_CLASS class __declspec(dllimport)
+#endif /* defined(ILU_BUILDING_PARSER) */
+#else
+#define ILU_PARSER_PUBLIC extern
+#define ILU_PARSER_PUBLIC_CLASS class
+#endif /* defined(WIN32) */
+
+
 typedef int boolean;
 #define TRUE 1
 #define FALSE 0
@@ -36,7 +53,6 @@ typedef int boolean;
 typedef unsigned long cardinal;
 
 typedef char * string;
-typedef unsigned char * bytes;
 typedef unsigned char byte;
 typedef char character;
 typedef void *refany;
@@ -66,6 +82,11 @@ typedef struct name_s {
 
 typedef enum {In, Out, InOut} ArgDirection;
 
+#define ISO_8859_1_CHARSET		4
+
+/* RFC 2044 */
+#define ISO_UNICODE_CHARSET		106
+
 /***********************************************************************
   Types needed by ILU parser
 
@@ -84,19 +105,29 @@ typedef cardinal			LineNumber;
 typedef struct ilu_constant_s	*	Constant;
 typedef struct ilu_constantvalue_s *	ConstantValue;
 typedef struct ilu_class_s	*	Class;
+typedef struct ilu_integerLiteral_s *	IntegerLiteral;
 
 enum PrimitiveTypes {
   invalid_Type, void_Type,
-  byte_Type, boolean_Type,
-  character_Type, shortcharacter_Type,
+  byte_Type, boolean_Type, fixedpoint_Type,
+  character_Type, shortcharacter_Type, string_Type,
   shortinteger_Type, integer_Type, longinteger_Type,
   shortcardinal_Type, cardinal_Type, longcardinal_Type,
   real_Type, shortreal_Type, longreal_Type,
-  object_Type, pipe_Type, optional_Type, alias_Type,
+  object_Type, pipe_Type, optional_Type, alias_Type, pickle_Type,
   union_Type, sequence_Type, record_Type, array_Type, enumeration_Type };
 
 typedef enum PrimitiveTypes TypeKind;
 
+struct ilu_integerLiteral_s {
+  boolean		small;
+  boolean		negative;
+  union {
+    unsigned long	direct;
+    char *		string;
+  } val;
+};
+
 struct ilu_imported_s {
   string		name;
   string		filename;
@@ -123,6 +154,10 @@ struct ilu_type_s {		/* A type-reference (in refman
 				 * terms) AKA type_stamp (in
 				 * ilu.bison terms). */
   Name            name;
+  list		  scoping;	/* when derived from OMG IDL, a type may have
+				   more than INTERFACE level of nesting.
+				   This list of strings contains scope names,
+				   in top down order, for this type. */
   TypeDescription description;
   /* When a type_stamp is defined in its file, this is the def. */
   Type            supertype;
@@ -164,12 +199,16 @@ struct ilu_argument_s {
   Type			type;
   ArgDirection		direction;
   boolean		sibling;
-  list			values;		/* used only for Unions */
+  list			values;		/* used only for Unions; a list of ConstantValue */
   LineNumber		def;
 };
 
 struct ilu_exception_s {
   Name			name;
+  list			scoping;	/* when derived from OMG IDL, an exception may have
+					   more than INTERFACE level of nesting.
+					   This list of strings contains scope names,
+					   in top down order, for this exception. */
   Type			type;
   boolean		valueOptional;	/* T if value may be NULL */
   LineNumber		def;
@@ -186,6 +225,10 @@ struct ilu_exception_s {
 
 struct ilu_constant_s {
   Name			name;
+  list			scoping;	/* when derived from OMG IDL, a constant may have
+					   more than INTERFACE level of nesting.
+					   This list of strings contains scope names,
+					   in top down order, for this constant. */
   LineNumber		def;
   Interface		interface;
   Type			type;
@@ -233,11 +276,11 @@ struct ilu_constantvalue_s {
   union {
     boolean b;
     struct {
-      int sign;
+      int sign;	/* -1 for negative, +1 for positive */
       unsigned long value;
     } i;
     struct {
-      int sign;
+      int sign;	/* -1 for negative, +1 for positive */
       string value;
       string fraction;
       long exponent;
@@ -270,6 +313,17 @@ struct ilu_typedes_s {
   TypeKind type;
   union
     {
+      struct {
+	IntegerLiteral min_numerator;
+	IntegerLiteral max_numerator;
+	IntegerLiteral denominator;
+      } fixed;
+
+      struct {
+	unsigned long max_length;
+	unsigned charset;
+      } string;
+
       struct {
 	Type type;
 	boolean optional;
@@ -286,9 +340,13 @@ struct ilu_typedes_s {
 	Type type;
 	list dimensions;	/* list of dimensions */
 	boolean optional;
-	} array;
+      } array;
 
-      list record;		/* list of Argument */
+      struct {
+	list fields;		/* list of Argument */
+	boolean extensible;
+	Type supertype;
+      } record;
 
       list enumeration;		/* list of EnumField */
 
@@ -301,24 +359,44 @@ struct ilu_typedes_s {
     } structuredDes;
 };
 
+/* communication with the IDL parser */
+struct idl_parse{
+  list definitions;
+  list defined_interfaces;
+  list imported_interfaces;
+  char *file;
+};
+
 /* various exported functionality */
-extern list ParseFile (string filename);	/* list of Interface found in file */
-extern Interface GetInterface (string interfacename, string filename);
-extern char *iluparser_CanonicalPathname (char *file);
-extern char *iluparser_GetProgramName (char *short_name);
-extern void iluparser_GenerateBoilerplate (FILE *file, Interface parse, char *programName, char *prefixes[2]);
-extern void iluparser_MultipleInterfaceBoilerplate (FILE *file, list interfaces, char *programName, char *prefixes[2]);
-extern char *iluparser_GetILUVersionString(void);
-extern void iluparser_RegisterInterfaceDirectories(list /* of directory names */);
-extern boolean iluparser_IsKeyword(string potential_keyword);
-extern void iluparser_ClearMarks(void);	/* clear mark bits on all known types */
+ILU_PARSER_PUBLIC list ParseFile (string filename);	/* list of Interface found in file */
+ILU_PARSER_PUBLIC Interface GetInterface (string interfacename, string filename);
+ILU_PARSER_PUBLIC char *iluparser_CanonicalPathname (char *file);
+ILU_PARSER_PUBLIC char *iluparser_GetProgramName (char *short_name);
+ILU_PARSER_PUBLIC void iluparser_GenerateBoilerplate (FILE *file, Interface parse, char *programName, char *prefixes[2]);
+ILU_PARSER_PUBLIC void iluparser_MultipleInterfaceBoilerplate (FILE *file, list interfaces, char *programName, char *prefixes[2]);
+ILU_PARSER_PUBLIC char *iluparser_GetILUVersionString(void);
+ILU_PARSER_PUBLIC void iluparser_RegisterInterfaceDirectories(list /* of directory names */);
+ILU_PARSER_PUBLIC boolean iluparser_IsKeyword(string potential_keyword);
+ILU_PARSER_PUBLIC void iluparser_ClearMarks(void);	/* clear mark bits on all known types */
+ILU_PARSER_PUBLIC string iluparser_FindFileInDir(char *,	/*dir*/
+						 char *);	/*filename*/
+/* return canonical name of file if found in dir, NULL otherwise */
+ILU_PARSER_PUBLIC string iluparser_FindFileInIncludes(string);	/* filename */
+/* return canonical name of file that appears in a #include preprocessor directive */
+
+ILU_PARSER_PUBLIC void iluparser_SetIdlIncludePath(string path);
+/*
+ * ParseFile will invoke idl2isl when given an IDL file.  (path) is
+ * the (space separated) series of "-I..." arguments to include in
+ * these invocations.
+ */
 
 /* memory management */
-extern void *iluparser_Malloc(unsigned long size);
-extern void iluparser_Free(void *ptr);
-extern void *iluparser_Realloc (void *ptr, unsigned long size);
-extern char * ilu_strdup (char *);
-extern int ilu_strcasecmp (char *s1, char *s2);	/* -n if s1 < s2, 0 if s1 == s2, n if s1 > s2 */
+ILU_PARSER_PUBLIC void *iluparser_Malloc(unsigned long size);
+ILU_PARSER_PUBLIC void iluparser_Free(void *ptr);
+ILU_PARSER_PUBLIC void *iluparser_Realloc (void *ptr, unsigned long size);
+ILU_PARSER_PUBLIC char * ilu_strdup (char *);
+ILU_PARSER_PUBLIC int ilu_strcasecmp (char *s1, char *s2);	/* -n if s1 < s2, 0 if s1 == s2, n if s1 > s2 */
 
 /* list primitives */
 
@@ -331,44 +409,53 @@ typedef boolean (*iluparser_FindProc) (refany element, refany rock);
 typedef boolean (*iluparser_CompareProc) (refany element1, refany element2);
 /* return TRUE if element1 and element2 should be swapped */
 
-extern cardinal list_size (list l);
-extern refany list_ref (list l, cardinal index);
-extern list list_cdr (list l);
-extern refany list_car (list l);
-extern void list_insert (list, refany new_element);
-extern boolean list_remove (list, refany element);
-extern void list_clear (list, boolean freeElements);
-extern void list_enumerate (list, iluparser_EnumProc proc, refany rock);
-extern refany list_find (list, iluparser_FindProc proc, refany rock);
-extern list iluparser_new_list(void);
-extern void list_push(list, refany new_element);
+ILU_PARSER_PUBLIC cardinal list_size (list l);
+ILU_PARSER_PUBLIC refany list_ref (list l, cardinal index);
+ILU_PARSER_PUBLIC list list_cdr (list l);
+ILU_PARSER_PUBLIC refany list_car (list l);
+ILU_PARSER_PUBLIC void list_insert (list, refany new_element);
+ILU_PARSER_PUBLIC boolean list_remove (list, refany element);
+ILU_PARSER_PUBLIC void list_clear (list, boolean freeElements);
+ILU_PARSER_PUBLIC void list_enumerate (list, iluparser_EnumProc proc, refany rock);
+ILU_PARSER_PUBLIC refany list_find (list, iluparser_FindProc proc, refany rock);
+ILU_PARSER_PUBLIC list iluparser_new_list(void);
+ILU_PARSER_PUBLIC void list_push(list, refany new_element);
 #define new_list iluparser_new_list
-extern void list_sort (list, iluparser_CompareProc);
+ILU_PARSER_PUBLIC void list_sort (list, iluparser_CompareProc);
 
 /* type primitives */
-extern TypeDescription type_description(Type);
-extern Type     ur_type(Type);
-extern Type     under_type(Type);
-extern TypeKind type_kind(Type);
-extern TypeKind type_basic_type(Type);	/* = type_kind */
-extern TypeKind type_ur_kind(Type);	/* type_kind(ur_type(..)) */
-extern string   type_name(Type);
-extern Interface type_interface(Type);
-extern void type_recurse(Type,
+ILU_PARSER_PUBLIC TypeDescription type_description(Type);
+ILU_PARSER_PUBLIC Type     ur_type(Type);
+ILU_PARSER_PUBLIC Type     under_type(Type);
+ILU_PARSER_PUBLIC TypeKind type_kind(Type);
+ILU_PARSER_PUBLIC TypeKind type_basic_type(Type);	/* = type_kind */
+ILU_PARSER_PUBLIC TypeKind type_ur_kind(Type);	/* type_kind(ur_type(..)) */
+ILU_PARSER_PUBLIC string   type_name(Type);
+ILU_PARSER_PUBLIC Interface type_interface(Type);
+#define type_uid(t)  ((t)->uid)
+ILU_PARSER_PUBLIC void type_recurse(Type,
 			 void (*proc) (Type, refany rock),
 			 refany rock);
-extern Class    class_object(Type t);
-extern Type	exception_type(Exception e);
+ILU_PARSER_PUBLIC Class    class_object(Type t);
+ILU_PARSER_PUBLIC Type	exception_type(Exception e);
+ILU_PARSER_PUBLIC Type	iluparser_CString_Type;
+
+/* function that combines sprintf with malloc
+   -- returns the allocated buffer containing the results
+   -- returns NIL if an error occurred
+*/
+char *aprintf (char *format, ...);
 
 /* name primitives */
-extern void name_set_base_name (Name n, string name);
-extern void name_set_lang_name (Name n, string lang, string name);
-extern string name_lang_name (Name n, string lang);
-extern string name_base_name (Name n);
+ILU_PARSER_PUBLIC void name_set_base_name (Name n, string name);
+ILU_PARSER_PUBLIC void name_set_lang_name (Name n, string lang, string name);
+ILU_PARSER_PUBLIC string name_lang_name (Name n, string lang);
+ILU_PARSER_PUBLIC string name_base_name (Name n);
 
 /* accessors for various kinds of names */
-extern string exception_name(Exception e);
-extern string interface_name(Interface i);
-extern string argument_name (Argument a);
-extern string procedure_name (Procedure);
-extern string constant_name (Constant);
+ILU_PARSER_PUBLIC string exception_name(Exception e);
+ILU_PARSER_PUBLIC string interface_name(Interface i);
+ILU_PARSER_PUBLIC string argument_name (Argument a);
+#define argument_type(a)  ((a)->type)
+ILU_PARSER_PUBLIC string procedure_name (Procedure);
+ILU_PARSER_PUBLIC string constant_name (Constant);
