@@ -14,17 +14,17 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: malloc.c,v 1.13 1996/03/27 00:54:42 mdavidso Exp $
+$Id: malloc.c,v 1.16 1997/05/16 20:50:59 janssen Exp $
 */
 
-#ifndef MACOS
 #include <stdlib.h>
 #if (defined(WIN32)||defined(WIN16))
 #include <iluwin.h>
+#elif defined( macintosh )
+#include <ilumac.h>
 #else
 #include <iluconf.h>
 #endif
-#endif
 
 #include <stdio.h>
 
@@ -80,7 +80,7 @@ void *iluparser_Realloc (void *ptr, unsigned long size)
   return (NULL);
 }
 
-#if !(defined(HAVE_ALLOCA) || defined(WIN32) || defined(WIN16))
+#if !(defined(__GNUC__) || defined(HAVE_ALLOCA) || defined(WIN32) || defined(WIN16))
 
 /*
 	alloca -- (mostly) portable public-domain implementation -- D A Gwyn
@@ -246,4 +246,4 @@ alloca (SIZE_T size)			/* returns pointer to storage */
   }
 }
 
-#endif /* !(defined(HAVE_ALLOCA) || defined(WIN32) || defined(WIN16)) */
+#endif /* !(defined(__GNUC__) || defined(HAVE_ALLOCA) || defined(WIN32) || defined(WIN16)) */
