@@ -14,13 +14,16 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: pathname.c,v 1.15 1996/03/27 00:54:13 mdavidso Exp $
+$Id: pathname.c,v 1.17 1997/03/15 21:22:37 janssen Exp $
 */
 
 #include <stdio.h>
 
 #if (defined(WIN32)||defined(WIN16))
 #include <iluwin.h>
+#elif defined( macintosh )
+#include	<unistd.h>
+#include	<ilumac.h>
 #else
 #include <iluconf.h>
 #endif
@@ -44,6 +47,10 @@ $Id: pathname.c,v 1.15 1996/03/27 00:54:13 mdavidso Exp $
 #define DIR_SEPARATOR_CHAR '\\'
 #define DIR_SEPARATOR_STR "\\"
 #define PATH_SEPARATOR_CHAR ';'
+#elif defined( macintosh )
+#define DIR_SEPARATOR_CHAR ':'
+#define DIR_SEPARATOR_STR ":"
+#define PATH_SEPARATOR_CHAR ','
 #else
 #define DIR_SEPARATOR_CHAR '/'
 #define DIR_SEPARATOR_STR "/"
@@ -62,7 +69,7 @@ char *iluparser_GetProgramName (char *shortName);
 static char *gnu_getcwd (void)
      /* from the info pages to glibc */
 {
-#if (defined(_IS_POSIX) || (defined(WIN32) || defined(WIN16)))
+#if (defined(_IS_POSIX) || (defined(WIN32) || defined(WIN16) || defined(macintosh)))
 
   int size = 100;
   char *buffer = (char *) iluparser_Malloc (size);
@@ -98,13 +105,15 @@ static char *gnu_getcwd (void)
 
 char *iluparser_CanonicalPathname (char *file)
 {
-#if (defined(_IS_BSD) || defined(_IS_POSIX) || (defined(WIN32) || defined(WIN16)))	/* in short, IS_UNIX or WIN32 */
+#if (defined(_IS_BSD) || defined(_IS_POSIX) || (defined(WIN32) || defined(WIN16) || defined(macintosh)))	/* in short, IS_UNIX or WIN32 */
   char *pathname;
   char *wd;
   unsigned int wdlen;
 
 #if (defined(WIN32) || defined(WIN16))
   if ((*file == DIR_SEPARATOR_CHAR) || ((*(file+1)) == ':')) /* path already starts at the root or has drive id*/
+#elif defined( macintosh )
+  if (*file != DIR_SEPARATOR_CHAR)	 /* path already starts at the root */
 #else
   if (*file == DIR_SEPARATOR_CHAR)	 /* path already starts at the root */
 #endif
@@ -117,7 +126,11 @@ char *iluparser_CanonicalPathname (char *file)
       wd = gnu_getcwd();
       wdlen = strlen(wd);
 
+#if defined( macintosh )
+      if (*file == DIR_SEPARATOR_CHAR) /* path relative to where we are */
+#else
       if (*file == '.' && file[1] == DIR_SEPARATOR_CHAR) /* path relative to where we are */
+#endif
 	{
 	  pathname = (char *) iluparser_Malloc (strlen(file+1) + wdlen + 1);
 	  strcpy (pathname, wd);
@@ -132,9 +145,11 @@ char *iluparser_CanonicalPathname (char *file)
 	}
     }
 
+#ifdef REMOVE_AUTOMOUNT_TMP_MNT
   if (strncmp("/tmp_mnt", pathname, 8) == 0)
     memmove (pathname, pathname + 8, strlen(pathname+7));
-  
+#endif
+
   return (pathname);
 
 #else	/* not BSD or POSIX */
@@ -242,8 +257,11 @@ char * /* result is NULL or full pathname */
     if( access(buf, X_OK) != 0 )
       continue;
 
+#ifdef REMOVE_AUTOMOUNT_TMP_MNT
     if (strncmp(buf, "/tmp_mnt/", 9) == 0)
       memmove (buf, buf+8, strlen(buf) - 7);
+#endif
+
     return buf;
   }
   /*NOTREACHED*/
