@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: typeuid.c,v 1.40 1996/07/16 01:15:29 janssen Exp $
+$Id: typeuid.c,v 1.49 1997/11/04 04:29:44 janssen Exp $
 */
 
 #include <string.h>
@@ -36,7 +36,11 @@ $Id: typeuid.c,v 1.40 1996/07/16 01:15:29 janssen Exp $
 
 #define TRUE  1
 #define FALSE 0
+
+#ifndef macintosh
 typedef int Boolean;
+#endif
+
 
 #define AND  &&
 #define OR   ||
@@ -241,16 +245,57 @@ static void ast_recurse (Type t, struct buffer_s *data)
     {
       printmToBuffer (data, "(%s)", type_name(t));
     }
+  else if (type_basic_type(t) == fixedpoint_Type)
+    {
+      IntegerLiteral min, max, denom;
+      printmToBuffer (data, "(fixedpoint ");
+      min = type_description(t)->structuredDes.fixed.min_numerator;
+      max = type_description(t)->structuredDes.fixed.max_numerator;
+      denom = type_description(t)->structuredDes.fixed.denominator;
+      if (min->negative)
+	print0ToBuffer (data, "-");
+      if (min->small)
+	printmToBuffer (data, "%lu", min->val.direct);
+      else
+	printmToBuffer (data, "%s", min->val.string);
+      print0ToBuffer (data, " ");
+      if (max->negative)
+	print0ToBuffer (data, "-");
+      if (max->small)
+	printmToBuffer (data, "%lu", max->val.direct);
+      else
+	printmToBuffer (data, "%s", max->val.string);
+      print0ToBuffer (data, " ");
+      if (denom->negative)
+	print0ToBuffer (data, "-");
+      if (denom->small)
+	printmToBuffer (data, "%lu", denom->val.direct);
+      else
+	printmToBuffer (data, "%s", denom->val.string);
+      print0ToBuffer (data, ")");
+    }
+  else if (type_basic_type(t) == string_Type)
+    {
+      printmToBuffer (data, "(string %lu %lu)",
+		      type_description(t)->structuredDes.string.max_length,
+		      type_description(t)->structuredDes.string.charset);
+    }
   else if (type_basic_type(t) == enumeration_Type)
     {
       printmToBuffer (data, "(enumeration non-extensible (supertype)");
-      list_enumerate (type_description(t)->structuredDes.record, (void (*)(refany, refany)) recurseEnumFields, data);
+      list_enumerate (type_description(t)->structuredDes.enumeration, (void (*)(refany, refany)) recurseEnumFields, data);
       print0ToBuffer (data, ")");
     }
   else if (type_basic_type(t) == record_Type)
     {
-      print0ToBuffer (data, "(record (supertype)");
-      list_enumerate (type_description(t)->structuredDes.record, (void (*)(refany, refany)) recurseRecordFields, data);
+      printmToBuffer (data, "(record %s(supertype",
+		      type_description(t)->structuredDes.record.extensible ? "extensible " : "");
+      if (type_description(t)->structuredDes.record.supertype != NULL) {
+	print0ToBuffer (data, " ");
+	ast_recurse(type_description(t)->structuredDes.record.supertype, data);
+      };
+      print0ToBuffer (data, ")");
+      list_enumerate (type_description(t)->structuredDes.record.fields, (void (*)(refany, refany)) recurseRecordFields, data);
       print0ToBuffer (data, ")");
     }
   else if (type_basic_type(t) == union_Type)
@@ -373,6 +418,8 @@ static void convbase(unsigned char *bstring,unsigned long len,char *outbuf)
 	*result='\0';
 }
 
+Type iluparser_CString_Type = NULL;
+
 char *FigureTypeUID (Type t)
 {
   struct buffer_s buffer;
@@ -441,5 +488,11 @@ char *FigureTypeUID (Type t)
   if (verbose && !t->builtIn)
     fprintf (stderr, "  uid is %s\n", t->uid);
 
+  if (iluparser_CString_Type == NULL &&
+      t->importInterfaceName == NULL &&
+      strcmp(interface_name(type_interface(t)), "ilu") == 0 &&
+      strcmp(type_name(t), "CString") == 0)
+    iluparser_CString_Type = t;
+
   return (t->uid);
 }
