@@ -14,8 +14,9 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: pprint.c,v 1.3 1996/06/21 01:48:23 janssen Exp $
+$Id: pprint.c,v 1.9 1997/11/04 02:41:48 janssen Exp $
 */
+/* Last edited by Mike Spreitzer June 16, 1997 3:11 pm PDT */
 
 #include <stdio.h>
 
@@ -138,11 +139,25 @@ static void PrintSupertype (Type t, boolean *first)
     printf (", %s", pprint_type_name(t));
 }
 
+static void PrintScoping (list scoping)
+{
+  int len, i;
+
+  if (list_size(scoping) > 1) {
+    printf ("(* IDL scoping:  ");
+    for (i = 0, len = list_size(scoping) - 1;  i < len;  i++) {
+      printf ("%s::", list_ref(scoping, i));
+    }
+    printf ("%s *)\n", list_ref(scoping, i));
+  };
+}
+
 static void PrintClass (Type t)
 {
   int len, pad;
   Class od = class_object(t);
 
+  PrintScoping(t->scoping);
   printf ("TYPE %s = OBJECT", pprint_type_name(t));
   len = strlen(pprint_type_name(t)) + 14;
   if (od->brand)
@@ -189,7 +204,7 @@ static void PrintClass (Type t)
   if (od->corba_rep_id != NULL)
     {
       string p = pprint_string (od->corba_rep_id, TRUE);
-      printf ("%sCORBAID \"%s\"", (len > 0) ? " " : "  ", p);
+      printf ("%sTYPEID \"%s\"", (len > 0) ? " " : "  ", p);
       len += strlen(p) + 10 + ((len > 0) ? 0 : 1);
       free(p);
     }
@@ -324,6 +339,7 @@ static void PrintType (Type t)
   int pad, len;
   TypeDescription d;
 
+  PrintScoping(t->scoping);
   printf ("TYPE %s = ", pprint_type_name(t));
   len = strlen(pprint_type_name(t)) + 8;
   d = type_description(t);
@@ -418,7 +434,7 @@ static void PrintType (Type t)
   if (type_basic_type(t) == record_Type)
     {
       boolean first = TRUE;
-      list_enumerate (d->structuredDes.record, (iluparser_EnumProc) PrintRecordField, &first);
+      list_enumerate (d->structuredDes.record.fields, (iluparser_EnumProc) PrintRecordField, &first);
       printf ("\n  END;\n");
     }
   else if (type_basic_type(t) == union_Type)
@@ -443,6 +459,7 @@ static void PrintException (Exception e)
   int len, pad;
   string p;
 
+  PrintScoping(e->scoping);
   len = strlen(pprint_exception_name(e)) +
     10 /* strlen("EXCEPTION ") */ + 1 /* strlen(";") */ +
     ((e->type == NULL) ? 0 : (3 + strlen(pprint_type_name(e->type))));
@@ -452,7 +469,7 @@ static void PrintException (Exception e)
   if (e->corba_rep_id != NULL)
     {
       p = pprint_string(e->corba_rep_id, TRUE);
-      printf (" CORBAID \"%s\"", p);
+      printf (" TYPEID \"%s\"", p);
       len += 11 + strlen(p);
       free(p);
     }
@@ -495,6 +512,7 @@ static void PrintConstant (Constant c)
   int len, pad;
   char buf[35];
 
+  PrintScoping(c->scoping);
   len = 9 /* strlen("CONSTANT ") */ +
     strlen(pprint_constant_name(c)) + 3 + strlen(pprint_type_name(c->type)) + 1;
   printf ("CONSTANT %s : %s ", name_base_name(c->name), pprint_type_name(c->type));
@@ -663,17 +681,17 @@ static boolean SortTypeListByName (Type a, Type b)
 
   if (a->builtIn || b->builtIn)
     return FALSE;
-  return (strcmp (pprint_type_name(a), pprint_type_name(b)) >= 0);
+  return (strcmp (pprint_type_name(a), pprint_type_name(b)) > 0);
 }
 
 static boolean SortExceptionsByName (Exception a, Exception b)
 {
-  return (strcmp (pprint_exception_name(a), pprint_exception_name(b)) >= 0);
+  return (strcmp (pprint_exception_name(a), pprint_exception_name(b)) > 0);
 }
 
 static boolean SortConstantsByName (Constant a, Constant b)
 {
-  return (strcmp (pprint_constant_name(a), pprint_constant_name(b)) >= 0);
+  return (strcmp (pprint_constant_name(a), pprint_constant_name(b)) > 0);
 }
 
 static void PrintParse (Interface s)
