@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: genstub.c,v 1.48 1996/07/03 19:23:11 janssen Exp $
+$Id: genstub.c,v 1.81 1997/11/01 03:56:17 janssen Exp $
 */
 
 #include <stdlib.h>
@@ -237,6 +237,12 @@ stConstant(Constant c, int *pCount)
 	printf("%s\n", suffix);
 }
 
+static void
+printTypeTable()
+{
+  printf ("__types__={}\n\n");
+}
+
 /************************************************************************/
 
 static void
@@ -251,6 +257,30 @@ stDefineIoFunc(Type t, const char *prefix)
 	printf("):\n");
 }
 
+static void
+stRegisterIoFuncs(Type t)
+{
+  printf ("%s.RegisterIoFuncs('%s', ",
+	  nameModuleIlu, type_uid(t));
+  printTypeIoFuncName(t, prefixFuncInput);
+  printf (", ");
+  printTypeIoFuncName(t, prefixFuncOutput);
+  printf (", ");
+  printTypeIoFuncName(t, prefixFuncSizeOf);
+  printf (", '%s.%s')\n", interface_name(type_interface(t)), type_name(t));
+}
+
+static void
+stRegisterEnumIoFuncs(Type t)
+{
+  printf ("%s.RegisterIoFuncs('%s', %s.%sEnum, %s.%sEnum, %s.%sEnum, '%s.%s')\n",
+	  nameModuleIlu, type_uid(t),
+	  nameModuleIlu, prefixFuncInput,
+	  nameModuleIlu, prefixFuncOutput,
+	  nameModuleIlu, prefixFuncSizeOf,
+	  interface_name(type_interface(t)), type_name(t));
+}
+
 /************************************/
 
 static void
@@ -297,6 +327,33 @@ stDefineEnumImageDict(Type t)
 	printf("}\n");
 }
 
+static void
+stEnumerationRegisterType(Type t)
+{
+  list fields = type_description(t)->structuredDes.enumeration;
+  EnumField field;
+  int i;
+
+  newline();
+#ifdef IIOP_PROTOCOL
+  printf ("__types__['%s'] = %s.RegisterEnumerationType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf (", %s%s%s, '%s', (",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t));
+  for (i = 0;  i < list_size(fields);  i++) {
+    field = list_ref(fields, i);
+    printf ("%s('%s', %d)",
+	    (i == 0) ? "" : ", ", field->name, field->id);
+  }
+  printf ("))\n");
+#endif /* IIOP_PROTOCOL */
+  stRegisterEnumIoFuncs(t);
+}
+
 static void
 stEnumerationType(Type t)
 {
@@ -304,6 +361,13 @@ stEnumerationType(Type t)
   printf ("class %s:", getTypeName(t));
   stDefineEnumLiterals(t);
   stDefineEnumImageDict(t);
+
+#ifdef ADD_VARIANT_SUPPORT
+
+  if (type_kind(t) != alias_Type)
+    stEnumerationRegisterType(t);
+
+#endif /* ADD_VARIANT_SUPPORT */
 }
 
 /************************************/
@@ -317,27 +381,32 @@ stRecordFieldInput(Argument f)
 }
 
 static void
-printNones (int count)
+printNone (Argument a, int *mcount)
 {
-  int i;
+  printf ("%sNone", (*mcount > 0) ? ", " : "");
+  *mcount += 1;
+}
 
-  for (i = 0;  i < count;  i++)
-    printf ("%sNone", (i > 0) ? ", " : "");
+static void enumerateRecordFields (Type t, EnumProc p, void * rock)
+{
+  if (type_description(t)->structuredDes.record.supertype != NULL)
+    enumerateRecordFields(type_description(t)->structuredDes.record.supertype, p, rock);
+  list_enumerate(type_description(t)->structuredDes.record.fields, p, rock);
 }
 
 static void
 stDefineRecordTypeInput(Type t)
 {
 	TypeDescription	d	= t->description;
-	list		fields	= d->structuredDes.record;
+	int count = 0;
 
 	stDefineIoFunc(t, prefixFuncInput);
 	printf("    %s.%sRecord(%s)\n", nameModuleIlu,
 		prefixFuncInput, nameVarCall);
 	printf("    %s = %s (", nameVarValue, getTypeName(t));
-	printNones(list_size(fields));
+	enumerateRecordFields (t, (EnumProc) printNone, &count);
 	printf(")\n");
-	list_enumerate(fields, (EnumProc) stRecordFieldInput, NULL);
+	enumerateRecordFields (t, (EnumProc) stRecordFieldInput, NULL);
 	printf("    %s.%sRecord(%s)\n", nameModuleIlu,
 		prefixFuncEnd, nameVarCall);
 	printf("    return %s\n", nameVarValue);
@@ -360,7 +429,6 @@ static void
 stDefineRecordTypeOutSize(Type t, const char *prefix)
 {
 	TypeDescription	d	= t->description;
-	list		fields	= d->structuredDes.record;
 
 	stDefineIoFunc(t, prefix);
 	if (prefix == prefixFuncSizeOf)
@@ -368,8 +436,8 @@ stDefineRecordTypeOutSize(Type t, const char *prefix)
 	else
 		printf("    ");
 	printf("%s.%sRecord(%s)", nameModuleIlu, prefix, nameVarCall);
-	list_enumerate(fields, (EnumProc) stRecordFieldOutSize,
-		(void *) prefix);
+	enumerateRecordFields (t, (EnumProc) stRecordFieldOutSize,
+			       (void *) prefix);
 	printf("\n    %s.%sRecord(%s)\n", nameModuleIlu,
 		prefixFuncEnd, nameVarCall);
 	if (prefix == prefixFuncSizeOf)
@@ -400,32 +468,96 @@ static void
 stDefineRecordTypeClass(Type t)
 {
   TypeDescription	d	= t->description;
-  list			fields	= d->structuredDes.record;
+  Type			supertype = d->structuredDes.record.supertype;
 
   newline();
-  printf ("class %s (iluRt.IluRecord):\n", getTypeName(t));
+  printf ("class %s (", getTypeName(t));
+  if (supertype != NULL) {
+    if (t->interface != supertype->interface)
+      printf ("%s.", getInterfaceName(supertype->interface));
+    printf("%s", getTypeName(supertype));
+  } else {
+    printf ("%s.IluRecord", nameModuleIlu);
+  }
+  printf ("):\n");
   printf ("    __ilu_type_name__ = '%s.%s'\n",
 	  interface_name(t->interface), type_name(t));
   printf ("    def __init__(self");
-  list_enumerate(fields, (EnumProc) listFieldParam, ", _");
+  enumerateRecordFields (t, (EnumProc) listFieldParam, ", _");
   printf ("):\n");
-  list_enumerate(fields, (EnumProc) initRecordField, 0);
-
+  enumerateRecordFields (t, (EnumProc) initRecordField, ", _");
   printf ("\n");
   printf ("    def __getinitargs__(self):\n");
   printf ("      return (");
-  list_enumerate(fields, (EnumProc) listPickleFieldParam, 0);
+  enumerateRecordFields (t, (EnumProc) listPickleFieldParam, ", _");
   printf (")");
   printf ("\n");
 }
 
+static void
+stRecordRegisterType(Type t)
+{
+  list fields = type_description(t)->structuredDes.record.fields;
+  Argument field;
+  int i;
+
+  newline();
+#ifdef IIOP_PROTOCOL
+  printf ("__types__['%s'] = %s.RegisterRecordType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf (", %s%s%s, '%s', (",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t));
+  for (i = 0;  i < list_size(fields);  i++) {
+    field = list_ref(fields, i);
+    printf ("('%s', '%s'),",
+	    argument_name(field), type_uid(argument_type(field)));
+  }
+  printf ("), %d, ", type_description(t)->structuredDes.record.extensible);
+  if (type_description(t)->structuredDes.record.supertype != NULL)
+    printf ("\"%s\")\n", type_uid(type_description(t)->structuredDes.record.supertype));
+  else
+    printf ("None)\n");
+#endif /* IIOP_PROTOCOL */
+  stRegisterIoFuncs(t);
+}
+
+static void stRecordType(Type t);
+
+static void
+  stRecordSupertype (Type t)
+{
+  Type st;
+
+  if (type_description(t)->structuredDes.record.supertype != NULL) {
+    st = ur_type(type_description(t)->structuredDes.record.supertype);
+    if ((st->interface == t->interface) && (st->importInterfaceName == NULL))
+      stRecordType(st);
+  }
+}
+
 static void
 stRecordType(Type t)
 {
+  if (t->marked) return;
+
+  stRecordSupertype(t);
+
   stDefineRecordTypeClass(t);
   stDefineRecordTypeInput(t);
   stDefineRecordTypeOutSize(t, prefixFuncOutput);
   stDefineRecordTypeOutSize(t, prefixFuncSizeOf);
+
+#ifdef ADD_VARIANT_SUPPORT
+
+  stRecordRegisterType(t);
+
+#endif /* ADD_VARIANT_SUPPORT */
+
+  t->marked = TRUE;
 }
 
 /************************************/
@@ -498,25 +630,31 @@ stUnionFieldInputOutSize(Argument f, DutRock *r)
 	}
 }
 
-static int
-unionDiscSize(Type t)
+static char *
+unionDiscType(Type t)
 {
   switch (type_ur_kind(t))
     {
     case cardinal_Type:
+      return "cardinal";
     case integer_Type:
+      return "integer";
     case enumeration_Type:
-      return (4);
+      return "enumeration";
     case shortcardinal_Type:
+      return "shortcardinal";
     case shortinteger_Type:
+      return "shortinteger";
     case character_Type:
-      return (2);
+      return "character";
     case boolean_Type:
+      return "boolean";
     case byte_Type:
+      return "byte";
     case shortcharacter_Type:
-      return (1);
+      return "shortcharacter";
     default:
-      return (4);
+      return NIL;
     }
 }
 
@@ -544,7 +682,8 @@ stDefineUnionTypeInputOutSize(Type t, const char *prefix)
 	printf("%s.%sUnion(%s", nameModuleIlu, prefix, nameVarCall);
 	if (prefix != prefixFuncInput)
 		printf(", %s", nameVarDiscrim);
-	printf(", %d)\n", unionDiscSize(d->structuredDes.uniond.discriminator_type));
+	printf(", %s.TypeKind_%s)\n", nameModuleIlu,
+	       unionDiscType(d->structuredDes.uniond.discriminator_type));
 
 	list_enumerate(fields, (EnumProc) stUnionFieldInputOutSize, &rock);
 	if (rock.count == 0)
@@ -568,12 +707,73 @@ stDefineUnionTypeInputOutSize(Type t, const char *prefix)
 		printf("    return %s\n", nameVarSize);
 }
 
+static void
+stUnionRegisterType(Type t)
+{
+  int i, default_arm, n_arms, j;
+  list arms;
+  Argument arm;
+
+  default_arm = 0;
+  arms = type_description(t)->structuredDes.uniond.types;
+  if (type_description(t)->structuredDes.uniond.default_arm != NULL) {
+    n_arms = list_size(arms);
+    for (i = 0;  i < n_arms;  i++) {
+      if (list_ref(arms, i) == type_description(t)->structuredDes.uniond.default_arm) {
+	default_arm = i+1;
+	break;
+      }
+    }
+  }
+  newline();
+#ifdef IIOP_PROTOCOL
+  n_arms = list_size(arms);
+  printf ("__types__['%s'] = %s.RegisterUnionType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf(", %s%s%s, '%s', ",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t));
+  printf ("'%s', %s.TypeKind_%s, %lu, %d, (",
+	  type_uid(type_description(t)->structuredDes.uniond.discriminator_type),
+	  nameModuleIlu, unionDiscType(ur_type(type_description(t)->structuredDes.uniond.discriminator_type)),
+	  default_arm, (type_description(t)->structuredDes.uniond.others_allowed) ? 1 : 0);
+  for (i = 0;  i < n_arms;  i++) {
+    arm = list_ref(arms, i);
+    printf("%s('%s', '%s', (", (i == 0) ? "" : ", ",
+	   (argument_name(arm) == NULL) ? "" : argument_name(arm),
+	   type_uid(ur_type(arm->type)));
+    for (j = 0;  j < list_size(arm->values);  j++) {
+      if (type_kind(ur_type(type_description(t)->structuredDes.uniond.discriminator_type)) == enumeration_Type)
+	printf("%s'%s'", (j == 0) ? "" : ", ", ((ConstantValue) (list_ref(arm->values, j)))->val.s);
+      else if (type_kind(ur_type(type_description(t)->structuredDes.uniond.discriminator_type)) == boolean_Type)
+	printf("%s%d", (j == 0) ? "" : ", ", ((ConstantValue) (list_ref(arm->values, j)))->val.b ? 1 : 0);
+      else
+	printf("%s%s%lu", (j == 0) ? "" : ", ",
+	       (((ConstantValue) (list_ref(arm->values, j)))->val.i.sign < 0) ? "-" : "",
+	       ((ConstantValue) (list_ref(arm->values, j)))->val.i.value);
+    }
+    printf("%s))", (list_size(arm->values) == 1) ? "," : "");
+  }
+  printf ("%s))\n", (n_arms == 1) ? "," : "");
+#endif /* IIOP_PROTOCOL */
+  stRegisterIoFuncs(t);
+}
+
 static void
 stUnionType(Type t)
 {
 	stDefineUnionTypeInputOutSize(t, prefixFuncInput);
 	stDefineUnionTypeInputOutSize(t, prefixFuncOutput);
 	stDefineUnionTypeInputOutSize(t, prefixFuncSizeOf);
+
+#ifdef ADD_VARIANT_SUPPORT
+
+	stUnionRegisterType(t);
+
+#endif /* ADD_VARIANT_SUPPORT */
 }
 
 /************************************/
@@ -600,7 +800,7 @@ totalArrayLength(list dimList)
 }
 
 static void
-stDefineArrayTypeInput(Type t)
+stDefineArrayTypeInput(Type t, const char *suffix)
 {
 	TypeDescription	d	= t->description;
 	Type		eType	= d->structuredDes.array.type;
@@ -611,42 +811,52 @@ stDefineArrayTypeInput(Type t)
 	int		i;
 
 	stDefineIoFunc(t, prefixFuncInput);
-	printf("    %s.%sArray(%s)\n", nameModuleIlu, prefixFuncInput,
-		nameVarCall);
 
-	for (i = 0; i < nVDims; i++)
-	{
-		long	length	= (long) list_ref(dimList, i);
+	if (suffix != NULL) {	/* special case:  1-dim string, byte, or wstring */
 
-		printLoopIndent(i);
-		printf("    %s%d = []\n", nameVarValue, i);
-		printLoopIndent(i);
-		printf("    for %s%d in range(0,%ld):\n", nameVarIndex, i,
-			length);
+	  printf ("    return %s.%s%s(%s, %lu);\n",
+		  nameModuleIlu, prefixFuncInput, suffix, nameVarCall,
+		  (unsigned long) list_ref(dimList, 0));
+
+	} else {		/* all other array types */
+
+	  printf("    %s.%sArray(%s)\n", nameModuleIlu, prefixFuncInput,
+		 nameVarCall);
+
+	  for (i = 0; i < nVDims; i++)
+	    {
+	      long	length	= (long) list_ref(dimList, i);
+
+	      printLoopIndent(i);
+	      printf("    %s%d = []\n", nameVarValue, i);
+	      printLoopIndent(i);
+	      printf("    for %s%d in range(0,%ld):\n", nameVarIndex, i,
+		     length);
+	    }
+	  printLoopIndent(nVDims);
+	  printf("    %s%d.append(", nameVarValue, nVDims - 1);
+	  if (eName)
+	    ioArraySpecialElemInput(eName,
+				    (long) list_ref(dimList, nVDims));
+	  else
+	    ioTypeInput(eType);
+	  printf(")\n");
+
+	  for (i = nVDims - 1; i > 0; i--)
+	    {
+	      printLoopIndent(i);
+	      printf("    %s%d.append(%s%d)\n", nameVarValue, i - 1,
+		     nameVarValue, i);
+	    }
+
+	  printf("    %s.%sArray(%s)\n", nameModuleIlu,
+		 prefixFuncEnd, nameVarCall);
+	  printf("    return %s0\n", nameVarValue);
 	}
-	printLoopIndent(nVDims);
-	printf("    %s%d.append(", nameVarValue, nVDims - 1);
-	if (eName)
-		ioArraySpecialElemInput(eName,
-			(long) list_ref(dimList, nVDims));
-	else
-		ioTypeInput(eType);
-	printf(")\n");
-
-	for (i = nVDims - 1; i > 0; i--)
-	{
-		printLoopIndent(i);
-		printf("    %s%d.append(%s%d)\n", nameVarValue, i - 1,
-			nameVarValue, i);
-	}
-
-	printf("    %s.%sArray(%s)\n", nameModuleIlu,
-		prefixFuncEnd, nameVarCall);
-	printf("    return %s0\n", nameVarValue);
 }
 
 static void
-stDefineArrayTypeOutSize(Type t, const char *prefix)
+stDefineArrayTypeOutSize(Type t, const char *prefix, const char *suffix)
 {
 	TypeDescription	d	= t->description;
 	Type		eType	= d->structuredDes.array.type;
@@ -659,43 +869,81 @@ stDefineArrayTypeOutSize(Type t, const char *prefix)
 	int		i;
 
 	stDefineIoFunc(t, prefix);
-	printf("    ");
-	if (prefix == prefixFuncSizeOf)
-		printf("%s = ", nameVarSize);
-	printf("%s.%sArray(%s, %ld)\n", nameModuleIlu, prefix,
-		nameVarCall, totLen);
 
-	for (i = 0; i < nVDims; i++)
-	{
-		long	length	= (long) list_ref(dimList, i);
+	if (suffix != NULL) {	/* special case:  1-dim bytes, chars, or wchars */
 
-		printLoopIndent(i);
-		printf("    for %s%d in range(0,%ld):\n", nameVarIndex, i,
-			length);
+	  printf ("    %s%s.%s%s (%s, %s, %lu)\n",
+		  (prefix == prefixFuncSizeOf) ? "return " : "",
+		  nameModuleIlu, prefix, suffix, nameVarCall,
+		  nameVarValue, (unsigned long) list_ref(dimList, 0));
+
+	} else {		/* all other arrays */
+
+	  printf("    ");
+	  if (prefix == prefixFuncSizeOf)
+	    printf("%s = ", nameVarSize);
+	  printf("%s.%sArray(%s, %ld)\n", nameModuleIlu, prefix,
+		 nameVarCall, totLen);
+
+	  for (i = 0; i < nVDims; i++)
+	    {
+	      long	length	= (long) list_ref(dimList, i);
+
+	      printLoopIndent(i);
+	      printf("    for %s%d in range(0,%ld):\n", nameVarIndex, i,
+		     length);
+	    }
+	  printLoopIndent(nVDims);
+	  printf("    ");
+	  if (prefix == prefixFuncSizeOf)
+	    printf("%s = %s + ", nameVarSize, nameVarSize);
+	  strcpy(argName, nameVarValue);
+	  for (i = 0; i < nVDims; i++)
+	    {
+	      char	subscript[64];
+
+	      sprintf(subscript, "[%s%d]", nameVarIndex, i);
+	      strcat(argName, subscript);
+	    }
+	  if (eName)
+	    ioArraySpecialElemOutSize(eName, argName, prefix,
+				      (long) list_ref(dimList, nVDims));
+	  else
+	    ioTypeOutSize(eType, argName, prefix);
+	  newline();
+
+	  printf("    %s.%sArray(%s)\n", nameModuleIlu,
+		 prefixFuncEnd, nameVarCall);
+	  if (prefix == prefixFuncSizeOf)
+	    printf("    return %s\n", nameVarSize);
 	}
-	printLoopIndent(nVDims);
-	printf("    ");
-	if (prefix == prefixFuncSizeOf)
-		printf("%s = %s + ", nameVarSize, nameVarSize);
-	strcpy(argName, nameVarValue);
-	for (i = 0; i < nVDims; i++)
-	{
-		char	subscript[64];
+}
 
-		sprintf(subscript, "[%s%d]", nameVarIndex, i);
-		strcat(argName, subscript);
-	}
-	if (eName)
-		ioArraySpecialElemOutSize(eName, argName, prefix,
-			(long) list_ref(dimList, nVDims));
-	else
-		ioTypeOutSize(eType, argName, prefix);
-	newline();
+static void
+stArrayRegisterType(Type t)
+{
+  int i;
+  list dims;
+  int dimCount = 0;
 
-	printf("    %s.%sArray(%s)\n", nameModuleIlu,
-		prefixFuncEnd, nameVarCall);
-	if (prefix == prefixFuncSizeOf)
-		printf("    return %s\n", nameVarSize);
+  newline();
+#ifdef IIOP_PROTOCOL
+  printf ("__types__['%s'] = %s.RegisterArrayType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf (", %s%s%s, '%s', ",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t));
+  printf ("'%s', (",
+	  type_uid(type_description(t)->structuredDes.array.type));
+  dims = type_description(t)->structuredDes.array.dimensions;
+  for (i = 0;  i < list_size(dims);  i++)
+    printf ("%lu,", (unsigned long) list_ref(dims, i));
+  printf ("))\n"); 
+#endif /* IIOP_PROTOCOL */
+  stRegisterIoFuncs(t);
 }
 
 static void
@@ -704,40 +952,53 @@ stArrayType(Type t)
 	TypeDescription	d	= baseTypeDescription(t);
 	Type		eType	= d->structuredDes.array.type;
 	list		dimList	= d->structuredDes.array.dimensions;
+	char *		suffix;
 
-	if (arraySpecialElemTypeName(eType) != 0 && list_size(dimList) == 1)
-		return;
-	stDefineArrayTypeInput(t);
-	stDefineArrayTypeOutSize(t, prefixFuncOutput);
-	stDefineArrayTypeOutSize(t, prefixFuncSizeOf);
+	if (list_size(dimList) == 1)
+	  suffix = arraySpecialElemTypeName(eType);
+	else
+	  suffix = NULL;	  
+
+	stDefineArrayTypeInput(t, suffix);
+	stDefineArrayTypeOutSize(t, prefixFuncOutput, suffix);
+	stDefineArrayTypeOutSize(t, prefixFuncSizeOf, suffix);
+
+#ifdef ADD_VARIANT_SUPPORT
+
+	stArrayRegisterType(t);
+
+#endif /* ADD_VARIANT_SUPPORT */
 }
 
 /************************************/
 
 static void
-stDefineSequenceTypeInput(Type t)
+stDefineSequenceTypeInput(Type t, const char *suffix)
 {
 	TypeDescription	d	= t->description;
 	Type		eType	= d->structuredDes.sequence.type;
 	cardinal	limit	= d->structuredDes.sequence.limit;
 
 	stDefineIoFunc(t, prefixFuncInput);
-	printf("    %s = %s.%sSequence(%s, %lu)\n", nameVarLength,
-		nameModuleIlu, prefixFuncInput, nameVarCall, limit);
-	printf("    %s = []\n", nameVarValue);
-	printf("    for %s in range(0, %s):\n", nameVarIndex, nameVarLength);
-
-	printf("        %s.append(", nameVarValue);
-	ioTypeInput(eType);
-	printf(")\n");
-
-	printf("    %s.%sSequence(%s)\n", nameModuleIlu,
-		prefixFuncEnd, nameVarCall);
-	printf("    return %s\n", nameVarValue);
+	if (suffix != NULL) {	/* string, or bytes, or wstring, type */
+	  printf ("    return %s.%s%s(%s, %lu)\n",
+		  nameModuleIlu, prefixFuncInput, suffix, nameVarCall, limit);
+	} else {		/* normal sequence */
+	  printf("    %s = %s.%sSequence(%s, %lu)\n", nameVarLength,
+		 nameModuleIlu, prefixFuncInput, nameVarCall, limit);
+	  printf("    %s = []\n", nameVarValue);
+	  printf("    for %s in range(0, %s):\n", nameVarIndex, nameVarLength);
+	  printf("        %s.append(", nameVarValue);
+	  ioTypeInput(eType);
+	  printf(")\n");
+	  printf("    %s.%sSequence(%s)\n", nameModuleIlu,
+		 prefixFuncEnd, nameVarCall);
+	  printf("    return %s\n", nameVarValue);
+	};
 }
 
 static void
-stDefineSequenceTypeOutSize(Type t, const char *prefix)
+stDefineSequenceTypeOutSize(Type t, const char *prefix, const char *suffix)
 {
 	TypeDescription	d	= t->description;
 	Type		eType	= d->structuredDes.sequence.type;
@@ -745,25 +1006,51 @@ stDefineSequenceTypeOutSize(Type t, const char *prefix)
 	char		argName[64];
 
 	stDefineIoFunc(t, prefix);
-	printf("    ");
-	if (prefix == prefixFuncSizeOf)
-		printf("%s = ", nameVarSize);
-	printf("%s.%sSequence(%s, len(%s), %lu)\n", nameModuleIlu, prefix,
-		nameVarCall, nameVarValue, limit);
-	printf("    for %s in range(0, len(%s)):\n", nameVarIndex,
-		nameVarValue);
+	if (suffix != NULL) {	/* string, or bytes, or wstring */
+	  printf ("    %s%s.%s%s (%s, %s, %lu)\n",
+		  (prefix == prefixFuncSizeOf) ? "return " : "",
+		  nameModuleIlu, prefix, suffix, nameVarCall,
+		  nameVarValue, limit);
+	} else {
+	  printf("    ");
+	  if (prefix == prefixFuncSizeOf)
+	    printf("%s = ", nameVarSize);
+	  printf("%s.%sSequence(%s, len(%s), %lu)\n", nameModuleIlu, prefix,
+		 nameVarCall, nameVarValue, limit);
+	  printf("    for %s in range(0, len(%s)):\n", nameVarIndex,
+		 nameVarValue);
+	  printf("        ");
+	  if (prefix == prefixFuncSizeOf)
+	    printf("%s = %s + ", nameVarSize, nameVarSize);
+	  sprintf(argName, "%s[%s]", nameVarValue, nameVarIndex);
+	  ioTypeOutSize(eType, argName, prefix);
+	  newline();
 
-	printf("        ");
-	if (prefix == prefixFuncSizeOf)
-		printf("%s = %s + ", nameVarSize, nameVarSize);
-	sprintf(argName, "%s[%s]", nameVarValue, nameVarIndex);
-	ioTypeOutSize(eType, argName, prefix);
-	newline();
+	  printf("    %s.%sSequence(%s)\n", nameModuleIlu,
+		 prefixFuncEnd, nameVarCall);
+	  if (prefix == prefixFuncSizeOf)
+	    printf("    return %s\n", nameVarSize);
+	}
+}
 
-	printf("    %s.%sSequence(%s)\n", nameModuleIlu,
-		prefixFuncEnd, nameVarCall);
-	if (prefix == prefixFuncSizeOf)
-		printf("    return %s\n", nameVarSize);
+static void
+stSequenceRegisterType(Type t, char *suffix)
+{
+  newline();
+#ifdef IIOP_PROTOCOL
+  printf ("__types__['%s'] = %s.RegisterSequenceType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf (", %s%s%s, '%s', ",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t));
+  printf ("'%s', %lu)\n",
+	  type_uid(type_description(t)->structuredDes.sequence.type),
+	  type_description(t)->structuredDes.sequence.limit);
+#endif /* IIOP_PROTOCOL */
+  stRegisterIoFuncs(t);
 }
 
 static void
@@ -771,20 +1058,46 @@ stSequenceType(Type t)
 {
 	TypeDescription	d	= baseTypeDescription(t);
 	Type		eType	= d->structuredDes.sequence.type;
+	char *		suffix;
 
-	if (sequenceSpecialElemTypeName(eType) != 0)
-		return;
-	stDefineSequenceTypeInput(t);
-	stDefineSequenceTypeOutSize(t, prefixFuncOutput);
-	stDefineSequenceTypeOutSize(t, prefixFuncSizeOf);
+	suffix = sequenceSpecialElemTypeName(eType);
+	stDefineSequenceTypeInput(t, suffix);
+	stDefineSequenceTypeOutSize(t, prefixFuncOutput, suffix);
+	stDefineSequenceTypeOutSize(t, prefixFuncSizeOf, suffix);
+
+#ifdef ADD_VARIANT_SUPPORT
+
+	stSequenceRegisterType(t, suffix);
+
+#endif /* ADD_VARIANT_SUPPORT */
 }
 
 /************************************/
 
+static void
+stAliasRegisterType(Type t)
+{
+  newline();
+#ifdef IIOP_PROTOCOL
+  printf ("__types__['%s'] = %s.RegisterAliasType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf (", %s%s%s, '%s', ",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t));
+  printf ("'%s')\n", type_uid(ur_type(t)));
+#endif /* IIOP_PROTOCOL */
+  printf ("%s.NoteAlias('%s.%s', '%s', '%s')\n",
+	  nameModuleIlu, interface_name(type_interface(t)), type_name(t),
+	  type_uid(t), type_uid(ur_type(t)));
+}
+
 static void
 stAliasType(Type t)
 {
-	switch (type_description(t)->type)
+	switch (type_kind(ur_type(t)))
 	{
 	case enumeration_Type:
 		stEnumerationType(t);
@@ -801,6 +1114,70 @@ stAliasType(Type t)
 		/* null */
 		break;
 	}
+#ifdef ADD_VARIANT_SUPPORT
+
+	stAliasRegisterType(t);
+
+#endif /* ADD_VARIANT_SUPPORT */
+}
+
+/************************************/
+
+static void
+stDefineOptionalTypeInput(Type t)
+{
+  stDefineIoFunc(t, prefixFuncInput);
+  printf("    return %s.%sOptional(%s) and ",
+	 nameModuleIlu, prefixFuncInput, nameVarCall);
+  ioTypeInput(type_description(t)->structuredDes.optional);
+  newline();
+}
+
+static void
+stDefineOptionalTypeOutSize(Type t, const char *prefix)
+{
+  Type base_type = type_description(t)->structuredDes.optional;
+
+  stDefineIoFunc(t, prefix);
+  if (prefix == prefixFuncSizeOf)
+    printf("    return ", nameVarSize);
+  else
+    printf("    ");
+  printf ("%s.%sOptional(%s, (%s != None)) %s ((%s != None) and ",
+	  nameModuleIlu, prefix, nameVarCall, nameVarValue,
+	  (prefix == prefixFuncSizeOf) ? "+" : "or", nameVarValue);
+  ioTypeOutSize(base_type, nameVarValue, prefix);
+  printf(")\n");
+}
+
+static void
+stOptionalRegisterType(Type t)
+{
+  newline();
+#ifdef IIOP_PROTOCOL
+  printf ("__types__['%s'] = %s.RegisterOptionalType(",
+	  getTypeName(t), nameModuleIlu);
+  printNameScopes(t->scoping);
+  printf (", %s%s%s, '%s', '%s')\n",
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	  (type_interface(t)->brand == NULL) ? "" : "'",
+	  type_uid(t), type_uid(type_description(t)->structuredDes.optional));
+#endif /* IIOP_PROTOCOL */
+  stRegisterIoFuncs(t);
+}
+
+static void	stOptional(Type t)
+{
+  stDefineOptionalTypeInput(t);
+  stDefineOptionalTypeOutSize(t, prefixFuncOutput);
+  stDefineOptionalTypeOutSize(t, prefixFuncSizeOf);
+
+#ifdef ADD_VARIANT_SUPPORT
+
+  stOptionalRegisterType(t);
+
+#endif /* ADD_VARIANT_SUPPORT */
 }
 
 /************************************/
@@ -810,21 +1187,21 @@ static void	stClass(Type t);		/* forward declaration */
 static void
 stType(Type t)
 {
-	if (t->description)
-	{
-		switch (t->description->type)
-		{
-		case enumeration_Type:	stEnumerationType(t);	break;
-		case record_Type:	stRecordType(t);	break;
-		case union_Type:	stUnionType(t);		break;
-		case array_Type:	stArrayType(t);		break;
-		case sequence_Type:	stSequenceType(t);	break;
-		case object_Type:	stClass(t);		break;
-		default:		/* null */		break;
-		}
-	}
-	else
-		stAliasType(t);
+  if (t->importInterfaceName == NULL)
+    {
+      switch (type_kind(t)) {
+	
+      case enumeration_Type:	stEnumerationType(t);	break;
+      case record_Type:		stRecordType(t);	break;
+      case union_Type:		stUnionType(t);		break;
+      case array_Type:		stArrayType(t);		break;
+      case sequence_Type:	stSequenceType(t);	break;
+      case object_Type:		stClass(t);		break;
+      case optional_Type:	stOptional(t);		break;
+      case alias_Type:		stAliasType(t);		break;
+      default:			/* null */		break;
+      }
+    }
 }
 
 /************************************************************************/
@@ -836,11 +1213,9 @@ stException(Exception e, int *pCount)
 		return;
 	if (++*pCount == 1)
 		newline();
-	if (e->corba_rep_id == NULL)
-	  printf("%s = 'ilu:%s.%s'\n", getExceptionName(e), ifcName,
-		 name_base_name(e->name));
-	else
-	  printf("%s = '%s'\n", getExceptionName(e), e->corba_rep_id);
+	printf ("%s = ", getExceptionName(e));
+	printExceptionIDString(e);
+	newline();
 }
 
 /************************************************************************/
@@ -859,26 +1234,54 @@ getMethodIndex(cardinal methodId)
 	fatal("getMethodIndex failed");
 }
 
+static void stArgumentListItem(Argument a, int *pCount)
+{
+  if (++*pCount > 1)
+    printf (", ");
+  printf("('%s', %u, %u, '%s')",
+	 getArgumentName(a),
+	 ((a->direction == In) ? 0 :
+	  ((a->direction == Out) ? 1 :
+	   ((a->direction == InOut) ? 2 : 3))),
+	 a->sibling,
+	 type_uid(a->type));
+}
+
 static void
 stExceptionListItem(Exception e, int *pCount)
 {
-	printf("%s%s", ++*pCount > 1 ? ", " : "", getExceptionName(e));
+  if (++*pCount > 1)
+    printf (", ");
+  printf ("(");
+  printExceptionIDString(e);
+  if (e->type != NULL)
+    printf(", '%s'", type_uid(e->type));
+  printf(",)");
 }
 
 static void
 stMethodRecord(Procedure m, int *pMethodIndex)
 {
-	int	nExceptions	= 0;
+	int	count	= 0;
 
 	methodIdTable[(*pMethodIndex)++ - 1] = m->id;
-	printf("\t    ('%s', %d, %s.%s, %s.%s, (",
+	printf("\t    ('%s', %d, %s.%s, %s.%s, %s%s%s, (",
 	       name_base_name(m->name),
 	       m->id,
 	       nameModuleIlu, booleanImage(m->functional),
-	       nameModuleIlu, booleanImage(m->asynch));
+	       nameModuleIlu, booleanImage(m->asynch),
+	       (m->returnType == NULL) ? "" : "\"",
+	       (m->returnType == NULL) ? "None" : type_uid(m->returnType),
+	       (m->returnType == NULL) ? "" : "\"");
+	list_enumerate(m->arguments, (EnumProc) stArgumentListItem,
+		&count);
+	if (count == 1)
+		printf(",");
+	printf("), (");
+	count = 0;
 	list_enumerate(m->exceptions, (EnumProc) stExceptionListItem,
-		&nExceptions);
-	if (nExceptions == 1)
+		&count);
+	if (count == 1)
 		printf(",");
 	printf(")),\\\n");
 }
@@ -939,23 +1342,40 @@ stArgOutSize(Argument a, const char *prefix)
 	if (a->direction == In || a->direction == InOut)
 	{
 		if (prefix == prefixFuncOutput)
-			printf("\t    ");
+			printf("\t        ");
 		else
-			printf("\\\n\t      + ");
+			printf("\\\n\t          + ");
 		ioTypeOutSize(a->type, getArgumentName(a), prefix);
 		if (prefix == prefixFuncOutput)
 			newline();
 	}
 }
 
+static void
+stMethodCheckCache(Procedure m)
+{
+  printf("\t%s = %s.%s('%s')\n\tif %s: return %s[0];\n",
+	 nameVarTemp, nameVarSelf, nameFuncCacheAccess, getProcedureName(m),
+	 nameVarTemp, nameVarTemp);
+}
+
+static void
+stMethodCacheResult(Procedure m)
+{
+  printf("\tself.%s('%s', %s);\n",
+	 nameFuncCacheValue, getProcedureName(m), nameVarResult);
+}
+
 static void
 stMethodSendRequest(Procedure m)
 {
 	Type	tClass	= m->object;
 	Class	c	= tClass->description->structuredDes.object;
-
-	printf("\t    %s.BeginRequest(%s,\\\n", nameModuleIlu, nameVarCall);
-	printf("\t      ");
+	
+	if (!(m->asynch))
+	  printf("\t    while 1:\n");
+	printf("\t        %s.BeginRequest(%s,\\\n", nameModuleIlu, nameVarCall);
+	printf("\t          ");
 	if (c->singleton == 0)
 		ioObjDiscrimOutSize(tClass, prefixFuncSizeOf);
 	else
@@ -966,14 +1386,14 @@ stMethodSendRequest(Procedure m)
 
 	if (c->singleton == 0)
 	{
-		printf("\t    ");
+		printf("\t        ");
 		ioObjDiscrimOutSize(tClass, prefixFuncOutput);
 		newline();
 	}
 	list_enumerate(m->arguments, (EnumProc) stArgOutSize,
 		(void *) prefixFuncOutput);
 
-	printf("\t    %s.FinishRequest(%s)\n", nameModuleIlu, nameVarCall);
+	printf("\t        %s.FinishRequest(%s)\n", nameModuleIlu, nameVarCall);
 }
 
 static void
@@ -981,16 +1401,20 @@ stMethodGetReply(Procedure m)
 {
 	int	nResults	= methodResultCount(m);
 
-	printf("\t    %s = %s.GetReply(%s)\n", nameVarExceptCode,
+	printf("\t        %s = %s.GetReply(%s)\n", nameVarExceptCode,
 		nameModuleIlu, nameVarCall);
-	printf("\t    if %s != 0:\n", nameVarExceptCode);
+	if (!(m->asynch))
+	  printf("\t        if %s != %s.%s: break\n",
+		 nameVarExceptCode, nameModuleIlu, nameExceptRetry);
+	printf("\t    try:\n");
+	printf("\t      if %s != 0:\n", nameVarExceptCode);
 	printf("\t\t%s(%s, %s.%s, %s)\n", nameFuncCatchExcept, nameVarCall,
 		getTypeName(m->object), nameVarClass, nameVarExceptCode);
 	if (nResults > 0)
 	{
 		int	count	= 0;
 
-		printf("\t    %s = ", nameVarResult);
+		printf("\t      %s = ", nameVarResult);
 		if (m->returnType)
 		{
 			++count;
@@ -998,7 +1422,28 @@ stMethodGetReply(Procedure m)
 		}
 		list_enumerate(m->arguments, (EnumProc) stArgInput, &count);
 		newline();
+		if (m->functional && (list_size(m->arguments) == 0))
+		  printf("\t      %s.%s('%s', %s)\n",
+			 nameVarSelf, nameFuncCacheValue,
+			 getProcedureName(m), nameVarResult);
 	}
+	printf("\t    finally:\n");
+}
+
+static void
+stCheckSibling(Argument a, refany unused)
+{
+  if ((type_ur_kind(a->type) == object_Type) &&
+      a->sibling &&
+      ((a->direction == In) || (a->direction == InOut)))
+    printf ("\tiluRt.CheckSibling(%s, %s)\n",
+	    nameVarSelf, getArgumentName(a));
+}
+
+static void
+stCheckSiblings(Procedure m)
+{
+  list_enumerate(m->arguments, (EnumProc) stCheckSibling, 0);
 }
 
 static void
@@ -1011,6 +1456,8 @@ stMethod(Procedure m)
 	printf("    def %s(%s", getProcedureName(m), nameVarSelf);
 	printArgList(m->arguments, 1);
 	printf("):\n");
+	if (m->functional && (list_size(m->arguments) == 0) && (methodResultCount(m) > 0))
+	  stMethodCheckCache(m);
 	if (methodResultCount(m) > 0)
 	  printf("\t%s = None\n", nameVarResult);
 
@@ -1035,6 +1482,7 @@ stMethod(Procedure m)
 
 #endif /* (ILUPYTHON_MAJOR_VERSION >= 1 && ILUPYTHON_MINOR_VERSION >= 2) */
 
+	stCheckSiblings(m);
 	printf("\t%s = %s.BeginCall(%s, %s.%s, %d)\n", nameVarCall, nameModuleIlu,
 		nameVarSelf, getTypeName(m->object), nameVarClass,
 		getMethodIndex(m->id));
@@ -1043,7 +1491,7 @@ stMethod(Procedure m)
 	if (m->asynch == 0)
 	  {
 	    stMethodGetReply(m);
-	    printf("\t    %s.ReplyRead(%s)\n", nameModuleIlu, nameVarCall);
+	    printf("\t      %s.ReplyRead(%s)\n", nameModuleIlu, nameVarCall);
 	  }
 	printf("\tfinally:\n\t    %s.FinishCall(%s)\n", nameModuleIlu, nameVarCall);
 	if (methodResultCount(m) > 0)
@@ -1109,10 +1557,20 @@ stClassClass(Type t)
 static void
 stRegisterClass(Type t)
 {
-	newline();
-	printf("%s.RegisterClass(", nameModuleIlu);
-	printClassVarName(t, NULL);
-	printf(")\n");
+  Class co = class_object(t);
+
+  newline();
+  printf("__types__['%s'] = %s.RegisterObjectType(", getTypeName(t), nameModuleIlu);
+  printClassVarName(t, NULL);
+  printf(", ");
+  printNameScopes(t->scoping);
+  printf(", %s%s%s, %s%s%s)\n",
+	 (type_interface(t)->brand == NULL) ? "" : "'",
+	 (type_interface(t)->brand == NULL) ? "None" : type_interface(t)->brand,
+	 (type_interface(t)->brand == NULL) ? "" : "'",
+	 (co->doc_string == NULL) ? "" : "'",
+	 (co->doc_string == NULL) ? "None" : co->doc_string,
+	 (co->doc_string == NULL) ? "" : "'");
 }
 
 static void
@@ -1127,18 +1585,17 @@ stClass(Type t)
 static void
 stCatchException(Exception e, int *pCount)
 {
-	Type	et	= e->import ? e->import->type : e->type;
-
-	printf("    %sif %s == %s:\n", ++*pCount > 1 ? "el" : "",
-	       nameVarExceptName, getExceptionName(e));
-	printf("\t%s = %s", nameVarExceptName, getExceptionName(e));
-	newline();
-	if (et)
-	  {
-	    printf("\t%s = ", nameVarExceptValue);
-	    ioTypeInput(et);
-	    newline();
-	  }
+  Type	et	= e->import ? e->import->type : e->type;
+  
+  printf("    %sif %s == %s:\n", ++*pCount > 1 ? "el" : "",
+	 nameVarExceptName, getExceptionName(e));
+  printf("\t%s = %s", nameVarExceptName, getExceptionName(e));
+  newline();
+  if (et) {
+    printf("\t%s = ", nameVarExceptValue);
+    ioTypeInput(et);
+    newline();
+  }
 }
 
 static void
@@ -1168,9 +1625,11 @@ generateStub(Interface ifc)
 	ifcName = interface_name(ifc);
 	printBanner("Stubs", ifc);
 	printImportTable();
+	printTypeTable();
 	list_enumerate(ifc->imports,    (EnumProc) stImported,  NULL);
 	list_enumerate(ifc->constants,  (EnumProc) stConstant,  &nConstants);
 	list_enumerate(ifc->exceptions, (EnumProc) stException, &nExceptions);
+	iluparser_ClearMarks();
 	list_enumerate(ifc->types,      (EnumProc) stType,      NULL);
 	stFuncCatchExceptions(ifc->exceptions);
 }
