@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: isl2python.c,v 1.11 1996/04/30 03:24:13 janssen Exp $
+$Id: isl2python.c,v 1.14 1997/10/24 02:29:15 janssen Exp $
 */
 
 #include <stdlib.h>
@@ -134,6 +134,7 @@ static int		flagPrintTree;
 static int		flagGenStub;
 static int		flagGenSkel;
 static int		flagRmFirst;
+static int		flagQuiet = 0;
 
 static void
 possibleUnlink (const char *filename)
@@ -153,22 +154,28 @@ possibleUnlink (const char *filename)
 }
 
 static void
-reopenStdout(const char *what, const char *fmtModuleName, Interface ifc)
+reopenStdout(const char *what, const char *fmtModuleName, Interface ifc, char *pathDir)
 {
-	char	moduleName[512];
-	char	fileName[512];
+	char	moduleName[2048];
+	char	fileName[2048];
 
 	sprintf(moduleName, fmtModuleName, getSimpleInterfaceName(ifc));
-	sprintf(fileName, fmtFileName, moduleName);
-	fprintf(stderr, "%s for interface \"%s\" to %s ...\n", what,
-		interface_name(ifc), fileName);
+	if (pathDir) {
+	  strcpy (fileName, pathDir);
+	  strcat (fileName, "/");
+	} else
+	  fileName[0] = 0;
+	sprintf(fileName + strlen(fileName), fmtFileName, moduleName);
+	if (! flagQuiet)
+	  fprintf(stderr, "%s for interface \"%s\" to %s ...\n", what,
+		  interface_name(ifc), fileName);
 	possibleUnlink(fileName);
 	if (freopen(fileName, "w", stdout) == NULL)
 		sysFatal(fileName);
 }
 
 static void
-generate(Interface ifc)
+generate(Interface ifc, char *pathDir)
 {
 	normalizeInterface(ifc);
 
@@ -178,14 +185,14 @@ generate(Interface ifc)
 	if (flagGenStub)
 	{
 		if (flagGenStub == 'f')
-			reopenStdout("client stubs", "%s", ifc);
+			reopenStdout("client stubs", "%s", ifc, pathDir);
 		generateStub(ifc);
 	}
 
 	if (flagGenSkel)
 	{
 		if (flagGenSkel == 'f')
-			reopenStdout("server stubs", fmtSkelModuleName, ifc);
+			reopenStdout("server stubs", fmtSkelModuleName, ifc, pathDir);
 		generateSkel(ifc);
 	}
 }
@@ -194,7 +201,7 @@ generate(Interface ifc)
 
 typedef enum
 {
-	OptTree, OptStub, OptSkel, OptRm,
+	OptTree, OptStub, OptSkel, OptRm, OptDir, OptQuiet,
 	OptNull
 } Option;
 
@@ -210,6 +217,8 @@ static OptTuple		optionTable[] =
     {	"stub",		OptStub	},
     {	"skel",		OptSkel	},
     {   "removefirst",	OptRm   },
+    {   "dir",		OptDir  },
+    {   "quiet",	OptQuiet},
 };
 
 static Option
@@ -230,7 +239,7 @@ getOption(char *name)
 static void
 usage(void)
 {
-	fprintf(stderr, "usage: %s [-removefirst] [-tree] files\n", programName);
+	fprintf(stderr, "usage: %s [-removefirst] [-tree] [-dir DIRECTORY] [-quiet] files\n", programName);
 	exit(0);
 }
 
@@ -238,8 +247,10 @@ int
 main(int argc, char **argv)
 {
 	int	optind	= 1;
+	char *	pathDir = NULL;
 
-	programName = iluparser_GetProgramName(argv[0]);
+	if ((programName = iluparser_GetProgramName(argv[0])) == NULL)
+	  programName = argv[0];
 
 	while (optind < argc && argv[optind][0] == '-')
 	{
@@ -247,10 +258,12 @@ main(int argc, char **argv)
 
 		switch (getOption(opt + 1))
 		{
-		case OptTree:		flagPrintTree = 1;	break;
-		case OptStub:		flagGenStub = 's';	break;
-		case OptSkel:		flagGenSkel = 's';	break;
-		case OptRm:		flagRmFirst = 1;	break;
+		case OptTree:		flagPrintTree = 1;		break;
+		case OptStub:		flagGenStub = 's';		break;
+		case OptSkel:		flagGenSkel = 's';		break;
+		case OptRm:		flagRmFirst = 1;		break;
+		case OptDir:		pathDir = argv[optind++];	break;
+		case OptQuiet:		flagQuiet = 1;			break;
 		default:		usage();
 		}
 	}
@@ -264,7 +277,7 @@ main(int argc, char **argv)
 
 		if ((s = ParseFile(fileName)) == 0)
 			fatal("Couldn't find or parse %s.\n", fileName);
-		list_enumerate(s, (EnumProc) generate, NULL);
+		list_enumerate(s, (EnumProc) generate, pathDir);
 	}
 
 	return 0;
