@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: name.c,v 1.6 1996/04/30 03:24:13 janssen Exp $
+$Id: name.c,v 1.9 1997/11/07 06:00:02 janssen Exp $
 */
 
 #include <stdio.h>
@@ -27,12 +27,45 @@ $Id: name.c,v 1.6 1996/04/30 03:24:13 janssen Exp $
 static void
 translateNameString(char *dst, const char *src)
 {
+ 	char *name = dst;
 	while (*src)
 	{
 		*dst++ = *src == '-' ? '_' : *src;
 		src++;
 	}
 	*dst = 0;
+ 	/* Python keywords that are not also reserved keywords in IDL
+ 	 * must be renamed by pre-pending an underscore to avoid trouble
+ 	 */
+ 	if (strcmp(name, "and") == 0
+ 	    || strcmp(name, "break") == 0
+ 	    || strcmp(name, "class") == 0
+ 	    || strcmp(name, "continue") == 0
+ 	    || strcmp(name, "def") == 0
+ 	    || strcmp(name, "del") == 0
+ 	    || strcmp(name, "elif") == 0
+ 	    || strcmp(name, "else") == 0
+ 	    || strcmp(name, "except") == 0
+ 	    || strcmp(name, "finally") == 0
+ 	    || strcmp(name, "for") == 0
+ 	    || strcmp(name, "from") == 0
+ 	    || strcmp(name, "global") == 0
+ 	    || strcmp(name, "if") == 0
+ 	    || strcmp(name, "import") == 0
+ 	    || strcmp(name, "is") == 0
+ 	    || strcmp(name, "lambda") == 0
+ 	    || strcmp(name, "not") == 0
+ 	    || strcmp(name, "or") == 0
+ 	    || strcmp(name, "pass") == 0
+ 	    || strcmp(name, "print") == 0
+ 	    || strcmp(name, "raise") == 0
+ 	    || strcmp(name, "return") == 0
+ 	    || strcmp(name, "try") == 0
+ 	    || strcmp(name, "while") == 0) {
+ 	   /* somewhat dangerous because we cannot check for overflow! */
+ 	   memmove(name + 1, name, strlen(name) + 1);
+ 	   name[0] = '_';
+ 	}
 }
 
 static char	language[] = "python";
@@ -146,15 +179,35 @@ char *
 getTypeName(Type t)
 {
 	char *	result;
+	char    buffer[1024];
+	int	idlish = 0;
 
 	result = name_lang_name(t->name, language);
-	if (result == 0)
-		result = setItemName(t->name, NULL);
+	if (result == 0) {
+	  char *basename = name_base_name(t->name);
+	  if (isPrefixOf(prefixIdlExcType, basename)) {
+	    basename += strlen(prefixIdlExcType);
+	    idlish = 1;
+	  };
+	  translateNameString(buffer, basename);
+	  if (idlish)
+	    strcat(buffer, "__omgidl_exctype");
+	  name_set_lang_name(t->name, language, buffer);
+	  result = name_lang_name(t->name, language);
+	};
 	if (t->importInterfaceName)
 		fatal("getTypeName: type is external (%s)", result);
 	return result;
 }
 
+char *
+getTypeUID(Type t)
+{
+	char *	result;
+
+	return (type_uid(t));
+}
+
 char *
 getIslProcedureName(Procedure p)
 {
