@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: util.c,v 1.7 1996/04/30 03:24:13 janssen Exp $
+$Id: util.c,v 1.14 1997/07/26 05:43:22 janssen Exp $
 */
 
 #include <stdlib.h>
@@ -106,6 +106,7 @@ simpleTypeName(Type t)
 	case shortcharacter_Type:	return "Byte";
 	case shortinteger_Type:		return "ShortInteger";
 	case shortreal_Type:		return "ShortReal";
+	case pickle_Type:		return "Pickle";
 	default:			return 0;
 	}
 }
@@ -208,6 +209,7 @@ void
 printImportTable (void)
 {
   printf ("_imported_modules = {};\n");
+  printf ("import ilu ; _imported_modules['ilu'] = ilu ; del ilu\n");
 }
 
 /************************************************************************/
@@ -252,6 +254,18 @@ printExceptionName(Exception e)
 	printf("%s", getExceptionName(e));
 }
 
+void
+printExceptionIDString(Exception e)
+{
+  Exception ur = e;
+  while (ur->import != NULL)
+    ur = ur->import;
+  if (ur->corba_rep_id == NULL)
+    printf("'ilu:%s.%s'", interface_name(ur->interface), name_base_name(e->name));
+  else
+    printf("'%s'", ur->corba_rep_id);
+}
+
 void
 printTypeIoFuncName(Type t, const char *prefix)
 {
@@ -260,3 +274,17 @@ printTypeIoFuncName(Type t, const char *prefix)
 	printIfcPrefix(urt->interface);
 	printf(fmtFuncIo, prefix, getTypeName(urt));
 }
+
+void
+printNameScopes(list scopes)
+{
+  int i;
+  string name;
+  int size = list_size(scopes);
+  printf ("(");
+  for (i = 0;  i < size;  i++) {
+    name = (string) list_ref(scopes, i);
+    printf("'%s',", name);
+  };
+  printf(")");
+}
