@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: util.h,v 1.4 1996/05/29 19:10:29 janssen Exp $
+$Id: util.h,v 1.8 1997/07/26 05:43:22 janssen Exp $
 */
 
 typedef iluparser_EnumProc	EnumProc;
@@ -45,4 +45,6 @@ extern void		printImportTable(void);
 extern void		printArgList(list argList, int nPrevArgs);
 extern void		printClassVarName(Type t, const char *varName);
 extern void		printExceptionName(Exception e);
+extern void		printExceptionIDString(Exception e);
 extern void		printTypeIoFuncName(Type t, const char *prefix);
+extern void		printNameScopes(list scopes);
